@@ -117,7 +117,6 @@ void CUnitHandler::IdleUnitUpdate(int frame) {
 }
 
 void CUnitHandler::UnitMoveFailed(int unitID) {
-	unitID = unitID;
 }
 
 // called when unit nanoframe first created
@@ -166,8 +165,9 @@ void CUnitHandler::UnitCreated(int unitID) {
 			NukeSiloAdd(unitID);
 		}
 	}
+	const bool iscommander = CUNIT::IsCommander(udef);
 
-	if (udef->isCommander && udef->canDGun) {
+	if ( iscommander && udef->canDGun) {
 		ai->dgunConHandler->AddController(unitID);
 	} else {
 		ai->GetUnit(unitID)->SetFireState(2);
