@@ -19,7 +19,8 @@ sRAIUnitDefBL::sRAIUnitDefBL(sRAIUnitDef* RAIud, sRAIBuildList* BuildList, float
 		task = Task;
 	else
 		task = -1;
-};
+}
+
 sRAIUnitDefBL::~sRAIUnitDefBL()
 {
 	for(int iUD=0; iUD<RBL->UDefSize; iUD++ )
@@ -42,7 +43,7 @@ sRAIUnitDefBL::~sRAIUnitDefBL()
 			iBL=RUD->ListSize; // end loop
 		}
 	}
-};
+}
 
 // ------------------------------------------------------------------------------------------------
 
@@ -337,7 +338,7 @@ int sRAIUnitDef::GetPrerequisite()
 	}
 
 	return ud->id;
-};
+}
 
 int sRAIUnitDef::GetPrerequisiteNewBuilder()
 {
@@ -592,19 +593,19 @@ void sRAIUnitDef::SetBestWeaponEff(sWeaponEfficiency *we, int type, float MaxFir
 
 // ------------------------------------------------------------------------------------------------
 
-sRAIBuildList::sRAIBuildList(int MaxDefSize, cRAIUnitDefHandler *UDRHandler)
+sRAIBuildList::sRAIBuildList(int MaxDefSize, cRAIUnitDefHandler *UDRHandler):
+	Name("Undefined"),
+	UDR(UDRHandler),
+	UDef(new sRAIUnitDefBL*[MaxDefSize]),
+	UDefActive(0),
+	UDefActiveTemp(0),
+	UDefSize(0),
+	priority(-1),
+	minUnits(0),
+	unitsActive(0),
+	index(UDR->BLSize),
+	minEfficiency(1.0)
 {
-	UDR = UDRHandler;
-	index = UDR->BLSize;
-	UDef = new sRAIUnitDefBL*[MaxDefSize];
-	UDefSize=0;
-	UDefActive=0;
-	UDefActiveTemp=0;
-	priority=-1;
-	minUnits=0;
-	minEfficiency=1.0;
-	unitsActive=0;
-	Name = "Undefined";
 }
 
 sRAIBuildList::~sRAIBuildList()
