@@ -1,7 +1,8 @@
-require "common"
+shard_include "common"
 
 local DebugEnabled = false
 
+
 local function EchoDebug(inStr)
 	if DebugEnabled then
 		game:SendToConsole("MexUpgradeBehaviour: " .. inStr)
@@ -44,7 +45,7 @@ function MexUpgradeBehaviour:UnitIdle(unit)
 					mohoName = "armmoho"
 				end
 				-- maybe we're underwater?
-				tmpType = game:GetTypeByName("comuwmme")
+				tmpType = game:GetTypeByName("coruwmme")
 				if builder:CanBuild(tmpType) then
 					mohoName = "coruwmme"
 				end
