@@ -31,7 +31,7 @@
 
 #include "Util.h"
 
-#include "System/maindefines.h"
+#include "System/MainDefines.h"
 #include "System/SafeCStrings.h"
 
 
