@@ -13,7 +13,7 @@
 #include <map>
 
 
-struct CommandDescription;
+struct SCommandDescription;
 
 namespace springLegacyAI {
 
@@ -132,7 +132,7 @@ public:
 	 * The commands that this group can understand, other commands will be
 	 * ignored.
 	 */
-	virtual const std::vector<CommandDescription>* GetGroupCommands(int groupId) = 0;
+	virtual const std::vector<SCommandDescription>* GetGroupCommands(int groupId) = 0;
 	virtual int GiveGroupOrder(int unitId, Command* c) = 0;
 
 	virtual int GiveOrder(int unitId, Command* c) = 0;
@@ -141,7 +141,7 @@ public:
 	 * The commands that this unit can understand, other commands will be
 	 * ignored.
 	 */
-	virtual const std::vector<CommandDescription>* GetUnitCommands(int unitId) = 0;
+	virtual const std::vector<SCommandDescription>* GetUnitCommands(int unitId) = 0;
 	virtual const CCommandQueue* GetCurrentUnitCommands(int unitId) = 0;
 
 	/*
