@@ -29,10 +29,8 @@ public:
 	virtual const T_interfaceSpecs& GetInterfaceKeys() const = 0;
 	virtual const T_skirmishAIKeys& GetSkirmishAIKeys() const = 0;
 
-	typedef std::map<const AIInterfaceKey, CAIInterfaceLibraryInfo*>
-			T_interfaceInfos;
-	typedef std::map<const SkirmishAIKey, CSkirmishAILibraryInfo*>
-			T_skirmishAIInfos;
+	typedef std::map<const AIInterfaceKey, CAIInterfaceLibraryInfo> T_interfaceInfos;
+	typedef std::map<const SkirmishAIKey, CSkirmishAILibraryInfo> T_skirmishAIInfos;
 
 	virtual const T_interfaceInfos& GetInterfaceInfos() const = 0;
 	virtual const T_skirmishAIInfos& GetSkirmishAIInfos() const = 0;
