@@ -26,7 +26,7 @@
 #include "System/FileSystem/FileSystem.h"
 #include "System/Log/ILog.h"
 #include "System/TimeProfiler.h"
-#include "System/Util.h"
+#include "System/StringUtil.h"
 
 #include <sstream>
 #include <iostream>
@@ -34,7 +34,7 @@
 
 #undef DeleteFile
 
-CR_BIND_DERIVED(CSkirmishAIWrapper, CObject, )
+CR_BIND(CSkirmishAIWrapper, )
 CR_REG_METADATA(CSkirmishAIWrapper, (
 	CR_MEMBER(key),
 
@@ -97,7 +97,7 @@ CSkirmishAIWrapper::CSkirmishAIWrapper(const int skirmishAIId):
 	teamId = aiData->team;
 	key    = aiLibManager->ResolveSkirmishAIKey(SkirmishAIKey(aiData->shortName, aiData->version));
 
-	timerName += ("AI t:" + IntToString(teamId));
+	timerName += ("AI::t:" + IntToString(teamId));
 	timerName += (" id:" + IntToString(skirmishAIId));
 	timerName += (" " + key.GetShortName());
 	timerName += (" " + key.GetVersion());
@@ -180,7 +180,7 @@ bool CSkirmishAIWrapper::LoadSkirmishAI(bool postLoad) {
 	Init();
 
 	for (size_t a = 0; a < unitHandler->MaxUnits(); a++) {
-		const CUnit* unit = unitHandler->units[a];
+		const CUnit* unit = unitHandler->GetUnit(a);
 
 		if (unit == nullptr)
 			continue;
@@ -245,6 +245,7 @@ void CSkirmishAIWrapper::Release(int reason) {
 static void streamCopy(/*const*/ std::istream* in, std::ostream* out)
 {
 	std::vector<char> buffer(128);
+	std::streampos start = in->tellg();
 
 	in->read(&buffer[0], buffer.size());
 
@@ -254,6 +255,9 @@ static void streamCopy(/*const*/ std::istream* in, std::ostream* out)
 	}
 
 	out->write(&buffer[0], in->gcount());
+
+	in->clear();  // clear fail and eof bits
+	in->seekg(start, std::ios::beg);  // back to the start!
 }
 
 static std::string createTempFileName(const char* action, int teamId, int skirmishAIId) {
