@@ -10,6 +10,7 @@ class COverheadController : public CCameraController
 {
 public:
 	COverheadController();
+	~COverheadController();
 
 	const std::string GetName() const { return "ta"; }
 
@@ -30,6 +31,9 @@ public:
 
 	bool flipped;
 
+	void ConfigNotify(const std::string& key, const std::string& value);
+	void ConfigUpdate();
+
 private:
 	float middleClickScrollSpeed;
 	float height;
