@@ -56,10 +56,9 @@ const std::string& GetBranch()
 	return patchSet;
 }
 
-std::string GetAdditional()
+inline const std::string CreateAdditionalVersion()
 {
 	std::string additional = SPRING_VERSION_ENGINE_ADDITIONAL;
-
 	additional += additional.empty() ? "" : " ";
 
 	additional += ""
@@ -123,6 +122,13 @@ std::string GetAdditional()
 	return additional;
 }
 
+const std::string& GetAdditional()
+{
+	const static std::string additional(CreateAdditionalVersion());
+
+	return additional;
+}
+
 #define QUOTEME_(x) #x
 #define QUOTEME(x) QUOTEME_(x)
 
