@@ -7,6 +7,7 @@
 
 #include "PreGame.h"
 
+#include "ClientData.h"
 #include "ClientSetup.h"
 #include "System/Sync/FPUCheck.h"
 #include "Game.h"
@@ -17,24 +18,27 @@
 #include "LoadScreen.h"
 #include "Game/Players/Player.h"
 #include "Game/Players/PlayerHandler.h"
-#include "Net/GameServer.h"
-#include "System/TimeProfiler.h"
 #include "UI/InfoConsole.h"
+#include "ExternalAI/SkirmishAIHandler.h"
 #include "Map/Generation/SimpleMapGenerator.h"
+#include "Menu/LuaMenuController.h"
+#include "Net/GameServer.h"
+#include "Net/Protocol/NetProtocol.h"
 
 #include "aGui/Gui.h"
-#include "ExternalAI/SkirmishAIHandler.h"
+
 #include "Rendering/Fonts/glFont.h"
 #include "Sim/Misc/GlobalSynced.h"
 #include "Sim/Misc/GlobalConstants.h"
 #include "Sim/Misc/TeamHandler.h"
 #include "System/Config/ConfigHandler.h"
 #include "System/Exceptions.h"
-#include "Net/Protocol/NetProtocol.h"
+#include "System/SafeUtil.h"
+#include "System/SpringExitCode.h"
+#include "System/TimeProfiler.h"
 #include "System/TdfParser.h"
 #include "System/Input/KeyInput.h"
 #include "System/FileSystem/ArchiveScanner.h"
-//// #include "System/FileSystem/FileHandler.h"
 #include "System/FileSystem/FileSystem.h"
 #include "System/FileSystem/VFSHandler.h"
 #include "System/LoadSave/DemoRecorder.h"
@@ -56,15 +60,18 @@ using std::string;
 
 CONFIG(bool, DemoFromDemo).defaultValue(false);
 
-CPreGame* pregame = NULL;
+static char mapChecksumMsgBuf[1024] = {0};
+static char modChecksumMsgBuf[1024] = {0};
 
-CPreGame::CPreGame(boost::shared_ptr<ClientSetup> setup)
+CPreGame* pregame = nullptr;
+
+CPreGame::CPreGame(std::shared_ptr<ClientSetup> setup)
 	: clientSetup(setup)
-	, savefile(NULL)
-	, timer(spring_gettime())
+	, savefile(nullptr)
+	, connectTimer(spring_gettime())
 	, wantDemo(true)
 {
-	assert(clientNet == NULL);
+	assert(clientNet == nullptr);
 
 	clientNet = new CNetProtocol();
 	activeController = this;
@@ -74,12 +81,12 @@ CPreGame::CPreGame(boost::shared_ptr<ClientSetup> setup)
 #endif
 
 	if (!clientSetup->isHost) {
-		//don't allow luasocket to connect to the host
-		LOG("Connecting to: %s:%i", clientSetup->hostIP.c_str(), clientSetup->hostPort);
+		LOG("[%s] client using IP %s and port %i", __func__, clientSetup->hostIP.c_str(), clientSetup->hostPort);
+		// don't allow luasocket to connect to the host
 		luaSocketRestrictions->addRule(CLuaSocketRestrictions::UDP_CONNECT, clientSetup->hostIP, clientSetup->hostPort, false);
 		clientNet->InitClient(clientSetup->hostIP.c_str(), clientSetup->hostPort, clientSetup->myPlayerName, clientSetup->myPasswd, SpringVersion::GetFull());
 	} else {
-		LOG("Hosting on: %s:%i", clientSetup->hostIP.c_str(), clientSetup->hostPort);
+		LOG("[%s] server using IP %s and port %i", __func__, clientSetup->hostIP.c_str(), clientSetup->hostPort);
 		clientNet->InitLocalClient();
 	}
 }
@@ -91,7 +98,7 @@ CPreGame::~CPreGame()
 	// but do not delete infoconsole, it is reused by CGame
 	agui::gui->Draw();
 
-	pregame = NULL;
+	pregame = nullptr;
 }
 
 void CPreGame::LoadSetupscript(const std::string& script)
@@ -121,21 +128,29 @@ void CPreGame::LoadSavefile(const std::string& save, bool usecreg)
 
 int CPreGame::KeyPressed(int k, bool isRepeat)
 {
-	if (k == SDLK_ESCAPE) {
-		if (KeyInput::GetKeyModState(KMOD_SHIFT)) {
-			LOG("[%s] user exited", __FUNCTION__);
-			gu->globalQuit = true;
-		} else {
-			LOG("Use shift-esc to quit");
-		}
+	if (k != SDLK_ESCAPE)
+		return 0;
+
+	if (!KeyInput::GetKeyModState(KMOD_SHIFT)) {
+		LOG("[PreGame::%s] press shift+escape to abort loading or exit", __func__);
+		return 0;
 	}
+
+	if (CLuaMenuController::ActivateInstance("[PreGame] User Aborted Loading")) {
+		assert(pregame == this);
+		spring::SafeDelete(pregame);
+		return 0;
+	}
+
+	LOG("[PreGame::%s] user exited", __func__);
+	gu->globalQuit = true;
 	return 0;
 }
 
 
 bool CPreGame::Draw()
 {
-	spring_msecs(10).sleep();
+	spring_msecs(10).sleep(true);
 	ClearScreen();
 	agui::gui->Draw();
 
@@ -145,13 +160,12 @@ bool CPreGame::Draw()
 		if (clientSetup->isHost)
 			font->glFormat(0.5f, 0.48f, 2.0f, FONT_CENTER | FONT_SCALE | FONT_NORM, "Waiting for server to start");
 		else
-			font->glFormat(0.5f, 0.48f, 2.0f, FONT_CENTER | FONT_SCALE | FONT_NORM, "Connecting to server (%ds)", (spring_gettime() - timer).toSecsi());
+			font->glFormat(0.5f, 0.48f, 2.0f, FONT_CENTER | FONT_SCALE | FONT_NORM, "Connecting to server (%ds)", (spring_gettime() - connectTimer).toSecsi());
 	} else {
 		font->glPrint(0.5f, 0.48f, 2.0f, FONT_CENTER | FONT_SCALE | FONT_NORM, "Waiting for server response");
 	}
 
-	font->glFormat(0.60f, 0.40f, 1.0f, FONT_SCALE | FONT_NORM, "Connecting to:   %s", clientNet->ConnectionStr().c_str());
-
+	font->glFormat(0.60f, 0.40f, 1.0f, FONT_SCALE | FONT_NORM, "Connecting to: %s", clientNet->ConnectionStr().c_str());
 	font->glFormat(0.60f, 0.35f, 1.0f, FONT_SCALE | FONT_NORM, "User name: %s", clientSetup->myPlayerName.c_str());
 
 	font->glFormat(0.5f,0.25f,0.8f,FONT_CENTER | FONT_SCALE | FONT_NORM, "Press SHIFT + ESC to quit");
@@ -177,7 +191,7 @@ bool CPreGame::Update()
 
 void CPreGame::AddGameSetupArchivesToVFS(const CGameSetup* setup, bool mapOnly)
 {
-	LOG("[%s] using map: %s", __FUNCTION__, setup->mapName.c_str());
+	LOG("[PreGame::%s] using map: %s", __func__, setup->mapName.c_str());
 	// Load Map archive
 	vfsHandler->AddArchiveWithDeps(setup->mapName, false);
 
@@ -186,7 +200,7 @@ void CPreGame::AddGameSetupArchivesToVFS(const CGameSetup* setup, bool mapOnly)
 
 	// Load Mutators (if any)
 	for (const std::string& mut: setup->GetMutatorsCont()) {
-		LOG("[%s] using mutator: %s", __FUNCTION__, mut.c_str());
+		LOG("[PreGame::%s] using mutator: %s", __func__, mut.c_str());
 		vfsHandler->AddArchiveWithDeps(mut, false);
 	}
 
@@ -194,29 +208,29 @@ void CPreGame::AddGameSetupArchivesToVFS(const CGameSetup* setup, bool mapOnly)
 	vfsHandler->AddArchiveWithDeps(setup->modName, false);
 
 	modArchive = archiveScanner->ArchiveFromName(setup->modName);
-	LOG("[%s] using game: %s (archive: %s)", __FUNCTION__, setup->modName.c_str(), modArchive.c_str());
+	LOG("[PreGame::%s] using game: %s (archive: %s)", __func__, setup->modName.c_str(), modArchive.c_str());
 }
 
 void CPreGame::StartServer(const std::string& setupscript)
 {
-	assert(!gameServer);
-	ScopedOnceTimer startserver("PreGame::StartServer");
+	assert(gameServer == nullptr);
+	ScopedOnceTimer timer("PreGame::StartServer");
 
-	boost::shared_ptr<GameData> startGameData(new GameData());
-	boost::shared_ptr<CGameSetup> startGameSetup(new CGameSetup());
+	std::shared_ptr<GameData> startGameData(new GameData());
+	std::shared_ptr<CGameSetup> startGameSetup(new CGameSetup());
 
 	startGameSetup->Init(setupscript);
-	startGameData->SetRandomSeed(static_cast<unsigned>(gu->RandInt()));
+	startGameData->SetRandomSeed(static_cast<unsigned>(guRNG.NextInt()));
 
-	if (startGameSetup->mapName.empty()) {
+	if (startGameSetup->mapName.empty())
 		throw content_error("No map selected in startscript");
-	}
 
 	if (startGameSetup->mapSeed != 0) {
 		CSimpleMapGenerator gen(startGameSetup.get());
 		gen.Generate();
 	}
 
+
 	// We must map the map into VFS this early, because server needs the start positions.
 	// Take care that MapInfo isn't loaded here, as map options aren't available to it yet.
 	AddGameSetupArchivesToVFS(startGameSetup.get(), true);
@@ -232,7 +246,7 @@ void CPreGame::StartServer(const std::string& setupscript)
 	const auto mapChecksum = archiveScanner->GetArchiveCompleteChecksum(mapArchive);
 	startGameData->SetModChecksum(modChecksum);
 	startGameData->SetMapChecksum(mapChecksum);
-	LOG("Checksums: game=0x%X map=0x%X", modChecksum, mapChecksum);
+	LOG("[PreGame::%s] checksums: game=0x%X map=0x%X", __func__, modChecksum, mapChecksum);
 
 	good_fpu_control_registers("before CGameServer creation");
 	startGameData->SetSetupText(startGameSetup->setupText);
@@ -250,19 +264,26 @@ void CPreGame::UpdateClientNet()
 	clientNet->Update();
 
 	if (clientNet->CheckTimeout(0, true)) {
-		LOG_L(L_WARNING, "Server not reachable");
-		SetExitCode(1);
+		if (CLuaMenuController::ActivateInstance("[PreGame] Server Connection Timeout")) {
+			assert(pregame == this);
+			spring::SafeDelete(pregame);
+			return;
+		}
+
+		LOG_L(L_ERROR, "[PreGame] Server Connection Timeout");
+
+		spring::exitCode = spring::EXIT_CODE_TIMEOUT;
 		gu->globalQuit = true;
 		return;
 	}
 
-	boost::shared_ptr<const RawPacket> packet;
+	std::shared_ptr<const RawPacket> packet;
 
 	while ((packet = clientNet->GetData(gs->frameNum))) {
 		const unsigned char* inbuf = packet->data;
 
 		if (packet->length <= 0) {
-			LOG_L(L_WARNING, "[PreGame::%s] zero-length packet (header: %i)", __FUNCTION__, inbuf[0]);
+			LOG_L(L_WARNING, "[PreGame::%s] zero-length packet (header: %i)", __func__, inbuf[0]);
 			continue;
 		}
 
@@ -272,14 +293,23 @@ void CPreGame::UpdateClientNet()
 				try {
 					netcode::UnpackPacket pckt(packet, 3);
 					std::string message;
+
 					pckt >> message;
-					LOG("%s", message.c_str());
-					handleerror(NULL, "Remote requested quit: " + message, "Quit message", MBF_OK | MBF_EXCL);
+
+					// (re)activate LuaMenu if user failed to connect
+					if (CLuaMenuController::ActivateInstance(message)) {
+						assert(pregame == this);
+						spring::SafeDelete(pregame);
+						return;
+					}
+
+					// force exit to system if no menu
+					LOG("[PreGame::%s] server requested quit or rejected connection (reason \"%s\")", __func__, message.c_str());
+					handleerror(nullptr, "server requested quit or rejected connection: " + message, "Quit message", MBF_OK | MBF_EXCL);
 				} catch (const netcode::UnpackPacketException& ex) {
-					LOG_L(L_ERROR, "Got invalid QuitMessage: %s", ex.what());
+					LOG_L(L_ERROR, "[PreGame::%s][NETMSG_{QUIT,REJECT_CONNECT}] exception \"%s\"", __func__, ex.what());
 				}
-				break;
-			}
+			} break;
 
 			case NETMSG_CREATE_NEWPLAYER: {
 				// server will send this first if we're using mid-game join
@@ -287,8 +317,12 @@ void CPreGame::UpdateClientNet()
 				// gamedata), otherwise skip to gamedata
 				try {
 					netcode::UnpackPacket pckt(packet, 3);
-					unsigned char spectator, team, playerNum;
 					std::string name;
+
+					uint8_t playerNum;
+					uint8_t spectator;
+					uint8_t team;
+
 					// since the >> operator uses dest size to extract data from
 					// the packet, we need to use temp variables of the same
 					// size of the packet, before converting to dest variable
@@ -302,55 +336,58 @@ void CPreGame::UpdateClientNet()
 					player.spectator = spectator;
 					player.team = team;
 					player.playerNum = playerNum;
-					// add ourself, to avoid crashing if our player num gets
-					// queried we will receive the same message later, in the
-					// game class, which is the global broadcast version
-					// the global broadcast will overwrite the user with the
-					// same values as here
+
+					// add ourselves to avoid crashing if our player-num gets queried
+					// we will receive this message a second time (the global broadcast
+					// version) which will overwrite the player with the same values as
+					// set here
 					playerHandler->AddPlayer(player);
 
-					LOG("[PreGame::%s] added new player %s with number %d to team %d", __FUNCTION__, name.c_str(), player.playerNum, player.team);
+					LOG("[PreGame::%s] added new player %s with number %d to team %d", __func__, name.c_str(), player.playerNum, player.team);
 				} catch (const netcode::UnpackPacketException& ex) {
-					LOG_L(L_ERROR, "[PreGame::%s] got invalid NETMSG_CREATE_NEWPLAYER: %s", __FUNCTION__, ex.what());
+					LOG_L(L_ERROR, "[PreGame::%s][NETMSG_CREATE_NEWPLAYER] exception \"%s\"", __func__, ex.what());
 				}
-				break;
-			}
+			} break;
 
 			case NETMSG_GAMEDATA: {
 				// server first sends this to let us know about teams, allyteams
 				// etc. (not if we are joining mid-game as an extra player), see
 				// NETMSG_SETPLAYERNUM
 				GameDataReceived(packet);
-				break;
-			}
+			} break;
 
 			case NETMSG_SETPLAYERNUM: {
 				// this is sent after NETMSG_GAMEDATA, to let us know which
 				// player number we have (server assigns them based on order
 				// of connection)
-				if (gameSetup == NULL)
+				if (gameSetup == nullptr)
 					throw content_error("No game data received from server");
 
-				const unsigned char playerNum = packet->data[1];
+				const uint8_t playerNum = packet->data[1];
 
 				if (!playerHandler->IsValidPlayer(playerNum))
 					throw content_error("Invalid player number received from server");
 
 				gu->SetMyPlayer(playerNum);
 
-				LOG("[PreGame::%s] user number %i (team %i, allyteam %i)", __FUNCTION__, gu->myPlayerNum, gu->myTeam, gu->myAllyTeam);
+				LOG("[PreGame::%s] received user number %i (team %i, allyteam %i), creating load-screen", __func__, gu->myPlayerNum, gu->myTeam, gu->myAllyTeam);
+
+				// respond with the client data and content checksums
+				clientNet->Send(CBaseNetProtocol::Get().SendClientData(playerNum, ClientData::GetCompressed()));
+
+				CLIENT_NETLOG(gu->myPlayerNum, LOG_LEVEL_INFO, mapChecksumMsgBuf);
+				CLIENT_NETLOG(gu->myPlayerNum, LOG_LEVEL_INFO, modChecksumMsgBuf);
 
 				CLoadScreen::CreateInstance(gameSetup->MapFile(), modArchive, savefile);
 
-				pregame = NULL;
-				delete this;
+				assert(pregame == this);
+				spring::SafeDelete(pregame);
 				return;
-			}
+			} break;
 
 			default: {
-				LOG_L(L_WARNING, "[PreGame::%s] unknown packet type (header: %i)", __FUNCTION__, inbuf[0]);
-				break;
-			}
+				LOG_L(L_WARNING, "[PreGame::%s] unknown packet type (header: %i)", __func__, inbuf[0]);
+			} break;
 		}
 	}
 }
@@ -365,7 +402,7 @@ void CPreGame::StartServerForDemo(const std::string& demoName)
 
 	{
 		// server will always use a modified copy of this
-		assert(gameSetup != NULL);
+		assert(gameSetup != nullptr);
 
 		// modify the demo's start-script so it can be used to watch the demo
 		tgame->AddPair("MapName", gameSetup->mapName);
@@ -378,7 +415,7 @@ void CPreGame::StartServerForDemo(const std::string& demoName)
 		tgame->remove("SourcePort", false);
 		//tgame->remove("IsHost", false);
 
-		for (std::map<std::string, TdfParser::TdfSection*>::iterator it = tgame->sections.begin(); it != tgame->sections.end(); ++it) {
+		for (auto it = tgame->sections.begin(); it != tgame->sections.end(); ++it) {
 			if (it->first.size() > 6 && it->first.substr(0, 6) == "player") {
 				it->second->AddPair("isfromdemo", 1);
 			}
@@ -394,30 +431,34 @@ void CPreGame::StartServerForDemo(const std::string& demoName)
 	gameData->SetSetupText(moddedDemoScript.str());
 
 	// create the server-private demo GameSetup containing the additional player
-	boost::shared_ptr<CGameSetup> demoGameSetup(new CGameSetup());
+	std::shared_ptr<CGameSetup> demoGameSetup(new CGameSetup());
 
 	if (!demoGameSetup->Init(moddedDemoScript.str()))
 		throw content_error("Demo contains incorrect script");
 
-	LOG("[%s] starting GameServer", __FUNCTION__);
+	LOG("[PreGame::%s] starting GameServer", __func__);
 	good_fpu_control_registers("before CGameServer creation");
 
 	gameServer = new CGameServer(clientSetup, gameData, demoGameSetup);
 	gameServer->AddLocalClient(clientSetup->myPlayerName, SpringVersion::GetFull());
 
 	good_fpu_control_registers("after CGameServer creation");
-	LOG("[%s] started GameServer", __FUNCTION__);
+	LOG("[PreGame::%s] started GameServer", __func__);
 }
 
 void CPreGame::ReadDataFromDemo(const std::string& demoName)
 {
-	ScopedOnceTimer startserver("PreGame::ReadDataFromDemo");
-	assert(gameServer == NULL);
-	LOG("[%s] pre-scanning demo file for game data...", __FUNCTION__);
-	CDemoReader scanner(demoName, 0);
+	ScopedOnceTimer timer("PreGame::ReadDataFromDemo");
+	assert(gameServer == nullptr);
+	LOG("[PreGame::%s] pre-scanning demo file for game data...", __func__);
+	CDemoReader scanner(demoName, 0.0f);
 
 	{
-		gameData.reset(new GameData(scanner.GetSetupScript()));
+		// this does not extract the RNG preseed, use first packet
+		// gameData.reset(new GameData(scanner.GetSetupScript()));
+		gameData.reset(new GameData(std::shared_ptr<netcode::RawPacket>(scanner.GetData(0.0f))));
+		assert(gameData->GetSetupText() == scanner.GetSetupScript());
+
 		if (CGameSetup::LoadReceivedScript(gameData->GetSetupText(), true)) {
 			StartServerForDemo(demoName);
 		} else {
@@ -425,28 +466,34 @@ void CPreGame::ReadDataFromDemo(const std::string& demoName)
 		}
 	}
 
-	assert(gameServer != NULL);
+	assert(gameServer != nullptr);
 }
 
-void CPreGame::GameDataReceived(boost::shared_ptr<const netcode::RawPacket> packet)
+void CPreGame::GameDataReceived(std::shared_ptr<const netcode::RawPacket> packet)
 {
-	ScopedOnceTimer startserver("PreGame::GameDataReceived");
+	ScopedOnceTimer timer("PreGame::GameDataReceived");
 
 	try {
+		// in demos, gameData is first new'ed in ReadDataFromDemo()
+		// in live games it will always still be NULL at this point
 		gameData.reset(new GameData(packet));
 	} catch (const netcode::UnpackPacketException& ex) {
 		throw content_error(std::string("Server sent us invalid GameData: ") + ex.what());
 	}
 
+	// preseed the synced RNG until GameID-based NETMSG_RANDSEED arrives
+	// allows proper randomness in LuaParser when executing defs.lua, etc
+	gsRNG.SetSeed(gameData->GetRandomSeed(), true);
+
 	// for demos, ReadDataFromDemo precedes UpdateClientNet -> GameDataReceived
 	// this means gameSetup contains data from the original game but we need the
 	// modified version (cf StartServerForDemo) which the server already has that
 	// contains an extra player
-	if (gameSetup != NULL)
-		SafeDelete(gameSetup);
+	if (gameSetup != nullptr)
+		spring::SafeDelete(gameSetup);
 
 	if (CGameSetup::LoadReceivedScript(gameData->GetSetupText(), clientSetup->isHost)) {
-		assert(gameSetup != NULL);
+		assert(gameSetup != nullptr);
 		gu->LoadFromSetup(gameSetup);
 		gs->LoadFromSetup(gameSetup);
 		// do we really need to do this so early?
@@ -458,45 +505,59 @@ void CPreGame::GameDataReceived(boost::shared_ptr<const netcode::RawPacket> pack
 	// some sanity checks
 	for (int p = 0; p < playerHandler->ActivePlayers(); ++p) {
 		const CPlayer* player = playerHandler->Player(p);
-		if (!playerHandler->IsValidPlayer(player->playerNum)) {
+
+		if (!playerHandler->IsValidPlayer(player->playerNum))
 			throw content_error("Invalid player in game data");
-		}
-		if (!teamHandler->IsValidTeam(player->team)) {
+
+		if (!teamHandler->IsValidTeam(player->team))
 			throw content_error("Invalid team in game data");
-		}
-		if (!teamHandler->IsValidAllyTeam(teamHandler->AllyTeam(player->team))) { // TODO: seems not to make sense really
+
+		// TODO: seems not to make sense really
+		if (!teamHandler->IsValidAllyTeam(teamHandler->AllyTeam(player->team)))
 			throw content_error("Invalid ally team in game data");
-		}
+
 	}
 
-	// Load archives into VFS
+	// load archives into VFS
 	AddGameSetupArchivesToVFS(gameSetup, false);
 
-	// Check checksums of map & game
+	// check checksums of map & game
+	// mismatches happen on dedicated servers between host and clients
+	// we want to know whether the *locally calculated* checksums also
+	// differ among clients so use the opportunity to send them
+	// NOTE: gu->myPlayerNum is not valid yet, GameData arrives first
+	std::pair<unsigned int, unsigned int> mapChecksums = {gameData->GetMapChecksum(), 0};
+	std::pair<unsigned int, unsigned int> modChecksums = {gameData->GetModChecksum(), 0};
+
 	try {
-		archiveScanner->CheckArchive(gameSetup->mapName, gameData->GetMapChecksum());
+		archiveScanner->CheckArchive(gameSetup->mapName, mapChecksums.first, mapChecksums.second);
 	} catch (const content_error& ex) {
-		LOG_L(L_WARNING, "Incompatible map-checksum: %s", ex.what());
+		LOG_L(L_WARNING, "[PreGame::%s] %s", __func__, ex.what());
 	}
 	try {
-		archiveScanner->CheckArchive(modArchive, gameData->GetModChecksum());
+		archiveScanner->CheckArchive(modArchive, modChecksums.first, modChecksums.second);
 	} catch (const content_error& ex) {
-		LOG_L(L_WARNING, "Incompatible game-checksum: %s", ex.what());
+		LOG_L(L_WARNING, "[PreGame::%s] %s", __func__, ex.what());
 	}
 
-	if (clientSetup->isHost && !gameSetup->recordDemo) { //script.txt allows to disable demo file recording (host only, used for menu)
+	std::memset(mapChecksumMsgBuf, 0, sizeof(mapChecksumMsgBuf));
+	std::memset(modChecksumMsgBuf, 0, sizeof(modChecksumMsgBuf));
+	std::snprintf(mapChecksumMsgBuf, sizeof(mapChecksumMsgBuf), "[PreGame::%s][map-checksums={0x%x,0x%x}]", __func__, mapChecksums.first, mapChecksums.second);
+	std::snprintf(modChecksumMsgBuf, sizeof(modChecksumMsgBuf), "[PreGame::%s][mod-checksums={0x%x,0x%x}]", __func__, modChecksums.first, modChecksums.second);
+
+	// script.txt allows to disable demo file recording (host only, used for menu)
+	if (clientSetup->isHost && !gameSetup->recordDemo)
 		wantDemo = false;
-	}
 
-	if (clientNet != NULL && wantDemo) {
-		assert(clientNet->GetDemoRecorder() == NULL);
+	if (clientNet != nullptr && wantDemo) {
+		assert(clientNet->GetDemoRecorder() == nullptr);
 
 		CDemoRecorder* recorder = new CDemoRecorder(gameSetup->mapName, gameSetup->modName, false);
 		recorder->WriteSetupText(gameData->GetSetupText());
 		recorder->SaveToDemo(packet->data, packet->length, clientNet->GetPacketTime(gs->frameNum));
 		clientNet->SetDemoRecorder(recorder);
 
-		LOG("Recording demo to: %s", (recorder->GetName()).c_str());
+		LOG("PreGame::%s] recording demo to \"%s\"", __func__, (recorder->GetName()).c_str());
 	}
 }
 
