@@ -14,7 +14,7 @@
 #include "Sim/Misc/ModInfo.h"
 #include "Sim/Path/IPathManager.h"
 #include "System/FileSystem/FileSystem.h"
-#include "System/Util.h"
+#include "System/StringUtil.h"
 
 #include <SDL_keycode.h>
 #include <cstdio>
@@ -54,6 +54,8 @@ static void StringListStats(
 
 
 
+static CGameInfo* instance = NULL;
+
 void CGameInfo::Enable()
 {
 	if (instance == NULL) {
@@ -64,6 +66,7 @@ void CGameInfo::Enable()
 void CGameInfo::Disable()
 {
 	delete instance;
+	instance = NULL;
 }
 
 bool CGameInfo::IsActive()
@@ -73,8 +76,6 @@ bool CGameInfo::IsActive()
 
 
 
-CGameInfo* CGameInfo::instance = NULL;
-
 CGameInfo::CGameInfo()
 {
 	box.x1 = 0.5f;
@@ -114,7 +115,7 @@ CGameInfo::CGameInfo()
 	values.push_back(buf);
 
 	labels.push_back("Map Size:");
-	sprintf(buf, "%ix%i", gs->mapx / 64, gs->mapy / 64);
+	sprintf(buf, "%ix%i", mapDims.mapx / 64, mapDims.mapy / 64);
 	values.push_back(buf);
 
 	labels.push_back("Map Name:");
