@@ -5,10 +5,8 @@
 
 #include "System/creg/creg_cond.h"
 
-#include <map>
 #include <string>
 #include <vector>
-#include <set>
 
 class CGroup;
 
@@ -49,7 +47,7 @@ protected:
 	 * This is always greater or equal FIRST_SPECIAL_GROUP.
 	 */
 	int firstUnusedGroup;
-	std::set<int> changedGroups;
+	std::vector<int> changedGroups;
 };
 
 extern std::vector<CGroupHandler*> grouphandlers;
