@@ -332,7 +332,7 @@ void CQuitBox::MouseRelease(int x,int y,int button)
 		LOG("[QuitBox] user exited to menu");
 
 		// signal SpringApp
-		gameSetup->setupText = "";
+		gu->reloadScript = "";
 		gu->globalReload = true;
 	}
 	else if (InBox(mx, my, box + quitBox)) {
