@@ -3,8 +3,8 @@
 #ifndef UNIT_TRACKER_H
 #define UNIT_TRACKER_H
 
-#include <set>
 #include "System/float3.h"
+#include "System/UnorderedSet.hpp"
 
 class CUnit;
 
@@ -45,8 +45,9 @@ class CUnitTracker
 
 		int trackMode;
 		int trackUnit;
-		std::set<int> trackGroup;
-		
+
+		spring::unordered_set<int> trackGroup;
+
 		int timeOut;
 		int lastFollowUnit;
 		float lastUpdateTime;
