@@ -16,27 +16,6 @@
 #include "System/Exceptions.h"
 
 
-/******************************************************************************/
-/******************************************************************************/
-
-LuaFonts::LuaFonts()
-{
-}
-
-
-LuaFonts::~LuaFonts()
-{
-}
-
-
-/******************************************************************************/
-/******************************************************************************/
-
-#define REGISTER_LUA_CFUNC(x)    \
-	lua_pushstring(L, #x);   \
-	lua_pushcfunction(L, x); \
-	lua_rawset(L, -3)
-
 bool LuaFonts::PushEntries(lua_State* L)
 {
 	CreateMetatable(L);
