@@ -1,33 +1,19 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-
-#include <set>
-#include <mutex>
-#include <boost/thread/mutex.hpp>
-
 #include "LuaGaia.h"
 
 #include "LuaInclude.h"
-
 #include "LuaUtils.h"
-#include "LuaSyncedCtrl.h"
-#include "LuaSyncedRead.h"
-#include "LuaUnitDefs.h"
-#include "LuaWeaponDefs.h"
-#include "LuaOpenGL.h"
 
-#include "Rendering/UnitDrawer.h"
 #include "Sim/Misc/GlobalSynced.h"
 #include "Sim/Misc/TeamHandler.h"
-#include "Sim/Units/Unit.h"
-#include "Sim/Units/UnitDef.h"
-#include "Sim/Units/CommandAI/Command.h"
-#include "System/FileSystem/FileHandler.h"
-#include "System/FileSystem/FileSystem.h"
-#include "System/Util.h"
+#include "System/FileSystem/FileHandler.h" // SPRING_VFS*
+#include "System/FileSystem/VFSHandler.h"
+#include "System/StringUtil.h"
+#include "System/Threading/SpringThreading.h"
 
 
-CLuaGaia* luaGaia = NULL;
+CLuaGaia* luaGaia = nullptr;
 
 static const char* LuaGaiaSyncedFilename   = "LuaGaia/main.lua";
 static const char* LuaGaiaUnsyncedFilename = "LuaGaia/draw.lua";
@@ -36,7 +22,7 @@ static const char* LuaGaiaUnsyncedFilename = "LuaGaia/draw.lua";
 /******************************************************************************/
 /******************************************************************************/
 
-static boost::mutex m_singleton;
+static spring::mutex m_singleton;
 
 DECL_LOAD_HANDLER(CLuaGaia, luaGaia)
 DECL_FREE_HANDLER(CLuaGaia, luaGaia)
@@ -45,8 +31,7 @@ DECL_FREE_HANDLER(CLuaGaia, luaGaia)
 /******************************************************************************/
 /******************************************************************************/
 
-CLuaGaia::CLuaGaia()
-: CLuaHandleSynced("LuaGaia", LUA_HANDLE_ORDER_GAIA)
+CLuaGaia::CLuaGaia(): CLuaHandleSynced("LuaGaia", LUA_HANDLE_ORDER_GAIA)
 {
 	if (!IsValid())
 		return;
@@ -58,26 +43,17 @@ CLuaGaia::CLuaGaia()
 	SetReadAllyTeam(CEventClient::AllAccessTeam);
 	SetSelectTeam(teamHandler->GaiaTeamID());
 
-	Init(LuaGaiaSyncedFilename, LuaGaiaUnsyncedFilename, SPRING_VFS_MAP);
+	Init(LuaGaiaSyncedFilename, LuaGaiaUnsyncedFilename, SPRING_VFS_MAP_BASE);
 }
 
-
 CLuaGaia::~CLuaGaia()
 {
-	luaGaia = NULL;
+	luaGaia = nullptr;
 }
 
 
-bool CLuaGaia::AddSyncedCode(lua_State* L)
+bool CLuaGaia::CanLoadHandler()
 {
-	return true;
+	return (gs->useLuaGaia && (vfsHandler->FileExists(LuaGaiaSyncedFilename, CVFSHandler::Map) || vfsHandler->FileExists(LuaGaiaUnsyncedFilename, CVFSHandler::Map)));
 }
 
-
-bool CLuaGaia::AddUnsyncedCode(lua_State* L)
-{
-	return true;
-}
-
-/******************************************************************************/
-/******************************************************************************/
