@@ -15,12 +15,27 @@
 #include "Sim/Objects/SolidObject.h"
 #include "Sim/Misc/GlobalSynced.h" // simFrame
 #include "System/Log/ILog.h"
-#include "System/Util.h"
+#include "System/StringUtil.h"
 
+#include <cctype>
+
+struct ActiveUniform {
+	GLint size = 0;
+	GLenum type = 0;
+	GLchar name[512] = {0};
+};
+
+struct LuaMatBinPtrLessThan {
+	bool operator()(const LuaMatBin* a, const LuaMatBin* b) const {
+		const LuaMaterial* ma = static_cast<const LuaMaterial*>(a);
+		const LuaMaterial* mb = static_cast<const LuaMaterial*>(b);
+		return (*ma < *mb);
+	}
+};
 
 LuaMatHandler LuaMatHandler::handler;
 LuaMatHandler& luaMatHandler = LuaMatHandler::handler;
-
+LuaMatBinPtrLessThan matBinCmp;
 
 /******************************************************************************/
 /******************************************************************************/
@@ -28,7 +43,7 @@ LuaMatHandler& luaMatHandler = LuaMatHandler::handler;
 //  Helper
 //
 
-static const char* GlUniformTypeToString(const GLenum uType)
+static const char* GLUniformTypeToString(const GLenum uType)
 {
 	switch (uType) {
 		case      GL_FLOAT: return "float";
@@ -43,6 +58,7 @@ static const char* GlUniformTypeToString(const GLenum uType)
 		case GL_FLOAT_MAT3: return "mat3";
 		case GL_FLOAT_MAT4: return "mat4";
 	}
+
 	return "unknown";
 }
 
@@ -61,7 +77,6 @@ bool LuaObjectMaterial::SetLODCount(unsigned int count)
 	return true;
 }
 
-
 bool LuaObjectMaterial::SetLastLOD(unsigned int lod)
 {
 	lastLOD = std::min(lod, lodCount - 1);
@@ -281,7 +296,7 @@ void LuaMaterial::Parse(
 			continue;
 		}
 
-		LOG_L(L_WARNING, "LuaMaterial: incorrect key \"%s\"", key.c_str());
+		LOG_L(L_WARNING, "[LuaMaterial::%s] incorrect key \"%s\"", __func__, key.c_str());
 	}
 }
 
@@ -382,10 +397,10 @@ int LuaMaterial::Compare(const LuaMaterial& a, const LuaMaterial& b)
 	if (a.postList != b.postList)
 		return ((a.postList > b.postList) * 2 - 1);
 
-	// have no influence on the performance cost
-	// still LuaMaterial/LuaMatBin are used in a std::set (:=LuaMatBinSet)
-	// and there there < operator is also used to detect equality (a == b iff !(a<b) && !(b<a))
-	// that's why need to check _all_ properties
+	// these have no influence on the performance cost
+	// previously LuaMaterial/LuaMatBin were kept in a std::set (LuaMatBinSet, now a sorted std::vector)
+	// for which the comparison operator < was also used to detect equality (a == b iff !(a<b) && !(b<a))
+	// so check _all_ properties
 	if ((cmp = LuaMatUniforms::Compare(a.uniforms[LuaMatShader::LUASHADER_PASS_FWD], b.uniforms[LuaMatShader::LUASHADER_PASS_FWD])) != 0)
 		return cmp;
 	if ((cmp = LuaMatUniforms::Compare(a.uniforms[LuaMatShader::LUASHADER_PASS_DFR], b.uniforms[LuaMatShader::LUASHADER_PASS_DFR])) != 0)
@@ -495,7 +510,7 @@ void LuaMaterial::Print(const string& indent) const
 //  LuaMatUniforms
 //
 
-std::unordered_map<LuaMatUniforms::IUniform*, std::string> LuaMatUniforms::GetUniformsAndStandardName()
+spring::unsynced_map<LuaMatUniforms::IUniform*, std::string> LuaMatUniforms::GetEngineUniformNamePairs()
 {
 	return {
 		{&viewMatrix,    "ViewMatrix"},
@@ -516,8 +531,7 @@ std::unordered_map<LuaMatUniforms::IUniform*, std::string> LuaMatUniforms::GetUn
 	};
 }
 
-
-std::unordered_map<std::string, LuaMatUniforms::IUniform*> LuaMatUniforms::GetUniformsAndPossibleNames()
+spring::unsynced_map<std::string, LuaMatUniforms::IUniform*> LuaMatUniforms::GetEngineNameUniformPairs()
 {
 	return {
 		{"Camera",                  &viewMatrix},
@@ -560,88 +574,132 @@ void LuaMatUniforms::AutoLink(LuaMatShader* shader)
 	if (!shader->IsCustomType())
 		return;
 
-	for (const auto& p: GetUniformsAndPossibleNames()) {
-		if (p.second->IsValid())
+	for (const auto& p: GetEngineNameUniformPairs()) {
+		IUniform* u = p.second;
+		ActiveUniform au;
+
+		if (u->IsValid())
 			continue;
 
-		// try CamelCase
-		p.second->loc = glGetUniformLocation(shader->openglID, p.first.c_str());
-		if (p.second->IsValid()) continue;
+		std::memset(au.name, 0, sizeof(au.name));
+		std::strncpy(au.name, p.first.c_str(), std::min(sizeof(au.name) - 1, p.first.size()));
 
-		// try camelCase
-		std::string s = p.first; s[0] = tolower(s[0]);
-		p.second->loc = glGetUniformLocation(shader->openglID, s.c_str());
-		if (p.second->IsValid()) continue;
+		{
+			// try getting location by CamelCase name, e.g. "ViewMatrix"
+			au.name[0] = std::toupper(au.name[0]);
 
-		// try lowercase
-		StringToLowerInPlace(s);
-		p.second->loc = glGetUniformLocation(shader->openglID, s.c_str());
+			if (u->SetLoc(glGetUniformLocation(shader->openglID, au.name)))
+				continue;
+		}
+		{
+			// try getting location by camelCase name, e.g. "viewMatrix"
+			au.name[0] = std::tolower(au.name[0]);
+
+			if (u->SetLoc(glGetUniformLocation(shader->openglID, au.name)))
+				continue;
+		}
+
+		// assume this uniform is not in the standard-set, mark as invalid for Execute
+		u->SetLoc(GL_INVALID_INDEX);
 	}
 }
 
 
 void LuaMatUniforms::Validate(LuaMatShader* s)
 {
+	constexpr const char* fmts[3] = {
+		"[LuaMatUniforms::%s] engine shaders prohibit the usage of uniform \"%s\"",
+		"[LuaMatUniforms::%s] missing \"TeamColor\" uniform",
+		"[LuaMatUniforms::%s] incorrect uniform-type for \"%s\" at location %d (declared %s, expected %s)",
+	};
+
 	if (!s->IsCustomType()) {
 		// print warning when uniforms are given for engine shaders
-		for (const auto& p: GetUniformsAndStandardName()) {
+		for (const auto& p: GetEngineUniformNamePairs()) {
 			if (!p.first->IsValid())
 				continue;
 
-			LOG_L(L_WARNING, "LuaMaterial: engine shaders prohibit the usage of uniform \"%s\"", p.second.c_str());
+			LOG_L(L_WARNING, fmts[0], __func__, p.second.c_str());
 		}
+
 		return;
 	}
 
 	// print warning when teamcolor is not bound
-	if (!teamColor.IsValid()) {
-		LOG_L(L_WARNING, "LuaMaterial: missing TeamColor uniform");
-	}
+	if (!teamColor.IsValid())
+		LOG_L(L_WARNING, fmts[1], __func__);
 
-	// check uniform types
-	for (const auto& p: GetUniformsAndStandardName()) {
-		if (!p.first->IsValid())
+
+	const decltype(GetEngineNameUniformPairs())& uniforms = GetEngineNameUniformPairs();
+	      decltype(uniforms.end()) uniformsIt;
+
+
+	ActiveUniform au;
+
+	GLsizei numUniforms = 0;
+	GLsizei uniformIndx = 0;
+
+	glGetProgramiv(s->openglID, GL_ACTIVE_UNIFORMS, &numUniforms);
+
+	while (uniformIndx < numUniforms) {
+		glGetActiveUniform(s->openglID, uniformIndx++, sizeof(au.name) - 1, nullptr, &au.size, &au.type, au.name);
+
+		// check if active uniform is in the standard-set
+		au.name[0] = std::toupper(au.name[0]);
+		uniformsIt = uniforms.find(au.name);
+
+		if (uniformsIt == uniforms.end()) {
+			au.name[0] = std::tolower(au.name[0]);
+			uniformsIt = uniforms.find(au.name);
+		}
+
+		if (uniformsIt == uniforms.end())
 			continue;
 
-		GLint size = 0;
-		GLenum uniformType = 0;
-		glGetActiveUniform(s->openglID, p.first->loc, 0, nullptr, &size, &uniformType, nullptr);
 
-		if (p.first->GetType() == uniformType)
+		// if so, check if its type matches the expected type
+		const GLint uniformLoc = (uniformsIt->second)->loc;
+		const GLenum expectedType = (uniformsIt->second)->GetType();
+
+		if (au.type == expectedType)
 			continue;
 
-		LOG_L(L_WARNING, "LuaMaterial: incorrect uniform type for \"%s\", got: %s expected: %s", p.second.c_str(), GlUniformTypeToString(uniformType), GlUniformTypeToString(p.first->GetType()));
+		LOG_L(L_WARNING, fmts[2], __func__, au.name, uniformLoc,  GLUniformTypeToString(au.type), GLUniformTypeToString(expectedType));
 	}
 }
 
 
 void LuaMatUniforms::Parse(lua_State* L, const int tableIdx)
 {
-	auto uniformAndNames = GetUniformsAndPossibleNames();
-	decltype(uniformAndNames) uniformAndNamesLower;
-	for (auto& p: uniformAndNames) {
-		uniformAndNamesLower[StringToLower(p.first)] = p.second;
+	decltype(GetEngineNameUniformPairs()) lcNameUniformPairs;
+
+	for (const auto& p: GetEngineNameUniformPairs()) {
+		lcNameUniformPairs[ StringToLower(p.first) ] = p.second;
 	}
 
 	for (lua_pushnil(L); lua_next(L, tableIdx) != 0; lua_pop(L, 1)) {
 		if (!lua_israwstring(L, -2) || !lua_isnumber(L, -1))
 			continue;
 
-		// get the lua table key
-		// and remove the "loc" at the end (cameraPosLoc -> camerapos)
-		std::string uniformLocStr = luaL_tosstring(L, -2);
-		StringToLowerInPlace(uniformLocStr);
-		if (StringEndsWith(uniformLocStr, "loc")) {
-			uniformLocStr.resize(uniformLocStr.size() - 3);
-		}
+		// convert LuaMaterial key to lower-case and remove the
+		// "loc" postfix from it ("cameraPosLoc" -> "camerapos")
+		std::string uniformLocStr = StringToLower(luaL_tosstring(L, -2));
 
-		const auto uniformLocIt = uniformAndNamesLower.find(uniformLocStr);
-		if (uniformLocIt == uniformAndNamesLower.end()) {
-			LOG_L(L_WARNING, "LuaMaterial: unknown uniform \"%s\"", lua_tostring(L, -2));
+		if (StringEndsWith(uniformLocStr, "loc"))
+			uniformLocStr.resize(uniformLocStr.size() - 3);
+
+		const auto uniformLocIt = lcNameUniformPairs.find(uniformLocStr);
+
+		if (uniformLocIt == lcNameUniformPairs.end()) {
+			LOG_L(L_WARNING, "[LuaMatUniforms::%s] unknown uniform \"%s\"", __func__, uniformLocStr.c_str());
 			continue;
 		}
 
-		uniformLocIt->second->loc = static_cast<GLint>(lua_tonumber(L, -1));
+		if ((uniformLocIt->second)->SetLoc(static_cast<GLint>(lua_tonumber(L, -1))))
+			continue;
+
+		// could log this too, but would be confusing without the shader/material name
+		// LOG_L(L_WARNING, "[LuaMatUniforms::%s] unused uniform \"%s\"", __func__, uniformLocStr.c_str());
 	}
 }
 
@@ -661,7 +719,7 @@ void LuaMatUniforms::Execute() const
 	camPos.Execute(camera->GetPos());
 	camDir.Execute(camera->GetDir());
 	sunDir.Execute(sky->GetLight()->GetLightDir());
-	rndVec.Execute(gu->RandVector());
+	rndVec.Execute(guRNG.NextVector());
 
 	simFrame.Execute(gs->frameNum);
 	visFrame.Execute(globalRendering->drawFrame);
@@ -708,57 +766,55 @@ void LuaMatUniforms::Print(const string& indent, bool isDeferred) const
 LuaMatRef::LuaMatRef(LuaMatBin* _bin)
 {
 	bin = _bin;
-	if (bin != NULL) {
+	if (bin != nullptr) {
 		bin->Ref();
 	}
 }
 
-
 LuaMatRef::~LuaMatRef()
 {
-	if (bin != NULL) {
+	if (bin != nullptr) {
 		bin->UnRef();
 	}
 }
 
+LuaMatRef::LuaMatRef(const LuaMatRef& mr)
+{
+	bin = mr.bin;
+	if (bin != nullptr) {
+		bin->Ref();
+	}
+}
+
 
 void LuaMatRef::Reset()
 {
-	if (bin != NULL) {
+	if (bin != nullptr) {
 		bin->UnRef();
 	}
-	bin = NULL;
+	bin = nullptr;
 }
 
 
 LuaMatRef& LuaMatRef::operator=(const LuaMatRef& mr)
 {
-	if (mr.bin != NULL) { mr.bin->Ref();   }
-	if (   bin != NULL) {    bin->UnRef(); }
+	if (mr.bin != nullptr) { mr.bin->Ref();   }
+	if (   bin != nullptr) {    bin->UnRef(); }
 	bin = mr.bin;
 	return *this;
 }
 
 
-LuaMatRef::LuaMatRef(const LuaMatRef& mr)
-{
-	bin = mr.bin;
-	if (bin != NULL) {
-		bin->Ref();
-	}
-}
-
-
 void LuaMatRef::AddUnit(CSolidObject* o)
 {
-	if (bin != NULL) {
+	if (bin != nullptr) {
 		bin->AddUnit(o);
 	}
 }
 
 void LuaMatRef::AddFeature(CSolidObject* o)
 {
-	if (bin != NULL) {
+	if (bin != nullptr) {
 		bin->AddFeature(o);
 	}
 }
@@ -771,18 +827,12 @@ void LuaMatRef::AddFeature(CSolidObject* o)
 //  LuaMatBin
 //
 
-void LuaMatBin::Ref()
-{
-	refCount++;
-}
-
-
 void LuaMatBin::UnRef()
 {
-	refCount--;
-	if (refCount <= 0) {
-		luaMatHandler.FreeBin(this);
-	}
+	if ((--refCount) > 0)
+		return;
+
+	luaMatHandler.FreeBin(this);
 }
 
 
@@ -823,23 +873,36 @@ LuaMatHandler::~LuaMatHandler()
 LuaMatRef LuaMatHandler::GetRef(const LuaMaterial& mat)
 {
 	if ((mat.type < 0) || (mat.type >= LUAMAT_TYPE_COUNT)) {
-		LOG_L(L_WARNING, "LuaMatHandler::GetRef() untyped material");
+		LOG_L(L_WARNING, "[LuaMatHandler::%s] untyped material %d", __func__, mat.type);
 		return LuaMatRef();
 	}
 
 	LuaMatBin* fakeBin = (LuaMatBin*) &mat;
 
 	LuaMatBinSet& binSet = binTypes[mat.type];
-	LuaMatBinSet::iterator it = binSet.find(fakeBin);
+	LuaMatBinSet::iterator it = std::find_if(binSet.begin(), binSet.end(), [&](LuaMatBin* bin) {
+		return (!matBinCmp(bin, fakeBin) && !matBinCmp(fakeBin, bin));
+	});
 
 	if (it != binSet.end()) {
-		assert(*fakeBin == **it);
-		return LuaMatRef(*it);
+		assert(*fakeBin == *(*it));
+		return (LuaMatRef(*it));
 	}
 
-	LuaMatBin* bin = new LuaMatBin(mat);
-	binSet.insert(bin);
-	return LuaMatRef(bin);
+	binSet.push_back(new LuaMatBin(mat));
+
+	LuaMatBin* bin = binSet.back();
+	LuaMatRef ref(bin);
+
+	// swap new bin into place, faster than resorting
+	for (size_t n = binSet.size() - 1; n > 0; n--) {
+		if (matBinCmp(binSet[n - 1], binSet[n]))
+			break;
+
+		std::swap(binSet[n - 1], binSet[n]);
+	}
+
+	return ref;
 }
 
 
@@ -866,22 +929,35 @@ void LuaMatHandler::ClearBins(LuaObjType objType)
 }
 
 
-void LuaMatHandler::FreeBin(LuaMatBin* bin)
+void LuaMatHandler::FreeBin(LuaMatBin* argBin)
 {
-	LuaMatBinSet& binSet = binTypes[bin->type];
-	LuaMatBinSet::iterator it = binSet.find(bin);
-	if (it != binSet.end()) {
-		delete bin;
-		binSet.erase(it);
+	LuaMatBinSet& binSet = binTypes[argBin->type];
+	LuaMatBinSet::iterator it = std::find_if(binSet.begin(), binSet.end(), [&](LuaMatBin* bin) {
+		return (!matBinCmp(bin, argBin) && !matBinCmp(argBin, bin));
+	});
+
+	if (it == binSet.end())
+		return;
+
+	assert((*it) == argBin);
+
+	// swap to-be-deleted bin to the back, faster than resorting
+	for (size_t n = (it - binSet.begin()); n < (binSet.size() - 1); n++) {
+		std::swap(binSet[n], binSet[n + 1]);
 	}
+
+	assert(binSet.back() == argBin);
+	binSet.pop_back();
+
+	delete argBin;
 }
 
 
 void LuaMatHandler::PrintBins(const string& indent, LuaMatType type) const
 {
-	if ((type < 0) || (type >= LUAMAT_TYPE_COUNT)) {
+	if ((type < 0) || (type >= LUAMAT_TYPE_COUNT))
 		return;
-	}
+
 	int num = 0;
 	LOG("%sBINCOUNT = " _STPF_, indent.c_str(), binTypes[type].size());
 	for (LuaMatBin* bin: binTypes[type]) {
@@ -895,9 +971,7 @@ void LuaMatHandler::PrintBins(const string& indent, LuaMatType type) const
 void LuaMatHandler::PrintAllBins(const string& indent) const
 {
 	for (int m = 0; m < LUAMAT_TYPE_COUNT; m++) {
-		std::string newIndent = indent + GetMatTypeName(LuaMatType(m));
-		newIndent += "  ";
-		PrintBins(newIndent, LuaMatType(m));
+		PrintBins(indent + GetMatTypeName(LuaMatType(m)) + "  ", LuaMatType(m));
 	}
 }
 
