@@ -5,7 +5,6 @@
 
 #include <algorithm>
 #include <limits.h>
-#include <boost/regex.hpp>
 
 #include "lib/streflop/streflop_cond.h"
 
@@ -13,19 +12,24 @@
 #include "System/float4.h"
 #include "LuaInclude.h"
 
+#include "LuaConstEngine.h"
 #include "LuaIO.h"
 #include "LuaUtils.h"
 
 #include "Game/GameVersion.h"
+#include "Sim/Misc/GlobalSynced.h" // gsRNG
 #include "System/Log/ILog.h"
 #include "System/FileSystem/FileHandler.h"
-#include "System/FileSystem/VFSHandler.h"
-#include "System/FileSystem/FileSystem.h"
 #include "System/Misc/SpringTime.h"
+#include "System/ContainerUtil.h"
 #include "System/ScopedFPUSettings.h"
-#include "System/Util.h"
+#include "System/StringUtil.h"
 
-LuaParser* LuaParser::currentParser = NULL;
+
+LuaParser* GetLuaParser(lua_State* L) {
+	assert(GetLuaContextData(L)->parser != nullptr);
+	return GetLuaContextData(L)->parser;
+}
 
 
 /******************************************************************************/
@@ -34,66 +38,75 @@ LuaParser* LuaParser::currentParser = NULL;
 //  LuaParser
 //
 
-LuaParser::LuaParser(const string& _fileName,
-                     const string& _fileModes,
-                     const string& _accessModes)
-: fileName(_fileName),
-  fileModes(_fileModes),
-  textChunk(""),
-  accessModes(_accessModes),
-  valid(false),
-  initDepth(0),
-  rootRef(LUA_NOREF),
-  currentRef(LUA_NOREF),
-  lowerKeys(true),
-  lowerCppKeys(true)
-{
-	L = LUA_OPEN();
+LuaParser::LuaParser(const string& _fileName, const string& _fileModes, const string& _accessModes, const boolean& synced)
+	: fileName(_fileName)
+	, fileModes(_fileModes)
+	, accessModes(_accessModes)
 
-	if (L != NULL) {
-		SetupEnv();
+	, D(false, false)
+
+	, initDepth(0)
+	, rootRef(LUA_NOREF)
+	, currentRef(LUA_NOREF)
+
+	, valid(false)
+	, lowerKeys(true)
+	, lowerCppKeys(true)
+{
+	// be on the safe side
+	D.synced = true;
+	D.parser = this;
+
+	if ((L = LUA_OPEN(&D)) != nullptr) {
+		SetupEnv(synced.b);
 	}
 }
 
+LuaParser::LuaParser(const string& _textChunk, const string& _accessModes, const boolean& synced)
+	: fileName("")
+	, fileModes("")
+	, textChunk(_textChunk)
+	, accessModes(_accessModes)
 
-LuaParser::LuaParser(const string& _textChunk,
-                     const string& _accessModes)
-: fileName(""),
-  fileModes(""),
-  textChunk(_textChunk),
-  accessModes(_accessModes),
-  valid(false),
-  initDepth(0),
-  rootRef(LUA_NOREF),
-  currentRef(LUA_NOREF),
-  lowerKeys(true),
-  lowerCppKeys(true)
+	, D(false, false)
+
+	, initDepth(0)
+	, rootRef(LUA_NOREF)
+	, currentRef(LUA_NOREF)
+
+	, valid(false)
+	, lowerKeys(true)
+	, lowerCppKeys(true)
 {
-	L = LUA_OPEN();
+	// be on the safe side
+	D.synced = true;
+	D.parser = this;
 
-	if (L != NULL) {
-		SetupEnv();
+	if ((L = LUA_OPEN(&D)) != nullptr) {
+		SetupEnv(synced.b);
 	}
 }
 
 
 LuaParser::~LuaParser()
 {
-	if (L != NULL)
+	// prevent crashes in glDelete* calls since LuaParser
+	// might be constructed by multiple different threads
+	D.Clear();
+
+	if (L != nullptr)
 		LUA_CLOSE(&L);
 
-	set<LuaTable*>::iterator it;
-	for (it = tables.begin(); it != tables.end(); ++it) {
-		LuaTable& table = **it;
-		table.parser  = NULL;
-		table.L       = NULL;
-		table.isValid = false;
-		table.refnum  = LUA_NOREF;
+	// invalidate leftover tables if parser is destroyed first
+	for (LuaTable* table: tables) {
+		table->parser  = nullptr;
+		table->L       = nullptr;
+		table->isValid = false;
+		table->refnum  = LUA_NOREF;
 	}
 }
 
-
-void LuaParser::SetupEnv()
+void LuaParser::SetupEnv(bool synced)
 {
 	LUA_OPEN_LIB(L, luaopen_base);
 	LUA_OPEN_LIB(L, luaopen_math);
@@ -113,12 +126,17 @@ void LuaParser::SetupEnv()
 	lua_pushnil(L); lua_setglobal(L, "collectgarbage");
 	lua_pushnil(L); lua_setglobal(L, "newproxy"); // not sync-safe cause of __gc
 
-	// FIXME: replace "random" _as in_ LuaHandleSynced (can write your own for now), we cannot use the LHS one cause gs-> is not valid in LuaParser
-	//        using streflop's RNG is possible
-	lua_getglobal(L, "math");
-	lua_pushliteral(L, "random");     lua_pushnil(L); lua_rawset(L, -3);
-	lua_pushliteral(L, "randomseed"); lua_pushnil(L); lua_rawset(L, -3);
-	lua_pop(L, 1); // pop "math"
+	{
+		lua_getglobal(L, "math");
+		if (synced) {
+			LuaPushNamedCFunc(L, "random", Random);
+			LuaPushNamedCFunc(L, "randomseed", RandomSeed);
+		} else {
+			LuaPushNamedCFunc(L, "random", DummyRandom);
+			LuaPushNamedCFunc(L, "randomseed", DummyRandomSeed);
+		}
+		lua_pop(L, 1); // pop "math"
+	}
 
 	AddFunc("DontMessWithMyCase", DontMessWithMyCase);
 
@@ -128,8 +146,8 @@ void LuaParser::SetupEnv()
 	AddFunc("TimeCheck", TimeCheck);
 	EndTable();
 
-	GetTable("Game");
-	LuaPushNamedString(L, "version", SpringVersion::GetSync());
+	GetTable("Engine");
+	LuaConstEngine::PushEntries(L);
 	EndTable();
 
 	GetTable("VFS");
@@ -162,6 +180,7 @@ bool LuaParser::Execute()
 
 	string code;
 	string codeLabel;
+
 	if (!textChunk.empty()) {
 		code = textChunk;
 		codeLabel = "text chunk";
@@ -181,44 +200,42 @@ bool LuaParser::Execute()
 		return false;
 	}
 
-	int error;
-	error = luaL_loadbuffer(L, code.c_str(), code.size(), codeLabel.c_str());
-	if (error != 0) {
-		errorLog = lua_tostring(L, -1);
-		LOG_L(L_ERROR, "%i, %s, %s",
-		                error, codeLabel.c_str(), errorLog.c_str());
-		LUA_CLOSE(&L);
-		return false;
-	}
-
-	currentParser = this;
-
-	// do not signal floating point exceptions in user Lua code
-	ScopedDisableFpuExceptions fe;
-	error = lua_pcall(L, 0, 1, 0);
-
-	currentParser = NULL;
+	int error = luaL_loadbuffer(L, code.c_str(), code.size(), codeLabel.c_str());
 
 	if (error != 0) {
 		errorLog = lua_tostring(L, -1);
-		LOG_L(L_ERROR, "%i, %s, %s",
-		                error, fileName.c_str(), errorLog.c_str());
+		LOG_L(L_ERROR, "%i, %s, %s", error, codeLabel.c_str(), errorLog.c_str());
 		LUA_CLOSE(&L);
 		return false;
 	}
 
-	if (!lua_istable(L, 1)) {
-		errorLog = "missing return table from " + fileName;
-		LOG_L(L_ERROR, "missing return table from %s", fileName.c_str());
-		LUA_CLOSE(&L);
-		return false;
-	}
+	{
+		// LuaParser::Execute can be called concurrently by the
+		// game-load and (e.g.) assimp model preloading threads
 
-	if (lowerKeys) {
-		LuaUtils::LowerKeys(L, 1);
-	}
+		// do not signal floating point exceptions in user Lua code
+		ScopedDisableFpuExceptions fe;
+		error = lua_pcall(L, 0, 1, 0);
 
-	LuaUtils::CheckTableForNaNs(L, 1, fileName);
+		if (error != 0) {
+			errorLog = lua_tostring(L, -1);
+			LOG_L(L_ERROR, "%i, %s, %s", error, fileName.c_str(), errorLog.c_str());
+			LUA_CLOSE(&L);
+			return false;
+		}
+
+		if (!lua_istable(L, 1)) {
+			errorLog = "missing return table from " + fileName;
+			LOG_L(L_ERROR, "missing return table from %s", fileName.c_str());
+			LUA_CLOSE(&L);
+			return false;
+		}
+
+		if (lowerKeys)
+			LuaUtils::LowerKeys(L, 1);
+
+		LuaUtils::CheckTableForNaNs(L, 1, fileName);
+	}
 
 	rootRef = luaL_ref(L, LUA_REGISTRYINDEX);
 	lua_settop(L, 0);
@@ -227,16 +244,8 @@ bool LuaParser::Execute()
 }
 
 
-void LuaParser::AddTable(LuaTable* tbl)
-{
-	tables.insert(tbl);
-}
-
-
-void LuaParser::RemoveTable(LuaTable* tbl)
-{
-	tables.erase(tbl);
-}
+void LuaParser::AddTable(LuaTable* tbl) { spring::VectorInsertUnique(tables, tbl); }
+void LuaParser::RemoveTable(LuaTable* tbl) { spring::VectorErase(tables, tbl); }
 
 
 LuaTable LuaParser::GetRoot()
@@ -249,7 +258,9 @@ LuaTable LuaParser::GetRoot()
 
 void LuaParser::PushParam()
 {
-	if (!IsValid() || (initDepth < 0)) { return; }
+	if (!IsValid() || (initDepth < 0))
+		return;
+
 	if (initDepth > 0) {
 		lua_rawset(L, -3);
 	} else {
@@ -260,7 +271,8 @@ void LuaParser::PushParam()
 
 void LuaParser::GetTable(const string& name, bool overwrite)
 {
-	if (!IsValid() || (initDepth < 0)) { return; }
+	if (!IsValid() || (initDepth < 0))
+		return;
 
 	lua_pushsstring(L, name);
 
@@ -282,14 +294,14 @@ void LuaParser::GetTable(const string& name, bool overwrite)
 
 void LuaParser::GetTable(int index, bool overwrite)
 {
-	if (!IsValid() || (initDepth < 0)) { return; }
+	if (!IsValid() || (initDepth < 0))
+		return;
 
 	lua_pushnumber(L, index);
 
 	if (overwrite) {
 		lua_newtable(L);
-	}
-	else {
+	} else {
 		lua_pushnumber(L, index);
 		lua_gettable(L, (initDepth == 0) ? LUA_GLOBALSINDEX : -3);
 		if (!lua_istable(L, -1)) {
@@ -304,7 +316,8 @@ void LuaParser::GetTable(int index, bool overwrite)
 
 void LuaParser::EndTable()
 {
-	if (!IsValid() || (initDepth < 0)) { return; }
+	if (!IsValid() || (initDepth < 0))
+		return;
 	assert(initDepth > 0);
 	initDepth--;
 	PushParam();
@@ -315,17 +328,25 @@ void LuaParser::EndTable()
 
 void LuaParser::AddFunc(const string& key, int (*func)(lua_State*))
 {
-	if (!IsValid() || (initDepth < 0)) { return; }
-	if (func == NULL) { return; }
+	if (!IsValid() || (initDepth < 0))
+		return;
+	if (func == nullptr)
+		return;
+
+	// can not use this; initDepth check in PushParam
+	// LuaPushNamedCFunc(L, key, func);
 	lua_pushsstring(L, key);
 	lua_pushcfunction(L, func);
+
+	// t[k] = v
 	PushParam();
 }
 
 
 void LuaParser::AddInt(const string& key, int value)
 {
-	if (!IsValid() || (initDepth < 0)) { return; }
+	if (!IsValid() || (initDepth < 0))
+		return;
 	lua_pushsstring(L, key);
 	lua_pushnumber(L, value);
 	PushParam();
@@ -334,7 +355,8 @@ void LuaParser::AddInt(const string& key, int value)
 
 void LuaParser::AddBool(const string& key, bool value)
 {
-	if (!IsValid() || (initDepth < 0)) { return; }
+	if (!IsValid() || (initDepth < 0))
+		return;
 	lua_pushsstring(L, key);
 	lua_pushboolean(L, value);
 	PushParam();
@@ -343,7 +365,8 @@ void LuaParser::AddBool(const string& key, bool value)
 
 void LuaParser::AddFloat(const string& key, float value)
 {
-	if (!IsValid() || (initDepth < 0)) { return; }
+	if (!IsValid() || (initDepth < 0))
+		return;
 	lua_pushsstring(L, key);
 	lua_pushnumber(L, value);
 	PushParam();
@@ -352,7 +375,8 @@ void LuaParser::AddFloat(const string& key, float value)
 
 void LuaParser::AddString(const string& key, const string& value)
 {
-	if (!IsValid() || (initDepth < 0)) { return; }
+	if (!IsValid() || (initDepth < 0))
+		return;
 	lua_pushsstring(L, key);
 	lua_pushsstring(L, value);
 	PushParam();
@@ -363,17 +387,22 @@ void LuaParser::AddString(const string& key, const string& value)
 
 void LuaParser::AddFunc(int key, int (*func)(lua_State*))
 {
-	if (!IsValid() || (initDepth < 0)) { return; }
-	if (func == NULL) { return; }
+	if (!IsValid() || (initDepth < 0))
+		return;
+	if (func == nullptr)
+		return;
+
 	lua_pushnumber(L, key);
 	lua_pushcfunction(L, func);
+
 	PushParam();
 }
 
 
 void LuaParser::AddInt(int key, int value)
 {
-	if (!IsValid() || (initDepth < 0)) { return; }
+	if (!IsValid() || (initDepth < 0))
+		return;
 	lua_pushnumber(L, key);
 	lua_pushnumber(L, value);
 	PushParam();
@@ -382,7 +411,8 @@ void LuaParser::AddInt(int key, int value)
 
 void LuaParser::AddBool(int key, bool value)
 {
-	if (!IsValid() || (initDepth < 0)) { return; }
+	if (!IsValid() || (initDepth < 0))
+		return;
 	lua_pushnumber(L, key);
 	lua_pushboolean(L, value);
 	PushParam();
@@ -391,7 +421,8 @@ void LuaParser::AddBool(int key, bool value)
 
 void LuaParser::AddFloat(int key, float value)
 {
-	if (!IsValid() || (initDepth < 0)) { return; }
+	if (!IsValid() || (initDepth < 0))
+		return;
 	lua_pushnumber(L, key);
 	lua_pushnumber(L, value);
 	PushParam();
@@ -400,7 +431,8 @@ void LuaParser::AddFloat(int key, float value)
 
 void LuaParser::AddString(int key, const string& value)
 {
-	if (!IsValid() || (initDepth < 0)) { return; }
+	if (!IsValid() || (initDepth < 0))
+		return;
 	lua_pushnumber(L, key);
 	lua_pushsstring(L, value);
 	PushParam();
@@ -436,21 +468,37 @@ int LuaParser::TimeCheck(lua_State* L)
 }
 
 
+/******************************************************************************/
+
+// seeding makes little sense inside LuaParser execution
+int LuaParser::RandomSeed(lua_State* L) { return (DummyRandomSeed(L)); }
+int LuaParser::Random(lua_State* L)
+{
+	// both US and DS depend on LuaParser via MapParser, etc
+	#if (!defined(UNITSYNC) && !defined(DEDICATED))
+	lua_pushnumber(L, gsRNG.NextFloat());
+	return 1;
+	#else
+	return (DummyRandom(L));
+	#endif
+}
+
+int LuaParser::DummyRandomSeed(lua_State* L) { return 0; }
+int LuaParser::DummyRandom(lua_State* L) { return 0; }
+
 /******************************************************************************/
 
 int LuaParser::DirList(lua_State* L)
 {
-	if (currentParser == NULL) {
-		luaL_error(L, "invalid call to DirList() after execution");
-	}
+	const LuaParser* currentParser = GetLuaParser(L);
 
-	const string dir = luaL_checkstring(L, 1);
-	if (!LuaIO::IsSimplePath(dir)) {
+	const std::string& dir = luaL_checkstring(L, 1);
+
+	if (!LuaIO::IsSimplePath(dir))
 		return 0;
-	}
-	const string pat = luaL_optstring(L, 2, "*");
-	string modes = luaL_optstring(L, 3, currentParser->accessModes.c_str());
-	modes = CFileHandler::AllowModes(modes, currentParser->accessModes);
+
+	const std::string& pat = luaL_optstring(L, 2, "*");
+	const std::string& modes = CFileHandler::AllowModes(luaL_optstring(L, 3, currentParser->accessModes.c_str()), currentParser->accessModes);
 
 	LuaUtils::PushStringVector(L, CFileHandler::DirList(dir, pat, modes));
 	return 1;
@@ -459,17 +507,15 @@ int LuaParser::DirList(lua_State* L)
 
 int LuaParser::SubDirs(lua_State* L)
 {
-	if (currentParser == NULL) {
-		luaL_error(L, "invalid call to SubDirs() after execution");
-	}
+	const LuaParser* currentParser = GetLuaParser(L);
 
-	const string dir = luaL_checkstring(L, 1);
-	if (!LuaIO::IsSimplePath(dir)) {
+	const std::string& dir = luaL_checkstring(L, 1);
+
+	if (!LuaIO::IsSimplePath(dir))
 		return 0;
-	}
-	const string pat = luaL_optstring(L, 2, "*");
-	string modes = luaL_optstring(L, 3, currentParser->accessModes.c_str());
-	modes = CFileHandler::AllowModes(modes, currentParser->accessModes);
+
+	const std::string& pat = luaL_optstring(L, 2, "*");
+	const std::string& modes = CFileHandler::AllowModes(luaL_optstring(L, 3, currentParser->accessModes.c_str()), currentParser->accessModes);
 
 	LuaUtils::PushStringVector(L, CFileHandler::SubDirs(dir, pat, modes));
 	return 1;
@@ -479,23 +525,20 @@ int LuaParser::SubDirs(lua_State* L)
 
 int LuaParser::Include(lua_State* L)
 {
-	if (currentParser == NULL) {
-		luaL_error(L, "invalid call to Include() after execution");
-	}
+	const LuaParser* currentParser = GetLuaParser(L);
 
 	// filename [, fenv]
-	const string filename = luaL_checkstring(L, 1);
-	if (!LuaIO::IsSimplePath(filename)) {
+	const std::string& filename = luaL_checkstring(L, 1);
+
+	if (!LuaIO::IsSimplePath(filename))
 		luaL_error(L, "bad pathname");
-	}
-	string modes = luaL_optstring(L, 3, currentParser->accessModes.c_str());
-	modes = CFileHandler::AllowModes(modes, currentParser->accessModes);
+
+	const std::string& modes = CFileHandler::AllowModes(luaL_optstring(L, 3, currentParser->accessModes.c_str()), currentParser->accessModes);
 
 	CFileHandler fh(filename, modes);
 	if (!fh.FileExists()) {
 		char buf[1024];
-		SNPRINTF(buf, sizeof(buf),
-		         "Include() file missing '%s'\n", filename.c_str());
+		SNPRINTF(buf, sizeof(buf), "Include() file missing '%s'\n", filename.c_str());
 		lua_pushstring(L, buf);
  		lua_error(L);
 	}
@@ -503,8 +546,7 @@ int LuaParser::Include(lua_State* L)
 	string code;
 	if (!fh.LoadStringData(code)) {
 		char buf[1024];
-		SNPRINTF(buf, sizeof(buf),
-		         "Include() could not load '%s'\n", filename.c_str());
+		SNPRINTF(buf, sizeof(buf), "Include() could not load '%s'\n", filename.c_str());
 		lua_pushstring(L, buf);
  		lua_error(L);
 	}
@@ -512,8 +554,7 @@ int LuaParser::Include(lua_State* L)
 	int error = luaL_loadbuffer(L, code.c_str(), code.size(), filename.c_str());
 	if (error != 0) {
 		char buf[1024];
-		SNPRINTF(buf, sizeof(buf), "error = %i, %s, %s\n",
-		         error, filename.c_str(), lua_tostring(L, -1));
+		SNPRINTF(buf, sizeof(buf), "error = %i, %s, %s\n", error, filename.c_str(), lua_tostring(L, -1));
 		lua_pushstring(L, buf);
 		lua_error(L);
 	}
@@ -536,15 +577,15 @@ int LuaParser::Include(lua_State* L)
 
 	if (error != 0) {
 		char buf[1024];
-		SNPRINTF(buf, sizeof(buf), "error = %i, %s, %s\n",
-		         error, filename.c_str(), lua_tostring(L, -1));
+		SNPRINTF(buf, sizeof(buf), "error = %i, %s, %s\n", error, filename.c_str(), lua_tostring(L, -1));
 		lua_pushstring(L, buf);
 		lua_error(L);
 	}
 
-	currentParser->accessedFiles.insert(StringToLower(filename));
-
-	return lua_gettop(L) - paramTop;
+	#if 0
+	spring::VectorInsertUnique(currentParser->accessedFiles, StringToLower(filename), true);
+	#endif
+	return (lua_gettop(L) - paramTop);
 }
 
 
@@ -552,16 +593,14 @@ int LuaParser::Include(lua_State* L)
 
 int LuaParser::LoadFile(lua_State* L)
 {
-	if (currentParser == NULL) {
-		luaL_error(L, "invalid call to LoadFile() after execution");
-	}
+	const LuaParser* currentParser = GetLuaParser(L);
 
-	const string filename = luaL_checkstring(L, 1);
-	if (!LuaIO::IsSimplePath(filename)) {
+	const std::string&  filename = luaL_checkstring(L, 1);
+
+	if (!LuaIO::IsSimplePath(filename))
 		return 0;
-	}
-	string modes = luaL_optstring(L, 2, currentParser->accessModes.c_str());
-	modes = CFileHandler::AllowModes(modes, currentParser->accessModes);
+
+	const std::string& modes = CFileHandler::AllowModes(luaL_optstring(L, 2, currentParser->accessModes.c_str()), currentParser->accessModes);
 
 	CFileHandler fh(filename, modes);
 	if (!fh.FileExists()) {
@@ -577,21 +616,22 @@ int LuaParser::LoadFile(lua_State* L)
 	}
 	lua_pushstring(L, data.c_str());
 
-	currentParser->accessedFiles.insert(StringToLower(filename));
-
+	#if 0
+	spring::VectorInsertUnique(currentParser->accessedFiles, StringToLower(filename), true);
+	#endif
 	return 1;
 }
 
 
 int LuaParser::FileExists(lua_State* L)
 {
-	if (currentParser == NULL) {
-		luaL_error(L, "invalid call to FileExists() after execution");
-	}
-	const string filename = luaL_checkstring(L, 1);
-	if (!LuaIO::IsSimplePath(filename)) {
+	const LuaParser* currentParser = GetLuaParser(L);
+
+	const std::string&  filename = luaL_checkstring(L, 1);
+
+	if (!LuaIO::IsSimplePath(filename))
 		return 0;
-	}
+
 	lua_pushboolean(L, CFileHandler::FileExists(filename, currentParser->accessModes));
 	return 1;
 }
@@ -599,9 +639,8 @@ int LuaParser::FileExists(lua_State* L)
 
 int LuaParser::DontMessWithMyCase(lua_State* L)
 {
-	if (currentParser == NULL) {
-		luaL_error(L, "invalid call to DontMessWithMyCase() after execution");
-	}
+	LuaParser* currentParser = GetLuaParser(L);
+
 	currentParser->SetLowerKeys(lua_toboolean(L, 1));
 	return 0;
 }
@@ -633,6 +672,8 @@ LuaTable::LuaTable(LuaParser* _parser)
 	L       = parser->L;
 	refnum  = parser->rootRef;
 
+	parser->AddTable(this);
+
 	if (PushTable()) {
 		lua_pushvalue(L, -1); // copy
 		refnum = luaL_ref(L, LUA_REGISTRYINDEX);
@@ -640,8 +681,6 @@ LuaTable::LuaTable(LuaParser* _parser)
 	 	refnum = LUA_NOREF;
 	}
 	isValid = (refnum != LUA_NOREF);
-
-	parser->AddTable(this);
 }
 
 
@@ -651,6 +690,9 @@ LuaTable::LuaTable(const LuaTable& tbl)
 	L      = tbl.L;
 	path   = tbl.path;
 
+	if (parser != nullptr)
+		parser->AddTable(this);
+
 	if (tbl.PushTable()) {
 		lua_pushvalue(L, -1); // copy
 		refnum = luaL_ref(L, LUA_REGISTRYINDEX);
@@ -658,31 +700,27 @@ LuaTable::LuaTable(const LuaTable& tbl)
 		refnum = LUA_NOREF;
 	}
 	isValid = (refnum != LUA_NOREF);
-
-	if (parser) {
-		parser->AddTable(this);
-	}
 }
 
 
 LuaTable& LuaTable::operator=(const LuaTable& tbl)
 {
-	if (parser && (refnum != LUA_NOREF) && (parser->currentRef == refnum)) {
+	if (parser != nullptr && (refnum != LUA_NOREF) && (parser->currentRef == refnum)) {
 		lua_settop(L, 0);
 		parser->currentRef = LUA_NOREF;
 	}
 
 	if (parser != tbl.parser) {
-		if (parser != NULL) {
+		if (parser != nullptr)
 			parser->RemoveTable(this);
-		}
-		if (L && (refnum != LUA_NOREF)) {
+
+		if (L != nullptr && (refnum != LUA_NOREF))
 			luaL_unref(L, LUA_REGISTRYINDEX, refnum);
-		}
+
+		if (tbl.parser != nullptr)
+			tbl.parser->AddTable(this);
+
 		parser = tbl.parser;
-		if (parser != NULL) {
-			parser->AddTable(this);
-		}
 	}
 
 	L    = tbl.L;
@@ -708,9 +746,8 @@ LuaTable LuaTable::SubTable(int key) const
 	SNPRINTF(buf, 32, "[%i]", key);
 	subTable.path = path + buf;
 
-	if (!PushTable()) {
+	if (!PushTable())
 		return subTable;
-	}
 
 	lua_pushnumber(L, key);
 	lua_gettable(L, -2);
@@ -725,22 +762,19 @@ LuaTable LuaTable::SubTable(int key) const
 	subTable.isValid = (subTable.refnum != LUA_NOREF);
 
 	parser->AddTable(&subTable);
-
 	return subTable;
 }
 
 
 LuaTable LuaTable::SubTable(const string& mixedKey) const
 {
-
 	const string key = !(parser ? parser->lowerCppKeys : true) ? mixedKey : StringToLower(mixedKey);
 
 	LuaTable subTable;
 	subTable.path = path + "." + key;
 
-	if (!PushTable()) {
+	if (!PushTable())
 		return subTable;
-	}
 
 	lua_pushstring(L, key.c_str());
 	lua_gettable(L, -2);
@@ -755,7 +789,6 @@ LuaTable LuaTable::SubTable(const string& mixedKey) const
 	subTable.isValid = (subTable.refnum != LUA_NOREF);
 
 	parser->AddTable(&subTable);
-
 	return subTable;
 }
 
@@ -803,16 +836,17 @@ LuaTable LuaTable::SubTableExpr(const string& expr) const
 
 LuaTable::~LuaTable()
 {
-	if (L && (refnum != LUA_NOREF)) {
+	if (parser != nullptr)
+		parser->RemoveTable(this);
+
+	if (L != nullptr && (refnum != LUA_NOREF)) {
 		luaL_unref(L, LUA_REGISTRYINDEX, refnum);
-		if (parser && (parser->currentRef == refnum)) {
+
+		if (parser != nullptr && (parser->currentRef == refnum)) {
 			lua_settop(L, 0);
 			parser->currentRef = LUA_NOREF;
 		}
 	}
-	if (parser) {
-		parser->RemoveTable(this);
-	}
 }
 
 
@@ -820,14 +854,12 @@ LuaTable::~LuaTable()
 
 bool LuaTable::PushTable() const
 {
-	if (!isValid) {
+	if (!isValid)
 		return false;
-	}
 
 	if ((refnum != LUA_NOREF) && (parser->currentRef == refnum)) {
 		if (!lua_istable(L, -1)) {
-			LOG_L(L_ERROR, "Internal Error: LuaTable::PushTable() = %s",
-					path.c_str());
+			LOG_L(L_ERROR, "Internal Error: LuaTable::PushTable() = %s", path.c_str());
 			parser->currentRef = LUA_NOREF;
 			lua_settop(L, 0);
 			return false;
@@ -1058,11 +1090,11 @@ int LuaTable::GetLength(const string& key) const
 //  Key list functions
 //
 
-bool LuaTable::GetKeys(vector<int>& data) const
+bool LuaTable::GetKeys(std::vector<int>& data) const
 {
-	if (!PushTable()) {
+	if (!PushTable())
 		return false;
-	}
+
 	const int table = lua_gettop(L);
 	for (lua_pushnil(L); lua_next(L, table) != 0; lua_pop(L, 1)) {
 		if (lua_israwnumber(L, -2)) {
@@ -1070,16 +1102,16 @@ bool LuaTable::GetKeys(vector<int>& data) const
 			data.push_back(value);
 		}
 	}
-	std::sort(data.begin(), data.end());
+	std::stable_sort(data.begin(), data.end());
 	return true;
 }
 
 
-bool LuaTable::GetKeys(vector<string>& data) const
+bool LuaTable::GetKeys(std::vector<string>& data) const
 {
-	if (!PushTable()) {
+	if (!PushTable())
 		return false;
-	}
+
 	const int table = lua_gettop(L);
 	for (lua_pushnil(L); lua_next(L, table) != 0; lua_pop(L, 1)) {
 		if (lua_israwstring(L, -2)) {
@@ -1087,7 +1119,7 @@ bool LuaTable::GetKeys(vector<string>& data) const
 			data.push_back(value);
 		}
 	}
-	std::sort(data.begin(), data.end());
+	std::stable_sort(data.begin(), data.end());
 	return true;
 }
 
@@ -1098,11 +1130,11 @@ bool LuaTable::GetKeys(vector<string>& data) const
 //  Map functions
 //
 
-bool LuaTable::GetMap(map<int, float>& data) const
+bool LuaTable::GetMap(spring::unordered_map<int, float>& data) const
 {
-	if (!PushTable()) {
+	if (!PushTable())
 		return false;
-	}
+
 	const int table = lua_gettop(L);
 	for (lua_pushnil(L); lua_next(L, table) != 0; lua_pop(L, 1)) {
 		if (lua_israwnumber(L, -2) && lua_isnumber(L, -1)) {
@@ -1115,11 +1147,11 @@ bool LuaTable::GetMap(map<int, float>& data) const
 }
 
 
-bool LuaTable::GetMap(map<int, string>& data) const
+bool LuaTable::GetMap(spring::unordered_map<int, string>& data) const
 {
-	if (!PushTable()) {
+	if (!PushTable())
 		return false;
-	}
+
 	const int table = lua_gettop(L);
 	for (lua_pushnil(L); lua_next(L, table) != 0; lua_pop(L, 1)) {
 		if (lua_israwnumber(L, -2) && lua_isstring(L, -1)) {
@@ -1138,11 +1170,11 @@ bool LuaTable::GetMap(map<int, string>& data) const
 }
 
 
-bool LuaTable::GetMap(map<string, float>& data) const
+bool LuaTable::GetMap(spring::unordered_map<string, float>& data) const
 {
-	if (!PushTable()) {
+	if (!PushTable())
 		return false;
-	}
+
 	const int table = lua_gettop(L);
 	for (lua_pushnil(L); lua_next(L, table) != 0; lua_pop(L, 1)) {
 		if (lua_israwstring(L, -2) && lua_isnumber(L, -1)) {
@@ -1155,11 +1187,11 @@ bool LuaTable::GetMap(map<string, float>& data) const
 }
 
 
-bool LuaTable::GetMap(map<string, string>& data) const
+bool LuaTable::GetMap(spring::unordered_map<string, string>& data) const
 {
-	if (!PushTable()) {
+	if (!PushTable())
 		return false;
-	}
+
 	const int table = lua_gettop(L);
 	for (lua_pushnil(L); lua_next(L, table) != 0; lua_pop(L, 1)) {
 		if (lua_israwstring(L, -2)) {
