@@ -4,7 +4,6 @@
 #include "LuaShaders.h"
 
 #include "LuaInclude.h"
-
 #include "LuaHashString.h"
 #include "LuaHandle.h"
 #include "LuaOpenGL.h"
@@ -12,15 +11,20 @@
 #include "LuaUtils.h"
 
 #include "Game/Camera.h"
-#include "Rendering/ShadowHandler.h"
 #include "System/Log/ILog.h"
-#include "System/Util.h"
+#include "System/StringUtil.h"
 
 #include <string>
 #include <vector>
-using std::string;
-using std::vector;
 
+struct ActiveUniform {
+	GLint size = 0;
+	GLenum type = 0;
+	GLchar name[512] = {0};
+};
+
+int   intUniformArrayBuf[1024] = {0   };
+float fltUniformArrayBuf[1024] = {0.0f};
 
 int LuaShaders::activeShaderDepth = 0;
 
@@ -30,11 +34,6 @@ int LuaShaders::activeShaderDepth = 0;
 
 bool LuaShaders::PushEntries(lua_State* L)
 {
-#define REGISTER_LUA_CFUNC(x) \
-	lua_pushstring(L, #x);      \
-	lua_pushcfunction(L, x);    \
-	lua_rawset(L, -3)
-
 	REGISTER_LUA_CFUNC(CreateShader);
 	REGISTER_LUA_CFUNC(DeleteShader);
 	REGISTER_LUA_CFUNC(UseShader);
@@ -79,10 +78,10 @@ LuaShaders::~LuaShaders()
 
 inline void CheckDrawingEnabled(lua_State* L, const char* caller)
 {
-	if (!LuaOpenGL::IsDrawingEnabled(L)) {
-		luaL_error(L, "%s(): OpenGL calls can only be used in Draw() "
-		              "call-ins, or while creating display lists", caller);
-	}
+	if (LuaOpenGL::IsDrawingEnabled(L))
+		return;
+
+	luaL_error(L, "%s(): OpenGL calls can only be used in Draw() " "call-ins, or while creating display lists", caller);
 }
 
 
@@ -166,9 +165,10 @@ int LuaShaders::GetShaderLog(lua_State* L)
 /******************************************************************************/
 
 enum {
-	UNIFORM_TYPE_INT          = 0, // includes arrays
-	UNIFORM_TYPE_FLOAT        = 1, // includes arrays
-	UNIFORM_TYPE_FLOAT_MATRIX = 2,
+	UNIFORM_TYPE_MIXED        = 0, // includes arrays; float or int
+	UNIFORM_TYPE_INT          = 1, // includes arrays
+	UNIFORM_TYPE_FLOAT        = 2, // includes arrays
+	UNIFORM_TYPE_FLOAT_MATRIX = 3,
 };
 
 static void ParseUniformType(lua_State* L, int loc, int type)
@@ -177,10 +177,11 @@ static void ParseUniformType(lua_State* L, int loc, int type)
 		case UNIFORM_TYPE_FLOAT: {
 			if (lua_israwnumber(L, -1)) {
 				glUniform1f(loc, lua_tofloat(L, -1));
+				return;
 			}
-			else if (lua_istable(L, -1)) {
-				float array[32] = {0.0f};
-				const int count = LuaUtils::ParseFloatArray(L, -1, array, sizeof(array) / sizeof(float));
+			if (lua_istable(L, -1)) {
+				const float* array = fltUniformArrayBuf;
+				const int count = LuaUtils::ParseFloatArray(L, -1, fltUniformArrayBuf, sizeof(fltUniformArrayBuf) / sizeof(float));
 
 				switch (count) {
 					case 1: { glUniform1f(loc, array[0]                              ); break; }
@@ -189,17 +190,19 @@ static void ParseUniformType(lua_State* L, int loc, int type)
 					case 4: { glUniform4f(loc, array[0], array[1], array[2], array[3]); break; }
 					default: { glUniform1fv(loc, count, &array[0]); } break;
 				}
-			}
 
-			return;
-		}
+				return;
+			}
+		} break;
+
 		case UNIFORM_TYPE_INT: {
 			if (lua_israwnumber(L, -1)) {
 				glUniform1i(loc, lua_toint(L, -1));
+				return;
 			}
-			else if (lua_istable(L, -1)) {
-				int array[32] = {0};
-				const int count = LuaUtils::ParseIntArray(L, -1, array, sizeof(array) / sizeof(int));
+			if (lua_istable(L, -1)) {
+				const int* array = intUniformArrayBuf;
+				const int count = LuaUtils::ParseIntArray(L, -1, intUniformArrayBuf, sizeof(intUniformArrayBuf) / sizeof(int));
 
 				switch (count) {
 					case 1: { glUniform1i(loc, array[0]                              ); break; }
@@ -208,8 +211,11 @@ static void ParseUniformType(lua_State* L, int loc, int type)
 					case 4: { glUniform4i(loc, array[0], array[1], array[2], array[3]); break; }
 					default: { glUniform1iv(loc, count, &array[0]); } break;
 				}
+
+				return;
 			}
 		} break;
+
 		case UNIFORM_TYPE_FLOAT_MATRIX: {
 			if (lua_istable(L, -1)) {
 				float array[16] = {0.0f};
@@ -220,15 +226,28 @@ static void ParseUniformType(lua_State* L, int loc, int type)
 					case (3 * 3): { glUniformMatrix3fv(loc, 1, GL_FALSE, array); break; }
 					case (4 * 4): { glUniformMatrix4fv(loc, 1, GL_FALSE, array); break; }
 				}
+
+				return;
 			}
 		} break;
 	}
 }
 
-static bool ParseUniformsTable(lua_State* L, const char* fieldName, int index, int type, GLuint progName)
-{
+static bool ParseUniformsTable(
+	lua_State* L,
+	const std::vector<ActiveUniform>& activeUniforms,
+	const std::function<bool(const ActiveUniform&, const ActiveUniform&)>& uniformPred,
+	int index,
+	int type,
+	GLuint progName
+) {
+	constexpr const char* fieldNames[] = {"uniform", "uniformInt", "uniformFloat", "uniformMatrix"};
+	          const char* fieldName    = fieldNames[type];
+
 	lua_getfield(L, index, fieldName);
 
+	ActiveUniform tmpUniform;
+
 	if (lua_istable(L, -1)) {
 		const int table = lua_gettop(L);
 
@@ -236,12 +255,56 @@ static bool ParseUniformsTable(lua_State* L, const char* fieldName, int index, i
 			if (!lua_israwstring(L, -2))
 				continue;
 
-			const string name = lua_tostring(L, -2);
+			const std::string name = lua_tostring(L, -2);
 			const GLint loc = glGetUniformLocation(progName, name.c_str());
 
 			if (loc < 0)
 				continue;
 
+			std::memset(tmpUniform.name, 0, sizeof(tmpUniform.name));
+			std::strncpy(tmpUniform.name, name.c_str(), std::min(sizeof(tmpUniform.name) - 1, name.size()));
+
+			const auto iter = std::lower_bound(activeUniforms.begin(), activeUniforms.end(), tmpUniform, uniformPred);
+
+			if (iter == activeUniforms.end() || std::strcmp(iter->name, name.c_str()) != 0) {
+				LOG_L(L_WARNING, "[%s] uniform \"%s\" from table \"%s\" not active in shader", __func__, name.c_str(), fieldName);
+				continue;
+			}
+
+			// should only need to auto-correct if type == UNIFORM_TYPE_MIXED, but GL debug-errors say otherwise
+			switch (iter->type) {
+				case GL_SAMPLER_1D       : { type = UNIFORM_TYPE_INT; } break;
+				case GL_SAMPLER_2D       : { type = UNIFORM_TYPE_INT; } break;
+				case GL_SAMPLER_3D       : { type = UNIFORM_TYPE_INT; } break;
+				case GL_SAMPLER_1D_SHADOW: { type = UNIFORM_TYPE_INT; } break;
+				case GL_SAMPLER_2D_SHADOW: { type = UNIFORM_TYPE_INT; } break;
+				case GL_SAMPLER_CUBE     : { type = UNIFORM_TYPE_INT; } break;
+
+				case GL_INT     : { type = UNIFORM_TYPE_INT; } break;
+				case GL_INT_VEC2: { type = UNIFORM_TYPE_INT; } break;
+				case GL_INT_VEC3: { type = UNIFORM_TYPE_INT; } break;
+				case GL_INT_VEC4: { type = UNIFORM_TYPE_INT; } break;
+
+				case GL_UNSIGNED_INT     : { type = UNIFORM_TYPE_INT; } break;
+				case GL_UNSIGNED_INT_VEC2: { type = UNIFORM_TYPE_INT; } break;
+				case GL_UNSIGNED_INT_VEC3: { type = UNIFORM_TYPE_INT; } break;
+				case GL_UNSIGNED_INT_VEC4: { type = UNIFORM_TYPE_INT; } break;
+
+				case GL_FLOAT     : { type = UNIFORM_TYPE_FLOAT; } break;
+				case GL_FLOAT_VEC2: { type = UNIFORM_TYPE_FLOAT; } break;
+				case GL_FLOAT_VEC3: { type = UNIFORM_TYPE_FLOAT; } break;
+				case GL_FLOAT_VEC4: { type = UNIFORM_TYPE_FLOAT; } break;
+
+				case GL_FLOAT_MAT2: { type = UNIFORM_TYPE_FLOAT_MATRIX; } break;
+				case GL_FLOAT_MAT3: { type = UNIFORM_TYPE_FLOAT_MATRIX; } break;
+				case GL_FLOAT_MAT4: { type = UNIFORM_TYPE_FLOAT_MATRIX; } break;
+
+				default: {
+					LOG_L(L_WARNING, "[%s] value for uniform \"%s\" from table \"%s\" (GL-type 0x%x) set as int", __func__, name.c_str(), fieldName, iter->type);
+					type = UNIFORM_TYPE_INT;
+				} break;
+			}
+
 			ParseUniformType(L, loc, type);
 		}
 	}
@@ -253,20 +316,38 @@ static bool ParseUniformsTable(lua_State* L, const char* fieldName, int index, i
 
 static bool ParseUniformSetupTables(lua_State* L, int index, GLuint progName)
 {
-	GLint currentProgram;
+	GLint currentProgram = 0;
+	GLint numUniforms = 0;
+
 	glGetIntegerv(GL_CURRENT_PROGRAM, &currentProgram);
-
 	glUseProgram(progName);
+	glGetProgramiv(progName, GL_ACTIVE_UNIFORMS, &numUniforms);
 
-	const bool success =
-		ParseUniformsTable(L, "uniform",       index, UNIFORM_TYPE_FLOAT,        progName) &&
-		ParseUniformsTable(L, "uniformFloat",  index, UNIFORM_TYPE_FLOAT,        progName) &&
-		ParseUniformsTable(L, "uniformInt",    index, UNIFORM_TYPE_INT,          progName) &&
-		ParseUniformsTable(L, "uniformMatrix", index, UNIFORM_TYPE_FLOAT_MATRIX, progName);
+
+	std::vector<ActiveUniform> uniforms;
+	std::function<bool(const ActiveUniform&, const ActiveUniform&)> uniformPred = [](const ActiveUniform& a, const ActiveUniform& b) {
+		return (std::strcmp(a.name, b.name) < 0);
+	};
+
+	if (numUniforms > 0) {
+		uniforms.resize(numUniforms);
+
+		for (size_t n = 0; n < uniforms.size(); n++) {
+			glGetActiveUniform(progName, n, sizeof(uniforms[0].name) - 1, nullptr, &uniforms[n].size, &uniforms[n].type, &uniforms[n].name[0]);
+		}
+
+		std::sort(uniforms.begin(), uniforms.end(), uniformPred);
+	}
+
+	bool ret = true;
+
+	if (ret && !ParseUniformsTable(L, uniforms, uniformPred, index, UNIFORM_TYPE_MIXED,        progName)) ret = false;
+	if (ret && !ParseUniformsTable(L, uniforms, uniformPred, index, UNIFORM_TYPE_INT,          progName)) ret = false;
+	if (ret && !ParseUniformsTable(L, uniforms, uniformPred, index, UNIFORM_TYPE_FLOAT,        progName)) ret = false;
+	if (ret && !ParseUniformsTable(L, uniforms, uniformPred, index, UNIFORM_TYPE_FLOAT_MATRIX, progName)) ret = false;
 
 	glUseProgram(currentProgram);
-
-	return success;
+	return ret;
 }
 
 
@@ -275,8 +356,8 @@ static bool ParseUniformSetupTables(lua_State* L, int index, GLuint progName)
 
 static GLuint CompileObject(
 	lua_State* L,
-	const vector<string>& defs,
-	const vector<string>& sources,
+	const std::vector<std::string>& defs,
+	const std::vector<std::string>& sources,
 	const GLenum type,
 	bool& success
 ) {
@@ -299,7 +380,7 @@ static GLuint CompileObject(
 	for (unsigned int i = 0; i < sources.size(); i++)
 		text[defs.size() + i] = sources[i].c_str();
 
-	glShaderSource(obj, text.size(), &text[0], NULL);
+	glShaderSource(obj, text.size(), &text[0], nullptr);
 	glCompileShader(obj);
 
 	GLint result;
@@ -310,12 +391,10 @@ static GLuint CompileObject(
 
 	LuaShaders& shaders = CLuaHandle::GetActiveShaders(L);
 	shaders.errorLog = log;
+
 	if (result != GL_TRUE) {
-		if (shaders.errorLog.empty()) {
-			shaders.errorLog = "Empty error message:  code = "
-							+ IntToString(result) + " (0x"
-							+ IntToString(result, "%04X") + ")";
-		}
+		if (shaders.errorLog.empty())
+			shaders.errorLog = "Empty error message:  code = " + IntToString(result) + " (0x" + IntToString(result, "%04X") + ")";
 
 		glDeleteShader(obj);
 
@@ -344,9 +423,8 @@ static bool ParseShaderTable(
 	if (lua_israwstring(L, -1)) {
 		const std::string& txt = lua_tostring(L, -1);
 
-		if (!txt.empty()) {
+		if (!txt.empty())
 			data.push_back(txt);
-		}
 
 		lua_pop(L, 1);
 		return true;
@@ -363,9 +441,8 @@ static bool ParseShaderTable(
 
 			const std::string& txt = lua_tostring(L, -1);
 
-			if (!txt.empty()) {
+			if (!txt.empty())
 				data.push_back(txt);
-			}
 		}
 
 		lua_pop(L, 1);
@@ -373,7 +450,7 @@ static bool ParseShaderTable(
 	}
 
 	LuaShaders& shaders = CLuaHandle::GetActiveShaders(L);
-	shaders.errorLog = "\"" + string(key) + "\" must be a string or a table value!";
+	shaders.errorLog = "\"" + std::string(key) + "\" must be a string or a table value!";
 
 	lua_pop(L, 1);
 	return false;
@@ -385,18 +462,20 @@ static void ApplyGeometryParameters(lua_State* L, int table, GLuint prog)
 	if (!IS_GL_FUNCTION_AVAILABLE(glProgramParameteriEXT))
 		return;
 
-	struct { const char* name; GLenum param; } parameters[] = {
+	constexpr struct { const char* name; GLenum param; } parameters[] = {
 		{ "geoInputType",   GL_GEOMETRY_INPUT_TYPE_EXT },
 		{ "geoOutputType",  GL_GEOMETRY_OUTPUT_TYPE_EXT },
 		{ "geoOutputVerts", GL_GEOMETRY_VERTICES_OUT_EXT }
 	};
 
-	const int count = sizeof(parameters) / sizeof(parameters[0]);
-	for (int i = 0; i < count; i++) {
+	constexpr size_t count = sizeof(parameters) / sizeof(parameters[0]);
+
+	for (size_t i = 0; i < count; i++) {
 		lua_getfield(L, table, parameters[i].name);
-		if (lua_israwnumber(L, -1)) {
+
+		if (lua_israwnumber(L, -1))
 			glProgramParameteriEXT(prog, parameters[i].param, /*type*/ lua_toint(L, -1));
-		}
+
 		lua_pop(L, 1);
 	}
 }
@@ -514,7 +593,7 @@ int LuaShaders::DeleteShader(lua_State* L)
 
 int LuaShaders::UseShader(lua_State* L)
 {
-	CheckDrawingEnabled(L, __FUNCTION__);
+	CheckDrawingEnabled(L, __func__);
 
 	const int progIdx = luaL_checkint(L, 1);
 	if (progIdx == 0) {
@@ -573,11 +652,11 @@ int LuaShaders::ActiveShader(lua_State* L)
 
 static const char* UniformTypeString(GLenum type)
 {
-#define UNIFORM_STRING_CASE(x)                   \
-  case (GL_ ## x): {                             \
-    static const string str = StringToLower(#x); \
-    return str.c_str();                          \
-	}
+#define UNIFORM_STRING_CASE(x)                        \
+  case (GL_ ## x): {                                  \
+    static const std::string str = StringToLower(#x); \
+    return str.c_str();                               \
+  }
 
 	switch (type) {
 		UNIFORM_STRING_CASE(FLOAT)
@@ -601,9 +680,7 @@ static const char* UniformTypeString(GLenum type)
 		UNIFORM_STRING_CASE(BOOL_VEC2)
 		UNIFORM_STRING_CASE(BOOL_VEC3)
 		UNIFORM_STRING_CASE(BOOL_VEC4)
-	  default: {
-	  	return "unknown_type";
-		}
+		default: { return "unknown_type"; }
 	}
 }
 
@@ -622,17 +699,16 @@ int LuaShaders::GetActiveUniforms(lua_State* L)
 	lua_newtable(L);
 
 	for (GLint i = 0; i < uniformCount; i++) {
+		ActiveUniform au;
 		GLsizei length;
-		GLint size;
-		GLenum type;
-		GLchar name[1024];
-		glGetActiveUniform(progName, i, sizeof(name), &length, &size, &type, name);
+
+		glGetActiveUniform(progName, i, sizeof(au.name), &length, &au.size, &au.type, au.name);
 
 		lua_newtable(L); {
-			HSTR_PUSH_STRING(L, "name",   name);
-			HSTR_PUSH_STRING(L, "type",   UniformTypeString(type));
+			HSTR_PUSH_STRING(L, "name",   au.name);
+			HSTR_PUSH_STRING(L, "type",   UniformTypeString(au.type));
 			HSTR_PUSH_NUMBER(L, "length", length);
-			HSTR_PUSH_NUMBER(L, "size",   size);
+			HSTR_PUSH_NUMBER(L, "size",   au.size);
 		}
 		lua_rawseti(L, -2, i + 1);
 	}
@@ -649,7 +725,7 @@ int LuaShaders::GetUniformLocation(lua_State* L)
 	if (progName == 0)
 		return 0;
 
-	const string name = luaL_checkstring(L, 2);
+	const std::string name = luaL_checkstring(L, 2);
 	const GLint location = glGetUniformLocation(progName, name.c_str());
 
 	lua_pushnumber(L, location);
@@ -663,9 +739,9 @@ int LuaShaders::GetUniformLocation(lua_State* L)
 int LuaShaders::Uniform(lua_State* L)
 {
 	if (activeShaderDepth <= 0)
-		CheckDrawingEnabled(L, __FUNCTION__);
+		CheckDrawingEnabled(L, __func__);
 
-	const GLuint location = (GLuint) luaL_checknumber(L, 1);
+	const GLuint location = luaL_checkint(L, 1);
 	const int numValues = lua_gettop(L) - 1;
 
 	switch (numValues) {
@@ -693,28 +769,24 @@ int LuaShaders::Uniform(lua_State* L)
 int LuaShaders::UniformInt(lua_State* L)
 {
 	if (activeShaderDepth <= 0)
-		CheckDrawingEnabled(L, __FUNCTION__);
+		CheckDrawingEnabled(L, __func__);
 
-	const GLuint location = (GLuint) luaL_checknumber(L, 1);
+	const GLuint location = luaL_checkint(L, 1);
 	const int numValues = lua_gettop(L) - 1;
 
 	switch (numValues) {
 		case 1: {
 			glUniform1i(location, luaL_checkint(L, 2));
-			break;
-		}
+		} break;
 		case 2: {
 			glUniform2i(location, luaL_checkint(L, 2), luaL_checkint(L, 3));
-			break;
-		}
+		} break;
 		case 3: {
 			glUniform3i(location, luaL_checkint(L, 2), luaL_checkint(L, 3), luaL_checkint(L, 4));
-			break;
-		}
+		} break;
 		case 4: {
 			glUniform4i(location, luaL_checkint(L, 2), luaL_checkint(L, 3), luaL_checkint(L, 4), luaL_checkint(L, 5));
-			break;
-		}
+		} break;
 		default: {
 			luaL_error(L, "Incorrect arguments to gl.UniformInt()");
 		}
@@ -723,29 +795,58 @@ int LuaShaders::UniformInt(lua_State* L)
 	return 0;
 }
 
+
+#if 0
+template<typename type, typename glUniformFunc, typename ParseArrayFunc>
+static bool GLUniformArray(lua_State* L, UniformFunc uf, ParseArrayFunc pf)
+{
+	const GLuint loc = luaL_checkint(L, 1);
+
+	switch (next_power_of_2(luaL_optint(L, 4, 32))) {
+		case (1 <<  3): { type array[1 <<  3] = {type(0)}; uf(loc, pf(L, 3, array, 1 <<  3), &array[0]); return true; } break;
+		case (1 <<  4): { type array[1 <<  4] = {type(0)}; uf(loc, pf(L, 3, array, 1 <<  4), &array[0]); return true; } break;
+		case (1 <<  5): { type array[1 <<  5] = {type(0)}; uf(loc, pf(L, 3, array, 1 <<  5), &array[0]); return true; } break;
+		case (1 <<  6): { type array[1 <<  6] = {type(0)}; uf(loc, pf(L, 3, array, 1 <<  6), &array[0]); return true; } break;
+		case (1 <<  7): { type array[1 <<  7] = {type(0)}; uf(loc, pf(L, 3, array, 1 <<  7), &array[0]); return true; } break;
+		case (1 <<  8): { type array[1 <<  8] = {type(0)}; uf(loc, pf(L, 3, array, 1 <<  8), &array[0]); return true; } break;
+		case (1 <<  9): { type array[1 <<  9] = {type(0)}; uf(loc, pf(L, 3, array, 1 <<  9), &array[0]); return true; } break;
+		case (1 << 10): { type array[1 << 10] = {type(0)}; uf(loc, pf(L, 3, array, 1 << 10), &array[0]); return true; } break;
+		default: {} break;
+	}
+
+	return false;
+}
+#endif
+
 int LuaShaders::UniformArray(lua_State* L)
 {
 	if (activeShaderDepth <= 0)
-		CheckDrawingEnabled(L, __FUNCTION__);
+		CheckDrawingEnabled(L, __func__);
 
 	if (!lua_istable(L, 3))
 		return 0;
 
-	const GLuint location = (GLuint) luaL_checknumber(L, 1);
-
 	switch (luaL_checkint(L, 2)) {
 		case UNIFORM_TYPE_INT: {
-			int array[32] = {0};
-			const int count = LuaUtils::ParseIntArray(L, 3, array, sizeof(array) / sizeof(int));
+			#if 0
+			GLUniformArray<int>(L, glUniform1iv, LuaUtils::ParseIntArray);
+			#else
+			const int loc = luaL_checkint(L, 1);
+			const int cnt = LuaUtils::ParseIntArray(L, 3, intUniformArrayBuf, sizeof(intUniformArrayBuf) / sizeof(int));
 
-			glUniform1iv(location, count, &array[0]);
+			glUniform1iv(loc, cnt, &intUniformArrayBuf[0]);
+			#endif
 		} break;
 
 		case UNIFORM_TYPE_FLOAT: {
-			float array[32] = {0.0f};
-			const int count = LuaUtils::ParseFloatArray(L, 3, array, sizeof(array) / sizeof(float));
+			#if 0
+			GLUniformArray<float>(L, glUniform1fv, LuaUtils::ParseFloatArray);
+			#else
+			const int loc = luaL_checkint(L, 1);
+			const int cnt = LuaUtils::ParseFloatArray(L, 3, fltUniformArrayBuf, sizeof(fltUniformArrayBuf) / sizeof(float));
 
-			glUniform1fv(location, count, &array[0]);
+			glUniform1fv(loc, cnt, &fltUniformArrayBuf[0]);
+			#endif
 		} break;
 
 		default: {
@@ -758,18 +859,17 @@ int LuaShaders::UniformArray(lua_State* L)
 int LuaShaders::UniformMatrix(lua_State* L)
 {
 	if (activeShaderDepth <= 0)
-		CheckDrawingEnabled(L, __FUNCTION__);
+		CheckDrawingEnabled(L, __func__);
 
 	const GLuint location = (GLuint)luaL_checknumber(L, 1);
 	const int numValues = lua_gettop(L) - 1;
 
 	switch (numValues) {
 		case 1: {
-			if (!lua_isstring(L, 2)) {
+			if (!lua_isstring(L, 2))
 				luaL_error(L, "Incorrect arguments to gl.UniformMatrix()");
-			}
 
-			const string matName = lua_tostring(L, 2);
+			const std::string matName = lua_tostring(L, 2);
 			const CMatrix44f* mat = LuaOpenGLUtils::GetNamedMatrix(matName);
 
 			if (mat) {
@@ -820,22 +920,19 @@ int LuaShaders::UniformMatrix(lua_State* L)
 
 int LuaShaders::SetShaderParameter(lua_State* L)
 {
-	if (activeShaderDepth <= 0) {
-		CheckDrawingEnabled(L, __FUNCTION__);
-	}
+	if (activeShaderDepth <= 0)
+		CheckDrawingEnabled(L, __func__);
 
 	const LuaShaders& shaders = CLuaHandle::GetActiveShaders(L);
 	const GLuint progName = shaders.GetProgramName(L, 1);
-	if (progName == 0) {
+	if (progName == 0)
 		return 0;
-	}
 
 	const GLenum param = (GLenum)luaL_checkint(L, 2);
 	const GLint  value =  (GLint)luaL_checkint(L, 3);
 
-	if (IS_GL_FUNCTION_AVAILABLE(glProgramParameteriEXT)) {
+	if (IS_GL_FUNCTION_AVAILABLE(glProgramParameteriEXT))
 		glProgramParameteriEXT(progName, param, value);
-	}
 
 	return 0;
 }
