@@ -4,33 +4,23 @@
 #include "MapDamage.h"
 #include "BasicMapDamage.h"
 #include "NoMapDamage.h"
-#include "ReadMap.h"
 #include "MapInfo.h"
 #include "Game/GameSetup.h"
 
-IMapDamage* mapDamage;
+IMapDamage* mapDamage = nullptr;
 
-IMapDamage::IMapDamage() :
+IMapDamage::IMapDamage():
 	disabled(true),
 	mapHardness(0.0f)
 {
 }
 
-IMapDamage::~IMapDamage()
-{
-}
-
 IMapDamage* IMapDamage::GetMapDamage()
 {
-	bool disable = false;
-	
 	if (mapInfo->map.notDeformable)
-		disable = true;
-	else if (gameSetup && gameSetup->disableMapDamage)
-		disable = true;
+		return (new CNoMapDamage());
+	if (gameSetup != nullptr && gameSetup->disableMapDamage)
+		return (new CNoMapDamage());
 	
-	if (disable)
-		return new CNoMapDamage;
-	else
-		return new CBasicMapDamage;
+	return (new CBasicMapDamage());
 }
