@@ -12,7 +12,7 @@ public:
 
 public:
 	IMapDamage();
-	virtual ~IMapDamage();
+	virtual ~IMapDamage() {}
 
 	virtual void Explosion(const float3& pos, float strength, float radius) = 0;
 	virtual void RecalcArea(int x1, int x2, int y1, int y2) = 0;
