@@ -1,7 +1,6 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
 #include "DebugColVolDrawer.h"
-#include <unordered_set>
 
 #include "Game/Camera.h"
 #include "Game/GlobalUnsynced.h"
@@ -17,6 +16,7 @@
 #include "Sim/Units/Unit.h"
 #include "Sim/Weapons/PlasmaRepulser.h"
 #include "Sim/Weapons/Weapon.h"
+#include "System/UnorderedSet.hpp"
 
 static const float4 DEFAULT_VOLUME_COLOR = float4(0.45f, 0.0f, 0.45f, 0.35f);
 static unsigned int volumeDisplayListIDs[3] = {0, 0, 0};
@@ -140,7 +140,7 @@ static inline void DrawObjectMidAndAimPos(const CSolidObject* o)
 	if (o->aimPos != o->midPos) {
 		// draw the aim-point
 		glPushMatrix();
-		glTranslatef3(o->relAimPos * WORLD_TO_OBJECT_SPACE);
+		glTranslatef3(o->relAimPos);
 		glColor4f(1.0f, 0.0f, 0.0f, 0.35f);
 		gluQuadricDrawStyle(q, GLU_FILL);
 		gluSphere(q, 2.0f, 5, 5);
@@ -149,7 +149,7 @@ static inline void DrawObjectMidAndAimPos(const CSolidObject* o)
 
 	{
 		// draw the mid-point, keep this transform on the stack
-		glTranslatef3(o->relMidPos * WORLD_TO_OBJECT_SPACE);
+		glTranslatef3(o->relMidPos);
 		glColor4f(1.0f, 0.0f, 1.0f, 0.35f);
 		gluQuadricDrawStyle(q, GLU_FILL);
 		gluSphere(q, 2.0f, 5, 5);
@@ -177,7 +177,7 @@ static inline void DrawFeatureColVol(const CFeature* f)
 	const bool vCustomDims = ((v->GetOffsets()).SqLength() >= 1.0f || std::fabs(v->GetBoundingRadius() - f->radius) >= 1.0f);
 
 	glPushMatrix();
-		glMultMatrixf(f->GetTransformMatrixRef());
+		glMultMatrixf(f->GetTransformMatrixRef(false));
 		DrawObjectMidAndAimPos(f);
 
 		if (v->DefaultToPieceTree()) {
@@ -185,9 +185,9 @@ static inline void DrawFeatureColVol(const CFeature* f)
 			// note: relMidPos transform is on the stack at this
 			// point but all piece-positions are relative to pos
 			// --> undo it
-			glTranslatef3(-f->relMidPos * WORLD_TO_OBJECT_SPACE);
+			glTranslatef3(-f->relMidPos);
 			DrawObjectDebugPieces(f);
-			glTranslatef3(f->relMidPos * WORLD_TO_OBJECT_SPACE);
+			glTranslatef3(f->relMidPos);
 		} else {
 			if (!v->IgnoreHits()) {
 				DrawCollisionVolume(v);
@@ -195,7 +195,7 @@ static inline void DrawFeatureColVol(const CFeature* f)
 		}
 
 		if (vCustomType || vCustomDims) {
-			// assume this is a custom volume
+			// assume this is a custom volume; draw radius-sphere next to it
 			glColor4f(0.5f, 0.5f, 0.5f, 0.35f);
 			glScalef(f->radius, f->radius, f->radius);
 			glWireSphere(&volumeDisplayListIDs[0], 20, 20);
@@ -252,7 +252,7 @@ static inline void DrawUnitColVol(const CUnit* u)
 
 
 	glPushMatrix();
-		glMultMatrixf(u->GetTransformMatrix());
+		glMultMatrixf(u->GetTransformMatrix(false));
 		DrawObjectMidAndAimPos(u);
 
 		if (v->DefaultToPieceTree()) {
@@ -260,9 +260,9 @@ static inline void DrawUnitColVol(const CUnit* u)
 			// note: relMidPos transform is on the stack at this
 			// point but all piece-positions are relative to pos
 			// --> undo it
-			glTranslatef3(-u->relMidPos * WORLD_TO_OBJECT_SPACE);
+			glTranslatef3(-u->relMidPos);
 			DrawObjectDebugPieces(u);
-			glTranslatef3(u->relMidPos * WORLD_TO_OBJECT_SPACE);
+			glTranslatef3(u->relMidPos);
 		} else {
 			if (!v->IgnoreHits()) {
 				// make it fade red under attack
@@ -286,7 +286,7 @@ static inline void DrawUnitColVol(const CUnit* u)
 		}
 
 		if (vCustomType || vCustomDims) {
-			// assume this is a custom volume
+			// assume this is a custom volume; draw radius-sphere next to it
 			glColor4f(0.5f, 0.5f, 0.5f, 0.35f);
 			glScalef(u->radius, u->radius, u->radius);
 			glWireSphere(&volumeDisplayListIDs[0], 20, 20);
@@ -318,7 +318,7 @@ public:
 		}
 	}
 
-	std::unordered_set<int> alreadyDrawnIds;
+	spring::unordered_set<int> alreadyDrawnIds;
 };
 
 
@@ -332,8 +332,6 @@ namespace DebugColVolDrawer
 		if (!enable)
 			return;
 
-		SCOPED_GMARKER("DebugColVolDrawer::Draw");
-
 		glPushAttrib(GL_ENABLE_BIT | GL_CURRENT_BIT);
 			glDisable(GL_LIGHTING);
 			glDisable(GL_LIGHT0);
