@@ -4,10 +4,10 @@
 #include "AdvSky.h"
 
 #include "Game/Camera.h"
+#include "Game/GlobalUnsynced.h"
 #include "Map/MapInfo.h"
 #include "Rendering/GlobalRendering.h"
 #include "Rendering/GL/VertexArray.h"
-#include "Rendering/Textures/Bitmap.h"
 #include "Sim/Misc/GlobalSynced.h"
 #include "System/Config/ConfigHandler.h"
 #include "System/Exceptions.h"
@@ -64,8 +64,8 @@ CAdvSky::CAdvSky()
 	memset(thicknessTransform, 0, 1024);
 	memset(covers, 0, 4 * 32 * sizeof(float));
 
-	domeheight = math::cos(PI / 16) * 1.01f;
-	domeWidth = math::sin(2 * PI / 32) * 400 * 1.7f;
+	domeheight = std::cos(math::PI / 16) * 1.01f;
+	domeWidth = std::sin(math::TWOPI / 32) * 400 * 1.7f;
 
 	UpdateSkyDir();
 	InitSun();
@@ -195,21 +195,21 @@ void CAdvSky::Draw()
 
 	glDisable(GL_DEPTH_TEST);
 	glEnable(GL_BLEND);
+	glPolygonMode(GL_FRONT_AND_BACK, GL_LINE * wireFrameMode + GL_FILL * (1 - wireFrameMode));
 
 	glPushMatrix();
-//	glTranslatef(camera->GetPos().x,camera->GetPos().y,camera->GetPos().z);
-	CMatrix44f m(camera->GetPos(),skydir1,UpVector,skydir2);
-	glMultMatrixf(m.m);
+	// glTranslatef3(camera->GetPos());
+	glMultMatrixf(CMatrix44f(camera->GetPos(), skydir1, UpVector, skydir2));
 
-	float3 modCamera=skydir1*camera->GetPos().x+skydir2*camera->GetPos().z;
+	const float3 modCamera = skydir1 * camera->GetPos().x + skydir2 * camera->GetPos().z;
 
 	glMatrixMode(GL_TEXTURE);
 	  glActiveTextureARB(GL_TEXTURE2_ARB);
 		glPushMatrix();
-		glTranslatef((gs->frameNum%20000)*0.00005f+modCamera.x*0.000025f,modCamera.z*0.000025f,0);
+		glTranslatef((gs->frameNum % 20000) * 0.00005f + modCamera.x * 0.000025f, modCamera.z * 0.000025f, 0);
 	  glActiveTextureARB(GL_TEXTURE3_ARB);
 		glPushMatrix();
-		glTranslatef((gs->frameNum%20000)*0.0020f+modCamera.x*0.001f,modCamera.z*0.001f,0);
+		glTranslatef((gs->frameNum % 20000) * 0.0020f + modCamera.x * 0.001f, modCamera.z * 0.001f, 0);
 	  glActiveTextureARB(GL_TEXTURE0_ARB);
 	glMatrixMode(GL_MODELVIEW);
 
@@ -225,6 +225,7 @@ void CAdvSky::Draw()
 
 	glPopMatrix();
 
+	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
 	glEnable(GL_DEPTH_TEST);
 	glDisable(GL_BLEND);
 
@@ -233,8 +234,8 @@ void CAdvSky::Draw()
 
 float3 CAdvSky::GetCoord(int x, int y)
 {
-	float fy = ((float)y/Y_PART) * 2 * PI;
-	float fx = ((float)x/X_PART) * 2 * PI;
+	float fy = ((float)y/Y_PART) * math::TWOPI;
+	float fx = ((float)x/X_PART) * math::TWOPI;
 
 	return float3(
 		fastmath::sin(fy/32) * fastmath::sin(fx),
@@ -368,7 +369,7 @@ void CAdvSky::Update()
 	if (!dynamicSky)
 		return;
 
-	SCOPED_TIMER("AdvSky::Update");
+	SCOPED_TIMER("Update::WorldDrawer::AdvSky");
 
 	CreateDetailTex();
 
@@ -380,7 +381,7 @@ void CAdvSky::Update()
 	case 0: {
 		for(int a=0; a<CLOUD_DETAIL; a++) {
 			float fade = gs->frameNum / (70.0f * (2<<(CLOUD_DETAIL-1-a)));
-			fade -= math::floor(fade/2)*2;
+			fade -= std::floor(fade/2)*2;
 			if(fade>1) {
 				fade = 2 - fade;
 				if(!cloudDown[a]) {
@@ -414,11 +415,11 @@ void CAdvSky::Update()
 		int qcda=(4<<CLOUD_DETAIL)>>a;
 		int *pkernel=kernel;
 		for(int y=0; y<cs4a; ++y, pkernel+=CLOUD_SIZE/4) {
-			float ydist=math::fabs(1.0f+y-cs8a)/cs8a;
+			float ydist=std::fabs(1.0f+y-cs8a)/cs8a;
 			ydist=ydist*ydist*(3-2*ydist);
 			int *pkrn=pkernel;
 			for(int x=0; x<cs4a; ++x) {
-				float xdist=math::fabs(1.0f+x-cs8a)/cs8a;
+				float xdist=std::fabs(1.0f+x-cs8a)/cs8a;
 				xdist=xdist*xdist*(3-2*xdist);
 
 				float contrib=(1-xdist)*(1-ydist);
@@ -508,19 +509,17 @@ void CAdvSky::Update()
 	}
 }
 
-void CAdvSky::CreateRandMatrix(int **matrix,float mod)
+void CAdvSky::CreateRandMatrix(int** matrix, float /*mod*/)
 {
-	for(int a=0, *pmat=*matrix; a<32*32; ++a) {
-		float r = ((float)( rand() )) / (float)RAND_MAX;
-		*pmat++=((int)(r * 255.0f));
+	for (int a = 0, *pmat = *matrix; a < 32*32; ++a) {
+		*pmat++ = ((int)(guRNG.NextFloat() * 255.0f));
 	}
 }
 
-void CAdvSky::CreateRandDetailMatrix(unsigned char* matrix,int size)
+void CAdvSky::CreateRandDetailMatrix(unsigned char* matrix, int size)
 {
-	for(int a=0;a<size*size;a++){
-		float  r = ((float)( rand() )) / (float)RAND_MAX;
-		*matrix++=((int)(r * 255.0f));
+	for (int a = 0; a < size*size; a++) {
+		*matrix++ = ((int)(guRNG.NextFloat() * 255.0f));
 	}
 }
 
@@ -530,7 +529,7 @@ void CAdvSky::CreateTransformVectors()
 	unsigned char *tt=thicknessTransform;
 	for(int a=0;a<1024;++a){
 		float f=(1023.0f-(a+cloudDensity*1024-512))/1023.0f;
-		float alpha=math::pow(f*2,3);
+		float alpha=std::pow(f*2,3);
 		if(alpha>1)
 			alpha=1;
 		*at=(int) (alpha*255);
@@ -548,8 +547,8 @@ void CAdvSky::DrawSun()
 		return;
 
 	const float3 xzSunCameraPos =
-		sundir1 * camera->GetPos().x +
-		sundir2 * camera->GetPos().z;
+		skyLight->GetLightDirX() * camera->GetPos().x +
+		skyLight->GetLightDirZ() * camera->GetPos().z;
 	const float3 modSunColor = sunColor * skyLight->GetLightIntensity();
 
 	// sun-disc vertices might be clipped against the
@@ -567,8 +566,8 @@ void CAdvSky::DrawSun()
 	const float ymod = (sunTexCoordY - 0.5f) * domeWidth * 0.025f * 256;
 	const float fx = gs->frameNum * 0.00005f * CLOUD_SIZE + xzSunCameraPos.x * CLOUD_SIZE * 0.000025f;
 	const float fy = ymod + xzSunCameraPos.z * CLOUD_SIZE * 0.000025f;
-	const float ffx = fx - math::floor(fx);
-	const float ffy = fy - math::floor(fy);
+	const float ffx = fx - std::floor(fx);
+	const float ffy = fy - std::floor(fy);
 	const int baseX = int(fx) & CLOUD_MASK;
 	const int baseY = int(fy) & CLOUD_MASK;
 
@@ -634,8 +633,8 @@ void CAdvSky::UpdateSunFlare() {
 		glBegin(GL_TRIANGLE_STRIP);
 
 		for (int x = 0; x < 257; ++x) {
-			const float dx = math::sin(x * 2.0f * PI / 256.0f);
-			const float dy = math::cos(x * 2.0f * PI / 256.0f);
+			const float dx = std::sin(x * math::TWOPI / 256.0f);
+			const float dy = std::cos(x * math::TWOPI / 256.0f);
 			const float dz = 5.0f;
 
 			glTexCoord2f(x / 256.0f, 0.125f); glVertexf3(zdir * dz + xdir * dx * 0.0014f + ydir * dy * 0.0014f);
@@ -662,7 +661,7 @@ void CAdvSky::InitSun()
 			mem[(y*128+x)*4+0]=255;
 			mem[(y*128+x)*4+1]=255;
 			mem[(y*128+x)*4+2]=255;
-			float dist=math::sqrt((float)(y-64)*(y-64)+(x-64)*(x-64));
+			float dist=std::sqrt((float)(y-64)*(y-64)+(x-64)*(x-64));
 			if(dist>60)
 				mem[(y*128+x)*4+3]=0;
 			else
@@ -765,7 +764,7 @@ void CAdvSky::CreateDetailTex()
 
 	for(int a=0;a<5;++a){
 		float fade = gs->frameNum / float(30<<a);
-		fade -= math::floor(fade/2)*2;
+		fade -= std::floor(fade/2)*2;
 		int size = std::min(32,256>>a);
 
 		if(fade>1){
@@ -786,7 +785,7 @@ void CAdvSky::CreateDetailTex()
 
 		}
 		float tSize = std::max(1,8>>a);
-		float c = math::pow(2.0f,a)*6/255.0f;
+		float c = std::pow(2.0f,a)*6/255.0f;
 		CVertexArray* va = GetVertexArray();
 		va->Initialize();
 		va->CheckInitSize(4*VA_SIZE_T);
@@ -813,23 +812,8 @@ void CAdvSky::CreateDetailTex()
 }
 
 void CAdvSky::UpdateSunDir() {
-	const float3& L = skyLight->GetLightDir();
-
-	sundir2 = L;
-	sundir2.y = 0.0f;
-
-	if (sundir2.SqLength() == 0.0f)
-		sundir2.x = 1.0f;
-
-	sundir2.ANormalize();
-	sundir1 = sundir2.cross(UpVector);
-
-	modSunDir.x = 0.0f;
-	modSunDir.y = L.y;
-	modSunDir.z = math::sqrt(L.x * L.x + L.z * L.z);
-
 	sunTexCoordX = 0.5f;
-	sunTexCoordY = GetTexCoordFromDir(modSunDir);
+	sunTexCoordY = GetTexCoordFromDir(skyLight->CalcPolarLightDir());
 
 	UpdateSunFlare();
 }
@@ -843,7 +827,7 @@ void CAdvSky::UpdateSkyDir() {
 
 	skydir2.ANormalize();
 	skydir1 = skydir2.cross(UpVector);
-	skyAngle = GetRadFromXY(skydir2.x, skydir2.z) + PI / 2.0f;
+	skyAngle = GetRadFromXY(skydir2.x, skydir2.z) + math::HALFPI;
 }
 
 void CAdvSky::UpdateSkyTexture() {
@@ -877,13 +861,13 @@ float3 CAdvSky::GetDirFromTexCoord(float x, float y)
 	dir.x = (x - 0.5f) * domeWidth;
 	dir.z = (y - 0.5f) * domeWidth;
 
-	const float hdist = math::sqrt(dir.x * dir.x + dir.z * dir.z);
+	const float hdist = std::sqrt(dir.x * dir.x + dir.z * dir.z);
 	const float ang = GetRadFromXY(dir.x, dir.z) + skyAngle;
-	const float fy = math::asin(hdist / 400);
+	const float fy = std::asin(hdist / 400);
 
-	dir.x = hdist * math::cos(ang);
-	dir.z = hdist * math::sin(ang);
-	dir.y = (fastmath::cos(fy) - domeheight) * 400;
+	dir.x = hdist * std::cos(ang);
+	dir.z = hdist * std::sin(ang);
+	dir.y = (std::cos(fy) - domeheight) * 400;
 
 	dir.ANormalize();
 	return dir;
@@ -915,8 +899,8 @@ void CAdvSky::UpdateTexPartDot3(int x, int y, unsigned char (*texp)[4]) {
 	const float3& dir = GetDirFromTexCoord(x / 256.0f, (255.0f - y) / 256.0f);
 
 	const float sunInt = skyLight->GetLightIntensity();
-	const float sunDist = math::acos(dir.dot(skyLight->GetLightDir())) * 50;
-	const float sunMod = sunInt * (0.3f / math::sqrt(sunDist) + 3.0f / (1 + sunDist));
+	const float sunDist = std::acos(dir.dot(skyLight->GetLightDir())) * 50;
+	const float sunMod = sunInt * (0.3f / std::sqrt(sunDist) + 3.0f / (1 + sunDist));
 
 	const float green = std::min(1.0f, (0.55f + sunMod));
 	const float blue  = 203 - sunInt * (40.0f / (3 + sunDist));
@@ -930,7 +914,7 @@ void CAdvSky::UpdateTexPartDot3(int x, int y, unsigned char (*texp)[4]) {
 void CAdvSky::UpdateTexPart(int x, int y, unsigned char (*texp)[4]) {
 	const float3& dir = GetDirFromTexCoord(x / 512.0f, (511.0f - y) / 512.0f);
 
-	const float sunDist = math::acos(dir.dot(skyLight->GetLightDir())) * 70;
+	const float sunDist = std::acos(dir.dot(skyLight->GetLightDir())) * 70;
 	const float sunMod = skyLight->GetLightIntensity() * 12.0f / (12 + sunDist);
 
 	const float red   = std::min(skyColor.x + sunMod * sunColor.x, 1.0f);
