@@ -6,6 +6,7 @@
 #include "AdvTreeGenerator.h"
 
 #include "Game/Camera.h"
+#include "Game/GlobalUnsynced.h"
 #include "Lua/LuaParser.h"
 #include "Map/MapInfo.h"
 #include "Map/ReadMap.h"
@@ -19,6 +20,7 @@
 #include "Rendering/Textures/Bitmap.h"
 #include "System/Log/ILog.h"
 #include "System/Exceptions.h"
+#include "System/myMath.h"
 
 using std::max;
 using std::min;
@@ -27,14 +29,12 @@ using std::min;
 // Construction/Destruction
 //////////////////////////////////////////////////////////////////////
 
-CAdvTreeGenerator* treeGen;
-
-CAdvTreeGenerator::CAdvTreeGenerator()
+void CAdvTreeGenerator::Init()
 {
 	LuaParser resourcesParser("gamedata/resources.lua", SPRING_VFS_MOD_BASE, SPRING_VFS_ZIP);
-	if (!resourcesParser.Execute()) {
+
+	if (!resourcesParser.Execute())
 		LOG_L(L_ERROR, "%s", resourcesParser.GetErrorLog().c_str());
-	}
 
 	const LuaTable treesTable = resourcesParser.GetRoot().SubTable("graphics").SubTable("trees");
 
@@ -55,16 +55,18 @@ CAdvTreeGenerator::CAdvTreeGenerator()
 	glDisable(GL_CULL_FACE);
 
 	unsigned char(* tree)[2048][4] = new unsigned char[256][2048][4];
+	const unsigned char* rmem = bm.GetRawMem();
+
 	memset(tree[0][0], 128, 256 * 2048 * 4);
 	for (int y = 0; y < 256; y++) {
 		for (int x = 0; x < 256; x++) {
-			tree[y][x][0] = bm.mem[(y * 256 + x) * 4    ];
-			tree[y][x][1] = bm.mem[(y * 256 + x) * 4 + 1];
-			tree[y][x][2] = bm.mem[(y * 256 + x) * 4 + 2];
+			tree[y][x][0] = rmem[(y * 256 + x) * 4    ];
+			tree[y][x][1] = rmem[(y * 256 + x) * 4 + 1];
+			tree[y][x][2] = rmem[(y * 256 + x) * 4 + 2];
 			tree[y][x][3] = 255;
-			tree[y][x + 1024][0] = (unsigned char)(bm.mem[(y * 256 + x) * 4    ] * 0.6f);
-			tree[y][x + 1024][1] = (unsigned char)(bm.mem[(y * 256 + x) * 4 + 1] * 0.6f);
-			tree[y][x + 1024][2] = (unsigned char)(bm.mem[(y * 256 + x) * 4 + 2] * 0.6f);
+			tree[y][x + 1024][0] = (unsigned char)(rmem[(y * 256 + x) * 4    ] * 0.6f);
+			tree[y][x + 1024][1] = (unsigned char)(rmem[(y * 256 + x) * 4 + 1] * 0.6f);
+			tree[y][x + 1024][2] = (unsigned char)(rmem[(y * 256 + x) * 4 + 2] * 0.6f);
 			tree[y][x + 1024][3] = 255;
 		}
 	}
@@ -85,7 +87,7 @@ CAdvTreeGenerator::CAdvTreeGenerator()
 	glBindTexture(GL_TEXTURE_2D, leafTex);
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
-	glBuildMipmaps(GL_TEXTURE_2D, GL_RGBA8, bm.xsize, bm.ysize, GL_RGBA, GL_UNSIGNED_BYTE, &bm.mem[0]);
+	glBuildMipmaps(GL_TEXTURE_2D, GL_RGBA8, bm.xsize, bm.ysize, GL_RGBA, GL_UNSIGNED_BYTE, bm.GetRawMem());
 
 	CreateLeafTex(leafTex, 256, 0, tree);
 	CreateLeafTex(leafTex, 512, 0, tree);
@@ -110,7 +112,7 @@ CAdvTreeGenerator::CAdvTreeGenerator()
 
 
 	leafDL = glGenLists(8);
-	srand(15U);
+
 	for (int a = 0; a < 8; ++a) {
 		va = GetVertexArray();
 		va->Initialize();
@@ -118,7 +120,7 @@ CAdvTreeGenerator::CAdvTreeGenerator()
 		barkva->Initialize();
 
 		glNewList(leafDL + a, GL_COMPILE);
-			float size = 0.65f + fRand(0.2f);
+			float size = 0.65f + 0.2f * guRNG.NextFloat();
 			MainTrunk(10, size * MAX_TREE_HEIGHT, size * 0.05f * MAX_TREE_HEIGHT);
 			va->DrawArrayTN(GL_QUADS);
 			barkva->DrawArrayTN(GL_TRIANGLE_STRIP);
@@ -126,14 +128,14 @@ CAdvTreeGenerator::CAdvTreeGenerator()
 	}
 
 	pineDL = glGenLists(8);
-	srand(15U);
+
 	for (int a = 0; a < 8; ++a) {
 		va = GetVertexArray();
 		va->Initialize();
 
 		glNewList(pineDL + a, GL_COMPILE);
-			float size = 0.7f + fRand(0.2f);
-			PineTree((int)(20 + fRand(10)), MAX_TREE_HEIGHT * size);
+			float size = 0.7f + 0.2f * guRNG.NextFloat();
+			PineTree((int)(20 + 10 * guRNG.NextFloat()), MAX_TREE_HEIGHT * size);
 			va->DrawArrayTN(GL_TRIANGLES);
 		glEndList();
 	}
@@ -158,10 +160,11 @@ void CAdvTreeGenerator::DrawTrunk(const float3& start, const float3& end, const
 
 	int numIter=(int)max(3.0f,size*10);
 	for(int a=0;a<=numIter;a++){
-		float angle=a/(float)numIter*2*PI;
-		float col=0.4f+(((orto1*math::sin(angle)+orto2*math::cos(angle)).dot(flatSun)))*0.3f;
-		barkva->AddVertexTN(start+orto1*math::sin(angle)*size+orto2*math::cos(angle)*size,angle/PI*0.125f*0.5f,0,float3(0,0,col));
-		barkva->AddVertexTN(end+orto1*math::sin(angle)*size*0.2f+orto2*math::cos(angle)*size*0.2f,angle/PI*0.125f*0.5f,3,float3(0,0,col));
+		float angle = a / (float)numIter * math::TWOPI;
+		float col = 0.4f + (((orto1 * std::sin(angle) + orto2 * std::cos(angle)).dot(flatSun))) * 0.3f;
+
+		barkva->AddVertexTN(start + orto1 * std::sin(angle) * size        + orto2 * std::cos(angle) * size       , angle / math::PI * 0.125f * 0.5f, 0, float3(0,0,col));
+		barkva->AddVertexTN(  end + orto1 * std::sin(angle) * size * 0.2f + orto2 * std::cos(angle) * size * 0.2f, angle / math::PI * 0.125f * 0.5f, 3, float3(0,0,col));
 	}
 	barkva->EndStrip();
 }
@@ -170,24 +173,24 @@ void CAdvTreeGenerator::MainTrunk(int numBranch,float height,float width)
 {
 	const float3 orto1 = RgtVector;
 	const float3 orto2 = FwdVector;
-	const float baseAngle = fRand(2 * PI);
+	const float baseAngle = math::TWOPI * guRNG.NextFloat();
 
 	DrawTrunk(ZeroVector, float3(0, height, 0), orto1, orto2, width);
 
 	for (int a = 0; a < numBranch; ++a) {
-		const float angle = baseAngle + (a * 3.88f) + fRand(0.5f);
-		float3 dir = orto1 * math::sin(angle) + orto2 * math::cos(angle);
-			dir.y = 0.3f + fRand(0.4f);
+		const float angle = baseAngle + (a * 3.88f) + 0.5f * guRNG.NextFloat();
+		float3 dir = orto1 * std::sin(angle) + orto2 * std::cos(angle);
+			dir.y = 0.3f + 0.4f * guRNG.NextFloat();
 			dir.ANormalize();
 		const float3 start(0, (a + 5) * height / (numBranch + 5), 0);
-		const float length = (height * (0.4f + fRand(0.1f))) * math::sqrt(float(numBranch - a) / numBranch);
+		const float length = (height * (0.4f + 0.1f * guRNG.NextFloat())) * std::sqrt(float(numBranch - a) / numBranch);
 
 		TrunkIterator(start, dir, length, length * 0.05f, 1);
 	}
 
 	for (int a = 0; a < 3; ++a) {
-		const float angle = (a * 3.88f) + fRand(0.5f);
-		float3 dir = orto1*math::sin(angle)+orto2*math::cos(angle);
+		const float angle = (a * 3.88f) + 0.5f * guRNG.NextFloat();
+		float3 dir = orto1*std::sin(angle)+orto2*std::cos(angle);
 			dir.y = 0.8f;
 			dir.ANormalize();
 		const float3 start(0, height - 0.3f, 0);
@@ -217,15 +220,15 @@ void CAdvTreeGenerator::TrunkIterator(const float3& start, const float3& dir, fl
 	if (depth == 0)
 		return;
 
-	const float dirDif = fRand(0.8f) + 1.0f;
+	const float dirDif = 0.8f * guRNG.NextFloat() + 1.0f;
 	const int numTrunks = (int) length * 5 / MAX_TREE_HEIGHT;
 
 	for (int a = 0; a < numTrunks; a++) {
-		const float angle = PI + float(a) * PI + fRand(0.3f);
+		const float angle = math::PI + float(a) * math::PI + 0.3f * guRNG.NextFloat();
 		const float newLength = length * (float(numTrunks - a) / (numTrunks + 1));
 
 		float3 newbase = start + dir * length * (float(a + 1) / (numTrunks + 1));
-		float3 newDir = dir + orto1 * math::cos(angle) * dirDif + orto2 * math::sin(angle) * dirDif;
+		float3 newDir = dir + orto1 * std::cos(angle) * dirDif + orto2 * std::sin(angle) * dirDif;
 		newDir.ANormalize();
 
 		TrunkIterator(newbase, newDir, newLength, newLength * 0.05f, depth - 1);
@@ -234,51 +237,44 @@ void CAdvTreeGenerator::TrunkIterator(const float3& start, const float3& dir, fl
 
 void CAdvTreeGenerator::CreateLeaves(const float3& start, const float3& dir, float length, float3& orto1,float3& orto2)
 {
-	const float baseRot = fRand(2 * PI);
+	const float baseRot = math::TWOPI * guRNG.NextFloat();
 	const int numLeaves = (int) length * 10 / MAX_TREE_HEIGHT;
 
 	float3 flatSun = sky->GetLight()->GetLightDir();
 	flatSun.y = 0.0f;
 
-	for (int a = 0; a < numLeaves + 1; a++) {
-		const float angle = baseRot + a * 0.618f * 2 * PI;
-
-		float3 pos = start + dir * length * (0.7f + fRand(0.3f));
-		pos +=
-			(orto1 * math::sin(angle) + orto2 * math::cos(angle)) *
-			(math::sqrt((float) a + 1) * 0.6f + fRand(0.4f)) *
-			0.1f * MAX_TREE_HEIGHT;
-
-		if (pos.y < 0.2f * MAX_TREE_HEIGHT)
-			pos.y = 0.2f * MAX_TREE_HEIGHT;
-
+	auto ADD_LEAF = [&](const float3& pos) {
 		float3 npos = pos;
 		npos.y = 0;
 		npos.ANormalize();
 
-		const float tex = float(int(rand() * 3 / RAND_MAX)) * 0.125f;
-		const float flipTex = float(int(rand()*2/RAND_MAX)) * 0.123f;
-		const float col = 0.5f + (npos.dot(flatSun) * 0.3f) + fRand(0.1f);
+		const float tex = (guRNG.NextInt(3)) * 0.125f;
+		const float flipTex = (guRNG.NextInt(2)) * 0.123f;
+		const float col = 0.5f + (npos.dot(flatSun) * 0.3f) + 0.1f * guRNG.NextFloat();
 
 		va->AddVertexTN(pos, 0.126f + tex + flipTex, 0.98f, float3( 0.09f * MAX_TREE_HEIGHT, -0.09f * MAX_TREE_HEIGHT, col));
 		va->AddVertexTN(pos, 0.249f + tex - flipTex, 0.98f, float3(-0.09f * MAX_TREE_HEIGHT, -0.09f * MAX_TREE_HEIGHT, col));
 		va->AddVertexTN(pos, 0.249f + tex - flipTex, 0.02f, float3(-0.09f * MAX_TREE_HEIGHT,  0.09f * MAX_TREE_HEIGHT, col));
 		va->AddVertexTN(pos, 0.126f + tex + flipTex, 0.02f, float3( 0.09f * MAX_TREE_HEIGHT,  0.09f * MAX_TREE_HEIGHT, col));
+	};
+
+	for (int a = 0; a < numLeaves + 1; a++) {
+		const float angle = baseRot + a * 0.618f * math::TWOPI;
+
+		float3 pos = start + dir * length * (0.7f + 0.3f * guRNG.NextFloat());
+		pos +=
+			(orto1 * std::sin(angle) + orto2 * std::cos(angle)) *
+			mix(guRNG.NextFloat(), std::sqrt((float) a + 1), 0.6f) *
+			0.1f * MAX_TREE_HEIGHT;
+
+		if (pos.y < 0.2f * MAX_TREE_HEIGHT)
+			pos.y = 0.2f * MAX_TREE_HEIGHT;
+
+		ADD_LEAF(pos);
 	}
 
 	float3 pos = start + dir * length * 1.03f;
-	float3 npos = pos;
-	npos.y = 0;
-	npos.ANormalize();
-
-	const float tex = float(int(rand() * 3 / RAND_MAX)) * 0.125f;
-	const float flipTex = float(int(rand() * 2 / RAND_MAX)) * 0.123f;
-	const float col = 0.5f + (npos.dot(flatSun) * 0.3f) + fRand(0.1f);
-
-	va->AddVertexTN(pos, 0.126f + tex + flipTex, 0.98f, float3( 0.09f * MAX_TREE_HEIGHT, -0.09f * MAX_TREE_HEIGHT, col));
-	va->AddVertexTN(pos, 0.249f + tex - flipTex, 0.98f, float3(-0.09f * MAX_TREE_HEIGHT, -0.09f * MAX_TREE_HEIGHT, col));
-	va->AddVertexTN(pos, 0.249f + tex - flipTex, 0.02f, float3(-0.09f * MAX_TREE_HEIGHT,  0.09f * MAX_TREE_HEIGHT, col));
-	va->AddVertexTN(pos, 0.126f + tex + flipTex, 0.02f, float3( 0.09f * MAX_TREE_HEIGHT,  0.09f * MAX_TREE_HEIGHT, col));
+	ADD_LEAF(pos);
 }
 
 void CAdvTreeGenerator::CreateFarTex(Shader::IProgramObject* treeShader)
@@ -456,14 +452,18 @@ void CAdvTreeGenerator::CreateGranTex(unsigned char* data, int xpos, int ypos, i
 	glMatrixMode(GL_PROJECTION);
 	glPushMatrix();
 	glOrtho(0,1,0,1,-4,4);
+	glViewport(0,0,256,256);
+
+	glPushAttrib(GL_ENABLE_BIT | GL_CURRENT_BIT | GL_COLOR_BUFFER_BIT);
 	glDisable(GL_TEXTURE_2D);
 	glDisable(GL_FOG);
 	glDisable(GL_BLEND);
 	glColor4f(1,1,1,1);
-	glViewport(0,0,256,256);
+
 	glAlphaFunc(GL_GREATER,0.5f);
 	glEnable(GL_ALPHA_TEST);
 	glDisable(GL_DEPTH_TEST);
+
 	glClearColor(0.0f,0.0f,0.0f,0);
 	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 
@@ -488,7 +488,8 @@ void CAdvTreeGenerator::CreateGranTex(unsigned char* data, int xpos, int ypos, i
 		}
 	}
 
-	glEnable(GL_DEPTH_TEST);
+	glPopAttrib();
+
 	glPopMatrix();
 	glMatrixMode(GL_MODELVIEW);
 	glPopMatrix();
@@ -502,7 +503,10 @@ void CAdvTreeGenerator::CreateGranTexBranch(const float3& start, const float3& e
 	float3 orto=dir.cross(FwdVector);
 
 	glBegin(GL_QUADS);
-		glColor3f(0.05f+fRand(0.02f),0.21f+fRand(0.05f),0.04f+fRand(0.01f));
+		float3 c = float3(guRNG.NextFloat(), guRNG.NextFloat(), guRNG.NextFloat());
+		c *= float3(0.02f, 0.05f, 0.01f);
+		c += float3(0.05f, 0.21f, 0.04f);
+		glColorf3(c);
 		glVertexf3(start+dir*0.006f+orto*0.007f);
 		glVertexf3(start+dir*0.006f-orto*0.007f);
 		glVertexf3(end-orto*0.007f);
@@ -515,14 +519,15 @@ void CAdvTreeGenerator::CreateGranTexBranch(const float3& start, const float3& e
 		glVertexf3(end+orto*0.001f);
 	glEnd();
 
-	float tipDist=0.025f;
-	float delta=0.013f;
-	int side=(rand()&1)*2-1;
-	while(tipDist<length*(0.83f+fRand(0.15f))){
+	float tipDist = 0.025f;
+	float delta = 0.013f;
+	int side = (guRNG.NextInt() & 1) * 2 - 1;
+
+	while(tipDist < length * (0.15f * guRNG.NextFloat() + 0.83f)){
 		float3 bstart=start+dir*(length-tipDist);
-		float3 bdir=dir+orto*side*(1.0f+fRand(0.7f));
+		float3 bdir=dir+orto*side*(1.0f + 0.7f * guRNG.NextFloat());
 		bdir.ANormalize();
-		float3 bend=bstart+bdir*tipDist*((6-tipDist)*(0.10f+fRand(0.05f)));
+		float3 bend=bstart+bdir*tipDist*((6-tipDist)*(0.10f + 0.05f * guRNG.NextFloat()));
 		CreateGranTexBranch(bstart,bend);
 		side*=-1;
 		tipDist+=delta;
@@ -535,19 +540,19 @@ void CAdvTreeGenerator::PineTree(int numBranch, float height)
 	DrawPineTrunk(ZeroVector,float3(0,height,0),height*0.025f);
 	float3 orto1 = RgtVector;
 	float3 orto2 = FwdVector;
-	float baseAngle=fRand(2*PI);
-	for(int a=0;a<numBranch;++a){
-		float sh = 0.2f + fRand(0.2f);
-		float h  = height * math::pow(sh + float(a)/numBranch * (1-sh), (float)0.7f);
-		float angle = baseAngle + (a * 0.618f + fRand(0.1f)) * 2 * PI;
-		float3 dir(orto1 * math::sin(angle) + orto2 * math::cos(angle));
-		dir.y = (a - numBranch) * 0.01f - (0.2f + fRand(0.2f));
+	float baseAngle = math::TWOPI * guRNG.NextFloat();
+	for (int a = 0; a < numBranch; ++a) {
+		float sh = 0.2f + 0.2f * guRNG.NextFloat();
+		float h  = height * std::pow(sh + float(a)/numBranch * (1-sh), (float)0.7f);
+		float angle = baseAngle + (a * 0.618f + 0.1f * guRNG.NextFloat()) * math::TWOPI;
+		float3 dir(orto1 * std::sin(angle) + orto2 * std::cos(angle));
+		dir.y = (a - numBranch) * 0.01f - (0.2f + 0.2f * guRNG.NextFloat());
 		dir.ANormalize();
-		float size = math::sqrt((float)numBranch - a + 5) * 0.08f * MAX_TREE_HEIGHT;
+		float size = std::sqrt((float)numBranch - a + 5) * 0.08f * MAX_TREE_HEIGHT;
 		DrawPineBranch(float3(0,h,0),dir,size);
 	}
 	//create the top
-	float col=0.55f+fRand(0.2f);
+	float col=0.55f + 0.2f * guRNG.NextFloat();
 	va->AddVertexTN(float3(0,height-0.09f*MAX_TREE_HEIGHT,0), 0.126f+0.5f, 0.02f, float3(0,0,col));
 	va->AddVertexTN(float3(0,height-0.03f*MAX_TREE_HEIGHT,0), 0.249f+0.5f, 0.02f, float3(0.05f*MAX_TREE_HEIGHT,0,col));
 	va->AddVertexTN(float3(0,height-0.03f*MAX_TREE_HEIGHT,0), 0.126f+0.5f, 0.98f, float3(-0.05f*MAX_TREE_HEIGHT,0,col));
@@ -567,18 +572,18 @@ void CAdvTreeGenerator::DrawPineTrunk(const float3 &start, const float3 &end, fl
 
 	int numIter=8;
 	for(int a=0;a<numIter;a++){
-		float angle=a/(float)numIter*2*PI;
-		float angle2=(a+1)/(float)numIter*2*PI;
-		float col=0.45f+(((orto1*math::sin(angle)+orto2*math::cos(angle)).dot(flatSun)))*0.3f;
-		float col2=0.45f+(((orto1*math::sin(angle2)+orto2*math::cos(angle2)).dot(flatSun)))*0.3f;
+		float angle=a/(float)numIter * math::TWOPI;
+		float angle2=(a+1)/(float)numIter * math::TWOPI;
+		float col=0.45f+(((orto1*std::sin(angle)+orto2*std::cos(angle)).dot(flatSun)))*0.3f;
+		float col2=0.45f+(((orto1*std::sin(angle2)+orto2*std::cos(angle2)).dot(flatSun)))*0.3f;
 
-		va->AddVertexTN(start+orto1*math::sin(angle)*size+orto2*math::cos(angle)*size,           angle/PI*0.125f*0.5f+0.5f,  0, float3(0,0,col));
-		va->AddVertexTN(end+orto1*math::sin(angle)*size*0.1f+orto2*math::cos(angle)*size*0.1f,   angle/PI*0.125f*0.5f+0.5f,  3, float3(0,0,col));
-		va->AddVertexTN(start+orto1*math::sin(angle2)*size+orto2*math::cos(angle2)*size,         angle2/PI*0.125f*0.5f+0.5f, 0, float3(0,0,col2));
+		va->AddVertexTN(start+orto1*std::sin(angle)*size+orto2*std::cos(angle)*size,           angle / math::PI*0.125f*0.5f+0.5f, 0, float3(0,0,col));
+		va->AddVertexTN(end+orto1*std::sin(angle)*size*0.1f+orto2*std::cos(angle)*size*0.1f,   angle / math::PI*0.125f*0.5f+0.5f, 3, float3(0,0,col));
+		va->AddVertexTN(start+orto1*std::sin(angle2)*size+orto2*std::cos(angle2)*size,         angle2/ math::PI*0.125f*0.5f+0.5f, 0, float3(0,0,col2));
 
-		va->AddVertexTN(start+orto1*math::sin(angle2)*size+orto2*math::cos(angle2)*size,         angle2/PI*0.125f*0.5f+0.5f, 0, float3(0,0,col2));
-		va->AddVertexTN(end+orto1*math::sin(angle)*size*0.1f+orto2*math::cos(angle)*size*0.1f,   angle/PI*0.125f*0.5f+0.5f,  3, float3(0,0,col));
-		va->AddVertexTN(end+orto1*math::sin(angle2)*size*0.1f+orto2*math::cos(angle2)*size*0.1f, angle2/PI*0.125f*0.5f+0.5f, 3, float3(0,0,col2));
+		va->AddVertexTN(start+orto1*std::sin(angle2)*size+orto2*std::cos(angle2)*size,         angle2 / math::PI*0.125f*0.5f+0.5f, 0, float3(0,0,col2));
+		va->AddVertexTN(end+orto1*std::sin(angle)*size*0.1f+orto2*std::cos(angle)*size*0.1f,   angle  / math::PI*0.125f*0.5f+0.5f, 3, float3(0,0,col));
+		va->AddVertexTN(end+orto1*std::sin(angle2)*size*0.1f+orto2*std::cos(angle2)*size*0.1f, angle2 / math::PI*0.125f*0.5f+0.5f, 3, float3(0,0,col2));
 	}
 }
 
@@ -591,14 +596,14 @@ void CAdvTreeGenerator::DrawPineBranch(const float3 &start, const float3 &dir, f
 	orto1.ANormalize();
 	float3 orto2 = dir.cross(orto1);
 
-	float tex = 0.0f;//int(rand() * 3.0f/RAND_MAX) * 0.125f;
-	float baseCol = 0.4f + 0.3f * dir.dot(flatSun) + fRand(0.1f);
+	float tex = 0.0f; // int(guRNG.NextFloat() * 3.0f) * 0.125f;
+	float baseCol = 0.4f + 0.3f * dir.dot(flatSun) + 0.1f * guRNG.NextFloat();
 
-	float col1=baseCol+fRand(0.2f);
-	float col2=baseCol+fRand(0.2f);
-	float col3=baseCol+fRand(0.2f);
-	float col4=baseCol+fRand(0.2f);
-	float col5=baseCol+fRand(0.2f);
+	float col1=baseCol + 0.2f * guRNG.NextFloat();
+	float col2=baseCol + 0.2f * guRNG.NextFloat();
+	float col3=baseCol + 0.2f * guRNG.NextFloat();
+	float col4=baseCol + 0.2f * guRNG.NextFloat();
+	float col5=baseCol + 0.2f * guRNG.NextFloat();
 
 	va->AddVertexTN(start,                                               0.5f+0.126f+tex,  0.02f, float3(0,0,col1));
 	va->AddVertexTN(start+dir*size*0.5f+orto1*size*0.5f+orto2*size*0.2f, 0.5f+0.249f+tex,  0.02f, float3(0,0,col2));
@@ -617,10 +622,6 @@ void CAdvTreeGenerator::DrawPineBranch(const float3 &start, const float3 &dir, f
 	va->AddVertexTN(start+dir*size+orto2*size*0.1f,                      0.5f+0.249f+tex,  0.98f, float3(0,0,col5));
 }
 
-float CAdvTreeGenerator::fRand(float size)
-{
-	return float(rand()) / RAND_MAX * size;
-}
 
 void CAdvTreeGenerator::CreateLeafTex(unsigned int baseTex, int xpos, int ypos,unsigned char buf[256][2048][4])
 {
@@ -646,25 +647,25 @@ void CAdvTreeGenerator::CreateLeafTex(unsigned int baseTex, int xpos, int ypos,u
 	glClearColor(0.0f,0.0f,0.0f,0);
 	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 
-	float baseCol=0.8f+fRand(0.2f);
+	float baseCol=0.8f + 0.2f * guRNG.NextFloat();
 
 	for(int a=0;a<84;++a){
-		float xp=0.9f-fRand(1.8f);
-		float yp=0.9f-fRand(1.8f);
+		float xp=0.9f - 1.8f * guRNG.NextFloat();
+		float yp=0.9f - 1.8f * guRNG.NextFloat();
 
-		float rot=fRand(360);
+		float rot=360 * guRNG.NextFloat();
 
-		float rCol=0.7f+fRand(0.3f);
-		float gCol=0.7f+fRand(0.3f);
-		float bCol=0.7f+fRand(0.3f);
+		float rCol=0.7f + 0.3f * guRNG.NextFloat();
+		float gCol=0.7f + 0.3f * guRNG.NextFloat();
+		float bCol=0.7f + 0.3f * guRNG.NextFloat();
 
 		glPushMatrix();
 		glLoadIdentity();
 		glColor3f(baseCol*rCol,baseCol*gCol,baseCol*bCol);
 		glTranslatef(xp,yp,0);
 		glRotatef(rot,0,0,1);
-		glRotatef(fRand(360),1,0,0);
-		glRotatef(fRand(360),0,1,0);
+		glRotatef(360 * guRNG.NextFloat(),1,0,0);
+		glRotatef(360 * guRNG.NextFloat(),0,1,0);
 
 		glBegin(GL_QUADS);
 			glTexCoord2f(0,0); glVertex3f(-0.1f,-0.2f,0);
