@@ -10,32 +10,32 @@
 #include "Rendering/GlobalRendering.h"
 #include "System/Config/ConfigHandler.h"
 #include "System/Exceptions.h"
+#include "System/SafeUtil.h"
 #include "System/Log/ILog.h"
 
-CONFIG(bool, DynamicSun).defaultValue(false).description("Sets whether the sun will move around the sky throughout the game.\nThe sun will always stay above DynamicSunMinElevation");
 CONFIG(bool, AdvSky).defaultValue(true).headlessValue(false).defaultValue(false).description("Enables High Resolution Clouds.");
 
-ISky* sky = NULL;
+ISky* sky = nullptr;
 
 ISky::ISky()
-	: wireframe(false)
-	, dynamicSky(false)
-	, skyColor(mapInfo->atmosphere.skyColor)
+	: skyColor(mapInfo->atmosphere.skyColor)
 	, sunColor(mapInfo->atmosphere.sunColor)
 	, cloudColor(mapInfo->atmosphere.cloudColor)
 	, fogColor(mapInfo->atmosphere.fogColor)
 	, fogStart(mapInfo->atmosphere.fogStart)
 	, fogEnd(mapInfo->atmosphere.fogEnd)
 	, cloudDensity(mapInfo->atmosphere.cloudDensity)
-	, skyLight(NULL)
-// 	, cloudDensity(0.0f)
+	, skyLight(nullptr)
+	, wireFrameMode(false)
+	, dynamicSky(false)
+
 {
-	SetLight(configHandler->GetBool("DynamicSun"));
+	skyLight = new ISkyLight();
 }
 
 ISky::~ISky()
 {
-	delete skyLight;
+	spring::SafeDelete(skyLight);
 }
 
 
@@ -59,7 +59,7 @@ void ISky::SetupFog() {
 
 ISky* ISky::GetSky()
 {
-	ISky* sky = NULL;
+	ISky* sky = nullptr;
 
 	try {
 		if (!mapInfo->atmosphere.skyBox.empty()) {
@@ -68,28 +68,17 @@ ISky* ISky::GetSky()
 			sky = new CAdvSky();
 		}
 	} catch (const content_error& ex) {
-		LOG_L(L_ERROR, "[%s] error: %s (falling back to BasicSky)",
-				__FUNCTION__, ex.what());
-		delete sky;
-		sky = NULL;
+		LOG_L(L_ERROR, "[%s] error: %s (falling back to BasicSky)", __FUNCTION__, ex.what());
+
+		spring::SafeDelete(sky);
 	}
 
-	if (sky == NULL) {
+	if (sky == nullptr)
 		sky = new CBasicSky();
-	}
 
 	return sky;
 }
 
-void ISky::SetLight(bool dynamic) {
-	delete skyLight;
-
-	if (dynamic)
-		skyLight = new DynamicSkyLight();
-	else
-		skyLight = new StaticSkyLight();
-}
-
 bool ISky::SunVisible(const float3 pos) const {
 	const CUnit* hitUnit = nullptr;
 	const CFeature* hitFeature = nullptr;
@@ -97,7 +86,7 @@ bool ISky::SunVisible(const float3 pos) const {
 	// cast a ray *toward* the sun from <pos>
 	// sun is visible if no terrain blocks it
 	const float3& sunDir = skyLight->GetLightDir();
-	const float sunDist = TraceRay::GuiTraceRay(pos, sunDir, globalRendering->viewRange, NULL, hitUnit, hitFeature, false, true, false);
+	const float sunDist = TraceRay::GuiTraceRay(pos, sunDir, globalRendering->viewRange, nullptr, hitUnit, hitFeature, false, true, false);
 
 	return (sunDist < 0.0f || sunDist >= globalRendering->viewRange);
 }
