@@ -22,12 +22,15 @@ public:
 		float alphaDecay
 	);
 
-	void Draw() override;
+	void Draw(CVertexArray* va) override;
 	void Update() override;
 
 	void Init(const CUnit* owner, const float3& offset) override;
+	// override this so the projectile does not instantly disappear
+	// after creation if it also happened to collide with something
+	void Collision() override {}
 
-	virtual int GetProjectilesCount() const override;
+	int GetProjectilesCount() const override;
 
 	static bool GetMemberInfo(SExpGenSpawnableMemberInfo& memberInfo);
 
