@@ -6,8 +6,9 @@
 #include "Rendering/Env/Particles/ProjectileDrawer.h"
 #include "Rendering/GL/VertexArray.h"
 #include "Rendering/Textures/TextureAtlas.h"
+#include "Sim/Projectiles/ProjectileMemPool.h"
 
-CR_BIND_DERIVED(CGeoSquareProjectile, CProjectile, )
+CR_BIND_DERIVED_POOL(CGeoSquareProjectile, CProjectile, , projMemPool.alloc, projMemPool.free)
 
 CR_REG_METADATA(CGeoSquareProjectile,(
 	CR_MEMBER(p1),
@@ -42,27 +43,20 @@ CGeoSquareProjectile::CGeoSquareProjectile(const float3& p1, const float3& p2, c
 	SetRadiusAndHeight(p1.distance(p2) * 0.55f, 0.0f);
 }
 
-CGeoSquareProjectile::~CGeoSquareProjectile()
-{
-}
 
-void CGeoSquareProjectile::Draw()
+void CGeoSquareProjectile::Draw(CVertexArray* va)
 {
-	inArray = true;
 	unsigned char col[4];
 	col[0] = (unsigned char) (r * a * 255);
 	col[1] = (unsigned char) (g * a * 255);
 	col[2] = (unsigned char) (b * a * 255);
 	col[3] = (unsigned char) (    a * 255);
 
-	float3 dif(p1 - camera->GetPos());
-	dif.ANormalize();
-	float3 dir1(dif.cross(v1));
-	dir1.ANormalize();
-	float3 dif2(p2 - camera->GetPos());
-	dif2.ANormalize();
-	float3 dir2(dif2.cross(v2));
-	dir2.ANormalize();
+	float3 dif(p1 - camera->GetPos()); dif.ANormalize();
+	float3 dir1(dif.cross(v1)); dir1.ANormalize();
+
+	float3 dif2(p2 - camera->GetPos()); dif2.ANormalize();
+	float3 dir2(dif2.cross(v2)); dir2.ANormalize();
 
 
 	const float u = (projectileDrawer->geosquaretex->xstart + projectileDrawer->geosquaretex->xend) / 2;
