@@ -6,34 +6,34 @@
 #include "Rendering/Env/Particles/ProjectileDrawer.h"
 #include "Rendering/GL/VertexArray.h"
 #include "Rendering/Textures/TextureAtlas.h"
+#include "Sim/Projectiles/ProjectileMemPool.h"
 #include "Sim/Units/Unit.h"
 
-CR_BIND_DERIVED(CRepulseGfx, CProjectile, )
+CR_BIND_DERIVED_POOL(CRepulseGfx, CProjectile, , projMemPool.alloc, projMemPool.free)
 
 CR_REG_METADATA(CRepulseGfx,(
 	CR_MEMBER(repulsed),
-	CR_MEMBER(sqMaxDist),
+	CR_MEMBER(sqMaxOwnerDist),
 	CR_MEMBER(age),
 	CR_MEMBER(color),
-	CR_MEMBER(difs)
+	CR_MEMBER(vertexDists)
 ))
 
 
-CRepulseGfx::CRepulseGfx(CUnit* owner, CProjectile* repulsed, float maxDist, const float3& color):
-	CProjectile(repulsed? repulsed->pos: ZeroVector, repulsed? repulsed->speed: ZeroVector, owner, false, false, false),
-	repulsed(repulsed),
-	sqMaxDist((maxDist * maxDist) + 100),
+CRepulseGfx::CRepulseGfx(CUnit* owner, CProjectile* repulsee, float maxOwnerDist, const float4& gfxColor):
+	CProjectile((repulsee != nullptr)? repulsee->pos: ZeroVector, (repulsee != nullptr)? repulsee->speed: ZeroVector, owner, false, false, false),
+	repulsed(repulsee),
 	age(0),
-	color(color)
+	sqMaxOwnerDist((maxOwnerDist * maxOwnerDist) + 100.0f),
+	color(gfxColor)
 {
-	if (repulsed) {
+	if (repulsed != nullptr)
 		AddDeathDependence(repulsed, DEPENDENCE_REPULSE);
-	}
 
 	checkCol = false;
 	useAirLos = true;
 
-	SetRadiusAndHeight(maxDist, 0.0f);
+	SetRadiusAndHeight(maxOwnerDist, 0.0f);
 
 	for (int y = 0; y < 5; ++y) {
 		float yp = (y / 4.0f - 0.5f);
@@ -41,32 +41,31 @@ CRepulseGfx::CRepulseGfx(CUnit* owner, CProjectile* repulsed, float maxDist, con
 		for (int x = 0; x < 5; ++x) {
 			float xp = (x / 4.0f - 0.5f);
 			float d = 0;
-			if (xp != 0 || yp != 0) {
+
+			if (xp != 0 || yp != 0)
 				d = fastmath::apxsqrt2(xp * xp + yp * yp);
-			}
-			difs[y * 5 + x] = (1 - fastmath::cos(d * 2)) * 20;
+
+			vertexDists[y * 5 + x] = (1.0f - fastmath::cos(d * 2.0f)) * 20.0f;
 		}
 	}
 }
 
-CRepulseGfx::~CRepulseGfx()
-{
-}
 
 void CRepulseGfx::DependentDied(CObject* o)
 {
-	if (o == repulsed) {
-		repulsed = NULL;
-		deleteMe = true;
-	}
+	if (o != repulsed)
+		return;
+
+	repulsed = nullptr;
+	deleteMe = true;
 }
 
-void CRepulseGfx::Draw()
+void CRepulseGfx::Draw(CVertexArray* va)
 {
 	const CUnit* owner = CProjectile::owner();
-	if (!owner || !repulsed) {
+
+	if (owner == nullptr || repulsed == nullptr)
 		return;
-	}
 
 	const float3 zdir = (repulsed->pos - owner->pos).SafeANormalize();
 	const float3 xdir = (zdir.cross(UpVector)).SafeANormalize();
@@ -76,16 +75,17 @@ void CRepulseGfx::Draw()
 	float3 ydirDS;
 
 	pos = (repulsed->pos - zdir * 10.0f) + (repulsed->speed * globalRendering->timeOffset);
-	inArray = true;
 
 	float drawsize = 10.0f;
 	float alpha = std::min(255.0f, age * 10.0f);
+
 	unsigned char col[4] = {
-		(unsigned char) (color.x * alpha),
-		(unsigned char) (color.y * alpha),
-		(unsigned char) (color.z * alpha),
-		(unsigned char) (0.2f    * alpha),
+		(unsigned char) (color.x * alpha       ),
+		(unsigned char) (color.y * alpha       ),
+		(unsigned char) (color.z * alpha       ),
+		(unsigned char) (color.w * alpha * 0.2f),
 	};
+	constexpr unsigned char col2[4] = {0, 0, 0, 0};
 
 	xdirDS = xdir * drawsize;
 	ydirDS = ydir * drawsize;
@@ -96,8 +96,8 @@ void CRepulseGfx::Draw()
 	const float txs = et->xend - et->xstart;
 	const float tys = et->yend - et->ystart;
 
-	static const int loopCountY = 4;
-	static const int loopCountX = 4;
+	static constexpr int loopCountY = 4;
+	static constexpr int loopCountX = 4;
 
 	va->EnlargeArrays(loopCountY * loopCountX * 4 + 16, 0, VA_SIZE_TC);
 
@@ -109,60 +109,52 @@ void CRepulseGfx::Draw()
 			const float dx = x - 2.00f;
 			const float rx = x * 0.25f;
 
-			va->AddVertexQTC(pos + xdirDS * (dx + 0) + ydirDS * (dy + 0) + zdir * difs[(y    ) * 5 + x    ],  txo + (ry        ) * txs, tyo + (rx        ) * tys,  col);
-			va->AddVertexQTC(pos + xdirDS * (dx + 0) + ydirDS * (dy + 1) + zdir * difs[(y + 1) * 5 + x    ],  txo + (ry + 0.25f) * txs, tyo + (rx        ) * tys,  col);
-			va->AddVertexQTC(pos + xdirDS * (dx + 1) + ydirDS * (dy + 1) + zdir * difs[(y + 1) * 5 + x + 1],  txo + (ry + 0.25f) * txs, tyo + (rx + 0.25f) * tys,  col);
-			va->AddVertexQTC(pos + xdirDS * (dx + 1) + ydirDS * (dy + 0) + zdir * difs[(y    ) * 5 + x + 1],  txo + (ry        ) * txs, tyo + (rx + 0.25f) * tys,  col);
+			va->AddVertexQTC(pos + xdirDS * (dx + 0) + ydirDS * (dy + 0) + zdir * vertexDists[(y    ) * 5 + x    ],  txo + (ry        ) * txs, tyo + (rx        ) * tys,  col);
+			va->AddVertexQTC(pos + xdirDS * (dx + 0) + ydirDS * (dy + 1) + zdir * vertexDists[(y + 1) * 5 + x    ],  txo + (ry + 0.25f) * txs, tyo + (rx        ) * tys,  col);
+			va->AddVertexQTC(pos + xdirDS * (dx + 1) + ydirDS * (dy + 1) + zdir * vertexDists[(y + 1) * 5 + x + 1],  txo + (ry + 0.25f) * txs, tyo + (rx + 0.25f) * tys,  col);
+			va->AddVertexQTC(pos + xdirDS * (dx + 1) + ydirDS * (dy + 0) + zdir * vertexDists[(y    ) * 5 + x + 1],  txo + (ry        ) * txs, tyo + (rx + 0.25f) * tys,  col);
 		}
 	}
 
 	drawsize = 7.0f;
 	alpha = std::min(10.0f, age / 2.0f);
-	col[0] = (unsigned char) (color.x * alpha);
-	col[1] = (unsigned char) (color.y * alpha);
-	col[2] = (unsigned char) (color.z * alpha);
-	col[3] = (unsigned char) (alpha * 0.4f);
+
+	col[0] = (unsigned char) (color.x * alpha       );
+	col[1] = (unsigned char) (color.y * alpha       );
+	col[2] = (unsigned char) (color.z * alpha       );
+	col[3] = (unsigned char) (color.w * alpha * 0.4f);
 
 	const AtlasedTexture* ct = projectileDrawer->repulsegfxtex;
 	const float tx = (ct->xend + ct->xstart) * 0.5f;
 	const float ty = (ct->yend + ct->ystart) * 0.5f;
 
-	static const unsigned char col2[4] = {0, 0, 0, 0};
-
 	xdirDS = xdir * drawsize;
 	ydirDS = ydir * drawsize;
 
 	va->AddVertexQTC(owner->pos + (-xdir + ydir) * drawsize * 0.2f,  tx, ty, col2);
 	va->AddVertexQTC(owner->pos + ( xdir + ydir) * drawsize * 0.2f,  tx, ty, col2);
-	va->AddVertexQTC(       pos + xdirDS + ydirDS + zdir * difs[6],  tx, ty, col);
-	va->AddVertexQTC(       pos - xdirDS + ydirDS + zdir * difs[6],  tx, ty, col);
+	va->AddVertexQTC(       pos + xdirDS + ydirDS + zdir * vertexDists[6],  tx, ty, col );
+	va->AddVertexQTC(       pos - xdirDS + ydirDS + zdir * vertexDists[6],  tx, ty, col );
 
 	va->AddVertexQTC(owner->pos + (-xdir - ydir) * drawsize * 0.2f,  tx, ty, col2);
 	va->AddVertexQTC(owner->pos + ( xdir - ydir) * drawsize * 0.2f,  tx, ty, col2);
-	va->AddVertexQTC(       pos + xdirDS - ydirDS + zdir * difs[6],  tx, ty, col);
-	va->AddVertexQTC(       pos - xdirDS - ydirDS + zdir * difs[6],  tx, ty, col);
+	va->AddVertexQTC(       pos + xdirDS - ydirDS + zdir * vertexDists[6],  tx, ty, col );
+	va->AddVertexQTC(       pos - xdirDS - ydirDS + zdir * vertexDists[6],  tx, ty, col );
 
 	va->AddVertexQTC(owner->pos + (xdir - ydir) * drawsize * 0.2f,   tx, ty, col2);
 	va->AddVertexQTC(owner->pos + (xdir + ydir) * drawsize * 0.2f,   tx, ty, col2);
-	va->AddVertexQTC(       pos + xdirDS + ydirDS + zdir * difs[6],  tx, ty, col);
-	va->AddVertexQTC(       pos + xdirDS - ydirDS + zdir * difs[6],  tx, ty, col);
+	va->AddVertexQTC(       pos + xdirDS + ydirDS + zdir * vertexDists[6],  tx, ty, col );
+	va->AddVertexQTC(       pos + xdirDS - ydirDS + zdir * vertexDists[6],  tx, ty, col );
 
 	va->AddVertexQTC(owner->pos + (-xdir - ydir) * drawsize * 0.2f,  tx, ty, col2);
 	va->AddVertexQTC(owner->pos + (-xdir + ydir) * drawsize * 0.2f,  tx, ty, col2);
-	va->AddVertexQTC(       pos - xdirDS + ydirDS + zdir * difs[6],  tx, ty, col);
-	va->AddVertexQTC(       pos - xdirDS - ydirDS + zdir * difs[6],  tx, ty, col);
+	va->AddVertexQTC(       pos - xdirDS + ydirDS + zdir * vertexDists[6],  tx, ty, col );
+	va->AddVertexQTC(       pos - xdirDS - ydirDS + zdir * vertexDists[6],  tx, ty, col );
 }
 
 void CRepulseGfx::Update()
 {
-	age++;
-
-	if (repulsed && owner() && (repulsed->pos - owner()->pos).SqLength() > sqMaxDist) {
-		deleteMe = true;
-	}
+	age += 1;
+	deleteMe |= (repulsed != nullptr && owner() != nullptr && (repulsed->pos - owner()->pos).SqLength() > sqMaxOwnerDist);
 }
 
-int CRepulseGfx::GetProjectilesCount() const
-{
-	return 20;
-}
