@@ -9,9 +9,10 @@
 #include "Rendering/Env/Particles/ProjectileDrawer.h"
 #include "Rendering/GL/VertexArray.h"
 #include "Rendering/Textures/TextureAtlas.h"
+#include "Sim/Projectiles/ProjectileMemPool.h"
 #include "System/myMath.h"
 
-CR_BIND_DERIVED(CSmokeTrailProjectile, CProjectile, )
+CR_BIND_DERIVED_POOL(CSmokeTrailProjectile, CProjectile, , projMemPool.alloc, projMemPool.free)
 
 CR_REG_METADATA(CSmokeTrailProjectile,(
 	CR_MEMBER(pos1),
@@ -59,7 +60,7 @@ CSmokeTrailProjectile::CSmokeTrailProjectile(
 	drawSegmented(false),
 	firstSegment(firstSegment),
 	lastSegment(lastSegment),
-	texture(texture == NULL ? projectileDrawer->smoketrailtex : texture)
+	texture((texture == nullptr)? projectileDrawer->smoketrailtex : texture)
 {
 	checkCol = false;
 	castShadow = true;
@@ -67,9 +68,7 @@ CSmokeTrailProjectile::CSmokeTrailProjectile(
 	UpdateEndPos(pos1, dir1);
 	SetRadiusAndHeight(pos1.distance(pos2), 0.0f);
 
-	if ((pos.y - CGround::GetApproximateHeight(pos.x, pos.z)) > 10) {
-		useAirLos = true;
-	}
+	useAirLos |= ((pos.y - CGround::GetApproximateHeight(pos.x, pos.z)) > 10.0f);
 }
 
 
@@ -95,9 +94,8 @@ void CSmokeTrailProjectile::UpdateEndPos(const float3 pos, const float3 dir)
 }
 
 
-void CSmokeTrailProjectile::Draw()
+void CSmokeTrailProjectile::Draw(CVertexArray* va)
 {
-	inArray = true;
 	const float age = gs->frameNum + globalRendering->timeOffset - creationTime;
 	const float invLifeTime = (1.0f / lifeTime);
 	va->EnlargeArrays(8, 0, VA_SIZE_TC);
