@@ -10,8 +10,9 @@
 #include "Rendering/GL/VertexArray.h"
 #include "Rendering/Textures/TextureAtlas.h"
 #include "Sim/Projectiles/ProjectileHandler.h"
+#include "Sim/Projectiles/ProjectileMemPool.h"
 
-CR_BIND_DERIVED(CWreckProjectile, CProjectile, )
+CR_BIND_DERIVED_POOL(CWreckProjectile, CProjectile, , projMemPool.alloc, projMemPool.free)
 CR_REG_METADATA(CWreckProjectile, )
 
 
@@ -28,24 +29,20 @@ void CWreckProjectile::Update()
 	speed.x *= 0.994f;
 	speed.z *= 0.994f;
 
-	if (speed.y > 0) {
+	if (speed.y > 0.0f)
 		speed.y *= 0.998f;
-	}
 
 	pos += speed;
 
 	if (!(gs->frameNum & (projectileHandler->GetParticleSaturation() < 0.5f? 1: 3))) {
-		CSmokeProjectile* hp = new CSmokeProjectile(owner(), pos, ZeroVector, 50, 4, 0.3f, 0.5f);
+		CSmokeProjectile* hp = projMemPool.alloc<CSmokeProjectile>(owner(), pos, ZeroVector, 50, 4, 0.3f, 0.5f);
 		hp->size += 0.1f;
 	}
-	if (pos.y + 0.3f < CGround::GetApproximateHeight(pos.x, pos.z)) {
-		deleteMe = true;
-	}
+	deleteMe |= (pos.y + 0.3f < CGround::GetApproximateHeight(pos.x, pos.z));
 }
 
-void CWreckProjectile::Draw()
+void CWreckProjectile::Draw(CVertexArray* va)
 {
-	inArray = true;
 	unsigned char col[4];
 	col[0] = (unsigned char) (0.15f * 200);
 	col[1] = (unsigned char) (0.1f  * 200);
