@@ -23,6 +23,7 @@ LOG_REGISTER_SECTION_GLOBAL(LOG_SECTION_FAR_TEXTURE_HANDLER)
 #define LOG_SECTION_CURRENT LOG_SECTION_FAR_TEXTURE_HANDLER
 
 #define NUM_ICON_ORIENTATIONS 8
+#define MAX_CREATE_QUEUE_SIZE 8
 
 
 CFarTextureHandler* farTextureHandler = nullptr;
@@ -32,18 +33,22 @@ CFarTextureHandler::CFarTextureHandler()
 	farTextureID = 0;
 	usedFarTextures = 0;
 
-	// ATI supports 16K textures, but that might be a bit too much
-	// for this purpose so limit the atlas to 4K (still sufficient)
-	iconSize.x = 32 * 4;
-	iconSize.y = 32 * 4;
-	texSize.x = std::min(globalRendering->maxTextureSize, 4096);
-	texSize.y = std::max(iconSize.y, 4 * NUM_ICON_ORIENTATIONS * iconSize.x * iconSize.y / texSize.x); // minimum space for 4 icons
-	texSize.y = next_power_of_2(texSize.y);
+	// assuming a maxTextureSize of 16384, we can fit at most 128
+	// (128-pixel) sprites or 16 far-textures into each row since
+	// each FT is rendered from 8 different orientations
+	// such an atlas would only allow 16*16=256 FT's but consume
+	// 16384*16384*4=1024MB when fully allocated, which is not a
+	// good tradeoff and necessitates limiting iconSize to 64x64
+	iconSize.x = 32 * 2;
+	iconSize.y = 32 * 2;
 
-	#ifdef HEADLESS
-	return;
-	#endif
+	texSize.x = globalRendering->maxTextureSize;
+	texSize.y = iconSize.y;
 
+	createQueue.reserve(MAX_CREATE_QUEUE_SIZE);
+	renderQueue.reserve(128);
+
+#ifndef HEADLESS
 	if (!fbo.IsValid()) {
 		LOG_L(L_WARNING, "framebuffer not valid!");
 		return;
@@ -67,13 +72,13 @@ CFarTextureHandler::CFarTextureHandler()
 	fbo.Unbind();
 
 	fbo.reloadOnAltTab = true;
+#endif
 }
 
 
 CFarTextureHandler::~CFarTextureHandler()
 {
 	glDeleteTextures(1, &farTextureID);
-	drawQueue.clear();
 }
 
 
@@ -114,20 +119,22 @@ bool CFarTextureHandler::HaveFarIcon(const CSolidObject* obj) const
 
 
 /**
- * @brief Really create the far texture for the given model.
+ * @brief Really create the far-texture for the given model.
  */
 void CFarTextureHandler::CreateFarTexture(const CSolidObject* obj)
 {
 	const S3DModel* model = obj->model;
 
-	// make space in the std::vectors
-	if (obj->team >= (int)iconCache.size())
-		iconCache.resize(obj->team + 1);
+	// make space
+	if (obj->team >= iconCache.size())
+		iconCache.resize(std::max(iconCache.size() * 2, size_t(obj->team + 1)), {});
 
-	if (model->id >= (int)iconCache[obj->team].size())
-		iconCache[obj->team].resize(model->id + 1, {0});
+	if (model->id >= iconCache[obj->team].size())
+		iconCache[obj->team].resize(std::max(iconCache[obj->team].size() * 2, size_t(model->id + 1)), {0});
 
-	assert(iconCache[obj->team][model->id].farTexNum == 0);
+	// same object can be queued multiple times in different passes
+	if (iconCache[obj->team][model->id].farTexNum != 0)
+		return;
 
 	// enough free space in the atlas?
 	if (!CheckResizeAtlas())
@@ -162,7 +169,7 @@ void CFarTextureHandler::CreateFarTexture(const CSolidObject* obj)
 	// twice the radius is not quite far away enough for some models
 	viewMat.Translate(float3(0.0f, 0.0f, -obj->GetDrawRadius() * (2.0f + 1.0f)));
 	viewMat.Scale(float3(-1.0f, 1.0f, 1.0f));
-	viewMat.RotateX(-60.0f * (M_PI / 180.0f));
+	viewMat.RotateX(-60.0f * math::DEG_TO_RAD);
 
 	// overwrite the matrices set by SetupOpaqueDrawing
 	//
@@ -240,29 +247,31 @@ void CFarTextureHandler::DrawFarTexture(const CSolidObject* obj, CVertexArray* v
 
 void CFarTextureHandler::Queue(const CSolidObject* obj)
 {
-	drawQueue.push_back(obj);
+	if (!fbo.IsValid())
+		return;
+
+	if (HaveFarIcon(obj)) {
+		renderQueue.push_back(obj);
+		return;
+	}
+
+	// limit the number of objects batch-processed this frame
+	if (createQueue.size() == MAX_CREATE_QUEUE_SIZE)
+		return;
+
+	createQueue.push_back(obj);
+	renderQueue.push_back(obj);
 }
 
 
 void CFarTextureHandler::Draw()
 {
-	if (drawQueue.empty())
-		return;
-
-	if (!fbo.IsValid()) {
-		drawQueue.clear();
-		return;
+	for (const CSolidObject* obj: createQueue) {
+		CreateFarTexture(obj);
 	}
 
-	// create new far-icons
-	for (const CSolidObject* obj: drawQueue) {
-		if (!HaveFarIcon(obj)) {
-			CreateFarTexture(obj);
-		}
-	}
-
-	// render current queued far icons on the screen
-	{
+	// render currently queued far-icons
+	if (!renderQueue.empty()) {
 		const float3 camNorm = ((camera->GetDir() * XZVector) - (UpVector * 0.1f)).ANormalize();
 
 		glEnable(GL_ALPHA_TEST);
@@ -277,9 +286,9 @@ void CFarTextureHandler::Draw()
 
 		CVertexArray* va = GetVertexArray();
 		va->Initialize();
-		va->EnlargeArrays(drawQueue.size() * 4, 0, VA_SIZE_T);
+		va->EnlargeArrays(renderQueue.size() * 4, 0, VA_SIZE_T);
 
-		for (const CSolidObject* obj: drawQueue) {
+		for (const CSolidObject* obj: renderQueue) {
 			DrawFarTexture(obj, va);
 		}
 
@@ -287,37 +296,46 @@ void CFarTextureHandler::Draw()
 		glDisable(GL_ALPHA_TEST);
 	}
 
-	drawQueue.clear();
+	renderQueue.clear();
+	createQueue.clear();
 }
 
 
 
 bool CFarTextureHandler::CheckResizeAtlas()
 {
-	const unsigned int maxSprites = ((texSize.x / iconSize.x) * (texSize.y / iconSize.y) / NUM_ICON_ORIENTATIONS) - 1;
-
-	if (usedFarTextures + 1 <= maxSprites)
-		return true;
-
 	const int oldTexSizeY = texSize.y;
+	const int maxTexSizeY = globalRendering->maxTextureSize;
+
+	while (texSize.y <= maxTexSizeY) {
+		const int maxSpritesX = texSize.x / iconSize.x;
+		const int maxSpritesY = texSize.y / iconSize.y;
+		const int maxSprites  = maxSpritesX * maxSpritesY;
+		const int numSprites  = usedFarTextures * NUM_ICON_ORIENTATIONS;
+
+		if ((numSprites + NUM_ICON_ORIENTATIONS) <= maxSprites)
+			break;
 
-	if (globalRendering->supportNPOTs) {
-		// make space for minimum 4 additional icons
-		texSize.y += std::max(iconSize.y,  4 * NUM_ICON_ORIENTATIONS * iconSize.x * iconSize.y / texSize.x);
-	} else {
 		texSize.y <<= 1;
 	}
 
-	if (texSize.y > globalRendering->maxTextureSize) {
-		LOG_L(L_DEBUG, "Out of farTextures"); 
-		texSize.y = oldTexSizeY;
+	if (texSize.y == oldTexSizeY)
+		return true;
+
+	if (texSize.y > maxTexSizeY) {
+		LOG_L(L_DEBUG, "[FTH::%s] out of far-texture atlas space", __func__);
 		return false;
 	}
 
-	std::vector<unsigned char> oldPixels(texSize.x * texSize.y * 4);
+
+	std::vector<unsigned char> atlasPixels(texSize.x * texSize.y * 4, 0);
+
 	glBindTexture(GL_TEXTURE_2D, farTextureID);
-	glGetTexImage(GL_TEXTURE_2D, 0, GL_RGBA, GL_UNSIGNED_BYTE, &oldPixels[0]); //TODO use the FBO?
-	memset(&oldPixels[0] + texSize.x*oldTexSizeY*4, 0, texSize.x*(texSize.y - oldTexSizeY)*4);
+	glGetTexImage(GL_TEXTURE_2D, 0, GL_RGBA, GL_UNSIGNED_BYTE, &atlasPixels[0]); //TODO use the FBO?
+	glDeleteTextures(1, &farTextureID);
+
+	// zero out the newly added atlas region
+	memset(&atlasPixels[0] + texSize.x * oldTexSizeY * 4, 0, texSize.x * (texSize.y - oldTexSizeY) * 4);
 
 	GLuint newFarTextureID;
 	glGenTextures(1, &newFarTextureID);
@@ -326,15 +344,11 @@ bool CFarTextureHandler::CheckResizeAtlas()
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_BORDER);
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
-	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, texSize.x, texSize.y, 0, GL_RGBA, GL_UNSIGNED_BYTE, &oldPixels[0]);
+	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, texSize.x, texSize.y, 0, GL_RGBA, GL_UNSIGNED_BYTE, &atlasPixels[0]);
 
 	fbo.Bind();
 	fbo.DetachAll();
-
-	glDeleteTextures(1, &farTextureID);
-	farTextureID = newFarTextureID;
-
-	fbo.AttachTexture(farTextureID);
+	fbo.AttachTexture(farTextureID = newFarTextureID);
 	fbo.CheckStatus("FARTEXTURE");
 	fbo.Unbind();
 
