@@ -3,12 +3,12 @@
 #ifndef ICON_HANDLER_H
 #define ICON_HANDLER_H
 
-#include <map>
 #include <string>
 #include <vector>
 
 #include "Icon.h"
 #include "System/float3.h"
+#include "System/UnorderedMap.hpp"
 
 class CVertexArray;
 
@@ -86,7 +86,7 @@ namespace icon {
 			unsigned int defTexID;
 			CIconData* defIconData;
 
-			typedef std::map<std::string, CIcon> IconMap;
+			typedef spring::unordered_map<std::string, CIcon> IconMap;
 			IconMap iconMap;
 
 		private:
