@@ -2,9 +2,10 @@
 
 // TODO: move this out of Sim, this is rendering code!
 
-
 #include "LineDrawer.h"
 
+#include <cmath>
+
 #include "Rendering/GlobalRendering.h"
 #include "Game/UI/CommandColors.h"
 
@@ -29,7 +30,7 @@ CLineDrawer::CLineDrawer()
 void CLineDrawer::UpdateLineStipple()
 {
 	stippleTimer += (globalRendering->lastFrameTime * 0.001f * cmdColors.StippleSpeed());
-	stippleTimer = math::fmod(stippleTimer, (16.0f / 20.0f));
+	stippleTimer = std::fmod(stippleTimer, (16.0f / 20.0f));
 }
 
 
