@@ -15,22 +15,23 @@ IInfoTextureHandler* infoTextureHandler = nullptr;
 void IInfoTextureHandler::Create()
 {
 	if (
-		globalRendering->haveGLSL && globalRendering->supportNPOTs
-		&& glGenerateMipmap && FBO::IsSupported()
-		&& GLEW_ARB_texture_query_lod && GLEW_VERSION_3_0
+		globalRendering->haveGLSL &&
+		globalRendering->supportNonPowerOfTwoTex &&
+		globalRendering->supportTextureQueryLOD &&
+		FBO::IsSupported() &&
+		glewIsSupported("GL_VERSION_3_0")
 	) {
 		try {
 			infoTextureHandler = new CInfoTextureHandler();
-		} catch(const opengl_error& glerr) {
+		} catch (const opengl_error& glerr) {
 			infoTextureHandler = nullptr;
 		}
 	}
 
-	if (infoTextureHandler == nullptr) {
+	if (infoTextureHandler == nullptr)
 		infoTextureHandler = new CLegacyInfoTextureHandler();
-	}
 
-	if (!!dynamic_cast<CInfoTextureHandler*>(infoTextureHandler)) {
+	if (dynamic_cast<CInfoTextureHandler*>(infoTextureHandler) != nullptr) {
 		LOG("InfoTexture: shaders");
 	} else {
 		LOG("InfoTexture: legacy");
