@@ -3,20 +3,19 @@
 #ifndef IMODELPARSER_H
 #define IMODELPARSER_H
 
-#include <unordered_map>
-#include <vector>
+#include <deque>
 #include <string>
-#include "System/Threading/SpringMutex.h"
 
+#include "3DModel.h"
+#include "System/UnorderedMap.hpp"
+#include "System/Threading/SpringThreading.h"
 
-struct S3DModel;
-struct S3DModelPiece;
 
 class IModelParser
 {
 public:
 	virtual ~IModelParser() {}
-	virtual S3DModel* Load(const std::string& name) = 0;
+	virtual S3DModel Load(const std::string& name) = 0;
 };
 
 
@@ -38,24 +37,21 @@ public:
 	void PreloadModel(const std::string& name);
 
 public:
-	typedef std::unordered_map<std::string, unsigned int> ModelMap; // "armflash.3do" --> id
-	typedef std::unordered_map<std::string, unsigned int> FormatMap; // "3do" --> MODELTYPE_3DO
-	typedef std::unordered_map<unsigned int, IModelParser*> ParserMap; // MODELTYPE_3DO --> parser
+	typedef spring::unordered_map<std::string, unsigned int> ModelMap; // "armflash.3do" --> id
+	typedef spring::unordered_map<std::string, unsigned int> FormatMap; // "3do" --> MODELTYPE_3DO
+	typedef spring::unordered_map<unsigned int, IModelParser*> ParserMap; // MODELTYPE_3DO --> parser
 
 private:
-	S3DModel* LoadCachedModel(const std::string& name, bool preload);
+	S3DModel ParseModel(const std::string& name, const std::string& path);
 	S3DModel* CreateModel(const std::string& name, const std::string& path, bool preload);
-	S3DModel* ParseModel(const std::string& name, const std::string& path);
+	S3DModel* LoadCachedModel(const std::string& name, bool preload);
 
 	IModelParser* GetFormatParser(const std::string& pathExt);
 
 	void KillModels();
 	void KillParsers();
 
-	void AddModelToCache(S3DModel* model, const std::string& name, const std::string& path);
-
 	void CreateLists(S3DModel* o);
-	void CreateListsNow(S3DModelPiece* o);
 
 private:
 	ModelMap cache;
@@ -65,7 +61,7 @@ private:
 	spring::mutex mutex;
 
 	// all unique models loaded so far
-	std::vector<S3DModel*> models;
+	std::deque<S3DModel> models;
 };
 
 #define modelLoader (CModelLoader::GetInstance())
