@@ -15,7 +15,7 @@ bool BuildingMaskMap::CheckBounds(unsigned int x, unsigned int z)
 }
 
 // sets mask value for tile[x,z] in 2*SQUARE_SIZE coordinates
-bool BuildingMaskMap::SetTileMask(unsigned int x, unsigned int z, boost::uint16_t value)
+bool BuildingMaskMap::SetTileMask(unsigned int x, unsigned int z, std::uint16_t value)
 {
 	if (!CheckBounds(x, z))
 		return false;
@@ -26,7 +26,7 @@ bool BuildingMaskMap::SetTileMask(unsigned int x, unsigned int z, boost::uint16_
 
 // tests previously set mask for tile[x,z] in 2*SQUARE_SIZE coordinates against supplied value
 // true - construction is allowed, false - it's not
-bool BuildingMaskMap::TestTileMaskUnsafe(unsigned int x, unsigned int z, boost::uint16_t value)
+bool BuildingMaskMap::TestTileMaskUnsafe(unsigned int x, unsigned int z, std::uint16_t value)
 {
 	assert(CheckBounds(x, z));
 	return (maskMap[x + z * mapDims.hmapx] & value) == value;
