@@ -14,25 +14,73 @@ class DamageArray
 	CR_DECLARE(DamageArray)
 
 public:
-	DamageArray(float damage = 1.0f);
-	DamageArray(const DamageArray& other) { *this = other; }
-	virtual ~DamageArray() { }
+	DamageArray(float damage = 1.0f)
+		: paralyzeDamageTime(0)
 
-	DamageArray operator * (float damageMult) const;
+		, impulseFactor(1.0f)
+		, impulseBoost(0.0f)
+
+		, craterMult(1.0f)
+		, craterBoost(0.0f)
+	{
+		SetDefaultDamage(damage);
+	}
 
 
+	DamageArray(const DamageArray& da) { *this = da; }
+	DamageArray(DamageArray&& da) { *this = std::move(da); }
 
+	virtual ~DamageArray() {}
+
+	DamageArray& operator = (const DamageArray& da) {
+		paralyzeDamageTime = da.paralyzeDamageTime;
+
+		impulseFactor = da.impulseFactor;
+		impulseBoost = da.impulseBoost;
+
+		craterMult = da.craterMult;
+		craterBoost = da.craterBoost;
+
+		damages = da.damages;
+		return *this;
+	}
+	DamageArray& operator = (DamageArray&& da) {
+		paralyzeDamageTime = da.paralyzeDamageTime;
+
+		impulseFactor = da.impulseFactor;
+		impulseBoost = da.impulseBoost;
+
+		craterMult = da.craterMult;
+		craterBoost = da.craterBoost;
+
+		damages = std::move(da.damages);
+		return *this;
+	}
+
+	DamageArray operator * (float damageMult) const {
+		DamageArray da(*this);
+
+		for (unsigned int a = 0; a < damages.size(); ++a)
+			da.damages[a] *= damageMult;
+
+		return da;
+	}
 
-	void SetDefaultDamage(float damage);
 
 	int GetNumTypes() const { return damages.size(); }
-	float Get(int typeIndex) const { return damages[typeIndex]; }
+
+	void SetDefaultDamage(float damage);
 	void Set(int typeIndex, float d) { damages[typeIndex] = d; }
+
+	float Get(int typeIndex) const { return damages[typeIndex]; }
 	float GetDefault() const { return damages[0]; }
 
+public:
 	int paralyzeDamageTime;
+
 	float impulseFactor;
 	float impulseBoost;
+
 	float craterMult;
 	float craterBoost;
 
@@ -40,13 +88,14 @@ protected:
 	std::vector<float> damages;
 };
 
+
 class DynDamageArray : public DamageArray
 {
 	CR_DECLARE(DynDamageArray)
 
 public:
 	DynDamageArray(float damage = 1.0f);
-	DynDamageArray(const DynDamageArray& other) { *this = other; refCount = 1; }
+	DynDamageArray(const DynDamageArray& da) { *this = da; fromDef = false; refCount = 1; }
 	~DynDamageArray();
 
 	void PostLoad();
