@@ -12,7 +12,7 @@ CR_BIND(CGeometricObjects::GeoGroup, )
 
 CR_REG_METADATA(CGeometricObjects, (
 	CR_MEMBER(geoGroups),
-	CR_MEMBER(toBeDeleted),
+	CR_MEMBER(timedGroups),
 	CR_MEMBER(firstFreeGroup)
 ))
 
@@ -21,15 +21,9 @@ CR_REG_METADATA_SUB(CGeometricObjects, GeoGroup, (CR_MEMBER(squares)))
 
 CGeometricObjects* geometricObjects;
 
-CGeometricObjects::CGeometricObjects()
-	: firstFreeGroup(1)
-{
-}
-
-
 CGeometricObjects::~CGeometricObjects()
 {
-	toBeDeleted.clear();
+	timedGroups.clear();
 	while (!geoGroups.empty()) {
 		DeleteGroup(geoGroups.begin()->first);
 	}
@@ -37,13 +31,13 @@ CGeometricObjects::~CGeometricObjects()
 
 int CGeometricObjects::AddSpline(float3 b1, float3 b2, float3 b3, float3 b4, float width, int arrow, int lifeTime, int group)
 {
-	if (group == 0) {
+	if (group == 0)
 		group = firstFreeGroup++;
-	}
 
 	float3 old1, old2;
 	old1 = CalcSpline(0.00f, b1, b2, b3, b4);
 	old2 = CalcSpline(0.05f, b1, b2, b3, b4);
+
 	for (int a = 0; a < 20; ++a) {
 		const float3 np = CalcSpline(a*0.05f + 0.1f, b1, b2, b3, b4);
 		const float3 dir1 = (old2 - old1).ANormalize();
@@ -57,14 +51,14 @@ int CGeometricObjects::AddSpline(float3 b1, float3 b2, float3 b3, float3 b4, flo
 			w1 = width * 0.5f;
 			w2 = w1;
 		}
-		CGeoSquareProjectile* gsp = new CGeoSquareProjectile(old1, old2, dir1, dir2, w1, w2);
+		CGeoSquareProjectile* gsp = projMemPool.alloc<CGeoSquareProjectile>(old1, old2, dir1, dir2, w1, w2);
 		geoGroups[group].squares.push_back(gsp);
 		old1 = old2;
 		old2 = np;
 	}
-	if (lifeTime > 0) {
-		toBeDeleted.insert(std::pair<int, int>(gs->frameNum + lifeTime, group));
-	}
+
+	if (lifeTime > 0)
+		timedGroups[gs->frameNum + lifeTime].push_back(group);
 
 	return group;
 }
@@ -74,9 +68,7 @@ void CGeometricObjects::DeleteGroup(int group)
 {
 	GeoGroup* gg = &geoGroups[group];
 
-	std::vector<CGeoSquareProjectile*>::iterator gi;
-
-	for (gi = gg->squares.begin(); gi != gg->squares.end(); ++gi) {
+	for (auto gi = gg->squares.begin(); gi != gg->squares.end(); ++gi) {
 		(*gi)->deleteMe = true;
 	}
 
@@ -88,9 +80,7 @@ void CGeometricObjects::SetColor(int group, float r, float g, float b, float a)
 {
 	GeoGroup* gg = &geoGroups[group];
 
-	std::vector<CGeoSquareProjectile*>::iterator gi;
-
-	for (gi = gg->squares.begin(); gi != gg->squares.end(); ++gi) {
+	for (auto gi = gg->squares.begin(); gi != gg->squares.end(); ++gi) {
 		(*gi)->SetColor(r, g, b, a);
 	}
 }
@@ -107,26 +97,24 @@ float3 CGeometricObjects::CalcSpline(float i, const float3& p1, const float3& p2
 
 int CGeometricObjects::AddLine(float3 start, float3 end, float width, int arrow, int lifetime, int group)
 {
-	if (group == 0) {
+	if (group == 0)
 		group = firstFreeGroup++;
-	}
 
 	float3 dir = (end - start).SafeANormalize();
 	if (arrow) {
-		CGeoSquareProjectile* gsp = new CGeoSquareProjectile(start, start*0.2f + end*0.8f, dir, dir, width*0.5f, width*0.5f);
+		CGeoSquareProjectile* gsp = projMemPool.alloc<CGeoSquareProjectile>(start, start*0.2f + end*0.8f, dir, dir, width*0.5f, width*0.5f);
 		geoGroups[group].squares.push_back(gsp);
 
-		gsp = new CGeoSquareProjectile(start*0.2f + end*0.8f, end, dir, dir, width, 0);
+		gsp = projMemPool.alloc<CGeoSquareProjectile>(start*0.2f + end*0.8f, end, dir, dir, width, 0);
 		geoGroups[group].squares.push_back(gsp);
 
 	} else {
-		CGeoSquareProjectile* gsp = new CGeoSquareProjectile(start, end, dir, dir, width*0.5f, width*0.5f);
+		CGeoSquareProjectile* gsp = projMemPool.alloc<CGeoSquareProjectile>(start, end, dir, dir, width*0.5f, width*0.5f);
 		geoGroups[group].squares.push_back(gsp);
 	}
 
-	if (lifetime > 0) {
-		toBeDeleted.insert(std::pair<int, int>(gs->frameNum + lifetime, group));
-	}
+	if (lifetime > 0)
+		timedGroups[gs->frameNum + lifetime].push_back(group);
 
 	return group;
 }
@@ -134,15 +122,21 @@ int CGeometricObjects::AddLine(float3 start, float3 end, float width, int arrow,
 
 void CGeometricObjects::Update()
 {
-	while(!toBeDeleted.empty() && (toBeDeleted.begin()->first <= gs->frameNum)) {
-		DeleteGroup(toBeDeleted.begin()->second);
-		toBeDeleted.erase(toBeDeleted.begin());
+	const auto iter = timedGroups.find(gs->frameNum);
+
+	if (iter == timedGroups.end())
+		return;
+
+	for (const int groupID: iter->second) {
+		DeleteGroup(groupID);
 	}
+
+	timedGroups.erase(iter->first);
 }
 
 
-void CGeometricObjects::MarkSquare(int mapSquare) {
-
+void CGeometricObjects::MarkSquare(int mapSquare)
+{
 	float3 startPos;
 	startPos.x = (int) (mapSquare * SQUARE_SIZE) % mapDims.mapx;
 	startPos.z = (int) (mapSquare * SQUARE_SIZE) / mapDims.mapx;
