@@ -3,15 +3,14 @@
 #ifndef GROUNDBLOCKINGOBJECTMAP_H
 #define GROUNDBLOCKINGOBJECTMAP_H
 
-#include <map>
-#include "System/creg/creg_cond.h"
+#include <vector>
 
 #include "Sim/Objects/SolidObject.h"
+#include "System/creg/creg_cond.h"
 #include "System/float3.h"
 
 
-typedef std::map<int, CSolidObject*> BlockingMapCell;
-typedef BlockingMapCell::const_iterator BlockingMapCellIt;
+typedef std::vector<CSolidObject*> BlockingMapCell;
 typedef std::vector<BlockingMapCell> BlockingMap;
 
 
@@ -25,6 +24,8 @@ public:
 		groundBlockingMap.resize(numSquares);
 	}
 
+	unsigned int CalcChecksum() const;
+
 	void AddGroundBlockingObject(CSolidObject* object);
 	void AddGroundBlockingObject(CSolidObject* object, const YardMapStatus& mask);
 	void RemoveGroundBlockingObject(CSolidObject* object);
@@ -34,22 +35,45 @@ public:
 	bool CanOpenYard(CSolidObject* object) const;
 	bool CanCloseYard(CSolidObject* object) const;
 
-	// these retrieve either the top-most or the bottom-most
-	// object in a given cell, or NULL if the cell is empty
+
+	// these retrieve either the first object in
+	// a given cell, or NULL if the cell is empty
 	CSolidObject* GroundBlocked(int x, int z) const;
 	CSolidObject* GroundBlocked(const float3& pos) const;
 
 	// same as GroundBlocked(), but does not bounds-check mapSquare
-	CSolidObject* GroundBlockedUnsafe(int mapSquare) const;
+	CSolidObject* GroundBlockedUnsafe(unsigned int mapSquare) const {
+		const BlockingMapCell& cell = GetCellUnsafeConst(mapSquare);
 
-	bool GroundBlocked(int x, int z, CSolidObject* ignoreObj) const;
-	bool GroundBlocked(const float3& pos, CSolidObject* ignoreObj) const;
+		if (cell.empty())
+			return nullptr;
 
-	// for full thread safety, access via GetCell would need to be mutexed, but it appears only sim thread uses it
-	const BlockingMapCell& GetCell(int mapSquare) const {
+		return (*cell.begin());
+	}
+
+
+	bool GroundBlocked(int x, int z, const CSolidObject* ignoreObj) const;
+	bool GroundBlocked(const float3& pos, const CSolidObject* ignoreObj) const;
+
+	bool ObjectInCell(unsigned int mapSquare, const CSolidObject* obj) const {
+		if (mapSquare >= groundBlockingMap.size())
+			return false;
+
+		const BlockingMapCell& cell = GetCellUnsafeConst(mapSquare);
+		const auto it = std::find(cell.cbegin(), cell.cend(), obj);
+		return (it != cell.cend());
+	}
+
+
+	const BlockingMapCell& GetCellUnsafeConst(unsigned int mapSquare) const {
+		assert(mapSquare < groundBlockingMap.size());
 		return groundBlockingMap[mapSquare];
 	}
 
+	BlockingMapCell& GetCellUnsafe(unsigned int mapSquare) {
+		return (const_cast<BlockingMapCell&>(GetCellUnsafeConst(mapSquare)));
+	}
+
 private:
 	bool CheckYard(CSolidObject* yardUnit, const YardMapStatus& mask) const;
 
