@@ -4,16 +4,15 @@
 #define INTERCEPT_HANDLER_H
 
 #include <deque>
-#include <boost/noncopyable.hpp>
+#include "System/Misc/NonCopyable.h"
 #include "System/Object.h"
 
 class CWeapon;
 class CWeaponProjectile;
-class CPlasmaRepulser;
 class CProjectile;
 class float3;
 
-class CInterceptHandler : public CObject, boost::noncopyable
+class CInterceptHandler : public CObject, spring::noncopyable
 {
 	CR_DECLARE(CInterceptHandler)
 
@@ -22,18 +21,13 @@ public:
 
 	void AddInterceptorWeapon(CWeapon* weapon);
 	void RemoveInterceptorWeapon(CWeapon* weapon);
-	void AddPlasmaRepulser(CPlasmaRepulser* shield);
-	void RemovePlasmaRepulser(CPlasmaRepulser* shield);
 
 	void AddInterceptTarget(CWeaponProjectile* target, const float3& destination);
-	void AddShieldInterceptableProjectile(CWeaponProjectile* p);
-	float AddShieldInterceptableBeam(CWeapon* emitter, const float3& start, const float3& dir, float length, float3& newDir, CPlasmaRepulser*& repulsedBy);
 
 	void DependentDied(CObject* o);
 
 private:
 	std::deque<CWeapon*> interceptors;
-	std::deque<CPlasmaRepulser*> repulsors;
 	std::deque<CWeaponProjectile*> interceptables;
 };
 
