@@ -16,15 +16,19 @@
 	#include "Sim/Weapons/PlasmaRepulser.h"
 #endif
 
-#include "System/Util.h"
-
 CR_BIND(CQuadField, (int2(1,1), 1))
 CR_REG_METADATA(CQuadField, (
 	CR_MEMBER(baseQuads),
 	CR_MEMBER(numQuadsX),
 	CR_MEMBER(numQuadsZ),
 	CR_MEMBER(quadSizeX),
-	CR_MEMBER(quadSizeZ)
+	CR_MEMBER(quadSizeZ),
+
+	CR_IGNORED(tempUnits),
+	CR_IGNORED(tempFeatures),
+	CR_IGNORED(tempProjectiles),
+	CR_IGNORED(tempSolids),
+	CR_IGNORED(tempQuads)
 ))
 
 CR_BIND(CQuadField::Quad, )
@@ -61,21 +65,21 @@ void CQuadField::Resize(int quad_size)
 			//   if a unit exists in multiple quads in the old field, it will
 			//   be removed from all of them and there is no danger of double
 			//   re-insertion (important if new grid has higher resolution)
-			const std::list<CUnit*      > units       = quad.units;
-			const std::list<CFeature*   > features    = quad.features;
-			const std::list<CProjectile*> projectiles = quad.projectiles;
+			const std::vector<CUnit*      > units       = quad.units;
+			const std::vector<CFeature*   > features    = quad.features;
+			const std::vector<CProjectile*> projectiles = quad.projectiles;
 
-			for (std::list<CUnit*>::const_iterator it = units.begin(); it != units.end(); ++it) {
+			for (auto it = units.cbegin(); it != units.cend(); ++it) {
 				oldQuadField->RemoveUnit(*it);
 				newQuadField->MovedUnit(*it); // handles addition
 			}
 
-			for (std::list<CFeature*>::const_iterator it = features.begin(); it != features.end(); ++it) {
+			for (auto it = features.cbegin(); it != features.cend(); ++it) {
 				oldQuadField->RemoveFeature(*it);
 				newQuadField->AddFeature(*it);
 			}
 
-			for (std::list<CProjectile*>::const_iterator it = projectiles.begin(); it != projectiles.end(); ++it) {
+			for (auto it = projectiles.cbegin(); it != projectiles.cend(); ++it) {
 				oldQuadField->RemoveProjectile(*it);
 				newQuadField->AddProjectile(*it);
 			}
@@ -103,7 +107,7 @@ void CQuadField::Quad::PostLoad()
 {
 #ifndef UNIT_TEST
 	for (CUnit* unit: units) {
-		VectorInsertUnique(teamUnits[unit->allyteam], unit, false);
+		spring::VectorInsertUnique(teamUnits[unit->allyteam], unit, false);
 	}
 #endif
 }
@@ -145,69 +149,67 @@ int CQuadField::WorldPosToQuadFieldIdx(const float3 p) const
 
 
 #ifndef UNIT_TEST
-std::vector<int> CQuadField::GetQuads(float3 pos, const float radius)
+void CQuadField::GetQuads(QuadFieldQuery& qfq, float3 pos, float radius)
 {
 	pos.AssertNaNs();
 	pos.ClampInBounds();
-	std::vector<int> ret;
+	qfq.quads = tempQuads.GetVector();
 
 	const int2 min = WorldPosToQuadField(pos - radius);
 	const int2 max = WorldPosToQuadField(pos + radius);
 
 	if (max.y < min.y || max.x < min.x)
-		return ret;
+		return;
 
 	// qsx and qsz are always equal
 	const float maxSqLength = (radius + quadSizeX * 0.72f) * (radius + quadSizeZ * 0.72f);
 
-	ret.reserve((max.y - min.y) * (max.x - min.x));
 	for (int z = min.y; z <= max.y; ++z) {
 		for (int x = min.x; x <= max.x; ++x) {
 			assert(x < numQuadsX);
 			assert(z < numQuadsZ);
 			const float3 quadPos = float3(x * quadSizeX + quadSizeX * 0.5f, 0, z * quadSizeZ + quadSizeZ * 0.5f);
 			if (pos.SqDistance2D(quadPos) < maxSqLength) {
-				ret.push_back(z * numQuadsX + x);
+				qfq.quads->push_back(z * numQuadsX + x);
 			}
 		}
 	}
 
-	return ret;
+	return;
 }
 
 
-std::vector<int> CQuadField::GetQuadsRectangle(const float3 mins, const float3 maxs)
+void CQuadField::GetQuadsRectangle(QuadFieldQuery& qfq, const float3& mins, const float3& maxs)
 {
 	mins.AssertNaNs();
 	maxs.AssertNaNs();
-	std::vector<int> ret;
+	qfq.quads = tempQuads.GetVector();
 
 	const int2 min = WorldPosToQuadField(mins);
 	const int2 max = WorldPosToQuadField(maxs);
 
 	if (max.y < min.y || max.x < min.x)
-		return ret;
+		return;
 
-	ret.reserve((max.y - min.y) * (max.x - min.x));
 	for (int z = min.y; z <= max.y; ++z) {
 		for (int x = min.x; x <= max.x; ++x) {
 			assert(x < numQuadsX);
 			assert(z < numQuadsZ);
-			ret.push_back(z * numQuadsX + x);
+			qfq.quads->push_back(z * numQuadsX + x);
 		}
 	}
 
-	return ret;
+	return;
 }
 #endif // UNIT_TEST
 
 
 /// note: this function got an UnitTest, check the tests/ folder!
-std::vector<int> CQuadField::GetQuadsOnRay(const float3 start, const float3 dir, const float length)
+void CQuadField::GetQuadsOnRay(QuadFieldQuery& qfq, const float3& start, const float3& dir, float length)
 {
 	dir.AssertNaNs();
 	start.AssertNaNs();
-	std::vector<int> ret;
+	qfq.quads = tempQuads.GetVector();
 
 	const float3 to = start + (dir * length);
 	const float3 invQuadSize = float3(1.0f / quadSizeX, 1.0f, 1.0f / quadSizeZ);
@@ -217,10 +219,9 @@ std::vector<int> CQuadField::GetQuadsOnRay(const float3 start, const float3 dir,
 
 	// often happened special case
 	if (noXdir && noZdir) {
-		ret.reserve(1);
-		ret.push_back(WorldPosToQuadFieldIdx(start));
-		assert(unsigned(ret.back()) < baseQuads.size());
-		return ret;
+		qfq.quads->push_back(WorldPosToQuadFieldIdx(start));
+		assert(unsigned(qfq.quads->back()) < baseQuads.size());
+		return;
 	}
 
 	// to prevent Div0
@@ -232,13 +233,12 @@ std::vector<int> CQuadField::GetQuadsOnRay(const float3 start, const float3 dir,
 
 		const int row = Clamp<int>(start.z * invQuadSize.z, 0, numQuadsZ - 1) * numQuadsX;
 
-		ret.reserve(finalX - startX + 1);
 		for (unsigned x = startX; x <= finalX; x++) {
-			ret.push_back(row + x);
-			assert(unsigned(ret.back()) < baseQuads.size());
+			qfq.quads->push_back(row + x);
+			assert(unsigned(qfq.quads->back()) < baseQuads.size());
 		}
 
-		return ret;
+		return;
 	}
 
 	// all other
@@ -251,7 +251,6 @@ std::vector<int> CQuadField::GetQuadsOnRay(const float3 start, const float3 dir,
 	assert(finalZ < quadSizeZ);
 	const float invDirZ = 1.0f / dir.z;
 
-	ret.reserve(std::max(numQuadsX, numQuadsZ));
 	for (int z = startZ; z <= finalZ; z++) {
 		float t0 = ((z    ) * quadSizeZ - start.z) * invDirZ;
 		float t1 = ((z + 1) * quadSizeZ - start.z) * invDirZ;
@@ -272,45 +271,46 @@ std::vector<int> CQuadField::GetQuadsOnRay(const float3 start, const float3 dir,
 		const int row = Clamp(z, 0, numQuadsZ - 1) * numQuadsX;
 
 		for (unsigned x = startX; x <= finalX; x++) {
-			ret.push_back(row + x);
-			assert(unsigned(ret.back()) < baseQuads.size());
+			qfq.quads->push_back(row + x);
+			assert(unsigned(qfq.quads->back()) < baseQuads.size());
 		}
 	}
 
-	return ret;
+	return;
 }
 
 
 #ifndef UNIT_TEST
 void CQuadField::MovedUnit(CUnit* unit)
 {
-	auto newQuads = std::move(GetQuads(unit->pos, unit->radius));
+	QuadFieldQuery qfQuery;
+	GetQuads(qfQuery, unit->pos, unit->radius);
 
 	// compare if the quads have changed, if not stop here
-	if (newQuads.size() == unit->quads.size()) {
-		if (std::equal(newQuads.begin(), newQuads.end(), unit->quads.begin())) {
+	if (qfQuery.quads->size() == unit->quads.size()) {
+		if (std::equal(qfQuery.quads->begin(), qfQuery.quads->end(), unit->quads.begin())) {
 			return;
 		}
 	}
 
 	for (const int qi: unit->quads) {
-		VectorErase(baseQuads[qi].units, unit);
-		VectorErase(baseQuads[qi].teamUnits[unit->allyteam], unit);
+		spring::VectorErase(baseQuads[qi].units, unit);
+		spring::VectorErase(baseQuads[qi].teamUnits[unit->allyteam], unit);
 	}
 
-	for (const int qi: newQuads) {
-		VectorInsertUnique(baseQuads[qi].units, unit, false);
-		VectorInsertUnique(baseQuads[qi].teamUnits[unit->allyteam], unit, false);
+	for (const int qi: *qfQuery.quads) {
+		spring::VectorInsertUnique(baseQuads[qi].units, unit, false);
+		spring::VectorInsertUnique(baseQuads[qi].teamUnits[unit->allyteam], unit, false);
 	}
 
-	unit->quads = std::move(newQuads);
+	unit->quads = std::move(*qfQuery.quads);
 }
 
 void CQuadField::RemoveUnit(CUnit* unit)
 {
 	for (const int qi: unit->quads) {
-		VectorErase(baseQuads[qi].units, unit);
-		VectorErase(baseQuads[qi].teamUnits[unit->allyteam], unit);
+		spring::VectorErase(baseQuads[qi].units, unit);
+		spring::VectorErase(baseQuads[qi].teamUnits[unit->allyteam], unit);
 	}
 
 	unit->quads.clear();
@@ -329,32 +329,31 @@ void CQuadField::RemoveUnit(CUnit* unit)
 
 void CQuadField::MovedRepulser(CPlasmaRepulser* repulser)
 {
-	auto newQuads = std::move(GetQuads(repulser->weaponMuzzlePos, repulser->GetRadius()));
+	QuadFieldQuery qfQuery;
+	GetQuads(qfQuery, repulser->weaponMuzzlePos, repulser->GetRadius());
 
 	// compare if the quads have changed, if not stop here
-	if (newQuads.size() == repulser->quads.size()) {
-		if (std::equal(newQuads.begin(), newQuads.end(), repulser->quads.begin())) {
+	if (qfQuery.quads->size() == repulser->quads.size()) {
+		if (std::equal(qfQuery.quads->begin(), qfQuery.quads->end(), repulser->quads.begin())) {
 			return;
 		}
 	}
 
 	for (const int qi: repulser->quads) {
-		VectorErase(baseQuads[qi].repulsers, repulser);
+		spring::VectorErase(baseQuads[qi].repulsers, repulser);
 	}
 
-	for (const int qi: newQuads) {
-		VectorInsertUnique(baseQuads[qi].repulsers, repulser, false);
+	for (const int qi: *qfQuery.quads) {
+		spring::VectorInsertUnique(baseQuads[qi].repulsers, repulser, false);
 	}
 
-	repulser->quads = std::move(newQuads);
+	repulser->quads = std::move(*qfQuery.quads);
 }
 
 void CQuadField::RemoveRepulser(CPlasmaRepulser* repulser)
 {
-	const auto& quads = GetQuads(repulser->weaponMuzzlePos, repulser->GetRadius());
-
 	for (const int qi: repulser->quads) {
-		VectorErase(baseQuads[qi].repulsers, repulser);
+		spring::VectorErase(baseQuads[qi].repulsers, repulser);
 	}
 
 	repulser->quads.clear();
@@ -371,19 +370,21 @@ void CQuadField::RemoveRepulser(CPlasmaRepulser* repulser)
 
 void CQuadField::AddFeature(CFeature* feature)
 {
-	const auto& newQuads = GetQuads(feature->pos, feature->radius);
+	QuadFieldQuery qfQuery;
+	GetQuads(qfQuery, feature->pos, feature->radius);
 
-	for (const int qi: newQuads) {
-		VectorInsertUnique(baseQuads[qi].features, feature, false);
+	for (const int qi: *qfQuery.quads) {
+		spring::VectorInsertUnique(baseQuads[qi].features, feature, false);
 	}
 }
 
 void CQuadField::RemoveFeature(CFeature* feature)
 {
-	const auto& quads = GetQuads(feature->pos, feature->radius);
+	QuadFieldQuery qfQuery;
+	GetQuads(qfQuery, feature->pos, feature->radius);
 
-	for (const int qi: quads) {
-		VectorErase(baseQuads[qi].features, feature);
+	for (const int qi: *qfQuery.quads) {
+		spring::VectorErase(baseQuads[qi].features, feature);
 	}
 
 	#ifdef DEBUG_QUADFIELD
@@ -417,16 +418,17 @@ void CQuadField::AddProjectile(CProjectile* p)
 	assert(p->synced);
 
 	if (p->hitscan) {
-		auto newQuads = std::move(GetQuadsOnRay(p->pos, p->dir, p->speed.w));
+		QuadFieldQuery qfQuery;
+		GetQuadsOnRay(qfQuery, p->pos, p->dir, p->speed.w);
 
-		for (const int qi: newQuads) {
-			VectorInsertUnique(baseQuads[qi].projectiles, p, false);
+		for (const int qi: *qfQuery.quads) {
+			spring::VectorInsertUnique(baseQuads[qi].projectiles, p, false);
 		}
 
-		p->quads = std::move(newQuads);
+		p->quads = std::move(*qfQuery.quads);
 	} else {
 		int newQuad = WorldPosToQuadFieldIdx(p->pos);
-		VectorInsertUnique(baseQuads[newQuad].projectiles, p, false);
+		spring::VectorInsertUnique(baseQuads[newQuad].projectiles, p, false);
 		p->quads.clear();
 		p->quads.push_back(newQuad);
 	}
@@ -437,7 +439,7 @@ void CQuadField::RemoveProjectile(CProjectile* p)
 	assert(p->synced);
 
 	for (const int qi: p->quads) {
-		VectorErase(baseQuads[qi].projectiles, p);
+		spring::VectorErase(baseQuads[qi].projectiles, p);
 	}
 
 	p->quads.clear();
@@ -447,36 +449,40 @@ void CQuadField::RemoveProjectile(CProjectile* p)
 
 
 
-std::vector<CUnit*> CQuadField::GetUnits(const float3& pos, float radius)
+void CQuadField::GetUnits(QuadFieldQuery& qfq, const float3& pos, float radius)
 {
-	const auto& quads = GetQuads(pos, radius);
+	QuadFieldQuery qfQuery;
+	GetQuads(qfQuery, pos, radius);
 	const int tempNum = gs->GetTempNum();
-	std::vector<CUnit*> units;
+	qfq.units = tempUnits.GetVector();
 
-	for (const int qi: quads) {
+	for (const int qi: *qfQuery.quads) {
 		for (CUnit* u: baseQuads[qi].units) {
 			if (u->tempNum == tempNum)
 				continue;
 
 			u->tempNum = tempNum;
-			units.push_back(u);
+			qfq.units->push_back(u);
 		}
 	}
 
-	return units;
+	return;
 }
 
-std::vector<CUnit*> CQuadField::GetUnitsExact(const float3& pos, float radius, bool spherical)
+void CQuadField::GetUnitsExact(QuadFieldQuery& qfq, const float3& pos, float radius, bool spherical)
 {
-	const auto& quads = GetQuads(pos, radius);
+	QuadFieldQuery qfQuery;
+	GetQuads(qfQuery, pos, radius);
 	const int tempNum = gs->GetTempNum();
-	std::vector<CUnit*> units;
+	qfq.units = tempUnits.GetVector();
 
-	for (const int qi: quads) {
+	for (const int qi: *qfQuery.quads) {
 		for (CUnit* u: baseQuads[qi].units) {
 			if (u->tempNum == tempNum)
 				continue;
 
+			u->tempNum = tempNum;
+
 			const float totRad       = radius + u->radius;
 			const float totRadSq     = totRad * totRad;
 			const float posUnitDstSq = spherical?
@@ -486,48 +492,56 @@ std::vector<CUnit*> CQuadField::GetUnitsExact(const float3& pos, float radius, b
 			if (posUnitDstSq >= totRadSq)
 				continue;
 
-			u->tempNum = tempNum;
-			units.push_back(u);
+			qfq.units->push_back(u);
 		}
 	}
 
-	return units;
+	return;
 }
 
-std::vector<CUnit*> CQuadField::GetUnitsExact(const float3& mins, const float3& maxs)
+void CQuadField::GetUnitsExact(QuadFieldQuery& qfq, const float3& mins, const float3& maxs)
 {
-	const auto& quads = GetQuadsRectangle(mins, maxs);
+	QuadFieldQuery qfQuery;
+	GetQuadsRectangle(qfQuery, mins, maxs);
 	const int tempNum = gs->GetTempNum();
-	std::vector<CUnit*> units;
+	qfq.units = tempUnits.GetVector();
 
-	for (const int qi: quads) {
+	for (const int qi: *qfQuery.quads) {
 		for (CUnit* unit: baseQuads[qi].units) {
-			const float3& pos = unit->pos;
 
-			if (unit->tempNum == tempNum) { continue; }
-			if (pos.x < mins.x || pos.x > maxs.x) { continue; }
-			if (pos.z < mins.z || pos.z > maxs.z) { continue; }
+			if (unit->tempNum == tempNum)
+				continue;
 
 			unit->tempNum = tempNum;
-			units.push_back(unit);
+
+			const float3& pos = unit->pos;
+			if (pos.x < mins.x || pos.x > maxs.x)
+				continue;
+			if (pos.z < mins.z || pos.z > maxs.z)
+				continue;
+
+			qfq.units->push_back(unit);
 		}
 	}
 
-	return units;
+	return;
 }
 
 
-std::vector<CFeature*> CQuadField::GetFeaturesExact(const float3& pos, float radius, bool spherical)
+void CQuadField::GetFeaturesExact(QuadFieldQuery& qfq, const float3& pos, float radius, bool spherical)
 {
-	const auto& quads = GetQuads(pos, radius);
+	QuadFieldQuery qfQuery;
+	GetQuads(qfQuery, pos, radius);
 	const int tempNum = gs->GetTempNum();
-	std::vector<CFeature*> features;
+	qfq.features = tempFeatures.GetVector();
 
-	for (const int qi: quads) {
+	for (const int qi: *qfQuery.quads) {
 		for (CFeature* f: baseQuads[qi].features) {
 			if (f->tempNum == tempNum)
 				continue;
 
+			f->tempNum = tempNum;
+
 			const float totRad       = radius + f->radius;
 			const float totRadSq     = totRad * totRad;
 			const float posDstSq = spherical?
@@ -537,91 +551,114 @@ std::vector<CFeature*> CQuadField::GetFeaturesExact(const float3& pos, float rad
 			if (posDstSq >= totRadSq)
 				continue;
 
-			f->tempNum = tempNum;
-			features.push_back(f);
+			qfq.features->push_back(f);
 		}
 	}
 
-	return features;
+	return;
 }
 
-std::vector<CFeature*> CQuadField::GetFeaturesExact(const float3& mins, const float3& maxs)
+void CQuadField::GetFeaturesExact(QuadFieldQuery& qfq, const float3& mins, const float3& maxs)
 {
-	const auto& quads = GetQuadsRectangle(mins, maxs);
+	QuadFieldQuery qfQuery;
+	GetQuadsRectangle(qfQuery, mins, maxs);
 	const int tempNum = gs->GetTempNum();
-	std::vector<CFeature*> features;
+	qfq.features = tempFeatures.GetVector();
 
-	for (const int qi: quads) {
+	for (const int qi: *qfQuery.quads) {
 		for (CFeature* feature: baseQuads[qi].features) {
-			const float3& pos = feature->pos;
-
-			if (feature->tempNum == tempNum) { continue; }
-			if (pos.x < mins.x || pos.x > maxs.x) { continue; }
-			if (pos.z < mins.z || pos.z > maxs.z) { continue; }
+			if (feature->tempNum == tempNum)
+				continue;
 
 			feature->tempNum = tempNum;
-			features.push_back(feature);
-		}
-	}
 
-	return features;
-}
-
-
-
-std::vector<CProjectile*> CQuadField::GetProjectilesExact(const float3& pos, float radius)
-{
-	const auto& quads = GetQuads(pos, radius);
-	std::vector<CProjectile*> projectiles;
-
-	for (const int qi: quads) {
-		for (CProjectile* p: baseQuads[qi].projectiles) {
-			if (pos.SqDistance(p->pos) >= Square(radius + p->radius)) {
+			const float3& pos = feature->pos;
+			if (pos.x < mins.x || pos.x > maxs.x)
+				continue;
+			if (pos.z < mins.z || pos.z > maxs.z)
 				continue;
-			}
 
-			projectiles.push_back(p);
+			qfq.features->push_back(feature);
 		}
 	}
 
-	return projectiles;
+	return;
 }
 
-std::vector<CProjectile*> CQuadField::GetProjectilesExact(const float3& mins, const float3& maxs)
+
+
+void CQuadField::GetProjectilesExact(QuadFieldQuery& qfq, const float3& pos, float radius)
 {
-	const auto& quads = GetQuadsRectangle(mins, maxs);
-	std::vector<CProjectile*> projectiles;
+	QuadFieldQuery qfQuery;
+	GetQuads(qfQuery, pos, radius);
+	const int tempNum = gs->GetTempNum();
+	qfq.projectiles = tempProjectiles.GetVector();
 
-	for (const int qi: quads) {
-		for (CProjectile* projectile: baseQuads[qi].projectiles) {
-			const float3& pos = projectile->pos;
+	for (const int qi: *qfQuery.quads) {
+		for (CProjectile* p: baseQuads[qi].projectiles) {
+			if (p->tempNum == tempNum)
+				continue;
 
-			if (pos.x < mins.x || pos.x > maxs.x) { continue; }
-			if (pos.z < mins.z || pos.z > maxs.z) { continue; }
+			p->tempNum = tempNum;
 
-			projectiles.push_back(projectile);
+			if (pos.SqDistance(p->pos) >= Square(radius + p->radius))
+				continue;
+
+			qfq.projectiles->push_back(p);
 		}
 	}
 
-	return projectiles;
+	return;
+}
+
+void CQuadField::GetProjectilesExact(QuadFieldQuery& qfq, const float3& mins, const float3& maxs)
+{
+	QuadFieldQuery qfQuery;
+	GetQuadsRectangle(qfQuery, mins, maxs);
+	const int tempNum = gs->GetTempNum();
+	qfq.projectiles = tempProjectiles.GetVector();
+
+	for (const int qi: *qfQuery.quads) {
+		for (CProjectile* p: baseQuads[qi].projectiles) {
+			if (p->tempNum == tempNum)
+				continue;
+
+			p->tempNum = tempNum;
+
+			const float3& pos = p->pos;
+			if (pos.x < mins.x || pos.x > maxs.x)
+				continue;
+			if (pos.z < mins.z || pos.z > maxs.z)
+				continue;
+
+			qfq.projectiles->push_back(p);
+		}
+	}
+
+	return;
 }
 
 
 
-std::vector<CSolidObject*> CQuadField::GetSolidsExact(
+void CQuadField::GetSolidsExact(
+	QuadFieldQuery& qfq,
 	const float3& pos,
 	const float radius,
 	const unsigned int physicalStateBits,
 	const unsigned int collisionStateBits
 ) {
-	const auto& quads = GetQuads(pos, radius);
+	QuadFieldQuery qfQuery;
+	GetQuads(qfQuery, pos, radius);
 	const int tempNum = gs->GetTempNum();
-	std::vector<CSolidObject*> solids;
+	qfq.solids = tempSolids.GetVector();
 
-	for (const int qi: quads) {
+	for (const int qi: *qfQuery.quads) {
 		for (CUnit* u: baseQuads[qi].units) {
 			if (u->tempNum == tempNum)
 				continue;
+
+			u->tempNum = tempNum;
+
 			if (!u->HasPhysicalStateBit(physicalStateBits))
 				continue;
 			if (!u->HasCollidableStateBit(collisionStateBits))
@@ -629,13 +666,15 @@ std::vector<CSolidObject*> CQuadField::GetSolidsExact(
 			if ((pos - u->pos).SqLength() >= Square(radius + u->radius))
 				continue;
 
-			u->tempNum = tempNum;
-			solids.push_back(u);
+			qfq.solids->push_back(u);
 		}
 
 		for (CFeature* f: baseQuads[qi].features) {
 			if (f->tempNum == tempNum)
 				continue;
+
+			f->tempNum = tempNum;
+
 			if (!f->HasPhysicalStateBit(physicalStateBits))
 				continue;
 			if (!f->HasCollidableStateBit(collisionStateBits))
@@ -643,12 +682,59 @@ std::vector<CSolidObject*> CQuadField::GetSolidsExact(
 			if ((pos - f->pos).SqLength() >= Square(radius + f->radius))
 				continue;
 
-			f->tempNum = tempNum;
-			solids.push_back(f);
+			qfq.solids->push_back(f);
 		}
 	}
 
-	return solids;
+	return;
+}
+
+
+bool CQuadField::NoSolidsExact(
+	const float3& pos,
+	const float radius,
+	const unsigned int physicalStateBits,
+	const unsigned int collisionStateBits
+) {
+	QuadFieldQuery qfQuery;
+	GetQuads(qfQuery, pos, radius);
+	const int tempNum = gs->GetTempNum();
+
+	for (const int qi: *qfQuery.quads) {
+		for (CUnit* u: baseQuads[qi].units) {
+			if (u->tempNum == tempNum)
+				continue;
+
+			u->tempNum = tempNum;
+
+			if (!u->HasPhysicalStateBit(physicalStateBits))
+				continue;
+			if (!u->HasCollidableStateBit(collisionStateBits))
+				continue;
+			if ((pos - u->pos).SqLength() >= Square(radius + u->radius))
+				continue;
+
+			return false;
+		}
+
+		for (CFeature* f: baseQuads[qi].features) {
+			if (f->tempNum == tempNum)
+				continue;
+
+			f->tempNum = tempNum;
+
+			if (!f->HasPhysicalStateBit(physicalStateBits))
+				continue;
+			if (!f->HasCollidableStateBit(collisionStateBits))
+				continue;
+			if ((pos - f->pos).SqLength() >= Square(radius + f->radius))
+				continue;
+
+			return false;
+		}
+	}
+
+	return true;
 }
 
 
@@ -662,10 +748,11 @@ void CQuadField::GetUnitsAndFeaturesColVol(
 ) {
 	const int tempNum = gs->GetTempNum();
 
+	QuadFieldQuery qfQuery;
+	GetQuads(qfQuery, pos, radius);
 	// start counting from the previous object-cache sizes
-	const auto& quads = GetQuads(pos, radius);
 
-	for (const int qi: quads) {
+	for (const int qi: *qfQuery.quads) {
 		const Quad& quad = baseQuads[qi];
 
 		for (CUnit* u: quad.units) {
@@ -673,13 +760,14 @@ void CQuadField::GetUnitsAndFeaturesColVol(
 			if (u->tempNum == tempNum)
 				continue;
 
+			u->tempNum = tempNum;
+
 			const auto* colvol = &u->collisionVolume;
 			const float totRad = radius + colvol->GetBoundingRadius();
 
 			if (pos.SqDistance(colvol->GetWorldSpacePos(u)) >= (totRad * totRad))
 				continue;
 
-			u->tempNum = tempNum;
 			units.push_back(u);
 		}
 
@@ -688,13 +776,14 @@ void CQuadField::GetUnitsAndFeaturesColVol(
 			if (f->tempNum == tempNum)
 				continue;
 
+			f->tempNum = tempNum;
+
 			const auto* colvol = &f->collisionVolume;
 			const float totRad = radius + colvol->GetBoundingRadius();
 
 			if (pos.SqDistance(colvol->GetWorldSpacePos(f)) >= (totRad * totRad))
 				continue;
 
-			f->tempNum = tempNum;
 			features.push_back(f);
 		}
 		if (repulsers != nullptr) {
@@ -703,13 +792,14 @@ void CQuadField::GetUnitsAndFeaturesColVol(
 				if (r->tempNum == tempNum)
 					continue;
 
+				r->tempNum = tempNum;
+
 				const auto* colvol = &r->collisionVolume;
 				const float totRad = radius + colvol->GetBoundingRadius();
 
 				if (pos.SqDistance(r->weaponMuzzlePos) >= (totRad * totRad))
 					continue;
 
-				r->tempNum = tempNum;
 				repulsers->push_back(r);
 			}
 		}
