@@ -3,20 +3,20 @@
 #ifndef WIND_H
 #define WIND_H
 
-#include <map>
-#include <boost/noncopyable.hpp>
+#include <vector>
 
+#include "System/Misc/NonCopyable.h"
 #include "System/float3.h"
 
 class CUnit;
 
-class CWind : public boost::noncopyable
+class CWind : public spring::noncopyable
 {
 	CR_DECLARE_STRUCT(CWind)
 
 public:
-	CWind();
-	~CWind();
+	CWind() { ResetState(); }
+	~CWind() { windGenIDs.clear(); }
 
 	void ResetState();
 	void LoadWind(float min, float max);
@@ -44,7 +44,7 @@ private:
 
 	int status;
 
-	std::map<int, CUnit*> windGens;
+	std::vector<int> windGenIDs;
 };
 
 extern CWind wind;
