@@ -1,6 +1,7 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
 #include "MoveMath.h"
+#include "Sim/Misc/ModInfo.h"
 #include "Sim/MoveTypes/MoveDefHandler.h"
 
 /*
@@ -20,18 +21,17 @@ float CMoveMath::HoverSpeedMod(const MoveDef& moveDef, float height, float slope
 
 float CMoveMath::HoverSpeedMod(const MoveDef& moveDef, float height, float slope, float dirSlopeMod)
 {
+	if (!modInfo.allowDirectionalPathing) {
+		return HoverSpeedMod(moveDef, height, slope);
+	}
+
+	// Only difference direction can have is making hills climbing slower.
+
 	// no speed-penalty if on water
 	if (height < 0.0f)
-		return 1.0f;
+		return (1.0f * !noHoverWaterMove);
 
-	if (slope > (moveDef.maxSlope * 2.0f))
-		return 0.0f;
-
-	// too steep downhill slope?
-	if (dirSlopeMod <= 0.0f && (slope * dirSlopeMod) < (-moveDef.maxSlope * 2.0f))
-		return 0.0f;
-	// too steep uphill slope?
-	if (dirSlopeMod  > 0.0f && (slope * dirSlopeMod) > ( moveDef.maxSlope       ))
+	if (slope > moveDef.maxSlope)
 		return 0.0f;
 
 	return (1.0f / (1.0f + std::max(0.0f, slope * dirSlopeMod) * moveDef.slopeMod));
