@@ -63,43 +63,82 @@ SolidObjectDef::SolidObjectDef()
 
 void SolidObjectDef::PreloadModel() const
 {
-	if (model == nullptr && !modelName.empty()) {
-		modelLoader.PreloadModel(modelName);
-	}
+	if (model != nullptr)
+		return;
+	if (modelName.empty())
+		return;
+
+	modelLoader.PreloadModel(modelName);
 }
 
 S3DModel* SolidObjectDef::LoadModel() const
 {
-	if (model == nullptr) {
-		if (!modelName.empty()) {
-			model = modelLoader.LoadModel(modelName);
-		} else {
-			// not useful, too much spam
-			// LOG_L(L_WARNING, "[SolidObjectDef::%s] object \"%s\" has no model defined", __FUNCTION__, name.c_str());
-		}
-	}
+	if (model != nullptr)
+		return model;
+	if (modelName.empty())
+		return nullptr;
 
-	return model;
+	return (model = modelLoader.LoadModel(modelName));
 }
 
 float SolidObjectDef::GetModelRadius() const
 {
-	return (LoadModel()->radius);
+	return ((LoadModel() != nullptr)? model->GetDrawRadius(): 0.0f);
 }
 
-void SolidObjectDef::ParseCollisionVolume(const LuaTable& table)
+
+void SolidObjectDef::ParseCollisionVolume(const LuaTable& odTable)
 {
-	collisionVolume = CollisionVolume(
-		table.GetString("collisionVolumeType", ""),
-		table.GetFloat3("collisionVolumeScales", ZeroVector),
-		table.GetFloat3("collisionVolumeOffsets", ZeroVector)
-	);
+	const LuaTable& cvTable = odTable.SubTable("collisionVolume");
+	const std::string& cvType = odTable.GetString("collisionVolumeType", "");
+
+	if (cvTable.IsValid()) {
+		collisionVolume = CollisionVolume(
+			cvTable.GetInt("type", 's'),
+			cvTable.GetInt("axis", 'z'),
+			cvTable.GetFloat3("scales" , ZeroVector),
+			cvTable.GetFloat3("offsets", ZeroVector)
+		);
+	} else {
+		collisionVolume = CollisionVolume(
+			((cvType.empty())? 's': cvType.front()),
+			((cvType.empty())? 'z': cvType.back ()),
+			odTable.GetFloat3("collisionVolumeScales" , ZeroVector),
+			odTable.GetFloat3("collisionVolumeOffsets", ZeroVector)
+		);
+	}
 
 	// if this unit wants per-piece volumes, make
 	// its main collision volume deferent and let
 	// it ignore hits
-	collisionVolume.SetDefaultToPieceTree(table.GetBool("usePieceCollisionVolumes", false));
-	collisionVolume.SetDefaultToFootPrint(table.GetBool("useFootPrintCollisionVolume", false));
+	collisionVolume.SetDefaultToPieceTree(odTable.GetBool("usePieceCollisionVolumes", false));
+	collisionVolume.SetDefaultToFootPrint(odTable.GetBool("useFootPrintCollisionVolume", false));
 	collisionVolume.SetIgnoreHits(collisionVolume.DefaultToPieceTree());
 }
 
+void SolidObjectDef::ParseSelectionVolume(const LuaTable& odTable)
+{
+	const LuaTable& svTable = odTable.SubTable("selectionVolume");
+	const std::string& svType = odTable.GetString("selectionVolumeType", odTable.GetString("collisionVolumeType", ""));
+
+	if (svTable.IsValid()) {
+		selectionVolume = CollisionVolume(
+			svTable.GetInt("type", 's'),
+			svTable.GetInt("axis", 'z'),
+			svTable.GetFloat3("scales" , ZeroVector),
+			svTable.GetFloat3("offsets", ZeroVector)
+		);
+	} else {
+		selectionVolume = CollisionVolume(
+			((svType.empty())? 's': svType.front()),
+			((svType.empty())? 'z': svType.back ()),
+			odTable.GetFloat3("selectionVolumeScales" , odTable.GetFloat3("collisionVolumeScales" , ZeroVector)),
+			odTable.GetFloat3("selectionVolumeOffsets", odTable.GetFloat3("collisionVolumeOffsets", ZeroVector))
+		);
+	}
+
+	selectionVolume.SetDefaultToPieceTree(odTable.GetBool("usePieceSelectionVolumes", false));
+	selectionVolume.SetDefaultToFootPrint(odTable.GetBool("useFootPrintSelectionVolume", false));
+	selectionVolume.SetIgnoreHits(selectionVolume.DefaultToPieceTree());
+}
+
