@@ -6,11 +6,8 @@
 #include "System/Object.h"
 #include "System/float4.h"
 
-#define WORLDOBJECT_DEFAULT_DRAWRADIUS 30.0f
-
 struct S3DModel;
 
-
 class CWorldObject: public CObject
 {
 public:
@@ -28,8 +25,7 @@ public:
 		, alwaysVisible(false)
 		, model(NULL)
 	{}
-	CWorldObject(const float3& pos, const float3& spd)
-		: CWorldObject()
+	CWorldObject(const float3& pos, const float3& spd): CWorldObject()
 	{
 		SetPosition(pos);
 		SetVelocity(spd);
@@ -37,12 +33,16 @@ public:
 
 	virtual ~CWorldObject() {}
 
+	// NOTE: used only by projectiles, SolidObject's override this!
+	virtual float GetDrawRadius() const { return drawRadius; }
+
 	virtual void SetPosition(const float3& p) {   pos = p; }
 	virtual void SetVelocity(const float3& v) { speed = v; }
 
 	virtual void SetVelocityAndSpeed(const float3& v) {
-		SetSpeed(v);
+		// set velocity first; do not assume f4::op=(f3) will not touch .w
 		SetVelocity(v);
+		SetSpeed(v);
 	}
 
 	// by default, SetVelocity does not set magnitude (for efficiency)
@@ -56,7 +56,11 @@ public:
 		drawRadius = r;
 	}
 
-	void SetRadiusAndHeight(S3DModel* model);
+	void SetRadiusAndHeight(const S3DModel* model);
+
+	// extrapolated base-positions; used in unsynced code
+	float3 GetDrawPos(                float t) const { return (pos + speed * t); }
+	float3 GetDrawPos(const float3 v, float t) const { return (pos +     v * t); }
 
 public:
 	int id;
@@ -67,7 +71,7 @@ public:
 	float radius;       ///< used for collisions
 	float height;       ///< The height of this object
 	float sqRadius;
-	float drawRadius;   ///< unsynced, used to see if in view etc.
+	float drawRadius;   ///< unsynced, used for projectile visibility culling
 
 	bool useAirLos;     ///< if true, the object's visibility is checked against airLosMap[allyteam]
 	bool alwaysVisible; ///< if true, object is drawn even if not in LOS
