@@ -7,54 +7,57 @@
 #include <array>
 #include "Sim/Misc/GlobalConstants.h"
 
-static const float PATHCOST_INFINITY = std::numeric_limits<float>::infinity();
+static constexpr float PATHCOST_INFINITY = std::numeric_limits<float>::infinity();
 
 // NOTE:
 //   PF and PE both use a PathNodeBuffer of size MAX_SEARCHED_NODES,
 //   thus MAX_SEARCHED_NODES_{PF, PE} MUST be <= MAX_SEARCHED_NODES
-static const unsigned int MAX_SEARCHED_NODES    = 65536U;
-static const unsigned int MAX_SEARCHED_NODES_PF = MAX_SEARCHED_NODES;
-static const unsigned int MAX_SEARCHED_NODES_PE = MAX_SEARCHED_NODES;
+static constexpr unsigned int MAX_SEARCHED_NODES    = 65536U;
+static constexpr unsigned int MAX_SEARCHED_NODES_PF = MAX_SEARCHED_NODES;
+static constexpr unsigned int MAX_SEARCHED_NODES_PE = MAX_SEARCHED_NODES;
 
 // PathManager distance thresholds (to use PF or PE)
-static const float MAXRES_SEARCH_DISTANCE =  50.0f;
-static const float MEDRES_SEARCH_DISTANCE = 100.0f;
+static constexpr float MAXRES_SEARCH_DISTANCE =  50.0f;
+static constexpr float MEDRES_SEARCH_DISTANCE = 100.0f;
 // path-refinement lookahead distances (MED to MAX and LOW to MED)
 static const float MAXRES_SEARCH_DISTANCE_EXT = (MAXRES_SEARCH_DISTANCE * 0.4f) * SQUARE_SIZE;
 static const float MEDRES_SEARCH_DISTANCE_EXT = (MEDRES_SEARCH_DISTANCE * 0.4f) * SQUARE_SIZE;
 
 // how many recursive refinement attempts NextWayPoint should make
-static const unsigned int MAX_PATH_REFINEMENT_DEPTH = 4;
+static constexpr unsigned int MAX_PATH_REFINEMENT_DEPTH = 4;
 
-static const unsigned int PATHESTIMATOR_VERSION = 74;
+static constexpr unsigned int PATHESTIMATOR_VERSION = 88;
 
-static const unsigned int MEDRES_PE_BLOCKSIZE =  8;
-static const unsigned int LOWRES_PE_BLOCKSIZE = 32;
+static constexpr unsigned int MEDRES_PE_BLOCKSIZE = 16;
+static constexpr unsigned int LOWRES_PE_BLOCKSIZE = 32;
 
-static const unsigned int SQUARES_TO_UPDATE = 1000;
-static const unsigned int MAX_SEARCHED_NODES_ON_REFINE = 2000;
+static constexpr unsigned int SQUARES_TO_UPDATE = 1000;
+static constexpr unsigned int MAX_SEARCHED_NODES_ON_REFINE = 2000;
 
-static const unsigned int PATH_HEATMAP_XSCALE =  1; // wrt. mapDims.hmapx
-static const unsigned int PATH_HEATMAP_ZSCALE =  1; // wrt. mapDims.hmapy
-static const unsigned int PATH_FLOWMAP_XSCALE = 32; // wrt. mapDims.mapx
-static const unsigned int PATH_FLOWMAP_ZSCALE = 32; // wrt. mapDims.mapy
+static constexpr unsigned int PATH_HEATMAP_XSCALE =  1; // wrt. mapDims.hmapx
+static constexpr unsigned int PATH_HEATMAP_ZSCALE =  1; // wrt. mapDims.hmapy
+static constexpr unsigned int PATH_FLOWMAP_XSCALE = 32; // wrt. mapDims.mapx
+static constexpr unsigned int PATH_FLOWMAP_ZSCALE = 32; // wrt. mapDims.mapy
 
 
 // PE-only flags (indices)
 enum {
-	PATHDIR_LEFT       = 0, // +x
+	PATHDIR_LEFT       = 0, // +x (LEFT *TO* RIGHT)
 	PATHDIR_LEFT_UP    = 1, // +x+z
-	PATHDIR_UP         = 2, // +z
+	PATHDIR_UP         = 2, // +z (UP *TO* DOWN)
 	PATHDIR_RIGHT_UP   = 3, // -x+z
-	PATHDIR_RIGHT      = 4, // -x
+
+	PATHDIR_RIGHT      = 4, // -x (RIGHT *TO* LEFT)
 	PATHDIR_RIGHT_DOWN = 5, // -x-z
-	PATHDIR_DOWN       = 6, // -z
+	PATHDIR_DOWN       = 6, // -z (DOWN *TO* UP)
 	PATHDIR_LEFT_DOWN  = 7, // +x-z
+
 	PATH_DIRECTIONS    = 8,
 };
-static const unsigned int PATHDIR_CARDINALS[4] = {PATHDIR_LEFT, PATHDIR_RIGHT, PATHDIR_UP, PATHDIR_DOWN};
-static const unsigned int PATH_DIRECTION_VERTICES = PATH_DIRECTIONS >> 1;
-static const unsigned int PATH_NODE_SPACING = 2;
+
+static constexpr unsigned int PATHDIR_CARDINALS[4] = {PATHDIR_LEFT, PATHDIR_RIGHT, PATHDIR_UP, PATHDIR_DOWN};
+static constexpr unsigned int PATH_DIRECTION_VERTICES = PATH_DIRECTIONS >> 1;
+static constexpr unsigned int PATH_NODE_SPACING = 2;
 
 // note: because the spacing between nodes is 2 (not 1) we
 // must also make sure not to skip across any intermediate
@@ -75,12 +78,14 @@ enum {
 
 	PATHOPT_SIZE      = 255, // size of PATHOPT bitmask
 };
-static const unsigned int PATHOPT_CARDINALS = (PATHOPT_RIGHT | PATHOPT_LEFT | PATHOPT_UP | PATHOPT_DOWN);
+
+static constexpr unsigned int PATHOPT_CARDINALS = (PATHOPT_RIGHT | PATHOPT_LEFT | PATHOPT_UP | PATHOPT_DOWN);
 
 
 static inline std::array<unsigned int, PATH_DIRECTIONS> GetPathDir2PathOpt()
 {
 	std::array<unsigned int, PATH_DIRECTIONS> a;
+
 	a[PATHDIR_LEFT]       = PATHOPT_LEFT;
 	a[PATHDIR_RIGHT]      = PATHOPT_RIGHT;
 	a[PATHDIR_UP]         = PATHOPT_UP;
@@ -89,6 +94,7 @@ static inline std::array<unsigned int, PATH_DIRECTIONS> GetPathDir2PathOpt()
 	a[PATHDIR_RIGHT_UP]   = (PATHOPT_RIGHT | PATHOPT_UP);
 	a[PATHDIR_RIGHT_DOWN] = (PATHOPT_RIGHT | PATHOPT_DOWN);
 	a[PATHDIR_LEFT_DOWN]  = (PATHOPT_LEFT  | PATHOPT_DOWN);
+
 	return a;
 }
 
@@ -96,16 +102,19 @@ static inline std::array<unsigned int, 15> GetPathOpt2PathDir()
 {
 	std::array<unsigned int, 15> a;
 	a.fill(0);
+
 	a[PATHOPT_LEFT]       = PATHDIR_LEFT;
 	a[PATHOPT_RIGHT]      = PATHDIR_RIGHT;
 	a[PATHOPT_UP]         = PATHDIR_UP;
 	a[PATHOPT_DOWN]       = PATHDIR_DOWN;
+
 	a[(PATHOPT_LEFT  | PATHOPT_UP)]   = PATHDIR_LEFT_UP;
 	a[(PATHOPT_RIGHT | PATHOPT_UP)]   = PATHDIR_RIGHT_UP;
 	a[(PATHOPT_RIGHT | PATHOPT_DOWN)] = PATHDIR_RIGHT_DOWN;
 	a[(PATHOPT_LEFT  | PATHOPT_DOWN)] = PATHDIR_LEFT_DOWN;
 	return a;
 }
+
 static const std::array<unsigned int, PATH_DIRECTIONS> DIR2OPT = GetPathDir2PathOpt();
 static const std::array<unsigned int, 15>              OPT2DIR = GetPathOpt2PathDir();
 
@@ -118,18 +127,14 @@ static inline unsigned int PathOpt2PathDir(unsigned int pathOptDir) { return OPT
 // (cost(A-->B) == cost(A<--B)) so we only need to store
 // (PATH_DIRECTIONS >> 1) values
 static inline int GetBlockVertexOffset(unsigned int pathDir, unsigned int numBlocks) {
-	int bvo = 0;
+	int bvo = pathDir;
 
 	switch (pathDir) {
-		case PATHDIR_LEFT:       { bvo = PATHDIR_LEFT;      } break;
-		case PATHDIR_LEFT_UP:    { bvo = PATHDIR_LEFT_UP;   } break;
-		case PATHDIR_UP:         { bvo = PATHDIR_UP;        } break;
-		case PATHDIR_RIGHT_UP:   { bvo = PATHDIR_RIGHT_UP;  } break;
-
 		case PATHDIR_RIGHT:      { bvo = int(PATHDIR_LEFT    ) -                                         PATH_DIRECTION_VERTICES; } break;
 		case PATHDIR_RIGHT_DOWN: { bvo = int(PATHDIR_LEFT_UP ) - (numBlocks * PATH_DIRECTION_VERTICES) - PATH_DIRECTION_VERTICES; } break;
-		case PATHDIR_DOWN:       { bvo = int(PATHDIR_UP      ) - (numBlocks * PATH_DIRECTION_VERTICES);                           } break;
+		case PATHDIR_DOWN:       { bvo = int(PATHDIR_UP      ) - (numBlocks * PATH_DIRECTION_VERTICES)                          ; } break;
 		case PATHDIR_LEFT_DOWN:  { bvo = int(PATHDIR_RIGHT_UP) - (numBlocks * PATH_DIRECTION_VERTICES) + PATH_DIRECTION_VERTICES; } break;
+		default: {} break;
 	}
 
 	return bvo;
