@@ -3,12 +3,12 @@
 #ifndef PATHMANAGER_H
 #define PATHMANAGER_H
 
-#include <map>
-#include <boost/cstdint.hpp> /* Replace with <stdint.h> if appropriate */
+#include <cinttypes>
 
 #include "Sim/Path/IPathManager.h"
 #include "IPath.h"
 #include "PathFinderDef.h"
+#include "System/UnorderedMap.hpp"
 
 class CSolidObject;
 class CPathFinder;
@@ -24,13 +24,23 @@ public:
 	~CPathManager();
 
 	unsigned int GetPathFinderType() const override { return PFS_TYPE_DEFAULT; }
-	boost::uint32_t GetPathCheckSum() const override;
+	std::uint32_t GetPathCheckSum() const override;
 
-	boost::int64_t Finalize() override;
+	std::int64_t Finalize() override;
 
 	void Update() override;
 	void UpdatePath(const CSolidObject*, unsigned int) override;
-	void DeletePath(unsigned int pathID) override;
+	void DeletePath(unsigned int pathID) override {
+		if (pathID == 0)
+			return;
+
+		const auto pi = pathMap.find(pathID);
+
+		if (pi == pathMap.end())
+			return;
+
+		pathMap.erase(pi);
+	}
 
 
 	float3 NextWayPoint(
@@ -83,30 +93,54 @@ public:
 
 private:
 	struct MultiPath {
-		MultiPath(const float3& pos, const CPathFinderDef* def, const MoveDef* moveDef)
+		MultiPath(): moveDef(nullptr), caller(nullptr) {}
+		MultiPath(const MoveDef* moveDef, const float3& startPos, const float3& goalPos, float goalRadius)
 			: searchResult(IPath::Error)
-			, start(pos)
-			, peDef(def)
+			, start(startPos)
+			, peDef(startPos, goalPos, goalRadius, 3.0f, 2000)
 			, moveDef(moveDef)
-			, finalGoal(ZeroVector)
-			, caller(NULL)
+			, caller(nullptr)
 		{}
 
-		~MultiPath() { delete peDef; }
+		MultiPath(const MultiPath& mp) = delete;
+		MultiPath(MultiPath&& mp) { *this = std::move(mp); }
 
-		// Paths
+		MultiPath& operator = (const MultiPath& mp) = delete;
+		MultiPath& operator = (MultiPath&& mp) {
+			lowResPath = std::move(mp.lowResPath);
+			medResPath = std::move(mp.medResPath);
+			maxResPath = std::move(mp.maxResPath);
+
+			searchResult = mp.searchResult;
+
+			start = mp.start;
+			finalGoal = mp.finalGoal;
+
+			peDef   = mp.peDef;
+			moveDef = mp.moveDef;
+			caller  = mp.caller;
+
+			mp.moveDef = nullptr;
+			mp.caller  = nullptr;
+			return *this;
+		}
+
+		// paths
 		IPath::Path lowResPath;
 		IPath::Path medResPath;
 		IPath::Path maxResPath;
+
 		IPath::SearchResult searchResult;
 
-		// Request definition
-		const float3 start;
-		const CPathFinderDef* peDef;
+		// request definition; start is const after ctor
+		float3 start;
+		float3 finalGoal;
+
+		CCircularSearchConstraint peDef;
+
 		const MoveDef* moveDef;
 
-		// Additional information.
-		float3 finalGoal;
+		// additional information
 		CSolidObject* caller;
 	};
 
@@ -116,17 +150,30 @@ private:
 		const MoveDef* moveDef,
 		const float3& startPos,
 		const float3& goalPos,
-		CPathFinderDef* peDef,
 		CSolidObject* caller
 	) const;
 
-	inline MultiPath* GetMultiPath(int pathID) const;
-	unsigned int Store(MultiPath* path);
+	MultiPath* GetMultiPath(int pathID) { return (const_cast<MultiPath*>(GetMultiPathConst(pathID))); }
+
+	const MultiPath* GetMultiPathConst(int pathID) const {
+		const auto pi = pathMap.find(pathID);
+		if (pi == pathMap.end())
+			return nullptr;
+		return &(pi->second);
+	}
+
+	unsigned int Store(MultiPath& path) {
+		pathMap[++nextPathID] = std::move(path);
+		return nextPathID;
+	}
+
+
 	static void FinalizePath(MultiPath* path, const float3 startPos, const float3 goalPos, const bool cantGetCloser);
+
 	void LowRes2MedRes(MultiPath& path, const float3& startPos, const CSolidObject* owner, bool synced) const;
 	void MedRes2MaxRes(MultiPath& path, const float3& startPos, const CSolidObject* owner, bool synced) const;
 
-	bool IsFinalized() const { return (maxResPF != NULL); }
+	bool IsFinalized() const { return (maxResPF != nullptr); }
 
 private:
 	CPathFinder* maxResPF;
@@ -136,15 +183,9 @@ private:
 	PathFlowMap* pathFlowMap;
 	PathHeatMap* pathHeatMap;
 
-	std::map<unsigned int, MultiPath*> pathMap;
+	spring::unordered_map<unsigned int, MultiPath> pathMap;
+
 	unsigned int nextPathID;
 };
 
-inline CPathManager::MultiPath* CPathManager::GetMultiPath(int pathID) const {
-	const std::map<unsigned int, MultiPath*>::const_iterator pi = pathMap.find(pathID);
-	if (pi == pathMap.end())
-		return NULL;
-	return pi->second;
-}
-
 #endif
