@@ -12,8 +12,7 @@ class CExpGenSpawnable: public CWorldObject
 {
 	CR_DECLARE(CExpGenSpawnable)
 public:
-	CExpGenSpawnable() : CWorldObject() {}
-	CExpGenSpawnable(const float3& pos, const float3& spd) : CWorldObject(pos, spd) {}
+	CExpGenSpawnable(const float3& pos, const float3& spd);
 
 	virtual ~CExpGenSpawnable() {}
 	virtual void Init(const CUnit* owner, const float3& offset) = 0;
@@ -25,6 +24,7 @@ public:
 	static CExpGenSpawnable* CreateSpawnable(int spawnableID);
 
 protected:
+	CExpGenSpawnable();
 	static bool GetMemberInfo(SExpGenSpawnableMemberInfo& memberInfo);
 };
 
