@@ -1,20 +1,21 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
 #include "ExpGenSpawner.h"
+
+#include "ExpGenSpawnableMemberInfo.h"
 #include "ExplosionGenerator.h"
 
 CR_BIND_DERIVED(CExpGenSpawner, CProjectile, )
-
 CR_REG_METADATA(CExpGenSpawner,
 (
 	CR_MEMBER_BEGINFLAG(CM_Config),
 		CR_MEMBER(delay),
 		CR_MEMBER(damage),
 		CR_MEMBER(explosionGenerator),
-	CR_MEMBER_ENDFLAG(CM_Config),
-	CR_RESERVED(8)
+	CR_MEMBER_ENDFLAG(CM_Config)
 ))
 
+
 CExpGenSpawner::CExpGenSpawner() :
 	CProjectile(),
 	delay(1),
@@ -25,6 +26,7 @@ CExpGenSpawner::CExpGenSpawner() :
 	deleteMe = false;
 }
 
+
 void CExpGenSpawner::Update()
 {
 	if ((deleteMe |= ((delay--) <= 0))) {
@@ -32,7 +34,21 @@ void CExpGenSpawner::Update()
 	}
 }
 
+
 int CExpGenSpawner::GetProjectilesCount() const
 {
 	return 0;
 }
+
+
+bool CExpGenSpawner::GetMemberInfo(SExpGenSpawnableMemberInfo& memberInfo)
+{
+	if (CProjectile::GetMemberInfo(memberInfo))
+		return true;
+
+	CHECK_MEMBER_INFO_INT  (CExpGenSpawner, delay )
+	CHECK_MEMBER_INFO_FLOAT(CExpGenSpawner, damage)
+	CHECK_MEMBER_INFO_PTR  (CExpGenSpawner, explosionGenerator, explGenHandler->LoadGenerator)
+
+	return false;
+}
