@@ -10,7 +10,7 @@ class IExplosionGenerator;
 //! spawns a given explosion-generator after \<delay\> frames
 class CExpGenSpawner : public CProjectile
 {
-	CR_DECLARE(CExpGenSpawner)
+	CR_DECLARE_DERIVED(CExpGenSpawner)
 public:
 	CExpGenSpawner();
 	~CExpGenSpawner() {}
@@ -19,6 +19,8 @@ public:
 
 	virtual int GetProjectilesCount() const override;
 
+	static bool GetMemberInfo(SExpGenSpawnableMemberInfo& memberInfo);
+
 private:
 	int delay;
 	float damage;
