@@ -15,9 +15,9 @@ public:
 	~CFlareProjectile();
 
 	void Update() override;
-	void Draw() override;
+	void Draw(CVertexArray* va) override;
 
-	virtual int GetProjectilesCount() const override;
+	int GetProjectilesCount() const override;
 
 public:
 	int activateFrame;
@@ -28,6 +28,10 @@ public:
 	std::vector<float3> subPos;
 	std::vector<float3> subSpeed;
 	float alphaFalloff;
+
+private:
+	CFlareProjectile() { }
+
 };
 
 #endif // FLARE_PROJECTILE_H
