@@ -4,6 +4,7 @@
 
 #include "Projectile.h"
 #include "ProjectileHandler.h"
+#include "ProjectileMemPool.h"
 #include "Game/GlobalUnsynced.h"
 #include "Game/TraceRay.h"
 #include "Map/Ground.h"
@@ -15,10 +16,14 @@
 #include "Sim/Misc/CollisionVolume.h"
 #include "Sim/Misc/QuadField.h"
 #include "Sim/Misc/TeamHandler.h"
-#include "Sim/Projectiles/Unsynced/FlyingPiece.h"
-#include "Sim/Projectiles/Unsynced/NanoProjectile.h"
+#include "Rendering/Env/Particles/Classes/FlyingPiece.h"
+#include "Rendering/Env/Particles/Classes/NanoProjectile.h"
+#include "Sim/Projectiles/WeaponProjectiles/WeaponProjectile.h"
 #include "Sim/Units/Unit.h"
 #include "Sim/Units/UnitDef.h"
+#include "Sim/Units/UnitHandler.h"
+#include "Sim/Weapons/WeaponDef.h"
+#include "Sim/Weapons/PlasmaRepulser.h"
 #include "System/Config/ConfigHandler.h"
 #include "System/EventHandler.h"
 #include "System/Log/ILog.h"
@@ -33,21 +38,17 @@
 
 using namespace std;
 
-CONFIG(int, MaxParticles).defaultValue(3000).headlessValue(1).minimumValue(1);
+CONFIG(int, MaxParticles).defaultValue(10000).headlessValue(1).minimumValue(1);
 CONFIG(int, MaxNanoParticles).defaultValue(2000).headlessValue(1).minimumValue(1);
 
-CProjectileHandler* projectileHandler = NULL;
-
 
 CR_BIND(CProjectileHandler, )
 CR_REG_METADATA(CProjectileHandler, (
 	CR_MEMBER(syncedProjectiles),
 	CR_MEMBER(unsyncedProjectiles),
-	CR_MEMBER_UN(flyingPieces3DO),
-	CR_MEMBER_UN(flyingPiecesS3O),
+	CR_MEMBER_UN(flyingPieces),
 	CR_MEMBER_UN(groundFlashes),
-	CR_MEMBER_UN(resortFlyingPieces3DO),
-	CR_MEMBER_UN(resortFlyingPiecesS3O),
+	CR_MEMBER_UN(resortFlyingPieces),
 
 	CR_MEMBER(maxParticles),
 	CR_MEMBER(maxNanoParticles),
@@ -59,24 +60,24 @@ CR_REG_METADATA(CProjectileHandler, (
 	CR_MEMBER(freeSyncedIDs),
 	CR_MEMBER(freeUnsyncedIDs),
 	CR_MEMBER(syncedProjectileIDs),
-	CR_MEMBER(unsyncedProjectileIDs),
-
-	CR_SERIALIZER(Serialize)
+	CR_MEMBER(unsyncedProjectileIDs)
 ))
 
 
 
-//////////////////////////////////////////////////////////////////////
-// Construction/Destruction
-//////////////////////////////////////////////////////////////////////
+// note: stores all ExpGenSpawnable types, not just projectiles
+ProjMemPool projMemPool;
+
+CProjectileHandler* projectileHandler = nullptr;
+
+
 
 CProjectileHandler::CProjectileHandler()
 : currentNanoParticles(0)
 , lastCurrentParticles(0)
 , lastSyncedProjectilesCount(0)
 , lastUnsyncedProjectilesCount(0)
-, resortFlyingPieces3DO(false)
-, resortFlyingPiecesS3O(false)
+, resortFlyingPieces({false})
 , syncedProjectileIDs(1024, nullptr)
 #if UNSYNCED_PROJ_NOEVENT
 , unsyncedProjectileIDs(0, nullptr)
@@ -87,33 +88,59 @@ CProjectileHandler::CProjectileHandler()
 	maxParticles     = configHandler->GetInt("MaxParticles");
 	maxNanoParticles = configHandler->GetInt("MaxNanoParticles");
 
+	projMemPool.clear();
+	projMemPool.reserve(1024);
+
 	// preload some IDs
 	for (int i = 0; i < syncedProjectileIDs.size(); i++) {
 		freeSyncedIDs.push_back(i);
 	}
-	std::random_shuffle(freeSyncedIDs.begin(), freeSyncedIDs.end(), gs->rng);
+	std::random_shuffle(freeSyncedIDs.begin(), freeSyncedIDs.end(), gsRNG);
 
 	for (int i = 0; i < unsyncedProjectileIDs.size(); i++) {
 		freeUnsyncedIDs.push_back(i);
 	}
-	std::random_shuffle(freeUnsyncedIDs.begin(), freeUnsyncedIDs.end(), gu->rng);
+	std::random_shuffle(freeUnsyncedIDs.begin(), freeUnsyncedIDs.end(), guRNG);
+
+	for (int modelType = 0; modelType < MODELTYPE_OTHER; ++modelType) {
+		flyingPieces[modelType].reserve(1000);
+	}
 
 	// register ConfigNotify()
-	configHandler->NotifyOnChange(this);
+	configHandler->NotifyOnChange(this, {"MaxParticles", "MaxNanoParticles"});
 }
 
 CProjectileHandler::~CProjectileHandler()
 {
 	configHandler->RemoveObserver(this);
 
-	for (CProjectile* p: syncedProjectiles) {
-		delete p;
+	{
+		// synced first, to avoid callback crashes
+		for (CProjectile* p: syncedProjectiles)
+			projMemPool.free(p);
+
+		syncedProjectiles.clear();
 	}
-	syncedProjectiles.clear(); // synced first, to avoid callback crashes
-	for (CProjectile* p: unsyncedProjectiles) {
-		delete p;
+
+	{
+		for (CProjectile* p: unsyncedProjectiles)
+			projMemPool.free(p);
+
+		unsyncedProjectiles.clear();
+	}
+
+	{
+		for (CGroundFlash* gf: groundFlashes)
+			projMemPool.free(gf);
+
+		groundFlashes.clear();
+	}
+
+	{
+		for (auto& fpc: flyingPieces) {
+			fpc.clear();
+		}
 	}
-	unsyncedProjectiles.clear();
 
 	freeSyncedIDs.clear();
 	freeUnsyncedIDs.clear();
@@ -124,51 +151,9 @@ CProjectileHandler::~CProjectileHandler()
 	CCollisionHandler::PrintStats();
 }
 
-void CProjectileHandler::Serialize(creg::ISerializer* s)
-{
-	if (s->IsWriting()) {
-		int ssize = int(syncedProjectiles.size());
-		int usize = int(unsyncedProjectiles.size());
-
-		s->SerializeInt(&ssize);
-		for (CProjectile* p: syncedProjectiles) {
-			void** ptr = (void**) &p;
-			s->SerializeObjectPtr(ptr, p->GetClass());
-		}
-
-		s->SerializeInt(&usize);
-		for (CProjectile* p: unsyncedProjectiles) {
-			void** ptr = (void**) &p;
-			s->SerializeObjectPtr(ptr, p->GetClass());
-		}
-	} else {
-		int ssize, usize;
-
-		s->SerializeInt(&ssize);
-		syncedProjectiles.resize(ssize);
-
-		for (CProjectile* p: syncedProjectiles) {
-			void** ptr = (void**) &p;
-			s->SerializeObjectPtr(ptr, 0/*FIXME*/);
-		}
-
-
-		s->SerializeInt(&usize);
-		unsyncedProjectiles.resize(usize);
-
-		for (CProjectile* p: unsyncedProjectiles) {
-			void** ptr = (void**) &p;
-			s->SerializeObjectPtr(ptr, 0/*FIXME*/);
-		}
-	}
-}
-
 
 void CProjectileHandler::ConfigNotify(const std::string& key, const std::string& value)
 {
-	if (key != "MaxParticles" && key != "MaxNanoParticles")
-		return;
-
 	maxParticles     = configHandler->GetInt("MaxParticles");
 	maxNanoParticles = configHandler->GetInt("MaxNanoParticles");
 }
@@ -188,55 +173,64 @@ static void MAPPOS_SANITY_CHECK(const float3 v)
 
 void CProjectileHandler::UpdateProjectileContainer(ProjectileContainer& pc, bool synced)
 {
-	// WARNING: we can't use iters here cause ProjectileCreated and ProjectileDestroyed events
-	// may add new projectiles to the container!
-	for (size_t i=0; i<pc.size();) {
+	// WARNING:
+	//   we can't use iterators here because ProjectileCreated
+	//   and ProjectileDestroyed events may add new projectiles
+	//   to the container!
+	for (size_t i = 0; i < pc.size(); /*no-op*/) {
 		CProjectile* p = pc[i];
-		assert(p);
+
+		assert(p != nullptr);
 		assert(p->synced == synced);
+#ifdef USING_CREG
 		assert(p->synced == !!(p->GetClass()->binder->flags & creg::CF_Synced));
+#endif
+
+		// creation
 		if (p->callEvent) {
-		#if UNSYNCED_PROJ_NOEVENT
-			if (synced) {
+			if (synced || !UNSYNCED_PROJ_NOEVENT)
 				eventHandler.ProjectileCreated(p, p->GetAllyteamID());
-			} else {
-				eventHandler.UnsyncedProjectileCreated(p);
-			}
-		#else
-			eventHandler.ProjectileCreated(p, p->GetAllyteamID());
-		#endif
+
+			eventHandler.RenderProjectileCreated(p);
 			p->callEvent = false;
 		}
-		if (!p->deleteMe) {
-			++i;
+
+		// deletion (FIXME: move outside of loop)
+		if (p->deleteMe) {
+			pc[i] = pc.back();
+			pc.pop_back();
+
+			eventHandler.RenderProjectileDestroyed(p);
+
+			if (synced) {
+				eventHandler.ProjectileDestroyed(p, p->GetAllyteamID());
+				syncedProjectileIDs[p->id] = nullptr;
+				freeSyncedIDs.push_back(p->id);
+
+				ASSERT_SYNCED(p->pos);
+				ASSERT_SYNCED(p->id);
+			} else {
+			#if !UNSYNCED_PROJ_NOEVENT
+				eventHandler.ProjectileDestroyed(p, p->GetAllyteamID());
+				unsyncedProjectileIDs[p->id] = nullptr;
+				freeUnsyncedIDs.push_back(p->id);
+			#endif
+			}
+
+			projMemPool.free(p);
 			continue;
 		}
-		pc[i] = pc.back();
-		pc.pop_back();
-		if (synced) { //FIXME move outside of loop!
-			eventHandler.ProjectileDestroyed(p, p->GetAllyteamID());
-			syncedProjectileIDs[p->id] = nullptr;
-			freeSyncedIDs.push_back(p->id);
-			ASSERT_SYNCED(p->pos);
-			ASSERT_SYNCED(p->id);
-		} else {
-		#if UNSYNCED_PROJ_NOEVENT
-			eventHandler.UnsyncedProjectileDestroyed(p);
-		#else
-			eventHandler.ProjectileDestroyed(p, p->GetAllyteamID());
-			unsyncedProjectileIDs[p->id] = nullptr;
-			freeUnsyncedIDs.push_back(p->id);
-		#endif
-		}
-		delete p;
+
+		// neither
+		++i;
 	}
 
-	SCOPED_TIMER("ProjectileHandler::Update::PP");
+	SCOPED_TIMER("Sim::Projectiles::Update");
 
-	//WARNING: we can't use iters here cause p->Update() may add new projectiles to the container!
-	for (size_t i=0; i<pc.size(); ++i) {
+	// WARNING: same as above but for p->Update()
+	for (size_t i = 0; i < pc.size(); ++i) {
 		CProjectile* p = pc[i];
-		assert(p);
+		assert(p != nullptr);
 
 		MAPPOS_SANITY_CHECK(p->pos);
 
@@ -249,33 +243,68 @@ void CProjectileHandler::UpdateProjectileContainer(ProjectileContainer& pc, bool
 
 
 template<class T>
-static void UPDATE_CONTAINER(T& cont) {
-#ifdef DEBUG
-	const auto* origStart = cont.empty() ? nullptr : &(*cont.begin());
-#endif
+static void UPDATE_PTR_CONTAINER(T& cont) {
+	if (cont.empty())
+		return;
 
-	auto pti = cont.begin();
-	while (pti != cont.end()) {
-		auto* p = *pti;
-		if (!p->Update()) {
-			*pti = cont.back();
-			cont.pop_back();
-			delete p;
-		} else {
-			++pti;
+#ifndef NDEBUG
+	const size_t origSize = cont.size();
+#endif
+	size_t size = cont.size();
+
+	for (unsigned int i = 0; i < size; /*no-op*/) {
+		CGroundFlash*& gf = cont[i];
+
+		if (!gf->Update()) {
+			projMemPool.free(gf);
+			gf = cont[size -= 1];
+			continue;
 		}
+
+		++i;
+	}
+
+	// WARNING:
+	//   check if the vector was enlarged while iterating, in
+	//   which case we will have missed updating newest items
+	assert(cont.size() == origSize);
+
+	cont.erase(cont.begin() + size, cont.end());
+}
+
+template<class T>
+static void UPDATE_REF_CONTAINER(T& cont) {
+	if (cont.empty())
+		return;
+
+#ifndef NDEBUG
+	const size_t origSize = cont.size();
+#endif
+	size_t size = cont.size();
+
+	for (unsigned int i = 0; i < size; /*no-op*/) {
+		auto& p = cont[i];
+
+		if (!p.Update()) {
+			p = std::move(cont[size -= 1]);
+			continue;
+		}
+
+		++i;
 	}
 
 	//WARNING: check if the vector got enlarged while iterating, in that case
-	// all iterators got invalidated in the loop, causing crashes
-	assert(cont.empty() || &(*cont.begin()) == origStart);
+	// we didn't update the newest items
+	assert(cont.size() == origSize);
+
+	cont.erase(cont.begin() + size, cont.end());
 }
 
 
 void CProjectileHandler::Update()
 {
 	{
-		SCOPED_TIMER("ProjectileHandler::Update");
+		SCOPED_TIMER("Sim::Projectiles");
 
 		// particles
 		CheckCollisions(); // before :Update() to check if the particles move into stuff
@@ -283,16 +312,18 @@ void CProjectileHandler::Update()
 		UpdateProjectileContainer(unsyncedProjectiles, false);
 
 		// groundflashes
-		UPDATE_CONTAINER(groundFlashes);
+		UPDATE_PTR_CONTAINER(groundFlashes);
 
-		// flying pieces
-		UPDATE_CONTAINER(flyingPiecesS3O);
-		UPDATE_CONTAINER(flyingPieces3DO);
+		// flying pieces; sort these every now and then
+		for (int modelType = 0; modelType < MODELTYPE_OTHER; ++modelType) {
+			auto& fpc = flyingPieces[modelType];
 
-		// sort those every now and then
-		FlyingPieceComparator fsort;
-		if (resortFlyingPiecesS3O) std::sort(flyingPiecesS3O.begin(), flyingPiecesS3O.end(), fsort);
-		if (resortFlyingPieces3DO) std::sort(flyingPieces3DO.begin(), flyingPieces3DO.end(), fsort);
+			UPDATE_REF_CONTAINER(fpc);
+
+			if (resortFlyingPieces[modelType]) {
+				std::stable_sort(fpc.begin(), fpc.end());
+			}
+		}
 	}
 
 	// precache part of particles count calculation that else becomes very heavy
@@ -314,11 +345,13 @@ void CProjectileHandler::AddProjectile(CProjectile* p)
 	// already initialized?
 	assert(p->id < 0);
 	assert(p->callEvent);
-	std::deque<int>* freeIDs = NULL;
-	ProjectileMap* proIDs = NULL;
+
+	std::deque<int>* freeIDs = nullptr;
+	ProjectileMap* proIDs = nullptr;
 
 	if (p->synced) {
 		syncedProjectiles.push_back(p);
+
 		freeIDs = &freeSyncedIDs;
 		proIDs = &syncedProjectileIDs;
 		ASSERT_SYNCED(freeIDs->size());
@@ -327,6 +360,7 @@ void CProjectileHandler::AddProjectile(CProjectile* p)
 #if UNSYNCED_PROJ_NOEVENT
 		return;
 #endif
+
 		freeIDs = &freeUnsyncedIDs;
 		proIDs = &unsyncedProjectileIDs;
 	}
@@ -338,9 +372,9 @@ void CProjectileHandler::AddProjectile(CProjectile* p)
 			freeIDs->push_back(i);
 		}
 		if (p->synced) {
-			std::random_shuffle(freeIDs->begin(), freeIDs->end(), gs->rng);
+			std::random_shuffle(freeIDs->begin(), freeIDs->end(), gsRNG);
 		} else{
-			std::random_shuffle(freeIDs->begin(), freeIDs->end(), gu->rng);
+			std::random_shuffle(freeIDs->begin(), freeIDs->end(), guRNG);
 		}
 		proIDs->resize(newSize, nullptr);
 	}
@@ -365,40 +399,62 @@ void CProjectileHandler::AddProjectile(CProjectile* p)
 
 
 
+static bool CheckProjectileCollisionFlags(const CProjectile* p, const CUnit* u)
+{
+	if (teamHandler->IsValidAllyTeam(p->GetAllyteamID())) {
+		const bool noFriendsBit = ((p->GetCollisionFlags() & Collision::NOFRIENDLIES) != 0);
+		const bool noEnemiesBit = ((p->GetCollisionFlags() & Collision::NOENEMIES   ) != 0);
+		const bool friendlyFire = teamHandler->AlliedAllyTeams(p->GetAllyteamID(), u->allyteam);
+
+		if (noFriendsBit && friendlyFire)
+			return false;
+
+		if (noEnemiesBit && !friendlyFire)
+			return false;
+	}
+
+	if ((p->GetCollisionFlags() & Collision::NONEUTRALS) != 0 && u->IsNeutral())
+		return false;
+
+	if ((p->GetCollisionFlags() & Collision::NOFIREBASES) != 0) {
+		const CUnit* owner = p->owner();
+		const CUnit* trans = (owner != nullptr)? owner->GetTransporter(): nullptr;
+
+		// check if the unit being collided with is occupied by p's owner
+		if (u == trans && trans->unitDef->isFirePlatform)
+			return false;
+	}
+
+	return true;
+}
+
+
 void CProjectileHandler::CheckUnitCollisions(
 	CProjectile* p,
 	std::vector<CUnit*>& tempUnits,
 	const float3 ppos0,
 	const float3 ppos1)
 {
+	if (!p->checkCol)
+		return;
+
 	CollisionQuery cq;
 
 	for (CUnit* unit: tempUnits) {
-		if (unit == NULL)
-			break;
+		assert(unit != nullptr);
 
 		// if this unit fired this projectile, always ignore
 		if (unit == p->owner())
 			continue;
 		if (!unit->HasCollidableStateBit(CSolidObject::CSTATE_BIT_PROJECTILES))
 			continue;
-		
-		if (p->GetAllyteamID() >= 0) {
-			if (p->GetCollisionFlags() & Collision::NOFRIENDLIES) {
-				if ( teamHandler->AlliedAllyTeams(p->GetAllyteamID(), unit->allyteam)) { continue; }
-			}
-			if (p->GetCollisionFlags() & Collision::NOENEMIES) {
-				if (!teamHandler->AlliedAllyTeams(p->GetAllyteamID(), unit->allyteam)) { continue; }
-			}
-			if (p->GetCollisionFlags() & Collision::NONEUTRALS) {
-				if (unit->IsNeutral()) { continue; }
-			}
-		}
 
-		if (CCollisionHandler::DetectHit(unit, ppos0, ppos1, &cq)) {
-			if (cq.GetHitPiece() != NULL) {
-				unit->SetLastAttackedPiece(cq.GetHitPiece(), gs->frameNum);
-			}
+		if (!CheckProjectileCollisionFlags(p, unit))
+			continue;
+
+		if (CCollisionHandler::DetectHit(unit, unit->GetTransformMatrix(true), ppos0, ppos1, &cq)) {
+			if (cq.GetHitPiece() != nullptr)
+				unit->SetLastHitPiece(cq.GetHitPiece(), gs->frameNum);
 
 			if (!cq.InsideHit()) {
 				p->SetPosition(cq.GetHitPos());
@@ -429,13 +485,15 @@ void CProjectileHandler::CheckFeatureCollisions(
 	CollisionQuery cq;
 
 	for (CFeature* feature: tempFeatures) {
-		if (feature == NULL)
-			break;
+		assert(feature != nullptr);
 
 		if (!feature->HasCollidableStateBit(CSolidObject::CSTATE_BIT_PROJECTILES))
 			continue;
 
-		if (CCollisionHandler::DetectHit(feature, ppos0, ppos1, &cq)) {
+		if (CCollisionHandler::DetectHit(feature, feature->GetTransformMatrix(true), ppos0, ppos1, &cq)) {
+			if (cq.GetHitPiece() != nullptr)
+				feature->SetLastHitPiece(cq.GetHitPiece(), gs->frameNum);
+
 			if (!cq.InsideHit()) {
 				p->SetPosition(cq.GetHitPos());
 				p->Collision(feature);
@@ -449,51 +507,107 @@ void CProjectileHandler::CheckFeatureCollisions(
 	}
 }
 
+
+void CProjectileHandler::CheckShieldCollisions(
+	CProjectile* p,
+	std::vector<CPlasmaRepulser*>& tempRepulsers,
+	const float3 ppos0,
+	const float3 ppos1)
+{
+	if (!p->checkCol)
+		return;
+
+	if (!p->weapon)
+		return;
+
+	CWeaponProjectile* wpro = static_cast<CWeaponProjectile*>(p);
+	const WeaponDef* wdef = wpro->GetWeaponDef();
+
+	//Bail early
+	if (wdef->interceptedByShieldType == 0)
+		return;
+
+	CollisionQuery cq;
+
+	for (CPlasmaRepulser* repulser: tempRepulsers) {
+		assert(repulser != nullptr);
+		if (!repulser->CanIntercept(wdef->interceptedByShieldType, p->GetAllyteamID()))
+			continue;
+
+		// we sometimes get false inside hits due to the movement of the shield
+		// a very hacky solution is to increase the ray that's intersecting
+		// by the last movement of the shield.
+		// it's not 100% accurate so there's a bit of a FIXME here to do a real solution
+		// (keep track in the projectile which shields it's in)
+
+		const float3 effectivePPos0 = ppos0 + (ppos0 - ppos1) * repulser->deltaPos.Length();
+		if (CCollisionHandler::DetectHit(repulser->owner, &repulser->collisionVolume, repulser->owner->GetTransformMatrix(true), effectivePPos0, ppos1, &cq)) {
+			if (!cq.InsideHit() || !repulser->weaponDef->exteriorShield || repulser->IsRepulsing(wpro)) {
+				if (repulser->IncomingProjectile(wpro, cq.GetHitPos()))
+					return;
+			}
+		}
+	}
+}
+
 void CProjectileHandler::CheckUnitFeatureCollisions(ProjectileContainer& pc)
 {
-	static std::vector<CUnit*> tempUnits(unitHandler->MaxUnits(), NULL);
-	static std::vector<CFeature*> tempFeatures(unitHandler->MaxUnits(), NULL);
+	static std::vector<CUnit*> tempUnits;
+	static std::vector<CFeature*> tempFeatures;
+	static std::vector<CPlasmaRepulser*> tempRepulsers;
 
-	for (size_t i=0; i<pc.size(); ++i) {
+	for (size_t i = 0; i < pc.size(); ++i) {
 		CProjectile* p = pc[i];
+
 		if (!p->checkCol) continue;
 		if ( p->deleteMe) continue;
 
 		const float3 ppos0 = p->pos;
 		const float3 ppos1 = p->pos + p->speed;
 
-		quadField->GetUnitsAndFeaturesColVol(p->pos, p->radius + p->speed.w, tempUnits, tempFeatures);
+		quadField->GetUnitsAndFeaturesColVol(p->pos, p->radius + p->speed.w, tempUnits, tempFeatures, &tempRepulsers);
 
-		CheckUnitCollisions(p, tempUnits, ppos0, ppos1);
-		CheckFeatureCollisions(p, tempFeatures, ppos0, ppos1);
+		CheckShieldCollisions(p, tempRepulsers, ppos0, ppos1); tempRepulsers.clear();
+		CheckUnitCollisions(p, tempUnits, ppos0, ppos1); tempUnits.clear();
+		CheckFeatureCollisions(p, tempFeatures, ppos0, ppos1); tempFeatures.clear();
 	}
 }
 
 void CProjectileHandler::CheckGroundCollisions(ProjectileContainer& pc)
 {
-	for (size_t i=0; i<pc.size(); ++i) {
+	for (size_t i = 0; i < pc.size(); ++i) {
 		CProjectile* p = pc[i];
+
 		if (!p->checkCol)
 			continue;
 
-		// NOTE: if <p> is a MissileProjectile and does not
-		// have selfExplode set, it will never be removed (!)
+		// NOTE:
+		//   if <p> is a MissileProjectile and does not have
+		//   selfExplode set, tbis will cause it to never be
+		//   removed (!)
 		if (p->GetCollisionFlags() & Collision::NOGROUND)
 			continue;
 
-		// NOTE: don't add p->radius to groundHeight, or most
-		// projectiles will collide with the ground too early
+		// don't collide with ground yet if last update scheduled a bounce
+		if (p->weapon && static_cast<const CWeaponProjectile*>(p)->HasScheduledBounce())
+			continue;
+
+		// NOTE:
+		//   don't add p->radius to groundHeight, or most (esp. modelled)
+		//   projectiles will collide with the ground one or more frames
+		//   too early
 		const float groundHeight = CGround::GetHeightReal(p->pos.x, p->pos.z);
+
 		const bool belowGround = (p->pos.y < groundHeight);
 		const bool insideWater = (p->pos.y <= 0.0f && !belowGround);
 		const bool ignoreWater = p->ignoreWater;
 
 		if (belowGround || (insideWater && !ignoreWater)) {
 			// if position has dropped below terrain or into water
-			// where we cannot live, adjust it and explode us now
+			// where we can not live, adjust it and explode us now
 			// (if the projectile does not set deleteMe = true, it
 			// will keep hugging the terrain)
-			p->pos.y = groundHeight * belowGround;
+			p->SetPosition(p->pos * XZVector + UpVector * groundHeight * belowGround);
 			p->Collision();
 		}
 	}
@@ -501,7 +615,7 @@ void CProjectileHandler::CheckGroundCollisions(ProjectileContainer& pc)
 
 void CProjectileHandler::CheckCollisions()
 {
-	SCOPED_TIMER("ProjectileHandler::Update::CheckCollisions");
+	SCOPED_TIMER("Sim::Projectiles::Collisions");
 
 	CheckUnitFeatureCollisions(syncedProjectiles); //! changes simulation state
 	CheckUnitFeatureCollisions(unsyncedProjectiles); //! does not change simulation state
@@ -519,29 +633,16 @@ void CProjectileHandler::AddGroundFlash(CGroundFlash* flash)
 
 
 void CProjectileHandler::AddFlyingPiece(
+	int modelType,
+	const S3DModelPiece* piece,
+	const CMatrix44f& m,
 	const float3 pos,
 	const float3 speed,
-	int team,
-	const S3DOPiece* piece,
-	const S3DOPrimitive* chunk)
-{
-	FlyingPiece* fp = new S3DOFlyingPiece(pos, speed, team, piece, chunk);
-	flyingPieces3DO.push_back(fp);
-	resortFlyingPieces3DO = true;
-}
-
-void CProjectileHandler::AddFlyingPiece(
-	const float3 pos,
-	const float3 speed,
-	int team,
-	int textureType,
-	const SS3OVertex* chunk)
-{
-	assert(textureType > 0);
-
-	FlyingPiece* fp = new SS3OFlyingPiece(pos, speed, team, textureType, chunk);
-	flyingPiecesS3O.push_back(fp);
-	resortFlyingPiecesS3O = true;
+	const float2 pieceParams,
+	const int2 renderParams
+) {
+	flyingPieces[modelType].emplace_back(piece, m, pos, speed, pieceParams, renderParams);
+	resortFlyingPieces[modelType] = true;
 }
 
 
@@ -563,15 +664,17 @@ void CProjectileHandler::AddNanoParticle(
 	const float l = fastmath::apxsqrt2(dif.SqLength());
 
 	dif /= l;
-	dif += gu->RandVector() * 0.15f;
+	dif += guRNG.NextVector() * 0.15f;
 
-	SColor color(unitDef->nanoColor.r, unitDef->nanoColor.g, unitDef->nanoColor.b);
-	if (globalRendering->teamNanospray) {
-		const CTeam* team = teamHandler->Team(teamNum);
-		color = SColor(team->color);
-	}
-	color.a = 20;
-	new CNanoProjectile(startPos, dif, int(l), color);
+	const float3 udColor = unitDef->nanoColor;
+	const uint8_t* tColor = (teamHandler->Team(teamNum))->color;
+
+	const SColor colors[2] = {
+		SColor(udColor.r, udColor.g, udColor.b, 20.0f / 255.0f),
+		SColor(tColor[0], tColor[1], tColor[2], uint8_t(20)),
+	};
+
+	projMemPool.alloc<CNanoProjectile>(startPos, dif, int(l), colors[globalRendering->teamNanospray]);
 }
 
 void CProjectileHandler::AddNanoParticle(
@@ -594,19 +697,20 @@ void CProjectileHandler::AddNanoParticle(
 	const float l = fastmath::apxsqrt2(dif.SqLength());
 	dif /= l;
 
-	float3 error = gu->RandVector() * (radius / l);
+	float3 error = guRNG.NextVector() * (radius / l);
 
-	SColor color(unitDef->nanoColor.r, unitDef->nanoColor.g, unitDef->nanoColor.b);
-	if (globalRendering->teamNanospray) {
-		const CTeam* team = teamHandler->Team(teamNum);
-		color = SColor(team->color);
-	}
-	color.a = 20;
+	const float3 udColor = unitDef->nanoColor;
+	const uint8_t* tColor = (teamHandler->Team(teamNum))->color;
+
+	const SColor colors[2] = {
+		SColor(udColor.r, udColor.g, udColor.b, 20.0f / 255.0f),
+		SColor(tColor[0], tColor[1], tColor[2], uint8_t(20)),
+	};
 
 	if (!inverse) {
-		new CNanoProjectile(startPos, (dif + error) * 3, int(l / 3), color);
+		projMemPool.alloc<CNanoProjectile>(startPos, (dif + error) * 3, int(l / 3), colors[globalRendering->teamNanospray]);
 	} else {
-		new CNanoProjectile(startPos + (dif + error) * l, -(dif + error) * 3, int(l / 3), color);
+		projMemPool.alloc<CNanoProjectile>(startPos + (dif + error) * l, -(dif + error) * 3, int(l / 3), colors[globalRendering->teamNanospray]);
 	}
 }
 
@@ -635,7 +739,7 @@ float CProjectileHandler::GetParticleSaturation(const bool withRandomization) co
 	// use the random mult to weaken the max limit a little
 	// so the chance is better spread when being close to the limit
 	// i.e. when there are rockets that spam CEGs this gives smaller CEGs still a chance
-	return (GetCurrentParticles() / float(maxParticles)) * (1.f + int(withRandomization) * 0.3f * gu->RandFloat());
+	return (GetCurrentParticles() / float(maxParticles)) * (1.f + int(withRandomization) * 0.3f * guRNG.NextFloat());
 }
 
 int CProjectileHandler::GetCurrentParticles() const
@@ -649,8 +753,11 @@ int CProjectileHandler::GetCurrentParticles() const
 	for (size_t i = lastUnsyncedProjectilesCount, e = unsyncedProjectiles.size(); i < e; ++i) {
 		partCount += unsyncedProjectiles[i]->GetProjectilesCount();
 	}
-	partCount += flyingPieces3DO.size();
-	partCount += flyingPiecesS3O.size();
+	for (const auto& c: flyingPieces) {
+		for (const auto& fp: c) {
+			partCount += fp.GetDrawCallCount();
+		}
+	}
 	partCount += groundFlashes.size();
 	return partCount;
 }
