@@ -9,14 +9,16 @@ class CBeamLaserProjectile: public CWeaponProjectile
 {
 	CR_DECLARE_DERIVED(CBeamLaserProjectile)
 public:
+	// creg only
 	CBeamLaserProjectile() { }
+
 	CBeamLaserProjectile(const ProjectileParams& params);
 
 	void Update() override;
-	void Draw() override;
+	void Draw(CVertexArray* va) override;
 	void DrawOnMinimap(CVertexArray& lines, CVertexArray& points) override;
 
-	virtual int GetProjectilesCount() const override;
+	int GetProjectilesCount() const override;
 
 private:
 	unsigned char coreColStart[4];
