@@ -9,13 +9,15 @@ class CEmgProjectile : public CWeaponProjectile
 {
 	CR_DECLARE_DERIVED(CEmgProjectile)
 public:
+	// creg only
 	CEmgProjectile() { }
+
 	CEmgProjectile(const ProjectileParams& params);
 
 	void Update() override;
-	void Draw() override;
+	void Draw(CVertexArray* va) override;
 
-	virtual int GetProjectilesCount() const override;
+	int GetProjectilesCount() const override;
 
 	int ShieldRepulse(const float3& shieldPos, float shieldForce, float shieldMaxSpeed) override;
 
