@@ -9,13 +9,15 @@ class CExplosiveProjectile : public CWeaponProjectile
 {
 	CR_DECLARE_DERIVED(CExplosiveProjectile)
 public:
+	// creg only
 	CExplosiveProjectile() { }
+
 	CExplosiveProjectile(const ProjectileParams& params);
 
 	void Update() override;
-	void Draw() override;
+	void Draw(CVertexArray* va) override;
 
-	virtual int GetProjectilesCount() const override;
+	int GetProjectilesCount() const override;
 
 	int ShieldRepulse(const float3& shieldPos, float shieldForce, float shieldMaxSpeed) override;
 
