@@ -9,17 +9,19 @@ class CLaserProjectile : public CWeaponProjectile
 {
 	CR_DECLARE_DERIVED(CLaserProjectile)
 public:
+	// creg only
 	CLaserProjectile() { }
+
 	CLaserProjectile(const ProjectileParams& params);
 
-	void Draw() override;
+	void Draw(CVertexArray* va) override;
 	void Update() override;
 	void Collision(CUnit* unit) override;
 	void Collision(CFeature* feature) override;
 	void Collision() override;
 	int ShieldRepulse(const float3& shieldPos, float shieldForce, float shieldMaxSpeed) override;
 
-	virtual int GetProjectilesCount() const override;
+	int GetProjectilesCount() const override;
 
 private:
 	void UpdateIntensity();
