@@ -11,14 +11,16 @@ class CLightningProjectile : public CWeaponProjectile
 {
 	CR_DECLARE_DERIVED(CLightningProjectile)
 public:
+	// creg only
 	CLightningProjectile() { }
+
 	CLightningProjectile(const ProjectileParams& params);
 
 	void Update() override;
-	void Draw() override;
+	void Draw(CVertexArray* va) override;
 	void DrawOnMinimap(CVertexArray& lines, CVertexArray& points) override;
 
-	virtual int GetProjectilesCount() const override;
+	int GetProjectilesCount() const override;
 
 private:
 	float3 color;
