@@ -15,7 +15,9 @@ class CMissileProjectile : public CWeaponProjectile
 protected:
 	void UpdateGroundBounce() override;
 public:
+	// creg only
 	CMissileProjectile() { }
+
 	CMissileProjectile(const ProjectileParams& params);
 
 	void Collision(CUnit* unit) override;
@@ -23,9 +25,9 @@ public:
 	void Collision() override;
 
 	void Update() override;
-	void Draw() override;
+	void Draw(CVertexArray* va) override;
 
-	virtual int GetProjectilesCount() const override;
+	int GetProjectilesCount() const override;
 
 	int ShieldRepulse(const float3& shieldPos, float shieldForce, float shieldMaxSpeed) override;
 
