@@ -25,14 +25,12 @@ public:
 	void FinishCommand();
 	void BuggerOff(const float3& pos, float radius);
 //	void StopMove();
-	
-	void SetGoal(const float3& pos, const float3& curPos, float goalRadius = SQUARE_SIZE);
-	
+
 	void ExecuteGuard(Command& c);
 	void ExecuteAreaAttack(Command& c);
 	void ExecuteAttack(Command& c);
 	void ExecuteFight(Command& c);
-//	void ExecuteMove(Command& c);
+	void ExecuteMove(Command& c);
 
 	bool IsValidTarget(const CUnit* enemy) const;
 
