@@ -1,10 +1,8 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
+#include <cassert>
 
 #include "BuilderCAI.h"
-
-#include <assert.h>
-
 #include "ExternalAI/EngineOutHandler.h"
 #include "Game/GameHelper.h"
 #include "Game/SelectedUnitsHandler.h"
@@ -19,14 +17,13 @@
 #include "Sim/Misc/Team.h"
 #include "Sim/Misc/TeamHandler.h"
 #include "Sim/MoveTypes/MoveType.h"
-#include "Sim/Units/UnitSet.h"
 #include "Sim/Units/UnitDefHandler.h"
 #include "Sim/Units/UnitHandler.h"
 #include "Sim/Units/UnitTypes/Builder.h"
 #include "Sim/Units/UnitTypes/Building.h"
 #include "Sim/Units/UnitTypes/Factory.h"
 #include "System/myMath.h"
-#include "System/Util.h"
+#include "System/StringUtil.h"
 #include "System/EventHandler.h"
 #include "System/Exceptions.h"
 #include "System/Log/ILog.h"
@@ -55,9 +52,9 @@ CR_REG_METADATA(CBuilderCAI , (
 ))
 
 // not adding to members, should repopulate itself
-CUnitSet CBuilderCAI::reclaimers;
-CUnitSet CBuilderCAI::featureReclaimers;
-CUnitSet CBuilderCAI::resurrecters;
+spring::unordered_set<int> CBuilderCAI::reclaimers;
+spring::unordered_set<int> CBuilderCAI::featureReclaimers;
+spring::unordered_set<int> CBuilderCAI::resurrecters;
 
 
 static std::string GetUnitDefBuildOptionToolTip(const UnitDef* ud, bool disabled) {
@@ -81,7 +78,7 @@ static std::string GetUnitDefBuildOptionToolTip(const UnitDef* ud, bool disabled
 
 CBuilderCAI::CBuilderCAI():
 	CMobileCAI(),
-	ownerBuilder(NULL),
+	ownerBuilder(nullptr),
 	building(false),
 	cachedRadiusId(0),
 	cachedRadius(0),
@@ -140,7 +137,7 @@ CBuilderCAI::CBuilderCAI(CUnit* owner):
 
 		c.action    = "reclaim";
 		c.name      = "Reclaim";
-		c.tooltip   = c.name + ": Sucks in the metal/energy content of a unit/feature and add it to your storage";
+		c.tooltip   = c.name + ": Sucks in the metal/energy content of a unit/feature\nand adds it to your storage";
 		c.mouseicon = c.name;
 		possibleCommands.push_back(commandDescriptionCache->GetPtr(c));
 	}
@@ -217,20 +214,21 @@ CBuilderCAI::CBuilderCAI(CUnit* owner):
 
 CBuilderCAI::~CBuilderCAI()
 {
-	// if uh == NULL then all pointers to units should be considered dangling pointers
-	if (unitHandler != NULL) {
-		RemoveUnitFromReclaimers(owner);
-		RemoveUnitFromFeatureReclaimers(owner);
-		RemoveUnitFromResurrecters(owner);
-		unitHandler->RemoveBuilderCAI(this);
-	}
+	// if uh == NULL then all pointers to units should be considered dangling
+	if (unitHandler == nullptr)
+		return;
+
+	RemoveUnitFromReclaimers(owner);
+	RemoveUnitFromFeatureReclaimers(owner);
+	RemoveUnitFromResurrecters(owner);
+	unitHandler->RemoveBuilderCAI(this);
 }
 
 void CBuilderCAI::InitStatic()
 {
-	reclaimers.clear();
-	featureReclaimers.clear();
-	resurrecters.clear();
+	spring::clear_unordered_set(reclaimers);
+	spring::clear_unordered_set(featureReclaimers);
+	spring::clear_unordered_set(resurrecters);
 }
 
 void CBuilderCAI::PostLoad()
@@ -261,9 +259,8 @@ inline float CBuilderCAI::GetBuildRange(const float targetRadius) const
 	// and so it would be impossible to get in buildrange (collision detection with units/features)
 	//
 	// what does this even mean?? IMMOBILE units cannot "get in range" of anything
-	if (owner->immobile) {
+	if (owner->immobile)
 		return (ownerBuilder->buildDistance + std::max(targetRadius - ownerBuilder->buildDistance, 0.0f));
-	}
 
 	return (ownerBuilder->buildDistance + targetRadius);
 }
@@ -313,7 +310,7 @@ bool CBuilderCAI::MoveInBuildRange(const float3& objPos, float objRadius, const
 	}
 
 	if (owner->unitDef->IsAirUnit()) {
-		StopMoveAndKeepPointing(objPos, objRadius, false);
+		StopMoveAndKeepPointing(objPos, GetBuildRange(objRadius) * 0.9f, false);
 	} else {
 		StopMoveAndKeepPointing(owner->moveType->goalPos, GetBuildRange(objRadius) * 0.9f, false);
 	}
@@ -322,23 +319,21 @@ bool CBuilderCAI::MoveInBuildRange(const float3& objPos, float objRadius, const
 }
 
 
-bool CBuilderCAI::IsBuildPosBlocked(const BuildInfo& build, const CUnit** nanoFrame) const
+bool CBuilderCAI::IsBuildPosBlocked(const BuildInfo& bi, const CUnit** nanoFrame) const
 {
-	CFeature* feature = NULL;
-	CGameHelper::BuildSquareStatus status = CGameHelper::TestUnitBuildSquare(build, feature, owner->allyteam, true);
+	CFeature* feature = nullptr;
+	CGameHelper::BuildSquareStatus status = CGameHelper::TestUnitBuildSquare(bi, feature, owner->allyteam, true);
 
-	if (feature != NULL && build.def->isFeature && build.def->wreckName == feature->def->name) {
-		// buildjob is a feature and it is finished already
+	// buildjob is a feature and it is finished already
+	if (feature != nullptr && bi.def->isFeature && bi.def->wreckName == feature->def->name)
 		return true;
-	}
 
-	if (status != CGameHelper::BUILDSQUARE_BLOCKED) {
-		// open area, reclaimable feature or movable unit
+	// open area, reclaimable feature or movable unit
+	if (status != CGameHelper::BUILDSQUARE_BLOCKED)
 		return false;
-	}
 
 	// status is blocked, check if there is a foreign object in the way
-	const CSolidObject* s = groundBlockingObjectMap->GroundBlocked(build.pos);
+	const CSolidObject* s = groundBlockingObjectMap->GroundBlocked(bi.pos);
 
 	// just ourselves, does not count
 	if (s == owner)
@@ -349,12 +344,12 @@ bool CBuilderCAI::IsBuildPosBlocked(const BuildInfo& build, const CUnit** nanoFr
 	// if a *unit* object is not present, then either
 	// there is a feature or the terrain is unsuitable
 	// (in the former case feature must be reclaimable)
-	if (u == NULL)
-		return (feature == NULL || !feature->def->reclaimable);
+	if (u == nullptr)
+		return (feature == nullptr || !feature->def->reclaimable);
 
 	// figure out if object is soft- or hard-blocking
 	if (u->beingBuilt) {
-		if (!ownerBuilder->CanAssistUnit(u, build.def)) {
+		if (!ownerBuilder->CanAssistUnit(u, bi.def)) {
 			if (u->immobile) {
 				// we can't or don't want assist finishing the nanoframe
 				return true;
@@ -365,7 +360,7 @@ bool CBuilderCAI::IsBuildPosBlocked(const BuildInfo& build, const CUnit** nanoFr
 		}
 
 		// unfinished nanoframe, assist it
-		if (nanoFrame != NULL && teamHandler->Ally(owner->allyteam, u->allyteam))
+		if (nanoFrame != nullptr && teamHandler->Ally(owner->allyteam, u->allyteam))
 			*nanoFrame = u;
 
 		return false;
@@ -379,23 +374,23 @@ bool CBuilderCAI::IsBuildPosBlocked(const BuildInfo& build, const CUnit** nanoFr
 
 inline bool CBuilderCAI::OutOfImmobileRange(const Command& cmd) const
 {
-	if (owner->unitDef->canmove) {
-		return false; // builder can move
-	}
-	if (((cmd.options & INTERNAL_ORDER) == 0) || (cmd.params.size() != 1)) {
-		return false; // not an internal object targetted command
-	}
+	// builder can move
+	if (owner->unitDef->canmove)
+		return false;
+
+	// not an internal object targetted command
+	if (((cmd.options & INTERNAL_ORDER) == 0) || (cmd.params.size() != 1))
+		return false;
 
 	const int objID = cmd.params[0];
 	const CWorldObject* obj = unitHandler->GetUnit(objID);
 
-	if (obj == NULL) {
+	if (obj == nullptr) {
 		// features don't move, but maybe the unit was transported?
 		obj = featureHandler->GetFeature(objID - unitHandler->MaxUnits());
 
-		if (obj == NULL) {
+		if (obj == nullptr)
 			return false;
-		}
 	}
 
 	switch (cmd.GetID()) {
@@ -403,26 +398,27 @@ inline bool CBuilderCAI::OutOfImmobileRange(const Command& cmd) const
 		case CMD_RECLAIM:
 		case CMD_RESURRECT:
 		case CMD_CAPTURE: {
-			if (!IsInBuildRange(obj)) {
+			if (!IsInBuildRange(obj))
 				return true;
-			}
+
 			break;
 		}
 	}
+
 	return false;
 }
 
 
 float CBuilderCAI::GetBuildOptionRadius(const UnitDef* ud, int cmdId)
 {
-	float radius;
-	if (cachedRadiusId == cmdId) {
-		radius = cachedRadius;
-	} else {
+	float radius = cachedRadius;
+
+	if (cachedRadiusId != cmdId) {
 		radius = ud->GetModelRadius();
 		cachedRadius = radius;
 		cachedRadiusId = cmdId;
 	}
+
 	return radius;
 }
 
@@ -430,8 +426,7 @@ float CBuilderCAI::GetBuildOptionRadius(const UnitDef* ud, int cmdId)
 void CBuilderCAI::CancelRestrictedUnit()
 {
 	if (owner->team == gu->myTeam) {
-		LOG_L(L_WARNING, "%s: Build failed, unit type limit reached",
-				owner->unitDef->humanName.c_str());
+		LOG_L(L_WARNING, "%s: Build failed, unit type limit reached", owner->unitDef->humanName.c_str());
 		eventHandler.LastMessagePosition(owner->pos);
 	}
 	StopMoveAndFinishCommand();
@@ -478,14 +473,14 @@ void CBuilderCAI::GiveCommandReal(const Command& c, bool fromSynced)
 			}
 		}
 
-		const CUnit* nanoFrame = NULL;
+		const CUnit* nanoFrame = nullptr;
 
 		// check if the buildpos is blocked
 		if (IsBuildPosBlocked(bi, &nanoFrame))
 			return;
 
 		// if it is a nanoframe help to finish it
-		if (nanoFrame != NULL) {
+		if (nanoFrame != nullptr) {
 			Command c2(CMD_REPAIR, c.options | INTERNAL_ORDER, nanoFrame->id);
 			CMobileCAI::GiveCommandReal(c2, fromSynced);
 			CMobileCAI::GiveCommandReal(c, fromSynced);
@@ -580,19 +575,26 @@ void CBuilderCAI::ExecuteBuildCmd(Command& c)
 
 	if (!inCommand) {
 		BuildInfo bi;
+
+		// note:
+		//   need at least 3 parameters or BuildInfo will fail to parse
+		//   this usually indicates a malformed command inserted by Lua
+		//   (most common with patrolling pseudo-factory hubs)
+		if (!bi.Parse(c)) {
+			StopMoveAndFinishCommand();
+			return;
+		}
+
+		#if 1
+		// snap build-position to multiples of SQUARE_SIZE plus a half-square offset (4, 12, 20, ...)
 		bi.pos.x = math::floor(c.params[0] / SQUARE_SIZE + 0.5f) * SQUARE_SIZE;
 		bi.pos.z = math::floor(c.params[2] / SQUARE_SIZE + 0.5f) * SQUARE_SIZE;
-		bi.pos.y = c.params[1];
+		#endif
 
-		if (c.params.size() == 4)
-			bi.buildFacing = abs((int)c.params[3]) % NUM_FACINGS;
-
-		bi.def = unitDefHandler->GetUnitDefByID(-c.GetID());
-
-		CFeature* f = NULL;
+		CFeature* f = nullptr;
 		CGameHelper::TestUnitBuildSquare(bi, f, owner->allyteam, true);
 
-		if (f != NULL) {
+		if (f != nullptr) {
 			if (!bi.def->isFeature || bi.def->wreckName != f->def->name) {
 				ReclaimFeature(f);
 			} else {
@@ -601,96 +603,98 @@ void CBuilderCAI::ExecuteBuildCmd(Command& c)
 			return;
 		}
 
+		// <build> is never parsed (except in PostLoad) so just copy it
+		build = bi;
 		inCommand = true;
-		build.Parse(c);
 	}
 
+	assert(build.def != nullptr);
 	assert(build.def->id == -c.GetID() && build.def->id != 0);
-	const float buildeeRadius = GetBuildOptionRadius(build.def, c.GetID());
 
 	if (building) {
 		// keep moving until 3D distance to buildPos is LEQ our buildDistance
 		MoveInBuildRange(build.pos, 0.0f);
 
-		if (!ownerBuilder->curBuild && !ownerBuilder->terraforming) {
+		if (ownerBuilder->curBuild == nullptr && !ownerBuilder->terraforming) {
 			building = false;
 			StopMoveAndFinishCommand();
+			return;
 		}
-		// This can only be true if two builders started building
-		// the restricted unit in the same simulation frame
-		else if (unitHandler->unitsByDefs[owner->team][build.def->id].size() > build.def->maxThisUnit) {
-			// unit restricted
-			building = false;
-			ownerBuilder->StopBuild();
-			CancelRestrictedUnit();
+
+		return;
+	}
+
+	build.pos = CGameHelper::Pos2BuildPos(build, true);
+
+	// keep moving until 3D distance to buildPos is LEQ our buildDistance
+	if (MoveInBuildRange(build.pos, 0.0f, true)) {
+		if (IsBuildPosBlocked(build)) {
+			StopMoveAndFinishCommand();
+			return;
 		}
-	} else {
-		if (unitHandler->unitsByDefs[owner->team][build.def->id].size() >= build.def->maxThisUnit) {
-			// unit restricted, don't bother moving all the way
-			// to the construction site first before telling us
-			// (since greyed-out icons can still be clicked etc,
-			// would be better to prevent that but doesn't cover
-			// case where limit reached while builder en-route)
+
+		if (!eventHandler.AllowUnitCreation(build.def, owner, &build)) {
+			StopMoveAndFinishCommand();
+			return;
+		}
+
+		if (teamHandler->Team(owner->team)->AtUnitLimit())
+			return;
+
+		CFeature* f = nullptr;
+
+		bool inWaitStance = false;
+		bool limitReached = false;
+
+		if (ownerBuilder->StartBuild(build, f, inWaitStance, limitReached) || (++buildRetries > 30)) {
+			building = true;
+			return;
+		}
+		// we can't reliably check if the unit-limit has been reached until
+		// the builder has reached the construction site, which is somewhat
+		// annoying (since greyed-out icons can still be clicked, etc)
+		if (limitReached) {
 			CancelRestrictedUnit();
 			StopMove();
 			return;
 		}
 
-		build.pos = CGameHelper::Pos2BuildPos(build, true);
+		if (f != nullptr && (!build.def->isFeature || build.def->wreckName != f->def->name)) {
+			inCommand = false;
+			ReclaimFeature(f);
+			return;
+		}
+		if (!inWaitStance) {
+			const float buildeeRadius = GetBuildOptionRadius(build.def, c.GetID());
+			const float fpSqRadius = (build.def->xsize * build.def->xsize + build.def->zsize * build.def->zsize);
+			const float fpRadius = (math::sqrt(fpSqRadius) * 0.5f) * SQUARE_SIZE;
 
-		// keep moving until 3D distance to buildPos is LEQ our buildDistance
-		if (MoveInBuildRange(build.pos, 0.0f, true)) {
-			if (IsBuildPosBlocked(build)) {
-				StopMoveAndFinishCommand();
-				return;
-			}
+			// tell everything within the radius of the soon-to-be buildee
+			// to get out of the way; using the model radius is not correct
+			// because this can be shorter than half the footprint diagonal
+			CGameHelper::BuggerOff(build.pos, std::max(buildeeRadius, fpRadius), false, true, owner->team, nullptr);
+			NonMoving();
+			return;
+		}
 
-			if (!eventHandler.AllowUnitCreation(build.def, owner, &build)) {
-				StopMoveAndFinishCommand();
-				return;
-			}
+		return;
+	}
 
-			if (teamHandler->Team(owner->team)->AtUnitLimit()) {
-				return;
-			}
 
-			CFeature* f = NULL;
+	if (owner->moveType->progressState == AMoveType::Failed) {
+		if (++buildRetries > 5) {
+			StopMoveAndFinishCommand();
+			return;
+		}
+	}
 
-			bool waitstance = false;
-			if (ownerBuilder->StartBuild(build, f, waitstance) || (++buildRetries > 30)) {
-				building = true;
-			}
-			else if (f != NULL && (!build.def->isFeature || build.def->wreckName != f->def->name)) {
-				inCommand = false;
-				ReclaimFeature(f);
-			}
-			else if (!waitstance) {
-				const float fpSqRadius = (build.def->xsize * build.def->xsize + build.def->zsize * build.def->zsize);
-				const float fpRadius = (math::sqrt(fpSqRadius) * 0.5f) * SQUARE_SIZE;
-
-				// tell everything within the radius of the soon-to-be buildee
-				// to get out of the way; using the model radius is not correct
-				// because this can be shorter than half the footprint diagonal
-				CGameHelper::BuggerOff(build.pos, std::max(buildeeRadius, fpRadius), false, true, owner->team, NULL);
-				NonMoving();
-			}
-		} else {
-			if (owner->moveType->progressState == AMoveType::Failed) {
-				if (++buildRetries > 5) {
-					StopMoveAndFinishCommand();
-					return;
-				}
-			}
-
-			// we are on the way to the buildpos, meanwhile it can happen
-			// that another builder already finished our buildcmd or blocked
-			// the buildpos with another building (skip our buildcmd then)
-			if ((++randomCounter % 5) == 0) {
-				if (IsBuildPosBlocked(build)) {
-					StopMoveAndFinishCommand();
-					return;
-				}
-			}
+	// we are on the way to the buildpos, meanwhile it can happen
+	// that another builder already finished our buildcmd or blocked
+	// the buildpos with another building (skip our buildcmd then)
+	if ((++randomCounter % 5) == 0) {
+		if (IsBuildPosBlocked(build)) {
+			StopMoveAndFinishCommand();
+			return;
 		}
 	}
 }
@@ -1176,10 +1180,8 @@ void CBuilderCAI::ExecuteFight(Command& c)
 		tempOrder = false;
 		inCommand = true;
 	}
-	if (c.params.size() < 3) { // this shouldnt happen but anyway ...
-		LOG_L(L_ERROR,
-				"Received a Fight command with less than 3 params on %s in BuilderCAI",
-				owner->unitDef->humanName.c_str());
+	if (c.params.size() < 3) {
+		LOG_L(L_ERROR, "[BuilderCAI::%s] invalid CMD_FIGHT for builder %d (%s)", __func__, owner->id, owner->unitDef->humanName.c_str());
 		return;
 	}
 
@@ -1324,37 +1326,14 @@ int CBuilderCAI::GetDefaultCmd(const CUnit* pointed, const CFeature* feature)
 }
 
 
-void CBuilderCAI::AddUnitToReclaimers(CUnit* unit)
-{
-	reclaimers.insert(unit);
-}
+void CBuilderCAI::AddUnitToReclaimers(CUnit* unit) { reclaimers.insert(unit->id); }
+void CBuilderCAI::RemoveUnitFromReclaimers(CUnit* unit) { reclaimers.erase(unit->id); }
 
+void CBuilderCAI::AddUnitToFeatureReclaimers(CUnit* unit) { featureReclaimers.insert(unit->id); }
+void CBuilderCAI::RemoveUnitFromFeatureReclaimers(CUnit* unit) { featureReclaimers.erase(unit->id); }
 
-void CBuilderCAI::RemoveUnitFromReclaimers(CUnit* unit)
-{
-	reclaimers.erase(unit);
-}
-
-
-void CBuilderCAI::AddUnitToFeatureReclaimers(CUnit* unit)
-{
-	featureReclaimers.insert(unit);
-}
-
-void CBuilderCAI::RemoveUnitFromFeatureReclaimers(CUnit* unit)
-{
-	featureReclaimers.erase(unit);
-}
-
-void CBuilderCAI::AddUnitToResurrecters(CUnit* unit)
-{
-	resurrecters.insert(unit);
-}
-
-void CBuilderCAI::RemoveUnitFromResurrecters(CUnit* unit)
-{
-	resurrecters.erase(unit);
-}
+void CBuilderCAI::AddUnitToResurrecters(CUnit* unit) { resurrecters.insert(unit->id); }
+void CBuilderCAI::RemoveUnitFromResurrecters(CUnit* unit) { resurrecters.erase(unit->id); }
 
 
 /**
@@ -1371,26 +1350,33 @@ void CBuilderCAI::RemoveUnitFromResurrecters(CUnit* unit)
 bool CBuilderCAI::IsUnitBeingReclaimed(const CUnit* unit, CUnit *friendUnit)
 {
 	bool retval = false;
-	std::list<CUnit*> rm;
 
-	for (CUnitSet::iterator it = reclaimers.begin(); it != reclaimers.end(); ++it) {
-		if ((*it)->commandAI->commandQue.empty()) {
-			rm.push_back(*it);
+	std::vector<int> rm;
+
+	for (auto it = reclaimers.begin(); it != reclaimers.end(); ++it) {
+		const CUnit* u = unitHandler->GetUnit(*it);
+		const CCommandAI* cai = u->commandAI;
+		const CCommandQueue& cq = cai->commandQue;
+
+		if (cq.empty()) {
+			rm.push_back(u->id);
 			continue;
 		}
-		const Command& c = (*it)->commandAI->commandQue.front();
+		const Command& c = cq.front();
 		if (c.GetID() != CMD_RECLAIM || (c.params.size() != 1 && c.params.size() != 5)) {
-			rm.push_back(*it);
+			rm.push_back(u->id);
 			continue;
 		}
 		const int cmdUnitId = (int)c.params[0];
-		if (cmdUnitId == unit->id && (!friendUnit || teamHandler->Ally(friendUnit->allyteam, (*it)->allyteam))) {
+		if (cmdUnitId == unit->id && (!friendUnit || teamHandler->Ally(friendUnit->allyteam, u->allyteam))) {
 			retval = true;
 			break;
 		}
 	}
-	for (std::list<CUnit*>::iterator it = rm.begin(); it != rm.end(); ++it)
-		RemoveUnitFromReclaimers(*it);
+
+	for (auto it = rm.begin(); it != rm.end(); ++it)
+		RemoveUnitFromReclaimers(unitHandler->GetUnit(*it));
+
 	return retval;
 }
 
@@ -1398,26 +1384,33 @@ bool CBuilderCAI::IsUnitBeingReclaimed(const CUnit* unit, CUnit *friendUnit)
 bool CBuilderCAI::IsFeatureBeingReclaimed(int featureId, CUnit *friendUnit)
 {
 	bool retval = false;
-	std::list<CUnit*> rm;
 
-	for (CUnitSet::iterator it = featureReclaimers.begin(); it != featureReclaimers.end(); ++it) {
-		if ((*it)->commandAI->commandQue.empty()) {
-			rm.push_back(*it);
+	std::vector<int> rm;
+
+	for (auto it = featureReclaimers.begin(); it != featureReclaimers.end(); ++it) {
+		const CUnit* u = unitHandler->GetUnit(*it);
+		const CCommandAI* cai = u->commandAI;
+		const CCommandQueue& cq = cai->commandQue;
+
+		if (cq.empty()) {
+			rm.push_back(u->id);
 			continue;
 		}
-		const Command& c = (*it)->commandAI->commandQue.front();
+		const Command& c = cq.front();
 		if (c.GetID() != CMD_RECLAIM || (c.params.size() != 1 && c.params.size() != 5)) {
-			rm.push_back(*it);
+			rm.push_back(u->id);
 			continue;
 		}
 		const int cmdFeatureId = (int)c.params[0];
-		if (cmdFeatureId-unitHandler->MaxUnits() == featureId && (!friendUnit || teamHandler->Ally(friendUnit->allyteam, (*it)->allyteam))) {
+		if (cmdFeatureId-unitHandler->MaxUnits() == featureId && (!friendUnit || teamHandler->Ally(friendUnit->allyteam, u->allyteam))) {
 			retval = true;
 			break;
 		}
 	}
-	for (std::list<CUnit*>::iterator it = rm.begin(); it != rm.end(); ++it)
-		RemoveUnitFromFeatureReclaimers(*it);
+
+	for (auto it = rm.begin(); it != rm.end(); ++it)
+		RemoveUnitFromFeatureReclaimers(unitHandler->GetUnit(*it));
+
 	return retval;
 }
 
@@ -1425,26 +1418,33 @@ bool CBuilderCAI::IsFeatureBeingReclaimed(int featureId, CUnit *friendUnit)
 bool CBuilderCAI::IsFeatureBeingResurrected(int featureId, CUnit *friendUnit)
 {
 	bool retval = false;
-	std::list<CUnit*> rm;
 
-	for (CUnitSet::iterator it = resurrecters.begin(); it != resurrecters.end(); ++it) {
-		if ((*it)->commandAI->commandQue.empty()) {
-			rm.push_back(*it);
+	std::vector<int> rm;
+
+	for (auto it = resurrecters.begin(); it != resurrecters.end(); ++it) {
+		const CUnit* u = unitHandler->GetUnit(*it);
+		const CCommandAI* cai = u->commandAI;
+		const CCommandQueue& cq = cai->commandQue;
+
+		if (cq.empty()) {
+			rm.push_back(u->id);
 			continue;
 		}
-		const Command& c = (*it)->commandAI->commandQue.front();
+		const Command& c = cq.front();
 		if (c.GetID() != CMD_RESURRECT || c.params.size() != 1) {
-			rm.push_back(*it);
+			rm.push_back(u->id);
 			continue;
 		}
 		const int cmdFeatureId = (int)c.params[0];
-		if (cmdFeatureId-unitHandler->MaxUnits() == featureId && (!friendUnit || teamHandler->Ally(friendUnit->allyteam, (*it)->allyteam))) {
+		if (cmdFeatureId-unitHandler->MaxUnits() == featureId && (!friendUnit || teamHandler->Ally(friendUnit->allyteam, u->allyteam))) {
 			retval = true;
 			break;
 		}
 	}
-	for (std::list<CUnit*>::iterator it = rm.begin(); it != rm.end(); ++it)
-		RemoveUnitFromResurrecters(*it);
+
+	for (auto it = rm.begin(); it != rm.end(); ++it)
+		RemoveUnitFromResurrecters(unitHandler->GetUnit(*it));
+
 	return retval;
 }
 
@@ -1477,9 +1477,9 @@ int CBuilderCAI::FindReclaimTarget(const float3& pos, float radius, unsigned cha
 	int rid = -1;
 
 	if (recUnits || recEnemy || recEnemyOnly) {
-		const std::vector<CUnit*>& units = quadField->GetUnitsExact(pos, radius, false);
-		for (std::vector<CUnit*>::const_iterator ui = units.begin(); ui != units.end(); ++ui) {
-			const CUnit* u = *ui;
+		QuadFieldQuery qfQuery;
+		quadField->GetUnitsExact(qfQuery, pos, radius, false);
+		for (const CUnit* u: *qfQuery.units) {
 
 			if (u == owner)
 				continue;
@@ -1517,10 +1517,10 @@ int CBuilderCAI::FindReclaimTarget(const float3& pos, float radius, unsigned cha
 	if ((!best || !stationary) && !recEnemyOnly) {
 		best = NULL;
 		const CTeam* team = teamHandler->Team(owner->team);
-		const std::vector<CFeature*>& features = quadField->GetFeaturesExact(pos, radius, false);
+		QuadFieldQuery qfQuery;
+		quadField->GetFeaturesExact(qfQuery, pos, radius, false);
 		bool metal = false;
-		for (std::vector<CFeature*>::const_iterator fi = features.begin(); fi != features.end(); ++fi) {
-			const CFeature* f = *fi;
+		for (const CFeature* f: *qfQuery.features) {
 			if (f->def->reclaimable && (recSpecial || f->def->autoreclaim) &&
 				(!recNonRez || !(f->def->destructable && f->udef != NULL))
 			) {
@@ -1588,13 +1588,13 @@ bool CBuilderCAI::FindResurrectableFeatureAndResurrect(const float3& pos,
                                                        unsigned char options,
 													   bool freshOnly)
 {
-	const std::vector<CFeature*> &features = quadField->GetFeaturesExact(pos, radius, false);
+	QuadFieldQuery qfQuery;
+	quadField->GetFeaturesExact(qfQuery, pos, radius, false);
 
 	const CFeature* best = NULL;
 	float bestDist = 1.0e30f;
 
-	for (std::vector<CFeature*>::const_iterator fi = features.begin(); fi != features.end(); ++fi) {
-		const CFeature* f = *fi;
+	for (const CFeature* f: *qfQuery.features) {
 		if (f->def->destructable && f->udef != NULL) {
 			if (!f->IsInLosForAllyTeam(owner->allyteam)) {
 				continue;
@@ -1631,16 +1631,15 @@ bool CBuilderCAI::FindCaptureTargetAndCapture(const float3& pos, float radius,
                                               unsigned char options,
 											  bool healthyOnly)
 {
-	const std::vector<CUnit*> &cu = quadField->GetUnitsExact(pos, radius, false);
+	QuadFieldQuery qfQuery;
+	quadField->GetUnitsExact(qfQuery, pos, radius, false);
 	std::vector<CUnit*>::const_iterator ui;
 
 	const CUnit* best = NULL;
 	float bestDist = 1.0e30f;
 	bool stationary = false;
 
-	for (ui = cu.begin(); ui != cu.end(); ++ui) {
-		CUnit* unit = *ui;
-
+	for (const CUnit* unit: *qfQuery.units) {
 		if ((((options & CONTROL_KEY) && owner->team != unit->team) ||
 			!teamHandler->Ally(owner->allyteam, unit->allyteam)) && (unit != owner) &&
 			(unit->losStatus[owner->allyteam] & (LOS_INRADAR|LOS_INLOS)) &&
@@ -1680,7 +1679,8 @@ bool CBuilderCAI::FindRepairTargetAndRepair(const float3& pos, float radius,
                                             bool attackEnemy,
 											bool builtOnly)
 {
-	const std::vector<CUnit*>& cu = quadField->GetUnitsExact(pos, radius, false);
+	QuadFieldQuery qfQuery;
+	quadField->GetUnitsExact(qfQuery, pos, radius, false);
 	const CUnit* bestUnit = NULL;
 
 	const float maxSpeed = owner->moveType->GetMaxSpeed();
@@ -1691,9 +1691,7 @@ bool CBuilderCAI::FindRepairTargetAndRepair(const float3& pos, float radius,
 	bool trySelfRepair = false;
 	bool stationary = false;
 
-	for (std::vector<CUnit*>::const_iterator ui = cu.begin(); ui != cu.end(); ++ui) {
-		const CUnit* unit = *ui;
-
+	for (const CUnit* unit: *qfQuery.units) {
 		if (teamHandler->Ally(owner->allyteam, unit->allyteam)) {
 			if (!haveEnemy && (unit->health < unit->maxHealth)) {
 				// don't help allies build unless set on roam
