@@ -314,6 +314,42 @@ public:
 		return false;
 	}
 
+	bool IsMoveCommand() const {
+		switch (id) {
+			case CMD_AREA_ATTACK:
+			case CMD_ATTACK:
+			case CMD_CAPTURE:
+			case CMD_FIGHT:
+			case CMD_GUARD:
+			case CMD_LOAD_UNITS:
+			case CMD_MANUALFIRE:
+			case CMD_MOVE:
+			case CMD_PATROL:
+			case CMD_RECLAIM:
+			case CMD_REPAIR:
+			case CMD_RESTORE:
+			case CMD_RESURRECT:
+			case CMD_UNLOAD_UNIT:
+			case CMD_UNLOAD_UNITS:
+				return true;
+
+			case CMD_DEATHWAIT:
+			case CMD_GATHERWAIT:
+			case CMD_SELFD:
+			case CMD_SQUADWAIT:
+			case CMD_STOP:
+			case CMD_TIMEWAIT:
+			case CMD_WAIT:
+				return false;
+
+			default:
+				// build commands are no different from reclaim or repair commands
+				// in that they can require a unit to move, so return true when we
+				// have one
+				return IsBuildCommand();
+		}
+	}
+
 	bool IsAreaCommand() const {
 		switch (id) {
 			case CMD_CAPTURE:
@@ -412,51 +448,4 @@ public:
 	#endif
 };
 
-
-struct CommandDescription {
-private:
-	CR_DECLARE_STRUCT(CommandDescription)
-
-public:
-	CommandDescription():
-		id(0),
-		type(CMDTYPE_ICON),
-		queueing(true),
-		hidden(false),
-		disabled(false),
-		showUnique(false),
-		onlyTexture(false) {}
-
-	/// CMD_xxx code (custom codes can also be used)
-	int id;
-	/// CMDTYPE_xxx code
-	int type;
-
-	/// command name
-	std::string name;
-	/// the associated command action binding name
-	std::string action;
-	/// button texture
-	std::string iconname;
-	/// mouse cursor
-	std::string mouseicon;
-	/// tooltip text
-	std::string tooltip;
-
-	bool queueing;
-	/// if true dont show a button for the command
-	bool hidden;
-	/// for greying-out commands
-	bool disabled;
-	/// command only applies to single units
-	bool showUnique;
-	/// do not draw the name if the texture is available
-	bool onlyTexture;
-
-	std::vector<std::string> params;
-};
-
-
-
 #endif // COMMAND_H
-
