@@ -9,7 +9,7 @@ class CBombDropper: public CWeapon
 {
 	CR_DECLARE_DERIVED(CBombDropper)
 public:
-	CBombDropper(CUnit* owner, const WeaponDef* def, bool useTorps);
+	CBombDropper(CUnit* owner = nullptr, const WeaponDef* def = nullptr, bool useTorps = false);
 
 	float GetPredictedImpactTime(float3 p) const override final;
 
@@ -18,7 +18,8 @@ private:
 
 	bool TestTarget(const float3 pos, const SWeaponTarget& trg) const override final;
 	bool TestRange(const float3 pos, const SWeaponTarget& trg) const override final;
-	bool HaveFreeLineOfFire(const float3 pos, const SWeaponTarget& trg, bool useMuzzle = false) const override final;
+	// TODO: requires sampling parabola from aimFromPos down to dropPos
+	bool HaveFreeLineOfFire(const float3 srcPos, const float3 tgtPos, const SWeaponTarget& trg) const override final { return true; }
 	void FireImpl(const bool scriptCall) override final;
 
 private:
