@@ -9,21 +9,16 @@
 #include "System/Platform/Threading.h"
 #include "System/GlobalConfig.h"
 
-using std::string;
-using std::vector;
-using std::map;
-
-
 CEventHandler eventHandler;
 
 
 /******************************************************************************/
 /******************************************************************************/
 
-void CEventHandler::SetupEvent(const string& eName,
-                               EventClientList* list, int props)
+void CEventHandler::SetupEvent(const std::string& eName, EventClientList* list, int props)
 {
-	eventMap[eName] = EventInfo(eName, list, props);
+	assert(std::find_if(eventMap.cbegin(), eventMap.cend(), [&](const EventPair& p) { return (p.first == eName); }) == eventMap.cend());
+	eventMap.push_back({eName, EventInfo(eName, list, props)});
 }
 
 /******************************************************************************/
@@ -31,21 +26,17 @@ void CEventHandler::SetupEvent(const string& eName,
 
 CEventHandler::CEventHandler()
 {
-	mouseOwner = NULL;
-
-	SetupEvents();
+	ResetState();
 }
 
-CEventHandler::~CEventHandler()
-{ }
-
-
 void CEventHandler::ResetState()
 {
-	mouseOwner = NULL;
+	mouseOwner = nullptr;
 
 	eventMap.clear();
+	eventMap.reserve(64);
 	handles.clear();
+	handles.reserve(16);
 
 	SetupEvents();
 }
@@ -55,8 +46,11 @@ void CEventHandler::SetupEvents()
 	#define SETUP_EVENT(name, props) SetupEvent(#name, &list ## name, props);
 	#define SETUP_UNMANAGED_EVENT(name, props) SetupEvent(#name, NULL, props);
 		#include "Events.def"
-	#undef SETUP_EVENT
 	#undef SETUP_UNMANAGED_EVENT
+	#undef SETUP_EVENT
+
+	// sort by name
+	std::stable_sort(eventMap.begin(), eventMap.end(), [](const EventPair& a, const EventPair& b) { return (a.first < b.first); });
 }
 
 
@@ -69,6 +63,7 @@ void CEventHandler::AddClient(CEventClient* ec)
 
 	for (auto it = eventMap.cbegin(); it != eventMap.cend(); ++it) {
 		const EventInfo& ei = it->second;
+
 		if (ei.HasPropBit(MANAGED_BIT)) {
 			if (ec->WantsEvent(it->first)) {
 				InsertEvent(ec, it->first);
@@ -79,9 +74,8 @@ void CEventHandler::AddClient(CEventClient* ec)
 
 void CEventHandler::RemoveClient(CEventClient* ec)
 {
-	if (mouseOwner == ec) {
-		mouseOwner = NULL;
-	}
+	if (mouseOwner == ec)
+		mouseOwner = nullptr;
 
 	ListRemove(handles, ec);
 
@@ -97,7 +91,7 @@ void CEventHandler::RemoveClient(CEventClient* ec)
 /******************************************************************************/
 /******************************************************************************/
 
-void CEventHandler::GetEventList(vector<string>& list) const
+void CEventHandler::GetEventList(std::vector<std::string>& list) const
 {
 	list.clear();
 
@@ -107,56 +101,66 @@ void CEventHandler::GetEventList(vector<string>& list) const
 }
 
 
-bool CEventHandler::IsKnown(const string& eName) const
+bool CEventHandler::IsKnown(const std::string& eName) const
 {
-	return (eventMap.find(eName) != eventMap.end());
+	// std::binary_search does not return an iterator
+	const auto comp = [](const EventPair& a, const EventPair& b) { return (a.first < b.first); };
+	const auto iter = std::lower_bound(eventMap.begin(), eventMap.end(), EventPair{eName, {}}, comp);
+	return (iter != eventMap.end() && iter->first == eName);
 }
 
 
-bool CEventHandler::IsManaged(const string& eName) const
+bool CEventHandler::IsManaged(const std::string& eName) const
 {
-	const EventMap::const_iterator it = eventMap.find(eName);
-	return ((it != eventMap.end()) && (it->second.HasPropBit(MANAGED_BIT)));
+	const auto comp = [](const EventPair& a, const EventPair& b) { return (a.first < b.first); };
+	const auto iter = std::lower_bound(eventMap.begin(), eventMap.end(), EventPair{eName, {}}, comp);
+	return (iter != eventMap.end() && iter->second.HasPropBit(MANAGED_BIT) && iter->first == eName);
 }
 
 
-bool CEventHandler::IsUnsynced(const string& eName) const
+bool CEventHandler::IsUnsynced(const std::string& eName) const
 {
-	const EventMap::const_iterator it = eventMap.find(eName);
-	return ((it != eventMap.end()) && (it->second.HasPropBit(UNSYNCED_BIT)));
+	const auto comp = [](const EventPair& a, const EventPair& b) { return (a.first < b.first); };
+	const auto iter = std::lower_bound(eventMap.begin(), eventMap.end(), EventPair{eName, {}}, comp);
+	return (iter != eventMap.end() && iter->second.HasPropBit(UNSYNCED_BIT) && iter->first == eName);
 }
 
 
-bool CEventHandler::IsController(const string& eName) const
+bool CEventHandler::IsController(const std::string& eName) const
 {
-	const EventMap::const_iterator it = eventMap.find(eName);
-	return ((it != eventMap.end()) && (it->second.HasPropBit(CONTROL_BIT)));
+	const auto comp = [](const EventPair& a, const EventPair& b) { return (a.first < b.first); };
+	const auto iter = std::lower_bound(eventMap.begin(), eventMap.end(), EventPair{eName, {}}, comp);
+	return (iter != eventMap.end() && iter->second.HasPropBit(CONTROL_BIT) && iter->first == eName);
 }
 
 
 /******************************************************************************/
 
-bool CEventHandler::InsertEvent(CEventClient* ec, const string& ciName)
+bool CEventHandler::InsertEvent(CEventClient* ec, const std::string& ciName)
 {
-	EventMap::iterator it = eventMap.find(ciName);
-	if ((it == eventMap.end()) || (it->second.GetList() == NULL)) {
+	const auto comp = [](const EventPair& a, const EventPair& b) { return (a.first < b.first); };
+	const auto iter = std::lower_bound(eventMap.begin(), eventMap.end(), EventPair{ciName, {}}, comp);
+
+	if ((iter == eventMap.end()) || (iter->second.GetList() == nullptr) || (iter->first != ciName))
 		return false;
-	}
-	if (ec->GetSynced() && it->second.HasPropBit(UNSYNCED_BIT)) {
+
+	if (ec->GetSynced() && iter->second.HasPropBit(UNSYNCED_BIT))
 		return false;
-	}
-	ListInsert(*it->second.GetList(), ec);
+
+	ListInsert(*iter->second.GetList(), ec);
 	return true;
 }
 
 
-bool CEventHandler::RemoveEvent(CEventClient* ec, const string& ciName)
+bool CEventHandler::RemoveEvent(CEventClient* ec, const std::string& ciName)
 {
-	EventMap::iterator it = eventMap.find(ciName);
-	if ((it == eventMap.end()) || (it->second.GetList() == NULL)) {
+	const auto comp = [](const EventPair& a, const EventPair& b) { return (a.first < b.first); };
+	const auto iter = std::lower_bound(eventMap.begin(), eventMap.end(), EventPair{ciName, {}}, comp);
+
+	if ((iter == eventMap.end()) || (iter->second.GetList() == nullptr) || (iter->first != ciName))
 		return false;
-	}
-	ListRemove(*it->second.GetList(), ec);
+
+	ListRemove(*(iter->second.GetList()), ec);
 	return true;
 }
 
@@ -165,19 +169,20 @@ bool CEventHandler::RemoveEvent(CEventClient* ec, const string& ciName)
 
 void CEventHandler::ListInsert(EventClientList& ecList, CEventClient* ec)
 {
-	EventClientList::iterator it;
-	for (it = ecList.begin(); it != ecList.end(); ++it) {
+	for (auto it = ecList.begin(); it != ecList.end(); ++it) {
 		const CEventClient* ecIt = *it;
-		if (ec == ecIt) {
+
+		if (ec == ecIt)
 			return; // already in the list
-		}
-		else if ((ec->GetOrder()  <  ecIt->GetOrder()) ||
+
+		if ((ec->GetOrder()  <  ecIt->GetOrder()) ||
 		         ((ec->GetOrder() == ecIt->GetOrder()) &&
 		          (ec->GetName()  <  ecIt->GetName()))) { // should not happen
 			ecList.insert(it, ec);
 			return;
 		}
 	}
+
 	ecList.push_back(ec);
 }
 
@@ -199,24 +204,30 @@ void CEventHandler::ListRemove(EventClientList& ecList, CEventClient* ec)
 /******************************************************************************/
 /******************************************************************************/
 
-#define CONTROL_ITERATE_DEF_TRUE(name, ...) \
-	bool result = true;                        \
-	for (int i = 0; i < list##name.size(); ) { \
-		CEventClient* ec = list##name[i];  \
-		result &= ec->name(__VA_ARGS__);   \
-		if (i < list##name.size() && ec == list##name[i]) \
+#define CONTROL_ITERATE_DEF_TRUE(name, ...)                        \
+	bool result = true;                                            \
+                                                                   \
+	for (size_t i = 0; i < list##name.size(); ) {                  \
+		CEventClient* ec = list##name[i];                          \
+		result &= ec->name(__VA_ARGS__);                           \
+                                                                   \
+		if (i < list##name.size() && ec == list##name[i])          \
 			++i; /* the call-in may remove itself from the list */ \
-	} \
+	}                                                              \
+                                                                   \
 	return result;
 
-#define CONTROL_ITERATE_DEF_FALSE(name, ...) \
-	bool result = false;                        \
-	for (int i = 0; i < list##name.size(); ) { \
-		CEventClient* ec = list##name[i];  \
-		result |= ec->name(__VA_ARGS__);   \
-		if (i < list##name.size() && ec == list##name[i]) \
+#define CONTROL_ITERATE_DEF_FALSE(name, ...)                       \
+	bool result = false;                                           \
+                                                                   \
+	for (size_t i = 0; i < list##name.size(); ) {                  \
+		CEventClient* ec = list##name[i];                          \
+		result |= ec->name(__VA_ARGS__);                           \
+                                                                   \
+		if (i < list##name.size() && ec == list##name[i])          \
 			++i; /* the call-in may remove itself from the list */ \
-	} \
+	}                                                              \
+                                                                   \
 	return result;
 
 
@@ -262,13 +273,13 @@ bool CEventHandler::AllowFeatureBuildStep(const CUnit* builder, const CFeature*
 }
 
 
-bool CEventHandler::AllowResourceLevel(int teamID, const string& type, float level)
+bool CEventHandler::AllowResourceLevel(int teamID, const std::string& type, float level)
 {
 	CONTROL_ITERATE_DEF_TRUE(AllowResourceLevel, teamID, type, level)
 }
 
 
-bool CEventHandler::AllowResourceTransfer(int oldTeam, int newTeam, const string& type, float amount)
+bool CEventHandler::AllowResourceTransfer(int oldTeam, int newTeam, const std::string& type, float amount)
 {
 	CONTROL_ITERATE_DEF_TRUE(AllowResourceTransfer, oldTeam, newTeam, type, amount)
 }
@@ -286,9 +297,9 @@ bool CEventHandler::AllowBuilderHoldFire(const CUnit* unit, int action)
 }
 
 
-bool CEventHandler::AllowStartPosition(int playerID, unsigned char readyState, const float3& clampedPos, const float3& rawPickPos)
+bool CEventHandler::AllowStartPosition(int playerID, int teamID, unsigned char readyState, const float3& clampedPos, const float3& rawPickPos)
 {
-	CONTROL_ITERATE_DEF_TRUE(AllowStartPosition, playerID, readyState, clampedPos, rawPickPos)
+	CONTROL_ITERATE_DEF_TRUE(AllowStartPosition, playerID, teamID, readyState, clampedPos, rawPickPos)
 }
 
 
@@ -308,13 +319,16 @@ bool CEventHandler::MoveCtrlNotify(const CUnit* unit, int data)
 int CEventHandler::AllowWeaponTargetCheck(unsigned int attackerID, unsigned int attackerWeaponNum, unsigned int attackerWeaponDefID)
 {
 	int result = -1;
-	for (int i = 0; i < listAllowWeaponTargetCheck.size(); ) {
+
+	for (size_t i = 0; i < listAllowWeaponTargetCheck.size(); ) {
 		CEventClient* ec = listAllowWeaponTargetCheck[i];
-		int result2 = ec->AllowWeaponTargetCheck(attackerID, attackerWeaponNum, attackerWeaponDefID);
-		if (result2 > result) result = result2;
+
+		result = std::max(result, ec->AllowWeaponTargetCheck(attackerID, attackerWeaponNum, attackerWeaponDefID));
+
 		if (i < listAllowWeaponTargetCheck.size() && ec == listAllowWeaponTargetCheck[i])
 			++i; /* the call-in may remove itself from the list */
 	}
+
 	return result;
 }
 
@@ -369,13 +383,15 @@ bool CEventHandler::ShieldPreDamaged(
 	const CUnit* shieldCarrier,
 	bool bounceProjectile,
 	const CWeapon* beamEmitter,
-	const CUnit* beamCarrier
+	const CUnit* beamCarrier,
+	const float3& startPos,
+	const float3& hitPos
 ) {
-	CONTROL_ITERATE_DEF_FALSE(ShieldPreDamaged, projectile, shieldEmitter, shieldCarrier, bounceProjectile, beamEmitter, beamCarrier)
+	CONTROL_ITERATE_DEF_FALSE(ShieldPreDamaged, projectile, shieldEmitter, shieldCarrier, bounceProjectile, beamEmitter, beamCarrier, startPos, hitPos)
 }
 
 
-bool CEventHandler::SyncedActionFallback(const string& line, int playerID)
+bool CEventHandler::SyncedActionFallback(const std::string& line, int playerID)
 {
 	for (int i = 0; i < listSyncedActionFallback.size(); ) {
 		CEventClient* ec = listSyncedActionFallback[i];
@@ -391,11 +407,12 @@ bool CEventHandler::SyncedActionFallback(const string& line, int playerID)
 /******************************************************************************/
 /******************************************************************************/
 
-#define ITERATE_EVENTCLIENTLIST(name, ...) \
-	for (int i = 0; i < list##name.size(); ) { \
-		CEventClient* ec = list##name[i]; \
-		ec->name(__VA_ARGS__); \
-		if (i < list##name.size() && ec == list##name[i]) \
+#define ITERATE_EVENTCLIENTLIST(name, ...)                         \
+	for (size_t i = 0; i < list##name.size(); ) {                  \
+		CEventClient* ec = list##name[i];                          \
+		ec->name(__VA_ARGS__);                                     \
+                                                                   \
+		if (i < list##name.size() && ec == list##name[i])          \
 			++i; /* the call-in may remove itself from the list */ \
 	}
 
@@ -532,28 +549,31 @@ void CEventHandler::GameProgress(int gameFrame)
 }
 
 
-#define DRAW_CALLIN(name)                                            \
-  void CEventHandler:: Draw ## name ()                               \
-  {                                                                  \
-    if (listDraw ## name.empty())                                    \
-      return;                                                        \
-    LuaOpenGL::EnableDraw ## name ();                                \
-    listDraw ## name [0]->Draw ## name ();                           \
-                                                                     \
-    for (int i = 1; i < listDraw ## name.size(); ) {                 \
-      LuaOpenGL::ResetDraw ## name ();                               \
-      CEventClient* ec = listDraw ## name [i];                       \
-      ec-> Draw ## name ();                                          \
-      if (i < listDraw ## name.size() && ec == listDraw ## name [i]) \
-	    ++i;                                                         \
-    }                                                                \
-                                                                     \
-    LuaOpenGL::DisableDraw ## name ();                               \
+#define DRAW_CALLIN(name)                                                   \
+	void CEventHandler:: Draw ## name ()                                    \
+	{                                                                       \
+		if (listDraw ## name.empty())                                       \
+			return;                                                         \
+                                                                            \
+		LuaOpenGL::EnableDraw ## name ();                                   \
+		listDraw ## name [0]->Draw ## name ();                              \
+                                                                            \
+		for (size_t i = 1; i < listDraw ## name.size(); ) {                 \
+			LuaOpenGL::ResetDraw ## name ();                                \
+			CEventClient* ec = listDraw ## name [i];                        \
+			ec-> Draw ## name ();                                           \
+                                                                            \
+			if (i < listDraw ## name.size() && ec == listDraw ## name [i])  \
+				++i;                                                        \
+		}                                                                   \
+                                                                            \
+		LuaOpenGL::DisableDraw ## name ();                                  \
   }
 
 DRAW_CALLIN(Genesis)
 DRAW_CALLIN(World)
 DRAW_CALLIN(WorldPreUnit)
+DRAW_CALLIN(WorldPreParticles)
 DRAW_CALLIN(WorldShadow)
 DRAW_CALLIN(WorldReflection)
 DRAW_CALLIN(WorldRefraction)
@@ -563,22 +583,26 @@ DRAW_CALLIN(GroundPostDeferred)
 DRAW_CALLIN(UnitsPostDeferred)
 DRAW_CALLIN(FeaturesPostDeferred)
 DRAW_CALLIN(ScreenEffects)
+DRAW_CALLIN(ScreenPost)
 DRAW_CALLIN(Screen)
 DRAW_CALLIN(InMiniMap)
 DRAW_CALLIN(InMiniMapBackground)
 
 
-#define DRAW_ENTITY_CALLIN(name, args, args2)     \
-  bool CEventHandler:: Draw ## name args        \
-  {                                               \
-    bool skipEngineDrawing = false;               \
-    for (int i = 0; i < listDraw ## name.size(); ) { \
-      CEventClient* ec = listDraw ## name [i];    \
-      skipEngineDrawing |= ec-> Draw ## name args2 ; \
-      if (i < listDraw ## name.size() && ec == listDraw ## name [i]) \
-	    ++i;                                      \
-    } \
-    return skipEngineDrawing; \
+#define DRAW_ENTITY_CALLIN(name, args, args2)                               \
+	bool CEventHandler:: Draw ## name args                                  \
+	{                                                                       \
+		bool skipEngineDrawing = false;                                     \
+                                                                            \
+		for (size_t i = 0; i < listDraw ## name.size(); ) {                 \
+			CEventClient* ec = listDraw ## name [i];                        \
+			skipEngineDrawing |= ec-> Draw ## name args2 ;                  \
+                                                                            \
+			if (i < listDraw ## name.size() && ec == listDraw ## name [i])  \
+				++i;                                                        \
+		}                                                                   \
+                                                                            \
+		return skipEngineDrawing;                                           \
   }
 
 DRAW_ENTITY_CALLIN(Unit, (const CUnit* unit), (unit))
@@ -589,19 +613,21 @@ DRAW_ENTITY_CALLIN(Projectile, (const CProjectile* projectile), (projectile))
 /******************************************************************************/
 /******************************************************************************/
 
-#define CONTROL_REVERSE_ITERATE_DEF_TRUE(name, ...) \
-	for (int i = list##name.size() - 1; i >= 0; --i) { \
-		CEventClient* ec = list##name[i]; \
-		if (ec->name(__VA_ARGS__)) \
-			return true; \
+#define CONTROL_REVERSE_ITERATE_DEF_TRUE(name, ...)                \
+	for (size_t i = 0; i < list##name.size(); ++i) {               \
+		CEventClient* ec = list##name[list##name.size() - 1 - i];  \
+                                                                   \
+		if (ec->name(__VA_ARGS__))                                 \
+			return true;                                           \
 	}
 
-#define CONTROL_REVERSE_ITERATE_STRING(name, ...) \
-	for (int i = list##name.size() - 1; i >= 0; --i) { \
-		CEventClient* ec = list##name[i]; \
-		const std::string& str = ec->name(__VA_ARGS__); \
-		if (!str.empty()) \
-			return str; \
+#define CONTROL_REVERSE_ITERATE_STRING(name, ...)                  \
+	for (size_t i = 0; i < list##name.size(); ++i) {               \
+		CEventClient* ec = list##name[list##name.size() - 1 - i];  \
+		const std::string& str = ec->name(__VA_ARGS__);            \
+                                                                   \
+		if (!str.empty())                                          \
+			return str;                                            \
 	}
 
 bool CEventHandler::CommandNotify(const Command& cmd)
@@ -634,32 +660,35 @@ bool CEventHandler::TextInput(const std::string& utf8)
 
 bool CEventHandler::MousePress(int x, int y, int button)
 {
-	for (int i = listMousePress.size() - 1; i >= 0; --i) {
-		CEventClient* ec = listMousePress[i];
-		if (ec->MousePress(x,y,button)) {
-			if (!mouseOwner)
+	for (size_t i = 0; i < listMousePress.size(); i++) {
+		CEventClient* ec = listMousePress[listMousePress.size() - 1 - i];
+
+		if (ec->MousePress(x, y, button)) {
+			if (mouseOwner == nullptr)
 				mouseOwner = ec;
+
 			return true;
 		}
 	}
+
 	return false;
 }
 
 
 void CEventHandler::MouseRelease(int x, int y, int button)
 {
-	if (mouseOwner) {
-		mouseOwner->MouseRelease(x, y, button);
-		mouseOwner = NULL;
-	}
+	if (mouseOwner == nullptr)
+		return;
+
+	mouseOwner->MouseRelease(x, y, button);
+	mouseOwner = nullptr;
 }
 
 
 bool CEventHandler::MouseMove(int x, int y, int dx, int dy, int button)
 {
-	if (mouseOwner == NULL) {
+	if (mouseOwner == nullptr)
 		return false;
-	}
 
 	return mouseOwner->MouseMove(x, y, dx, dy, button);
 }
@@ -683,7 +712,7 @@ bool CEventHandler::IsAbove(int x, int y)
 	return false;
 }
 
-string CEventHandler::GetTooltip(int x, int y)
+std::string CEventHandler::GetTooltip(int x, int y)
 {
 	CONTROL_REVERSE_ITERATE_STRING(GetTooltip, x, y)
 	return "";
@@ -714,14 +743,14 @@ bool CEventHandler::GroupChanged(int groupID)
 
 
 
-bool CEventHandler::GameSetup(const string& state, bool& ready,
-                                  const map<int, string>& playerStates)
+bool CEventHandler::GameSetup(const std::string& state, bool& ready,
+                                  const std::vector< std::pair<int, std::string> >& playerStates)
 {
 	CONTROL_REVERSE_ITERATE_DEF_TRUE(GameSetup, state, ready, playerStates)
 	return false;
 }
 
-void CEventHandler::DownloadQueued(int ID, const string& archiveName, const string& archiveType)
+void CEventHandler::DownloadQueued(int ID, const std::string& archiveName, const std::string& archiveType)
 {
 	ITERATE_EVENTCLIENTLIST(DownloadQueued, ID, archiveName, archiveType);
 }
@@ -746,7 +775,7 @@ void CEventHandler::DownloadProgress(int ID, long downloaded, long total)
 	ITERATE_EVENTCLIENTLIST(DownloadProgress, ID, downloaded, total);
 }
 
-string CEventHandler::WorldTooltip(const CUnit* unit,
+std::string CEventHandler::WorldTooltip(const CUnit* unit,
                                    const CFeature* feature,
                                    const float3* groundPos)
 {
@@ -757,7 +786,7 @@ string CEventHandler::WorldTooltip(const CUnit* unit,
 
 bool CEventHandler::MapDrawCmd(int playerID, int type,
                                const float3* pos0, const float3* pos1,
-                                   const string* label)
+                               const std::string* label)
 {
 	CONTROL_REVERSE_ITERATE_DEF_TRUE(MapDrawCmd, playerID, type, pos0, pos1, label)
 	return false;
