@@ -18,7 +18,7 @@
 #ifndef TOOLS
 	#include "VFSHandler.h"
 	#include "DataDirsAccess.h"
-	#include "System/Util.h"
+	#include "System/StringUtil.h"
 	#include "System/Platform/Misc.h"
 #endif
 
@@ -30,24 +30,19 @@ using std::string;
 /******************************************************************************/
 
 CFileHandler::CFileHandler(const char* fileName, const char* modes)
-	: filePos(0), fileSize(-1)
 {
+	Close();
 	Open(fileName, modes);
 }
 
 
 CFileHandler::CFileHandler(const string& fileName, const string& modes)
-	: filePos(0), fileSize(-1)
 {
+	Close();
 	Open(fileName, modes);
 }
 
 
-CFileHandler::~CFileHandler()
-{
-	ifs.close();
-}
-
 
 /******************************************************************************/
 
@@ -89,18 +84,15 @@ bool CFileHandler::TryReadFromRawFS(const string& fileName)
 }
 
 
-bool CFileHandler::TryReadFromModFS(const string& fileName)
+bool CFileHandler::TryReadFromVFS(const string& fileName, int section)
 {
 #ifndef TOOLS
-	if (vfsHandler == NULL) {
+	if (vfsHandler == nullptr)
 		return false;
-	}
 
-	const string file = StringToLower(fileName);
-	if (vfsHandler->LoadFile(file, fileBuffer)) {
-		// did we allocated more mem than needed
-		// (e.g. because of incorrect usage of std::vector)?
-		assert(fileBuffer.size() == fileBuffer.capacity());
+	if (vfsHandler->LoadFile(StringToLower(fileName), fileBuffer, (CVFSHandler::Section) section)) {
+		// capacity can exceed size if FH was used to open more than one file
+		// assert(fileBuffer.size() == fileBuffer.capacity());
 
 		fileSize = fileBuffer.size();
 		return true;
@@ -110,71 +102,64 @@ bool CFileHandler::TryReadFromModFS(const string& fileName)
 }
 
 
-bool CFileHandler::TryReadFromMapFS(const string& fileName)
-{
-	return TryReadFromModFS(fileName); // FIXME
-}
-
-
-bool CFileHandler::TryReadFromBaseFS(const string& fileName)
-{
-	return TryReadFromModFS(fileName); // FIXME
-}
-
-
 void CFileHandler::Open(const string& fileName, const string& modes)
 {
 	this->fileName = fileName;
-	const char* c = modes.c_str();
-	while (c[0] != 0) {
-		if ((c[0] == SPRING_VFS_RAW[0])  && TryReadFromRawFS(fileName))  break;
-		if ((c[0] == SPRING_VFS_MOD[0])  && TryReadFromModFS(fileName))  break;
-		if ((c[0] == SPRING_VFS_MAP[0])  && TryReadFromMapFS(fileName))  break;
-		if ((c[0] == SPRING_VFS_BASE[0]) && TryReadFromBaseFS(fileName)) break;
-		if ((c[0] == SPRING_VFS_PWD[0])  && TryReadFromPWD(fileName)) break;
-		c++;
+	for (char c: modes) {
+#ifndef TOOLS
+		CVFSHandler::Section section = CVFSHandler::GetModeSection(c);
+		if ((section != CVFSHandler::Section::Error) && TryReadFromVFS(fileName, section))
+			break;
+
+		if ((c == SPRING_VFS_RAW[0]) && TryReadFromRawFS(fileName))
+			break;
+
+#endif
+		if ((c == SPRING_VFS_PWD[0]) && TryReadFromPWD(fileName))
+			break;
 	}
 }
 
+void CFileHandler::Close()
+{
+	filePos = 0;
+	fileSize = -1;
+
+	ifs.close();
+	fileBuffer.clear();
+}
+
+
 
 /******************************************************************************/
 
 bool CFileHandler::FileExists(const std::string& filePath, const std::string& modes)
 {
-	bool fileExists = false;
-
+	for (char c: modes) {
 #ifndef TOOLS
-	// VFS
-	if (modes.find_first_of(SPRING_VFS_ZIP) != string::npos) {
-		fileExists = fileExists || vfsHandler->FileExists(filePath);
-	}
+		CVFSHandler::Section section = CVFSHandler::GetModeSection(c);
+		if ((section != CVFSHandler::Section::Error) && vfsHandler->FileExists(filePath, section))
+			return true;
 
-	// Raw
-	if (modes.find(SPRING_VFS_RAW) != string::npos) {
-		const std::string rawpath = dataDirsAccess.LocateFile(filePath);
-		fileExists = fileExists || FileSystem::FileExists(rawpath);
-	}
+		if ((c == SPRING_VFS_RAW[0]) && FileSystem::FileExists(dataDirsAccess.LocateFile(filePath)))
+			return true;
 #endif
-
-	// PWD
-	if (modes.find(SPRING_VFS_PWD) != string::npos) {
-	#ifndef TOOLS
-		if (FileSystem::IsAbsolutePath(filePath))
-			return false;
-		const std::string fullpath(Platform::GetOrigCWD() + filePath);
-	#else
-		const std::string fullpath(filePath);
-	#endif
-		fileExists = fileExists || FileSystem::FileExists(fullpath);
+		if (c == SPRING_VFS_PWD[0]) {
+#ifndef TOOLS
+			if (!FileSystem::IsAbsolutePath(filePath)) {
+				const std::string fullpath(Platform::GetOrigCWD() + filePath);
+				if (FileSystem::FileExists(fullpath))
+					return true;
+			}
+#else
+			const std::string fullpath(filePath);
+			if (FileSystem::FileExists(fullpath))
+				return true;
+#endif
+		}
 	}
 
-	return fileExists;
-}
-
-
-bool CFileHandler::FileExists() const
-{
-	return (fileSize >= 0);
+	return false;
 }
 
 
@@ -184,19 +169,20 @@ int CFileHandler::Read(void* buf, int length)
 		ifs.read(static_cast<char*>(buf), length);
 		return ifs.gcount();
 	}
-	else if (!fileBuffer.empty()) {
-		if ((length + filePos) > fileSize) {
-			length = fileSize - filePos;
-		}
-		if (length > 0) {
-			assert(fileBuffer.size() >= (filePos + length));
-			memcpy(buf, &fileBuffer[filePos], length);
-			filePos += length;
-		}
-		return length;
+
+	if (fileBuffer.empty())
+		return 0;
+
+	if ((length + filePos) > fileSize)
+		length = fileSize - filePos;
+
+	if (length > 0) {
+		assert(fileBuffer.size() >= (filePos + length));
+		memcpy(buf, &fileBuffer[filePos], length);
+		filePos += length;
 	}
 
-	return 0;
+	return length;
 }
 
 
@@ -218,64 +204,51 @@ int CFileHandler::ReadString(void* buf, int length)
 
 void CFileHandler::Seek(int length, std::ios_base::seekdir where)
 {
-	if (ifs.is_open())
-	{
+	if (ifs.is_open()) {
 		// Status bits must be cleared before seeking, otherwise it might fail
 		// in the common case of EOF
 		// http://en.cppreference.com/w/cpp/io/basic_istream/seekg
 		ifs.clear();
 		ifs.seekg(length, where);
+		return;
 	}
-	else if (!fileBuffer.empty())
-	{
-		if (where == std::ios_base::beg)
-		{
-			filePos = length;
-		}
-		else if (where == std::ios_base::cur)
-		{
-			filePos += length;
-		}
-		else if (where == std::ios_base::end)
-		{
-			filePos = fileSize + length;
-		}
+	if (fileBuffer.empty())
+		return;
+
+	switch (where) {
+		case std::ios_base::beg: { filePos = length; } break;
+		case std::ios_base::cur: { filePos += length; } break;
+		case std::ios_base::end: { filePos = fileSize + length; } break;
+		default: {} break;
 	}
 }
 
 bool CFileHandler::Eof() const
 {
-	if (ifs.is_open()) {
+	if (ifs.is_open())
 		return ifs.eof();
-	}
-	if (!fileBuffer.empty()) {
+
+	if (!fileBuffer.empty())
 		return (filePos >= fileSize);
-	}
+
 	return true;
 }
 
 
-int CFileHandler::FileSize() const
-{
-	return fileSize;
-}
-
-
 int CFileHandler::GetPos()
 {
-	if (ifs.is_open()) {
+	if (ifs.is_open())
 		return ifs.tellg();
-	} else {
-		return filePos;
-	}
+
+	return filePos;
 }
 
 
 bool CFileHandler::LoadStringData(string& data)
 {
-	if (!FileExists()) {
+	if (!FileExists())
 		return false;
-	}
+
 	char* buf = new char[fileSize];
 	Read(buf, fileSize);
 	data.append(buf, fileSize);
@@ -294,21 +267,7 @@ std::vector<string> CFileHandler::FindFiles(const string& path,
 		const string& pattern)
 {
 #ifndef TOOLS
-	std::vector<string> found = dataDirsAccess.FindFiles(path, pattern);
-	boost::regex regexpattern(FileSystem::ConvertGlobToRegex(pattern),
-			boost::regex::icase);
-	std::vector<string> f;
-
-	if (vfsHandler) {
-		f = vfsHandler->GetFilesInDir(path);
-	}
-
-	for (std::vector<string>::iterator fi = f.begin(); fi != f.end(); ++fi) {
-		if (boost::regex_match(*fi, regexpattern)) {
-			found.push_back(path + *fi);
-		}
-	}
-	return found;
+	return DirList(path, pattern, SPRING_VFS_ALL);
 #else
 	return std::vector<std::string>();
 #endif
@@ -320,22 +279,24 @@ std::vector<string> CFileHandler::FindFiles(const string& path,
 std::vector<string> CFileHandler::DirList(const string& path,
 		const string& pattern, const string& modes)
 {
-	const string pat = pattern.empty() ? "*" : pattern;
 
-	std::set<string> fileSet;
-	const char* c = modes.c_str();
-	while (c[0] != 0) {
-		if (c[0] == SPRING_VFS_RAW[0])  { InsertRawFiles(fileSet, path, pat);  }
-		if (c[0] == SPRING_VFS_MOD[0])  { InsertModFiles(fileSet, path, pat);  }
-		if (c[0] == SPRING_VFS_MAP[0])  { InsertMapFiles(fileSet, path, pat);  }
-		if (c[0] == SPRING_VFS_BASE[0]) { InsertBaseFiles(fileSet, path, pat); }
-		c++;
-	}
 	std::vector<string> fileVec;
+#ifndef TOOLS
+	const string pat = pattern.empty() ? "*" : pattern;
+	std::set<string> fileSet;
+	for (char c: modes) {
+		CVFSHandler::Section section = CVFSHandler::GetModeSection(c);
+		if (section != CVFSHandler::Section::Error)
+			InsertVFSFiles(fileSet, path, pat, section);
+
+		if (c == SPRING_VFS_RAW[0])
+			InsertRawFiles(fileSet, path, pat);
+	}
 	std::set<string>::const_iterator it;
 	for (it = fileSet.begin(); it != fileSet.end(); ++it) {
 		fileVec.push_back(*it);
 	}
+#endif
 	return fileVec;
 }
 
@@ -344,23 +305,18 @@ bool CFileHandler::InsertRawFiles(std::set<string>& fileSet,
 		const string& path, const string& pattern)
 {
 #ifndef TOOLS
-	boost::regex regexpattern(FileSystem::ConvertGlobToRegex(pattern),
-	                          boost::regex::icase);
-
 	const std::vector<string> &found = dataDirsAccess.FindFiles(path, pattern);
 	std::vector<string>::const_iterator fi;
 	for (fi = found.begin(); fi != found.end(); ++fi) {
-		if (boost::regex_match(*fi, regexpattern)) {
-			fileSet.insert(fi->c_str());
-		}
+		fileSet.insert(fi->c_str());
 	}
 #endif
 	return true;
 }
 
 
-bool CFileHandler::InsertModFiles(std::set<string>& fileSet,
-		const string& path, const string& pattern)
+bool CFileHandler::InsertVFSFiles(std::set<string>& fileSet,
+		const string& path, const string& pattern, int section)
 {
 #ifndef TOOLS
 	if (!vfsHandler) {
@@ -375,7 +331,7 @@ bool CFileHandler::InsertModFiles(std::set<string>& fileSet,
 	boost::regex regexpattern(FileSystem::ConvertGlobToRegex(pattern),
 			boost::regex::icase);
 
-	const std::vector<string> &found = vfsHandler->GetFilesInDir(path);
+	const std::vector<string> &found = vfsHandler->GetFilesInDir(path, (CVFSHandler::Section) section);
 	std::vector<string>::const_iterator fi;
 	for (fi = found.begin(); fi != found.end(); ++fi) {
 		if (boost::regex_match(*fi, regexpattern)) {
@@ -387,41 +343,29 @@ bool CFileHandler::InsertModFiles(std::set<string>& fileSet,
 }
 
 
-bool CFileHandler::InsertMapFiles(std::set<string>& fileSet,
-		const string& path, const string& pattern)
-{
-	return InsertModFiles(fileSet, path, pattern); // FIXME
-}
-
-
-bool CFileHandler::InsertBaseFiles(std::set<string>& fileSet,
-		const string& path, const string& pattern)
-{
-	return InsertModFiles(fileSet, path, pattern); // FIXME
-}
-
-
 /******************************************************************************/
 
 std::vector<string> CFileHandler::SubDirs(const string& path,
 		const string& pattern, const string& modes)
 {
+	std::vector<string> dirVec;
+#ifndef TOOLS
 	const string pat = pattern.empty() ? "*" : pattern;
 
 	std::set<string> dirSet;
-	const char* c = modes.c_str();
-	while (c[0] != 0) {
-		if (c[0] == SPRING_VFS_RAW[0])  { InsertRawDirs(dirSet, path, pat);  }
-		if (c[0] == SPRING_VFS_MOD[0])  { InsertModDirs(dirSet, path, pat);  }
-		if (c[0] == SPRING_VFS_MAP[0])  { InsertMapDirs(dirSet, path, pat);  }
-		if (c[0] == SPRING_VFS_BASE[0]) { InsertBaseDirs(dirSet, path, pat); }
-		c++;
+	for (char c: modes) {
+		CVFSHandler::Section section = CVFSHandler::GetModeSection(c);
+		if (section != CVFSHandler::Section::Error)
+			InsertVFSDirs(dirSet, path, pat, section);
+
+		if (c == SPRING_VFS_RAW[0])
+			InsertRawDirs(dirSet, path, pat);
 	}
-	std::vector<string> dirVec;
 	std::set<string>::const_iterator it;
 	for (it = dirSet.begin(); it != dirSet.end(); ++it) {
 		dirVec.push_back(*it);
 	}
+#endif
 	return dirVec;
 }
 
@@ -447,8 +391,8 @@ bool CFileHandler::InsertRawDirs(std::set<string>& dirSet,
 }
 
 
-bool CFileHandler::InsertModDirs(std::set<string>& dirSet,
-		const string& path, const string& pattern)
+bool CFileHandler::InsertVFSDirs(std::set<string>& dirSet,
+		const string& path, const string& pattern, int section)
 {
 #ifndef TOOLS
 	if (!vfsHandler) {
@@ -463,7 +407,7 @@ bool CFileHandler::InsertModDirs(std::set<string>& dirSet,
 	boost::regex regexpattern(FileSystem::ConvertGlobToRegex(pattern),
 			boost::regex::icase);
 
-	const std::vector<string> &found = vfsHandler->GetDirsInDir(path);
+	const std::vector<string> &found = vfsHandler->GetDirsInDir(path, (CVFSHandler::Section) section);
 	std::vector<string>::const_iterator fi;
 	for (fi = found.begin(); fi != found.end(); ++fi) {
 		if (boost::regex_match(*fi, regexpattern)) {
@@ -475,20 +419,6 @@ bool CFileHandler::InsertModDirs(std::set<string>& dirSet,
 }
 
 
-bool CFileHandler::InsertMapDirs(std::set<string>& dirSet,
-		const string& path, const string& pattern)
-{
-	return InsertModDirs(dirSet, path, pattern); // FIXME
-}
-
-
-bool CFileHandler::InsertBaseDirs(std::set<string>& dirSet,
-		const string& path, const string& pattern)
-{
-	return InsertModDirs(dirSet, path, pattern); // FIXME
-}
-
-
 /******************************************************************************/
 
 string CFileHandler::AllowModes(const string& modes, const string& allowed)
