@@ -2,7 +2,7 @@
 
 #include "System/Info.h"
 
-#include "System/Util.h"
+#include "System/StringUtil.h"
 #include "System/Exceptions.h"
 #include "System/Log/ILog.h"
 #include "Lua/LuaParser.h"
