@@ -4,7 +4,8 @@
 #define MOUSE_INPUT_H
 
 #include <SDL_events.h>
-#include <boost/signals2/connection.hpp>
+#include <slimsig/connection.h>
+#include "System/Input/InputHandler.h"
 
 #include "System/type2.h"
 
@@ -15,7 +16,7 @@ public:
 	static void FreeInstance(IMouseInput*);
 
 	IMouseInput ();
-	virtual ~IMouseInput() {}
+	virtual ~IMouseInput();
 
 	virtual void InstallWndCallback() {}
 
@@ -25,12 +26,12 @@ public:
 	bool HandleSDLMouseEvent(const SDL_Event& event);
 
 	virtual void SetWMMouseCursor(void* wmcursor) {}
-	
+
 protected:
 	int2 mousepos;
-	boost::signals2::connection inputCon;
+	InputHandler::SignalType::connection_type inputCon;
 };
 
 extern IMouseInput* mouseInput;
 
-#endif 
+#endif
