@@ -57,6 +57,8 @@ public:
 	 * Log()s system informations (CPU, 32/64bit, gcc/boost version, ...)
 	 */
 	static void LogSystemInfo();
+	static void LogConfigInfo();
+	static void LogExceptionInfo(const char* src, const char* msg);
 
 private:
 	/**
