@@ -1,48 +1,44 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
 #include "SpringTime.h"
-#include "System/maindefines.h"
+#include "System/MainDefines.h"
 #include "System/myMath.h"
 
-#include <boost/thread/mutex.hpp>
+#define FORCE_CHRONO_TIMERS
 
+#include <chrono>
+#include <atomic>
 
-#ifndef UNIT_TEST
-#ifdef USING_CREG
-#include "System/creg/Serializer.h"
+#if defined(USING_CREG) && !defined(UNIT_TEST)
+	#include "System/creg/Serializer.h"
 
-//FIXME always use class even in non-debug! for creg!
-CR_BIND(spring_time, )
-CR_REG_METADATA(spring_time,(
-	CR_IGNORED(x),
-	CR_SERIALIZER(Serialize)
-))
-#endif
+	//FIXME always use class even in non-debug! for creg!
+	CR_BIND(spring_time, )
+	CR_REG_METADATA(spring_time,(
+		CR_IGNORED(x),
+		CR_SERIALIZER(Serialize)
+	))
 #endif
 
 
-// mingw doesn't support std::this_thread (yet?)
-#if defined(__MINGW32__) || defined(SPRINGTIME_USING_BOOST)
-	#undef gt
-	#include <boost/thread/thread.hpp>
-	namespace this_thread { using namespace boost::this_thread; };
-#else
-	#define SPRINGTIME_USING_STD_SLEEP
-	#ifdef _GLIBCXX_USE_SCHED_YIELD
-	#undef _GLIBCXX_USE_SCHED_YIELD
-	#endif
-	#define _GLIBCXX_USE_SCHED_YIELD // workaround a gcc <4.8 bug
-	#include <thread>
-	#include <mutex>
-	namespace this_thread { using namespace std::this_thread; }
+#if (defined(WIN32) && !defined(FORCE_CHRONO_TIMERS))
+	#define USE_NATIVE_WINDOWS_CLOCK 1
 #endif
 
-#define USE_NATIVE_WINDOWS_CLOCK (defined(WIN32) && !defined(FORCE_CHRONO_TIMERS))
 #if USE_NATIVE_WINDOWS_CLOCK
-#include <windows.h>
+	#include <windows.h>
 #endif
 
 
+#ifdef _GLIBCXX_USE_SCHED_YIELD
+#undef _GLIBCXX_USE_SCHED_YIELD
+#endif
+#define _GLIBCXX_USE_SCHED_YIELD // workaround a gcc <4.8 bug
+
+#include "System/Threading/SpringThreading.h"
+namespace this_thread { using namespace std::this_thread; }
+
+
 
 namespace spring_clock {
 	static bool highResMode = false;
@@ -76,7 +72,7 @@ namespace spring_clock {
 	#if USE_NATIVE_WINDOWS_CLOCK
 	// QPC wants the LARGE_INTEGER's to be qword-aligned
 	__FORCE_ALIGN_STACK__
-	boost::int64_t GetTicksNative() {
+	std::int64_t GetTicksNative() {
 		assert(timerInited);
 
 		if (highResMode) {
@@ -111,7 +107,7 @@ namespace spring_clock {
 			LARGE_INTEGER currTick;
 
 			if (!QueryPerformanceFrequency(&tickFreq))
-				return (FromMilliSecs<boost::int64_t>(0));
+				return (FromMilliSecs<std::int64_t>(0));
 
 			QueryPerformanceCounter(&currTick);
 
@@ -127,11 +123,11 @@ namespace spring_clock {
 			//
 			// currTick.QuadPart /= tickFreq.QuadPart;
 
-			if (tickFreq.QuadPart >= boost::int64_t(1e9)) return (FromNanoSecs <boost::uint64_t>(std::max(0.0, currTick.QuadPart / (tickFreq.QuadPart * 1e-9))));
-			if (tickFreq.QuadPart >= boost::int64_t(1e6)) return (FromMicroSecs<boost::uint64_t>(std::max(0.0, currTick.QuadPart / (tickFreq.QuadPart * 1e-6))));
-			if (tickFreq.QuadPart >= boost::int64_t(1e3)) return (FromMilliSecs<boost::uint64_t>(std::max(0.0, currTick.QuadPart / (tickFreq.QuadPart * 1e-3))));
+			if (tickFreq.QuadPart >= std::int64_t(double(1e9))) return (FromNanoSecs <std::uint64_t>(std::max(double(0.0), currTick.QuadPart / (tickFreq.QuadPart * double(1e-9)))));
+			if (tickFreq.QuadPart >= std::int64_t(double(1e6))) return (FromMicroSecs<std::uint64_t>(std::max(double(0.0), currTick.QuadPart / (tickFreq.QuadPart * double(1e-6)))));
+			if (tickFreq.QuadPart >= std::int64_t(double(1e3))) return (FromMilliSecs<std::uint64_t>(std::max(double(0.0), currTick.QuadPart / (tickFreq.QuadPart * double(1e-3)))));
 
-			return (FromSecs<boost::int64_t>(std::max(0LL, currTick.QuadPart)));
+			return (FromSecs<std::int64_t>(std::max(0LL, currTick.QuadPart)));
 		} else {
 			// timeGetTime is affected by time{Begin,End}Period whereas
 			// GetTickCount is not ---> resolution of the former can be
@@ -143,12 +139,12 @@ namespace spring_clock {
 			// risk of overflowing)
 			//
 			// note: there is a GetTickCount64 but no timeGetTime64
-			return (FromMilliSecs<boost::uint32_t>(timeGetTime()));
+			return (FromMilliSecs<std::uint32_t>(timeGetTime()));
 		}
 	}
 	#endif
 
-	boost::int64_t GetTicks() {
+	std::int64_t GetTicks() {
 		assert(timerInited);
 
 		#if USE_NATIVE_WINDOWS_CLOCK
@@ -170,27 +166,18 @@ namespace spring_clock {
 		}
 
 		#else
-
-		#ifdef SPRINGTIME_USING_BOOST
-		return "boost::chrono::high_resolution_clock";
-		#endif
-		#ifdef SPRINGTIME_USING_STDCHRONO
 		return "std::chrono::high_resolution_clock";
 		#endif
-
-		#endif
 	}
 }
 
 
 
-boost::int64_t spring_time::xs = 0;
+std::int64_t spring_time::xs = 0;
 
-static float avgThreadYieldTimeMilliSecs = 0.0f;
-static float avgThreadSleepTimeMilliSecs = 0.0f;
+static std::atomic_int avgThreadYieldTimeMicroSecs = {0};
+static std::atomic_int avgThreadSleepTimeMicroSecs = {0};
 
-static boost::mutex yieldTimeMutex;
-static boost::mutex sleepTimeMutex;
 
 static void thread_yield()
 {
@@ -200,16 +187,24 @@ static void thread_yield()
 	const spring_time dt = t1 - t0;
 
 	if (t1 >= t0) {
-		boost::mutex::scoped_lock lock(yieldTimeMutex);
-		avgThreadYieldTimeMilliSecs = mix(avgThreadYieldTimeMilliSecs, dt.toMilliSecsf(), 0.1f);
+		// yes, it's not 100% thread correct, but it's okay when 1 of 1 million writes is dropped
+		int avg = avgThreadYieldTimeMicroSecs.load();
+		int newAvg = mix<float>(avg, dt.toMicroSecsf(), 0.1f);
+		avgThreadYieldTimeMicroSecs.store(newAvg);
 	}
 }
 
 
-void spring_time::sleep()
+void spring_time::sleep(bool forceThreadSleep)
 {
+	if (forceThreadSleep) {
+		spring::this_thread::sleep_for(chrono::nanoseconds(toNanoSecsi()));
+		return;
+	}
+
+
 	// for very short time intervals use a yielding loop (yield is ~5x more accurate than sleep(), check the UnitTest)
-	if (toMilliSecsf() < (avgThreadSleepTimeMilliSecs + avgThreadYieldTimeMilliSecs * 5.0f)) {
+	if (toMicroSecsi() < (avgThreadSleepTimeMicroSecs + avgThreadYieldTimeMicroSecs * 5)) {
 		const spring_time s = gettime();
 
 		while ((gettime() - s) < *this)
@@ -221,39 +216,23 @@ void spring_time::sleep()
 	// expected wakeup time
 	const spring_time t0 = gettime() + *this;
 
-	#if defined(SPRINGTIME_USING_STD_SLEEP)
-		this_thread::sleep_for(chrono::nanoseconds(toNanoSecsi()));
-	#else
-		boost::this_thread::sleep(boost::posix_time::microseconds(std::ceil(toNanoSecsf() * 1e-3)));
-	#endif
+	spring::this_thread::sleep_for(chrono::nanoseconds(toNanoSecsi()));
 
 	const spring_time t1 = gettime();
 	const spring_time dt = t1 - t0;
 
 	if (t1 >= t0) {
-		boost::mutex::scoped_lock lock(sleepTimeMutex);
-		avgThreadSleepTimeMilliSecs = mix(avgThreadSleepTimeMilliSecs, dt.toMilliSecsf(), 0.1f);
+		// yes, it's not 100% thread correct, but it's okay when 1 of 1 million writes is dropped
+		int avg = avgThreadSleepTimeMicroSecs.load();
+		int newAvg = mix<float>(avg, dt.toMicroSecsf(), 0.1f);
+		avgThreadSleepTimeMicroSecs.store(newAvg);
 	}
 }
 
 void spring_time::sleep_until()
 {
-#if defined(SPRINGTIME_USING_STD_SLEEP)
 	auto tp = chrono::time_point<chrono::high_resolution_clock, chrono::nanoseconds>(chrono::nanoseconds(toNanoSecsi()));
 	this_thread::sleep_until(tp);
-#else
-	spring_time napTime = gettime() - *this;
-
-	if (napTime.toMilliSecsf() < avgThreadYieldTimeMilliSecs) {
-		while (napTime.isTime()) {
-			thread_yield();
-			napTime = gettime() - *this;
-		}
-		return;
-	}
-
-	napTime.sleep();
-#endif
 }
 
 #if defined USING_CREG && !defined UNIT_TEST
