@@ -3,11 +3,9 @@
 #ifndef _UDP_LISTENER_H
 #define _UDP_LISTENER_H
 
-#include <boost/noncopyable.hpp>
-#include <boost/shared_ptr.hpp>
-#include <boost/weak_ptr.hpp>
-#include <boost/asio/ip/udp.hpp>
-#include <list>
+#include "System/Misc/NonCopyable.h"
+#include <memory>
+#include <asio/ip/udp.hpp>
 #include <map>
 #include <queue>
 #include <string>
@@ -15,7 +13,7 @@
 namespace netcode
 {
 class UDPConnection;
-typedef boost::shared_ptr<boost::asio::ip::udp::socket> SocketPtr;
+typedef std::shared_ptr<asio::ip::udp::socket> SocketPtr;
 
 /**
  * @brief Class for handling Connections on an UDPSocket
@@ -24,7 +22,7 @@ typedef boost::shared_ptr<boost::asio::ip::udp::socket> SocketPtr;
  * You can Listen for new connections, initiate new ones and send/recieve data
  * to/from them.
  */
-class UDPListener : boost::noncopyable
+class UDPListener : spring::noncopyable
 {
 public:
 	/**
@@ -37,7 +35,7 @@ public:
 	/**
 	 * @brief close the socket and DELETE all connections
 	 */
-	~UDPListener() {}
+	~UDPListener();
 
 	/**
 	 * Try to bind a socket to a local address and port.
@@ -49,8 +47,7 @@ public:
 	 *         the default value "" results in the v6 any address "::",
 	 *         or the v4 equivalent "0.0.0.0", if v6 is no supported
 	 */
-	static std::string TryBindSocket(int port, SocketPtr* socket,
-			const std::string& ip = "");
+	static std::string TryBindSocket(int port, SocketPtr* socket, const std::string& ip = "");
 
 	/**
 	 * @brief Run this from time to time
@@ -63,8 +60,7 @@ public:
 	 * @brief Initiate a connection
 	 * Make a new connection to ip:port. It will be pushed back in conn.
 	 */
-	boost::shared_ptr<UDPConnection> SpawnConnection(const std::string& ip,
-			const unsigned port);
+	std::shared_ptr<UDPConnection> SpawnConnection(const std::string& ip, const unsigned port);
 
 	/**
 	 * Set if we are accepting new connections
@@ -72,12 +68,12 @@ public:
 	 */
 	void SetAcceptingConnections(const bool enable);
 	bool IsAcceptingConnections() const;
-
 	bool HasIncomingConnections() const;
-	boost::weak_ptr<UDPConnection> PreviewConnection();
-	boost::shared_ptr<UDPConnection> AcceptConnection();
-	void RejectConnection();
 
+	std::weak_ptr<UDPConnection> PreviewConnection();
+	std::shared_ptr<UDPConnection> AcceptConnection();
+
+	void RejectConnection();
 	void UpdateConnections(); // Updates connections when the endpoint has been reconnected
 
 private:
@@ -88,14 +84,14 @@ private:
 	bool acceptNewConnections;
 
 	/// Our socket
-	/// typedef boost::shared_ptr<boost::asio::ip::udp::socket> SocketPtr;
+	/// typedef std::shared_ptr<asio::ip::udp::socket> SocketPtr;
 	SocketPtr mySocket;
 
 	/// all connections
-	typedef std::map< boost::asio::ip::udp::endpoint, boost::weak_ptr<UDPConnection> > ConnMap;
-	ConnMap conn;
+	std::map< asio::ip::udp::endpoint, std::weak_ptr<UDPConnection> > connMap;
+	std::map< std::string, size_t> dropMap;
 
-	std::queue< boost::shared_ptr<UDPConnection> > waiting;
+	std::queue< std::shared_ptr<UDPConnection> > waiting;
 };
 
 }
