@@ -3,15 +3,20 @@
 #include "lib/streflop/streflop_cond.h" //! must happen before OffscreenGLContext.h, which includes agl.h
 #include "System/OffscreenGLContext.h"
 
+#include <functional>
+
 #include "System/Exceptions.h"
-#include "System/maindefines.h"
+#include "System/MainDefines.h"
+#include "System/SafeUtil.h"
 #include "System/Log/ILog.h"
 #include "System/Platform/errorhandler.h"
 #include "System/Platform/Threading.h"
-#include <boost/thread.hpp>
+#include "System/Threading/SpringThreading.h"
 
 
+#ifndef HEADLESS
 static PFNGLACTIVETEXTUREPROC mainGlActiveTexture = NULL;
+#endif
 
 
 #ifdef HEADLESS
@@ -146,7 +151,7 @@ void COffscreenGLContext::WorkerThreadPost()
 {
 	CGLError err = CGLSetCurrentContext(cglWorkerCtx);
 	if (kCGLNoError != err) {
-		LOG_L(L_ERROR, CGLErrorString(err));
+		LOG_L(L_ERROR, "%s", CGLErrorString(err));
 		throw opengl_error("Could not activate worker rendering context");
 	}
 }
@@ -156,7 +161,7 @@ void COffscreenGLContext::WorkerThreadFree()
 {
 	CGLError err = CGLSetCurrentContext(NULL);
 	if (kCGLNoError != err) {
-		LOG_L(L_ERROR, CGLErrorString(err));
+		LOG_L(L_ERROR, "%s", CGLErrorString(err));
 		throw opengl_error("Could not deactivate worker rendering context");
 	}
 }
@@ -265,54 +270,42 @@ void COffscreenGLContext::WorkerThreadFree()
 /******************************************************************************/
 /******************************************************************************/
 
-COffscreenGLThread::COffscreenGLThread(boost::function<void()> f) :
+COffscreenGLThread::COffscreenGLThread(std::function<void()> f) :
 	thread(NULL),
 	glOffscreenCtx() //! may trigger an opengl_error exception!
 {
-	thread = new boost::thread( boost::bind(&COffscreenGLThread::WrapFunc, this, f) );
+	thread = new spring::thread( std::bind(&COffscreenGLThread::WrapFunc, this, f) );
 }
 
 
 COffscreenGLThread::~COffscreenGLThread()
 {
-	if (thread)
-		Join();
-	delete thread; thread = NULL;
-}
-
-
-bool COffscreenGLThread::IsFinished(boost::posix_time::time_duration wait)
-{
-	return thread->timed_join(wait);
+	Join();
 }
 
 
 void COffscreenGLThread::Join()
 {
-	while(thread->joinable())
-		if(thread->timed_join(boost::posix_time::seconds(1)))
-			break;
+	if (thread)
+		thread->join();
+	spring::SafeDelete(thread);
 }
 
 
 __FORCE_ALIGN_STACK__
-void COffscreenGLThread::WrapFunc(boost::function<void()> f)
+void COffscreenGLThread::WrapFunc(std::function<void()> f)
 {
 	Threading::SetThreadName("OffscreenGLThread");
 
 	glOffscreenCtx.WorkerThreadPost();
 
-#ifdef STREFLOP_H
 	// init streflop
 	// not needed for sync'ness (precision flags are per-process)
 	// but fpu exceptions are per-thread
 	streflop::streflop_init<streflop::Simple>();
-#endif
 
 	try {
 		f();
-	} catch(boost::thread_interrupted const&) {
-		// do nothing
 	} CATCH_SPRING_ERRORS
 
 	glOffscreenCtx.WorkerThreadFree();
