@@ -14,7 +14,7 @@ namespace CrashHandler {
 	void Stacktrace(Threading::NativeThreadHandle thread, const std::string& threadName, const int logLevel = LOG_LEVEL_ERROR);
 	void PrepareStacktrace(const int logLevel = LOG_LEVEL_ERROR);
 	void CleanupStacktrace(const int logLevel = LOG_LEVEL_ERROR);
-
+	
 #ifndef WIN32
 	/*
 	 * The following method breaks the Stacktrace() interface, but it is unavoidable since we need to provide the
@@ -25,6 +25,8 @@ namespace CrashHandler {
 	 *   the parameter is specific to the needs of one platform.
 	 */
 	void SuspendedStacktrace(Threading::ThreadControls* ctls, const std::string& threadName);
+#else
+	bool InitImageHlpDll();
 #endif
 
 	void OutputStacktrace();
