@@ -14,21 +14,18 @@
 #endif
 
 #include "System/Platform/Win/win32.h"
+#include "System/Threading/SpringThreading.h"
 #include <functional>
 #include <atomic>
-#include <boost/thread.hpp>
-#include <boost/cstdint.hpp>
+#include <cinttypes>
 
 
 
-class CGameController;
-
-
 namespace Threading {
 
 	class ThreadControls;
 
-	extern boost::thread_specific_ptr<std::shared_ptr<Threading::ThreadControls>> threadCtls;
+	extern thread_local std::shared_ptr<Threading::ThreadControls> threadCtls;
 
 	/**
 	 * Generic types & functions to handle OS native threads
@@ -53,14 +50,14 @@ namespace Threading {
 	};
 
 	/**
-	 * Creates a new boost::thread whose entry function is wrapped by some boilerplate code that allows for suspend/resume.
+	 * Creates a new spring::thread whose entry function is wrapped by some boilerplate code that allows for suspend/resume.
 	 * These suspend/resume controls are exposed via the ThreadControls object that is provided by the caller and initialized by the thread.
 	 * The thread is guaranteed to be in a running and initialized state when this function returns.
 	 *
 	 * The ppThreadCtls object is an optional return parameter that gives access to the Suspend/Resume controls under Linux.
 	 *
 	 */
-	boost::thread CreateNewThread(boost::function<void()> taskFunc, std::shared_ptr<Threading::ThreadControls>* ppThreadCtls = nullptr);
+	spring::thread CreateNewThread(std::function<void()> taskFunc, std::shared_ptr<Threading::ThreadControls>* ppThreadCtls = nullptr);
 
 	/**
 	 * Retrieves a shared pointer to the current ThreadControls for the calling thread.
@@ -87,17 +84,11 @@ namespace Threading {
 		NativeThreadHandle      handle;
 		std::atomic<bool>       running;
 	#ifndef WIN32
-		boost::mutex            mutSuspend;
-		boost::condition_variable condInitialized;
+		spring::mutex            mutSuspend;
+		spring::condition_variable condInitialized;
 		ucontext_t              ucontext;
 		pid_t                   thread_id;
 	#endif
-
-		friend void ThreadStart(
-			boost::function<void()> taskFunc,
-			std::shared_ptr<ThreadControls>* ppCtlsReturn,
-			ThreadControls* tempCtls
-		);
 	};
 
 	inline bool NativeThreadIdsEqual(const NativeThreadId thID1, const NativeThreadId thID2);
@@ -111,10 +102,10 @@ namespace Threading {
 	 * want to run. Note that this approach will fail when N > 32.
 	 */
 	void DetectCores();
-	boost::uint32_t GetAffinity();
-	boost::uint32_t SetAffinity(boost::uint32_t cores_bitmask, bool hard = true);
-	void SetAffinityHelper(const char* threadName, boost::uint32_t affinity);
-	boost::uint32_t GetAvailableCoresMask();
+	std::uint32_t GetAffinity();
+	std::uint32_t SetAffinity(std::uint32_t cores_bitmask, bool hard = true);
+	void SetAffinityHelper(const char* threadName, std::uint32_t affinity);
+	std::uint32_t GetAvailableCoresMask();
 
 	/**
 	 * returns count of cpu cores/ hyperthreadings cores
@@ -123,11 +114,6 @@ namespace Threading {
 	int GetLogicalCpuCores();  /// physical + hyperthreading
 	bool HasHyperThreading();
 
-	/**
-	 * threadpool related stuff
-	 */
-	void InitThreadPool();
-
 	/**
 	 * Inform the OS kernel that we are a cpu-intensive task
 	 */
@@ -137,14 +123,19 @@ namespace Threading {
 	 * Used to detect the main-thread which runs SDL, GL, Input, Sim, ...
 	 */
 	void SetMainThread();
+	void SetGameLoadThread();
+	void SetAudioThread();
+	void SetWatchDogThread();
+
 	bool IsMainThread();
 	bool IsMainThread(NativeThreadId threadID);
 
-	void SetGameLoadThread();
 	bool IsGameLoadThread();
 	bool IsGameLoadThread(NativeThreadId threadID);
 
-	void SetWatchDogThread();
+	bool IsAudioThread();
+	bool IsAudioThread(NativeThreadId threadID);
+
 	bool IsWatchDogThread();
 	bool IsWatchDogThread(NativeThreadId threadID);
 
@@ -160,18 +151,14 @@ namespace Threading {
 		Error() : flags(0) {}
 		Error(const std::string& _caption, const std::string& _message, const unsigned int _flags) : caption(_caption), message(_message), flags(_flags) {}
 
+		bool Empty() const { return (caption.empty() && message.empty() && flags == 0); }
+
 		std::string caption;
 		std::string message;
 		unsigned int flags;
 	};
 	void SetThreadError(const Error& err);
 	Error* GetThreadError();
-
-
-	/**
-	 * A 64bit atomic counter
-	 */
-	struct AtomicCounterInt64;
 }
 
 
@@ -196,54 +183,6 @@ namespace Threading {
 
 		return (thID1 == thID2);
 	}
-
-
-	struct AtomicCounterInt64 {
-	public:
-		AtomicCounterInt64(boost::int64_t start = 0) : num(start) {}
-
-		// prefix
-		boost::int64_t operator++() {
-	#ifdef _MSC_VER
-			return InterlockedIncrement64(&num);
-	#elif defined(__APPLE__)
-			return OSAtomicIncrement64(&num);
-	#else // assuming GCC (__sync_add_and_fetch is a builtin)
-			return __sync_add_and_fetch(&num, boost::int64_t(1));
-	#endif
-		}
-
-		boost::int64_t operator+=(int x) {
-	#ifdef _MSC_VER
-			return InterlockedExchangeAdd64(&num, boost::int64_t(x));
-	#elif defined(__APPLE__)
-			return OSAtomicAdd64(boost::int64_t(x), &num);
-	#else // assuming GCC (__sync_fetch_and_add is a builtin)
-			return __sync_fetch_and_add(&num, boost::int64_t(x));
-	#endif
-		}
-
-		boost::int64_t operator-=(int x) {
-	#ifdef _MSC_VER
-			return InterlockedExchangeAdd64(&num, boost::int64_t(-x));
-	#elif defined(__APPLE__)
-			return OSAtomicAdd64(boost::int64_t(-x), &num);
-	#else // assuming GCC (__sync_fetch_and_add is a builtin)
-			return __sync_fetch_and_add(&num, boost::int64_t(-x));
-	#endif
-		}
-
-		operator boost::int64_t() {
-			return num;
-		}
-
-	private:
-	#ifdef _MSC_VER
-		__declspec(align(8)) boost::int64_t num;
-	#else
-		__attribute__ ((aligned (8))) boost::int64_t num;
-	#endif
-	};
 }
 
 #endif // _THREADING_H_
