@@ -9,10 +9,9 @@
 // Update Watchdog::threadNames also if adding threads
 enum WatchdogThreadnum {
 	WDT_MAIN  = 0,
-	WDT_SIM   = 1,
-	WDT_LOAD  = 2,
-	WDT_AUDIO = 3,
-	WDT_COUNT = 4,
+	WDT_LOAD  = 1,
+	WDT_AUDIO = 2,
+	WDT_COUNT = 3,
 };
 
 namespace Watchdog
@@ -22,7 +21,7 @@ namespace Watchdog
 	void Uninstall();
 
 	//! Call this to reset the watchdog timer of the current thread
-	void ClearTimer(bool disable = false, Threading::NativeThreadId* _threadId = NULL);
+	void ClearTimer(bool disable = false, Threading::NativeThreadId* _threadId = nullptr);
 	void ClearTimer(const WatchdogThreadnum num, bool disable = false);
 	void ClearTimer(const std::string& name, bool disable = false);
 	inline void ClearTimer(const char* name, bool disable = false) {
@@ -32,7 +31,7 @@ namespace Watchdog
 
 	//! Call these in the threads you want to monitor
 	void RegisterThread(WatchdogThreadnum num, bool primary = false);
-	void DeregisterThread(WatchdogThreadnum num);
+	bool DeregisterThread(WatchdogThreadnum num);
 }
 
 #endif // _WATCHDOG_H
