@@ -1,6 +1,6 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "maindefines.h"
+#include "MainDefines.h"
 #include <string.h>
 
 /*
