@@ -5,6 +5,7 @@
 
 // AIs which want to use creg have to specify this when compiling:
 // '-DUSING_CREG'
+
 #if defined BUILDING_AI && !defined USING_CREG
 	#if !defined NOT_USING_CREG
 		#define NOT_USING_CREG
@@ -17,9 +18,11 @@
 #endif // defined BUILDING_AI && !defined USING_CREG
 
 #ifdef NOT_USING_CREG
+#undef USING_CREG
 #include "ISerializer.h" // prevent some compiler errors
 // define all creg preprocessor macros from creg_cond.h to nothing
 #define CR_DECLARE(TCls)
+#define CR_DECLARE_DERIVED(TCls)
 #define CR_DECLARE_STRUCT(TStr)
 #define CR_DECLARE_SUB(cl)
 #define CR_BIND_DERIVED(TCls, TBase, ctor_args)
@@ -32,7 +35,6 @@
 #define CR_MEMBER(Member)
 #define CR_IGNORED(Member)
 #define CR_MEMBER_UN(Member)
-#define CR_RESERVED(Size)
 #define CR_SETFLAG(Flag)
 #define CR_MEMBER_SETFLAG(Member, Flag)
 #define CR_MEMBER_BEGINFLAG(Flag)
