@@ -91,7 +91,7 @@ bool Window::HandleEventSelf(const SDL_Event& ev)
 		case SDL_KEYDOWN: {
 			if (ev.key.keysym.sym == SDLK_ESCAPE)
 			{
-				WantClose();
+				WantClose.emit();
 				return true;
 			}
 			break;
