@@ -1,234 +1,278 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include <map>
 #include <array>
-#include <boost/cstdint.hpp>
-#include <boost/thread.hpp>
+#include <cinttypes>
+
 #include "lib/streflop/streflop_cond.h"
 
 #include "LuaInclude.h"
-#include "Game/GameVersion.h"
 #include "Lua/LuaHandle.h"
+#include "Lua/LuaMemPool.h"
+
 #include "System/myMath.h"
-#include "System/Platform/Threading.h"
-#include "System/Threading/SpringMutex.h"
-#include "System/Log/ILog.h"
-#if (!defined(DEDICATED) && !defined(UNITSYNC) && !defined(BUILDING_AI))
-	#include "System/Misc/SpringTime.h"
+
+#if (ENABLE_USERSTATE_LOCKS != 0)
+	#include "System/UnorderedMap.hpp"
+	#include "System/Threading/SpringThreading.h"
 #endif
 
+#include "System/Log/ILog.h"
+#include "System/Misc/SpringTime.h"
+
+#if defined(DEDICATED) || defined(UNITSYNC) || defined(BUILDING_AI)
+#error liblua should be built only once!
+#endif
+
+
+
+
 
 ///////////////////////////////////////////////////////////////////////////
 // Custom Lua Mutexes
 
-static std::map<lua_State*, spring::recursive_mutex*> mutexes;
-static std::map<lua_State*, bool> coroutines;
+#if (ENABLE_USERSTATE_LOCKS != 0)
+static spring::unsynced_map<lua_State*, bool> coroutines;
+static spring::unsynced_map<lua_State*, spring::recursive_mutex*> mutexes;
 
 static spring::recursive_mutex* GetLuaMutex(lua_State* L)
 {
-	assert(!mutexes[L]);
+	assert(mutexes[L] == nullptr);
 	return new spring::recursive_mutex();
 }
-
+#endif
 
 
 void LuaCreateMutex(lua_State* L)
 {
-	#if (ENABLE_USERSTATE_LOCKS == 0)
-	// if LoadingMT=1, everything runs in the game-load thread (on startup)
-	//assert(Threading::IsMainThread() || Threading::IsGameLoadThread() || SpringVersion::IsUnitsync());
-	return;
-	#endif
-
+#if (ENABLE_USERSTATE_LOCKS != 0)
 	luaContextData* lcd = GetLuaContextData(L);
-	if (!lcd) return; // CLuaParser
-	assert(lcd);
+
+	if (lcd == nullptr)
+		return; // CLuaParser
+
+	assert(lcd != nullptr);
 
 	spring::recursive_mutex* mutex = GetLuaMutex(L);
 	lcd->luamutex = mutex;
 	mutexes[L] = mutex;
+#endif
 }
 
 
 void LuaDestroyMutex(lua_State* L)
 {
-	#if (ENABLE_USERSTATE_LOCKS == 0)
-	//assert(Threading::IsMainThread() || Threading::IsGameLoadThread() || SpringVersion::IsUnitsync());
-	return;
-	#endif
+#if (ENABLE_USERSTATE_LOCKS != 0)
+	if (GetLuaContextData(L) == nullptr)
+		return; // CLuaParser
 
-	if (!GetLuaContextData(L)) return; // CLuaParser
-	assert(GetLuaContextData(L));
+	assert(GetLuaContextData(L) != nullptr);
 
 	if (coroutines.find(L) != coroutines.end()) {
 		mutexes.erase(L);
 		coroutines.erase(L);
-	} else {
-		lua_unlock(L);
-		assert(mutexes.find(L) != mutexes.end());
-		spring::recursive_mutex* mutex = GetLuaContextData(L)->luamutex;
-		assert(mutex);
-		delete mutex;
-		mutexes.erase(L);
-		//TODO erase all related coroutines too?
+		return;
 	}
+
+	lua_unlock(L);
+	assert(mutexes.find(L) != mutexes.end());
+	spring::recursive_mutex* mutex = GetLuaContextData(L)->luamutex;
+	assert(mutex);
+	delete mutex;
+	mutexes.erase(L);
+	//TODO erase all related coroutines too?
+#endif
 }
 
 
 void LuaLinkMutex(lua_State* L_parent, lua_State* L_child)
 {
-	#if (ENABLE_USERSTATE_LOCKS == 0)
-	//assert(Threading::IsMainThread() || Threading::IsGameLoadThread() || SpringVersion::IsUnitsync());
-	return;
-	#endif
-
+#if (ENABLE_USERSTATE_LOCKS != 0)
 	luaContextData* plcd = GetLuaContextData(L_parent);
-	assert(plcd);
-
 	luaContextData* clcd = GetLuaContextData(L_child);
-	assert(clcd);
 
+	assert(plcd != nullptr);
+	assert(clcd != nullptr);
 	assert(plcd == clcd);
 
 	coroutines[L_child] = true;
 	mutexes[L_child] = plcd->luamutex;
+#endif
 }
 
 
 void LuaMutexLock(lua_State* L)
 {
-	#if (ENABLE_USERSTATE_LOCKS == 0)
-	//assert(Threading::IsMainThread() || Threading::IsGameLoadThread() || SpringVersion::IsUnitsync());
-	return;
-	#endif
+#if (ENABLE_USERSTATE_LOCKS != 0)
 
-	if (!GetLuaContextData(L)) return; // CLuaParser
+	if (GetLuaContextData(L) == nullptr)
+		return; // CLuaParser
 
 	spring::recursive_mutex* mutex = GetLuaContextData(L)->luamutex;
 
 	if (mutex->try_lock())
 		return;
 
-	//static int failedLocks = 0;
-	//LOG("LuaMutexLock %i", ++failedLocks);
 	mutex->lock();
+#endif
 }
 
 
 void LuaMutexUnlock(lua_State* L)
 {
-	#if (ENABLE_USERSTATE_LOCKS == 0)
-	//assert(Threading::IsMainThread() || Threading::IsGameLoadThread() || SpringVersion::IsUnitsync());
-	return;
-	#endif
-
-	if (!GetLuaContextData(L)) return; // CLuaParser
+#if (ENABLE_USERSTATE_LOCKS != 0)
+	if (GetLuaContextData(L) == nullptr)
+		return; // CLuaParser
 
 	spring::recursive_mutex* mutex = GetLuaContextData(L)->luamutex;
 	mutex->unlock();
+#endif
 }
 
 
 void LuaMutexYield(lua_State* L)
 {
-	#if (ENABLE_USERSTATE_LOCKS == 0)
-	//assert(Threading::IsMainThread() || Threading::IsGameLoadThread() || SpringVersion::IsUnitsync());
-	return;
-	#endif
-
+#if (ENABLE_USERSTATE_LOCKS != 0)
 	assert(GetLuaContextData(L));
 	/*mutexes[L]->unlock();
 	if (!mutexes[L]->try_lock()) {
 		// only yield if another thread is waiting for the mutex
-		boost::this_thread::yield();
+		spring::this_thread::yield();
 		mutexes[L]->lock();
 	}*/
 
 	static int count = 0;
-	bool y = false;
-	if (count-- <= 0) { y = true; count = 30; }
+
+	if (count-- <= 0)
+		count = 30;
+
 	LuaMutexUnlock(L);
 
-	if (y) boost::this_thread::yield();
+	if (count == 30)
+		spring::this_thread::yield();
+
 	LuaMutexLock(L);
+#endif
 }
 
 
 ///////////////////////////////////////////////////////////////////////////
 //
 
-const char* spring_lua_getName(lua_State* L)
-{
-	auto ld = GetLuaContextData(L);
-	if (ld) {
-		return ld->owner->GetName().c_str();
-	}
-
-	static const char* c = "";
-	return c;
+const char* spring_lua_getHandleName(const CLuaHandle* h) {
+	return ((h != nullptr)? (h->GetName()).c_str(): "<null>");
 }
 
+const char* spring_lua_getHandleName(lua_State* L)
+{
+	const luaContextData* lcd = GetLuaContextData(L);
+
+	if (lcd != nullptr)
+		return (spring_lua_getHandleName(lcd->owner));
+
+	return "";
+}
+
+
 
 ///////////////////////////////////////////////////////////////////////////
 // Custom Memory Allocator
 //
-// these track allocations across all states
-static Threading::AtomicCounterInt64 totalBytesAlloced = 0;
-static Threading::AtomicCounterInt64 totalNumLuaAllocs = 0;
-static Threading::AtomicCounterInt64 totalLuaAllocTime = 0;
+static constexpr uint32_t maxAllocedBytes = 768u * (1024u * 1024u);
+static constexpr const char* maxAllocFmtStr = "[%s][handle=%s][OOM] synced=%d {alloced,maximum}={%u,%u}bytes\n";
 
-static const unsigned int maxAllocedBytes = 768u * 1024u*1024u;
-static const char* maxAllocFmtStr = "%s: cannot allocate more memory! (%u bytes already used, %u bytes maximum)";
+// tracks allocations across all states
+static SLuaAllocState gLuaAllocState = {};
+static SLuaAllocError gLuaAllocError = {};
 
+void spring_lua_alloc_log_error(const luaContextData* lcd)
+{
+	const CLuaHandle* lho = lcd->owner;
+
+	const char* lhn = spring_lua_getHandleName(lho);
+	const char* fmt = maxAllocFmtStr;
+
+	SLuaAllocState& s = gLuaAllocState;
+	SLuaAllocError& e = gLuaAllocError;
+
+	if (e.msgPtr == nullptr)
+		e.msgPtr = &e.msgBuf[0];
+
+	// append to buffer until it fills up or get_error is called
+	e.msgPtr += SNPRINTF(e.msgPtr, sizeof(e.msgBuf) - (e.msgPtr - &e.msgBuf[0]), fmt, __func__, lhn, lcd->synced, uint32_t(s.allocedBytes), maxAllocedBytes);
+}
 
 void* spring_lua_alloc(void* ud, void* ptr, size_t osize, size_t nsize)
 {
-	auto lcd = (luaContextData*) ud;
+	luaContextData* lcd = static_cast<luaContextData*>(ud);
+	LuaMemPool* lmp = lcd->memPool;
+
+	gLuaAllocState.allocedBytes -= osize;
+	gLuaAllocState.allocedBytes += nsize;
 
 	if (nsize == 0) {
-		totalBytesAlloced -= osize;
-		free(ptr);
-		return NULL;
+		// deallocation; must return NULL
+		lmp->Free(ptr, osize);
+		return nullptr;
 	}
 
-	if ((nsize > osize) && (totalBytesAlloced > maxAllocedBytes)) {
-		// better kill Lua than whole engine
-		// NOTE: this will trigger luaD_throw --> exit(EXIT_FAILURE)
-		LOG_L(L_FATAL, maxAllocFmtStr, (lcd->owner->GetName()).c_str(), (unsigned int) totalBytesAlloced, maxAllocedBytes);
-		return NULL;
+	if ((nsize > osize) && (gLuaAllocState.allocedBytes.load() > maxAllocedBytes)) {
+		// (re)allocation
+		// better kill Lua than whole engine; instant desync if synced handle
+		// NOTE: this will trigger luaD_throw, which calls exit(EXIT_FAILURE)
+		spring_lua_alloc_log_error(lcd);
+		return nullptr;
 	}
 
-	#if (!defined(DEDICATED) && !defined(UNITSYNC) && !defined(BUILDING_AI))
+	// ptr is NULL if and only if osize is zero
+	// behaves like realloc when nsize!=0 and osize!=0 (ptr != NULL)
+	// behaves like malloc when nsize!=0 and osize==0 (ptr == NULL)
 	const spring_time t0 = spring_gettime();
-	void* mem = realloc(ptr, nsize);
+	void* mem = lmp->Realloc(ptr, nsize, osize);
 	const spring_time t1 = spring_gettime();
 
-	totalBytesAlloced += (nsize - osize);
-	totalNumLuaAllocs += 1;
-	totalLuaAllocTime += (t1 - t0).toMicroSecsi();
+	gLuaAllocState.numLuaAllocs += 1;
+	gLuaAllocState.luaAllocTime += (t1 - t0).toMicroSecsi();
 
 	return mem;
-	#else
-	return (realloc(ptr, nsize));
-	#endif
 }
 
-void spring_lua_alloc_get_stats(SLuaInfo* info)
+void spring_lua_alloc_get_stats(SLuaAllocState* state)
 {
-	info->allocedBytes = totalBytesAlloced;
-	info->numLuaAllocs = totalNumLuaAllocs;
-	info->luaAllocTime = totalLuaAllocTime;
-	info->numLuaStates = mutexes.size() - coroutines.size();
+	state->allocedBytes.store(gLuaAllocState.allocedBytes.load());
+	state->numLuaAllocs.store(gLuaAllocState.numLuaAllocs.load());
+	state->luaAllocTime.store(gLuaAllocState.luaAllocTime.load());
+
+#if (ENABLE_USERSTATE_LOCKS != 0)
+	state->numLuaStates.store(mutexes.size() - coroutines.size();
+#else
+	state->numLuaStates.store(LuaMemPool::GetPoolCount());
+#endif
 }
 
-void spring_lua_alloc_update_stats(bool clear)
+bool spring_lua_alloc_get_error(SLuaAllocError* error)
 {
-	if (clear) {
-		totalNumLuaAllocs = 0;
-		totalLuaAllocTime = 0;
-	}
+	if (gLuaAllocError.msgBuf[0] == 0)
+		return false;
+
+	// copy and clear
+	std::memcpy(error->msgBuf, gLuaAllocError.msgBuf, sizeof(error->msgBuf));
+	std::memset(gLuaAllocError.msgBuf, 0, sizeof(gLuaAllocError.msgBuf));
+
+	gLuaAllocError.msgPtr = &gLuaAllocError.msgBuf[0];
+	return true;
 }
 
+void spring_lua_alloc_update_stats(int clearStatsFrame)
+{
+	gLuaAllocState.numLuaAllocs.store(gLuaAllocState.numLuaAllocs * (1 - clearStatsFrame));
+	gLuaAllocState.luaAllocTime.store(gLuaAllocState.luaAllocTime * (1 - clearStatsFrame));
+}
+
+
+
+
+
+
 //////////////////////////////////////////////////////////
 ////// Custom synced float to string
 //////////////////////////////////////////////////////////
@@ -236,7 +280,7 @@ void spring_lua_alloc_update_stats(bool clear)
 #ifdef WIN32
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wformat"
-static inline int sprintf64(char* dst, boost::int64_t x) { return sprintf(dst, "%I64d", x); }
+static inline int sprintf64(char* dst, std::int64_t x) { return sprintf(dst, "%I64d", x); }
 #pragma GCC diagnostic pop
 #else
 static inline int sprintf64(char* dst, long int x)      { return sprintf(dst, "%ld", x); }
@@ -247,7 +291,7 @@ static inline int sprintf64(char* dst, long long int x) { return sprintf(dst, "%
 // int numbers in that range are 100% exact, and don't suffer float precision issues
 static constexpr int MAX_PRECISE_DIGITS_IN_FLOAT = std::numeric_limits<float>::digits10;
 static constexpr auto SPRING_FLOAT_MAX = std::numeric_limits<float>::max();
-static constexpr auto SPRING_INT64_MAX = std::numeric_limits<boost::int64_t>::max();
+static constexpr auto SPRING_INT64_MAX = std::numeric_limits<std::int64_t>::max();
 
 static constexpr std::array<double, 11> v = {
 	1, 1e1, 1e2, 1e3, 1e4, 1e5, 1e6, 1e7, 1e8, 1e9, 1e10
@@ -256,24 +300,27 @@ static constexpr std::array<double, 11> v = {
 
 static constexpr inline double Pow10d(unsigned i)
 {
-	return (i<v.size()) ? v[i] : std::pow(double(10), i);
+	return (i < v.size()) ? v[i] : std::pow(double(10), i);
 }
 
 
 static const inline int FastLog10(const float f)
 {
 	assert(f != 0.0f); // log10(0) = -inf
-	if (f>=1.f && f<(SPRING_INT64_MAX >> 1)) {
-		const boost::int64_t i = f;
-		int log10 = 0;
-		boost::int64_t n = 10;
-		while (i >= n) {
-			++log10;
-			n *= 10;
-		}
-		return log10;
+
+	if (f < 1.0f || f >= (SPRING_INT64_MAX >> 1))
+		return std::floor(std::log10(f));
+
+	const std::int64_t i = f;
+	      std::int64_t n = 10;
+
+	int log10 = 0;
+	while (i >= n) {
+		++log10;
+		n *= 10;
 	}
-	return std::floor(std::log10(f));
+
+	return log10;
 }
 
 
@@ -296,11 +343,10 @@ static inline int PrintIntPart(char* buf, float f, const bool carrierBit = false
 		return sprintf(buf, "%d", int(f) + carrierBit);
 	} else
 #endif
-	if (f < (SPRING_INT64_MAX - carrierBit)) {
-		return sprintf64(buf, boost::int64_t(f) + carrierBit); // much faster than printing a float!
-	} else {
-		return sprintf(buf, "%1.0f", f + carrierBit);
-	}
+	if (f < (SPRING_INT64_MAX - carrierBit))
+		return sprintf64(buf, std::int64_t(f) + carrierBit); // much faster than printing a float!
+
+	return sprintf(buf, "%1.0f", f + carrierBit);
 }
 
 
@@ -317,8 +363,8 @@ static inline int PrintFractPart(char* buf, float f, int digits, int precision)
 	const auto old = buf;
 
 	assert(digits <= 15);
-	assert(digits <= std::numeric_limits<boost::int64_t>::digits10);
-	const boost::int64_t i = double(f) * Pow10d(digits) + 0.5;
+	assert(digits <= std::numeric_limits<std::int64_t>::digits10);
+	const std::int64_t i = double(f) * Pow10d(digits) + 0.5;
 	char s[16];
 	const int len = sprintf64(s, i);
 	if (len < digits) {
@@ -343,27 +389,21 @@ static inline bool HandleRounding(float* fractF, int log10, int charsInStdNotati
 	// We handle here the case when rounding in the
 	// fract part carries into the integer part.
 	// We don't handle the fract rounding itself!
+	// fDigits excludes the dot when precision is < 0
+	const int iDigits = mix(1, charsInStdNotation, (!scienceNotation && log10 >= 0));
+	const int fDigits = mix(std::max(0, nDigits - (iDigits + 1)), precision, (precision >= 0));
 
-	int iDigits = 1;
-	if (!scienceNotation) {
-		if (log10 >= 0) {
-			iDigits = charsInStdNotation;
-		}
-	}
-
-	int fDigits = std::max(0, nDigits - (iDigits + 1)); // excluding dot
-	if (precision >= 0) {
-		fDigits = precision;
-	}
-
+	// check fractional part against the rounding limit
 	// 1 -> 0.95   -%.1f-> 1.0
 	// 2 -> 0.995  -%.2f-> 1.00
 	// 3 -> 0.9995 -%.3f-> 1.000
-	const float roundLimit = 1.f - 0.5f * std::pow(0.1f, fDigits);
+	const float roundLimit = 1.0f - 0.5f * std::pow(0.1f, fDigits);
+
 	if (*fractF >= roundLimit) {
 		*fractF = 0.0f;
 		return true;
 	}
+
 	return false;
 }
 
@@ -410,17 +450,24 @@ void spring_lua_ftoa(float f, char* buf, int precision)
 	if ((charsInStdNotation > nDigits) && (precision == -1)) {
 		e10 = log10;
 		nDigits -= 4; // space needed for "e+01"
-		f *= std::pow(10.f, -e10);
+		f *= std::pow(10.0f, -e10);
 	}
-	const bool scienceNotation = (e10 != 0);
 
 	float truncF;
 	float fractF = std::modf(f, &truncF);
+
+	const bool scienceNotation = (e10 != 0);
 	const bool carrierBit = HandleRounding(&fractF, log10, charsInStdNotation, nDigits, scienceNotation, precision);
 
-	const int iDigits = PrintIntPart(buf, truncF, carrierBit);
-	if (scienceNotation)
+	int iDigits = PrintIntPart(buf, truncF, carrierBit);
+	if (scienceNotation) {
+		if (iDigits == 2) {
+			iDigits = 1;
+			e10 += 1;
+			assert(fractF == 0);
+		}
 		assert(iDigits == 1);
+	}
 	nDigits -= iDigits;
 	buf += iDigits;
 
@@ -436,9 +483,10 @@ void spring_lua_ftoa(float f, char* buf, int precision)
 		buf += fDigits;
 	}
 
-	if (scienceNotation) {
-		sprintf(buf, "e%+02d", e10);
-	}
+	if (!scienceNotation)
+		return;
+
+	sprintf(buf, "e%+02d", e10);
 }
 
 
@@ -485,3 +533,4 @@ void spring_lua_format(float f, const char* fmt, char* buf)
 	// copy the float string into dst
 	memcpy(buf, bufC, len+1);
 }
+
