@@ -9,26 +9,56 @@
 #ifndef STREFLOP_COND_H
 #define STREFLOP_COND_H
 
-#if (defined(STREFLOP_X87) || defined(STREFLOP_SSE) || defined(STREFLOP_SOFT)) && (!defined(NOT_USING_STREFLOP))
+#if (!defined(NOT_USING_STREFLOP) && (defined(STREFLOP_SSE) || defined(STREFLOP_X87) || defined(STREFLOP_SOFT)))
+#define STREFLOP_ENABLED 1
+#endif
+
+// these need to be known in FastMath.h and myMath.h which both include us
+#ifdef __GNUC__
+	#define _const __attribute__((const))
+	#define _pure __attribute__((pure))
+	#define _warn_unused_result __attribute__((warn_unused_result))
+#else
+	#define _const
+	#define _pure
+	#define _warn_unused_result
+#endif
+
+
+
+#if STREFLOP_ENABLED
 #include "streflop.h"
+
 namespace math {
 	using namespace streflop;
 }
+
 #else
 #include <cmath>
 
+namespace streflop {
+	typedef float Simple;
+	typedef double Double;
+	template<typename T> void streflop_init() {}
+};
+
+
+
 #ifdef __APPLE__
 // macosx's cmath doesn't include c++11's std::hypot yet (tested 2013)
 namespace std {
 	template<typename T> T hypot(T x, T y);
-}
+};
 #endif
 
 
+
 namespace math {
 	using std::fabs;
-	// We are using fastmath::sqrt_sse instead!
+
+	// see FastMath NOTE below
 	// using std::sqrt;
+
 	using std::sin;
 	using std::cos;
 
@@ -47,18 +77,31 @@ namespace math {
 	using std::hypot;
 	using std::pow;
 	using std::log;
+	using std::log2;
 	using std::log10;
 	using std::exp;
 	using std::frexp;
 	using std::ldexp;
+	using std::round;
+	using std::erf;
+
+	// these are in streflop:: but not in std::, FastMath adds sqrtf
+	// static inline float sqrtf(float x) { return std::sqrt(x); }
+	static inline float cosf(float x) { return std::cos(x); }
+	static inline float sinf(float x) { return std::sin(x); }
+	static inline float tanf(float x) { return std::tan(x); }
+	static inline float acosf(float x) { return std::acos(x); }
+	static inline float fabsf(float x) { return std::fabs(x); }
+
+
 // the following are C99 functions -> not supported by VS C
 #if !defined(_MSC_VER) || _MSC_VER < 1500
 	using std::isnan;
 	using std::isinf;
 	using std::isfinite;
-
-#elif __cplusplus
+#else
 }
+
 #include <limits>
 namespace math {
 	template<typename T> inline bool isnan(T value) {
@@ -76,9 +119,16 @@ namespace math {
 }
 
 
-#ifdef __APPLE__
-// see above
 
+// NOTE:
+//   for non-streflop builds we replace std::sqrt by fastmath::sqrt_sse in math::
+//   any code that only includes streflop_cond.h (assimp, etc) and not FastMath.h
+//   would not know about this without also including the latter, do so here
+#include "System/FastMath.h"
+
+
+
+#ifdef __APPLE__
 #include <algorithm>
 
 namespace std {
@@ -86,14 +136,16 @@ namespace std {
 	T hypot(T x, T y) {
 		x = std::abs(x);
 		y = std::abs(y);
-		auto t = std::min(x,y);
-		     x = std::max(x,y);
+		auto t = std::min(x, y);
+		     x = std::max(x, y);
 		t = t / x;
-		return x * sqrtf(1.f + t*t);
+		return x * math::sqrtf(1.0f + t*t);
 	}
 }
 #endif
 
-#endif
+
+
+#endif // STREFLOP_ENABLED
 
 #endif // STREFLOP_COND_H
