@@ -4,6 +4,7 @@
 #include <boost/test/unit_test.hpp>
 
 #include "System/TimeProfiler.h"
+#include "System/Misc/SpringTime.h"
 #include "System/Log/ILog.h"
 #include "System/Misc/SpringTime.h"
 #include <cmath>
@@ -11,6 +12,8 @@
 #include <boost/chrono/include.hpp> // boost chrono
 #include <boost/thread.hpp>
 
+BOOST_GLOBAL_FIXTURE(InitSpringTime);
+
 // #define BOOST_MONOTONIC_RAW_CLOCK
 
 static const int testRuns = 1000000;
@@ -159,8 +162,6 @@ struct TestProcessor {
 BOOST_AUTO_TEST_CASE( ClockQualityCheck )
 {
 	LOG("Clock Precision Test");
-	spring_clock::PushTickRate();
-	spring_time::setstarttime(spring_time::gettime(true));
 
 	#ifdef BOOST_CHRONO_HAS_CLOCK_STEADY
 	LOG("[%s] BOOST_CHRONO_HAS_CLOCK_STEADY defined --> CLOCK_MONOTONIC", __FUNCTION__);
@@ -361,3 +362,19 @@ BOOST_AUTO_TEST_CASE( ThreadSleepTime )
 	BenchmarkSleepFnc("sleep_spring", &sleep_spring, 500, 1e0);
 	BenchmarkSleepFnc("sleep_spring2", &sleep_spring2, 500, 1e6);
 }
+
+
+BOOST_AUTO_TEST_CASE(Timer)
+{
+
+	ScopedTimer t2("test");
+	ScopedOnceTimer t("test");
+	sleep_spring(500);
+
+	BOOST_CHECK(t2.GetDuration().toMilliSecsi() >= 450);
+	BOOST_CHECK(t.GetDuration().toMilliSecsi() >= 450);
+
+	BOOST_CHECK(t2.GetDuration().toMilliSecsi() <= 550);
+	BOOST_CHECK(t.GetDuration().toMilliSecsi() <= 550);
+}
+
