@@ -23,6 +23,8 @@ if [ "$(cat /proc/sys/kernel/core_uses_pid)" != "1" ]; then
 	exit 1
 fi
 
+echo "Env: GAME=$GAME MAP=$MAP AI=$AI AIVER=$AIVER"
+
 # enable core dumps
 ulimit -c unlimited
 
@@ -36,15 +38,18 @@ fi
 
 # limit to 1.5GB RAM
 #ulimit -v 1500000
-# max 4 min cpu time
-ulimit -t 240
+# max 5 min cpu time
+ulimit -t 300
 
 # delete path cache
 rm -rf ~/.config/spring/cache/
 
-# start up the client in background
-$RUNCLIENT "$1 --nocolor" &
-PID_CLIENT=$!
+if [ "$GAME" != "devgame:test" ];
+then
+	# start up the client in background
+	$RUNCLIENT "$1 --nocolor" &
+	PID_CLIENT=$!
+fi
 
 # start host
 echo "Starting Host"
@@ -60,20 +65,26 @@ echo waiting for host to exit, pid: $PID_HOST
 wait $PID_HOST
 EXIT=$?
 
-echo waiting for client to exit, pid: $PID_CLIENT
-# get spring client process exit code / wait for exit
-wait $PID_CLIENT
-EXITCHILD=$?
 
-#reenable abbort on error
-set -e
-
-# exit with exit code of server/client if failed
-if [ $EXITCHILD -ne 0 ];
+if [ "$GAME" != "devgame:test" ];
 then
-	echo Client exited with $EXITCHILD
-	exit $EXITCHILD
+	echo waiting for client to exit, pid: $PID_CLIENT
+	# get spring client process exit code / wait for exit
+	wait $PID_CLIENT
+	EXITCHILD=$?
+
+	#reenable abbort on error
+	set -e
+
+	# exit with exit code of server/client if failed
+	if [ $EXITCHILD -ne 0 ];
+	then
+		echo Client exited with $EXITCHILD
+		exit $EXITCHILD
+	fi
+
 fi
 
+echo Server exited with $EXIT
 exit $EXIT
 
