@@ -6,12 +6,12 @@
 #include "Logger.h"
 #include <string.h>
 
-FileData::FileData():
-	name(""),
-	size(0),
-	compsize(0),
-	download(false),
-	mode(0644)
+FileData::FileData()
+    : name("")
+    , size(0)
+    , compsize(0)
+    , download(false)
+    , mode(0644)
 {
 	memset(md5, 0, sizeof(md5));
 	memset(crc32, 0, sizeof(crc32));
