@@ -10,7 +10,7 @@
 	#warning PLAIN_API_STRUCTURE is defined -> functions will NOT be properly exported!
 #else
 	#include "unitsync.h"
-	#include "System/exportdefines.h"
+	#include "System/ExportDefines.h"
 #endif
 
 // from unitsync.cpp:
@@ -1138,81 +1138,6 @@ EXPORT(const char*) lpGetStrKeyStrVal(const char* key, const char* defValue);
 /* deprecated functions */
 
 #ifdef ENABLE_DEPRECATED_FUNCTIONS
-/**
- * @deprecated in June 2011, use ProcessUnits() instead
- */
-EXPORT(int         ) ProcessUnitsNoChecksum();
-#if       !defined(PLAIN_API_STRUCTURE)
-/**
- * @deprecated use GetMapOptionCount instead
- */
-EXPORT(int         ) GetMapInfoEx(const char* mapName, MapInfo* outInfo, int version);
-/**
- * @deprecated use GetMapOptionCount instead
- */
-EXPORT(int         ) GetMapInfo(const char* mapName, MapInfo* outInfo);
-#endif // !defined(PLAIN_API_STRUCTURE)
-
-/**
- * @deprecated use GetMapInfoCount instead
- */
-EXPORT(const char* ) GetMapDescription(int index);
-EXPORT(const char* ) GetMapAuthor(int index);
-EXPORT(int         ) GetMapWidth(int index);
-EXPORT(int         ) GetMapHeight(int index);
-EXPORT(int         ) GetMapTidalStrength(int index);
-EXPORT(int         ) GetMapWindMin(int index);
-EXPORT(int         ) GetMapWindMax(int index);
-EXPORT(int         ) GetMapGravity(int index);
-EXPORT(int         ) GetMapResourceCount(int index);
-EXPORT(const char* ) GetMapResourceName(int index, int resourceIndex);
-EXPORT(float       ) GetMapResourceMax(int index, int resourceIndex);
-EXPORT(int         ) GetMapResourceExtractorRadius(int index, int resourceIndex);
-EXPORT(int         ) GetMapPosCount(int index);
-EXPORT(float       ) GetMapPosX(int index, int posIndex);
-EXPORT(float       ) GetMapPosZ(int index, int posIndex);
-
-/**
- * @deprecated use GetInfoValue* instead
- */
-EXPORT(const char* ) GetInfoValue(int index);
-/**
- * @deprecated use the mod info item with key "name" instead, see GetPrimaryModInfoCount / GetInfoKey
- */
-EXPORT(const char* ) GetPrimaryModName(int index);
-/**
- * @deprecated use the mod info item with key "shortName" instead, see GetPrimaryModInfoCount / GetInfoKey
- */
-EXPORT(const char* ) GetPrimaryModShortName(int index);
-/**
- * @deprecated use the mod info item with key "version" instead, see GetPrimaryModInfoCount / GetInfoKey
- */
-EXPORT(const char* ) GetPrimaryModVersion(int index);
-/**
- * @deprecated use the mod info item with key "mutator" instead, see GetPrimaryModInfoCount / GetInfoKey
- */
-EXPORT(const char* ) GetPrimaryModMutator(int index);
-/**
- * @deprecated use the mod info item with key "game" instead, see GetPrimaryModInfoCount / GetInfoKey
- */
-EXPORT(const char* ) GetPrimaryModGame(int index);
-/**
- * @deprecated use the mod info item with key "shortGame" instead, see GetPrimaryModInfoCount / GetInfoKey
- */
-EXPORT(const char* ) GetPrimaryModShortGame(int index);
-/**
- * @deprecated use the mod info item with key "description" instead, see GetPrimaryModInfoCount / GetInfoKey
- */
-EXPORT(const char* ) GetPrimaryModDescription(int index);
-/**
- * @deprecated use OpenArchive instead
- */
-EXPORT(int         ) OpenArchiveType(const char* name, const char* type);
-/**
- * @deprecated unused
- */
-EXPORT(const char* ) GetOptionStyle(int optIndex);
-
 #endif // ENABLE_DEPRECATED_FUNCTIONS
 /** @} */
 
